/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2004 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Interchange.c  :                                                  //
'**  '// version        :   1.0                                            //
'**  '// Language       :   C                                              //
'**  '// Development    :   Windows 2000 PentiumIII 500Mhz                 //
'**  '// Application    :   DTP                                            //
'**  '// Authors        :   Welch Allyn Inc. MedR&D                        //
'**  '// Sources        :   COOA                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include <stddef.h>

#include "InterChange.h"
#include "Mem.h"            /* CRC and ReadIntxx()/WriteIntxx() */

/* Local variables */

typedef enum
{
    GET_IDLE,               // waiting for 1st preamble character
    GET_WACPRE2,            // waiting for 2nd WAC preamble character
    GET_WACPRE3,            // waiting for 3rd WAC preamble character
    GET_WACLN1,             // waiting for 1st packet length character
    GET_WACLN2,
    GET_WACLN3,
    GET_WACLN4,
    GET_WACPT1,             // waiting for 1st port character */
    GET_WACPT2,
    GET_WACDTA,
    GET_WACCRC1,            // waiting for 1st CRC charcter */
    GET_WACCRC2
} CM_STATE;

#define MAXDATA 180          /* Total size of buffer */

static uint8 DataBuf[MAXDATA];
static uint8 OutBuffer[PACKET_LITE_HEADER_SIZE];
static uint8 CRCBuffer[CRCSIZE];

static uint16  Port;

/* Local function prototypes */
static DataReadyEventCallBack pDataReadyEvent;
static uint32 InterChange_ProcessPackets();

/* -------------------------------------------------------------------- */
/* Public Interface                                                     */
/* -------------------------------------------------------------------- */

/*-D******************************************************************
*
*  FUNCTION:  void comm_thread(void)
*
*  SYNOPSIS:  This module controls the serial communications
*             Simplified flow code:
*             forever loop
*               get a packet / command
*               process it
*             end loop
*
*  ARGUMENTS: None
*
*  DATA ALTERED: Potentially any and all RAM and PROM
*
*  RETURNS:   This function never returns.
*
**-E*****************************************************************/

#pragma FUNC_EXT_CALLED (comm_thread)
void comm_thread(void)
{
    uint32 DataSize;
    for(;;)
    {               
        DataSize = InterChange_ProcessPackets();    
        
        /* Process WACP data */              
        if (pDataReadyEvent != NULL)
            pDataReadyEvent(DataBuf, DataSize);
    }   
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */

void InterChange_Init(DataReadyEventCallBack pFuncPtr)
{
    /* Initailize local variables */
    Port= DF_PORT;
    
    /* Initialize the callback function pointer */
    pDataReadyEvent = pFuncPtr;
    
    /* Initialize the uart layer */
    initUart();
}

void InterChange_SetPort(uint16 port)
{
    Port = port;
}

/*================================================================================
InterChange Send Packet: Wraps the passed in Buffer into a PACKET structure and
    sends the packet to the appropriate port.
================================================================================*/
boolean InterChange_SendPacket(uint8 *pBuf, uint32 Size)
{
    boolean RetVal = FALSE;
    boolean ByteAligned = TRUE;    
    uint32 PacketLen;
    uint16 CRC;
    uint8 *pTemp;
    uint32 Result;
    
    /* Initailize CRC and packet header buffer pointer */
    CRC   = CRC_SEED;
    pTemp = OutBuffer; 

    /* Set the Size of the entire packet */
    PacketLen = PACKET_LITE_HEADER_SIZE + Size + CRCSIZE;

    /* Serialize the packet header struct */
    /* Set the PREAMBLE */    
    WriteInt8(&pTemp, CTL_W, &ByteAligned);
    WriteInt8(&pTemp, CTL_A, &ByteAligned);
    WriteInt8(&pTemp, CTL_L, &ByteAligned);

    /* Write the Packet Length */
    WriteInt32(&pTemp, PacketLen, &ByteAligned);

    /* Write the Port */
    WriteInt16(&pTemp, Port, &ByteAligned);

    /* Compute CRC on header */
    CRC = CalculateCRC(CRC, OutBuffer, PACKET_LITE_HEADER_SIZE, TRUE);

    /* Compute CRC on the data */
    CRC = CalculateCRC(CRC, pBuf, Size, TRUE);

    /* Reset the ByteAligned flag */
    ByteAligned = TRUE;
    
    /* Serialize the CRC value */
    pTemp = CRCBuffer; 

    WriteInt16(&pTemp, CRC, &ByteAligned);

    /* Send out the packet header */
    Result = TxBuf(OutBuffer, (uint32)PACKET_LITE_HEADER_SIZE);

    /* Uncomment the following checks when things stabilize */
    TOUCH(Result);
    //if (Result == PACKET_HEADER_SIZE)
    {
        /* Send out the data */
        Result = TxBuf(pBuf, Size);
        //if (Result == Size)
        {   
            /* Send out the CRC */
            Result = TxBuf(CRCBuffer, (uint32)CRCSIZE);
            //if (Result == CRCSIZE)
            {   
                /* Set the return value to TRUE */
                RetVal = TRUE;
            }
        }
    }
    
    return( RetVal );
}

/* -------------------------------------------------------------------- */
/* Local functions                                                      */
/* -------------------------------------------------------------------- */

/*================================================================================
InterChange Process Packets: This function loops until a full Packet has been
    received. It then calls the appropriate callback function to either process
    the Packet's contents or handle receive errors.
================================================================================*/
uint32 InterChange_ProcessPackets()
{
    CM_STATE state = GET_IDLE;          /* Command decoding state */
    byte c;                             /* Holder for received byte */
    uint16 i = 0;                       /* Data byte counter */
    uint16 crc = CRC_SEED;
    boolean ByteAligned = TRUE;
    uint8 *pTemp = DataBuf;    
    PACKET_HDR Cmd;
    
    for (;;) {        
        RxBuf(&c, 1);
        switch (state) {

            case GET_IDLE:                      /* Waiting for first preamble */
                Cmd.Preamble[0] = c;
                state = (c == CTL_W) ? GET_WACPRE2 : GET_IDLE;
                crc = CRC_SEED;                   /* Init CRC */
                pTemp = DataBuf;                  /* Init the temp pointer */
                crc = CalculateCRC(crc, &c, sizeof(c), FALSE);
                break;

            case GET_WACPRE2:                    /* Expectin g 2nd preamble */
                Cmd.Preamble[1] = c;
                state = (c == CTL_A) ? GET_WACPRE3 : GET_IDLE;
                crc = CalculateCRC(crc, &c, sizeof(c), FALSE);
                break;

            case GET_WACPRE3:                    /* Expecting for 3rd preamble */
                Cmd.Preamble[2] = c;
                state = (c == CTL_L) ? GET_WACLN1 : GET_IDLE;
                crc = CalculateCRC(crc, &c, sizeof(c), FALSE);
                break;

            case GET_WACLN1:                    /* Received 1st length char */
                Cmd.PacketLen = (uint32)c << 24;    /* MSB of packet length */
                state = GET_WACLN2;             /* Get 2nd packet length byte */
                crc = CalculateCRC(crc, &c, sizeof(c), FALSE);
                break;
                
            case GET_WACLN2:                    /* Received 2nd length charcater */
                Cmd.PacketLen += (uint32)c << 16;
                state = GET_WACLN3;
                crc = CalculateCRC(crc, &c, sizeof(c), FALSE);
                break;
                
            case GET_WACLN3:                    /* Received 3rd length character */
                Cmd.PacketLen += (uint16)c << 8;
                state = GET_WACLN4;
                crc = CalculateCRC(crc, &c, sizeof(c), FALSE);
                break;
                
            case GET_WACLN4:                    /* Received 4th length character */
                Cmd.PacketLen += c;
                state = GET_WACPT1;
                crc = CalculateCRC(crc, &c, sizeof(c), FALSE);
               break;
                
            case GET_WACPT1:                    /* Recived 1st "port" character */
                Cmd.Port = (uint16)c << 8;
                state = GET_WACPT2;
                crc = CalculateCRC(crc, &c, sizeof(c), FALSE);
                break;
                
            case GET_WACPT2:                    /* Received 2nd "port" character */
                Cmd.Port = Cmd.Port + c;
                state = GET_WACDTA;
                Cmd.Length = Cmd.PacketLen - PACKET_LITE_HEADER_SIZE - CRCSIZE;
                i = 0;
                crc = CalculateCRC(crc, &c, sizeof(c), FALSE);
                break;                

            case GET_WACDTA:                    /* Received data or CRC */                
                WriteInt8(&pTemp, c, &ByteAligned); /* Buffer it */
                crc = CalculateCRC(crc, &c, sizeof(c), FALSE);
                i++;
                if (i >= Cmd.Length)             /* Last data byte? */
                    state = GET_WACCRC1;        /* Yes, CRC bytes next */
                break;
                
            case GET_WACCRC1:                   /* Received CRC high byte */
                crc = crc - ((uint16)c << 8);     /* "Compare" high byte */
                state = GET_WACCRC2;
                break;
                    
            case GET_WACCRC2:                   /* Received CRC low byte */
                crc = crc - c;                  /* "Compare" low byte */
                if (crc != 0)                   /* CRC error? */
                {
                    Cmd.Length = 0;             /* Yes, destroy data length */
                    return (Cmd.Length);        
                }
                else
                    return (Cmd.Length);           /* Return data for processing */

            default:
                state = GET_IDLE;
                break;

        } /* switch */
    } /* forever loop */
}




