/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CConObjSvr.c   :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CConObjSvr.h"
#include "Wacp.h"

/*********************************************************************************
CConObjSvr Start up: 
*********************************************************************************/
void CConObjSvr_Init(CConObjSvr* pObject, CLinkTable* pLinkTable, int32 ThreadPriority)
{
    pObject->ThreadStatus = TRUE;
    pObject->pLinkTable = pLinkTable;
    pObject->Count = 0;
    
    /* Initialize the mutex for accessing the Connection Object server data */
    CWACPMutex_Init(&pObject->ConnMutex, NULL);
    CWACPMutex_Init(&pObject->ExitMutex, NULL);

    /* Initialize the semaphore for the KeepAliveThread */
    CWACPSemaphore_Init(&pObject->TimeOutSem, NULL, 0, 1);

    /* Start up the KeepAlive thread to monitor connection timeouts */
    CWACPThread_Init(&pObject->ConnObjThread, CONNOBJSVR_THREAD);
    CWACPThread_StartThread(&pObject->ConnObjThread, CConObjSvr_KeepAliveThread, pObject, 
                            ThreadPriority);
}

void CConObjSvr_Destroy(CConObjSvr* pObject)
{
    pObject->ThreadStatus = FALSE;

    /* Notify the KeepAlive thread to check the ThreadStatus */
    CWACPSemaphore_Raise(&pObject->TimeOutSem);

    /* Allow the KeepAliveThread to terminate */
    SYS_SLEEP(1);

    /* Wait for the Thread to terminate */
    CWACPMutex_Lock(&pObject->ExitMutex, INFINITE);

    /* Destroy the Exit mutex */
    CWACPMutex_Destroy(&pObject->ExitMutex);

    /* Destroy the Exit Semaphore object */
    CWACPSemaphore_Destroy(&pObject->TimeOutSem);

    /* Destroy the Connection Object server thread */
    CWACPThread_Destroy(&pObject->ConnObjThread);
    
    /* Destroy the Connection Object server mutex */
    CWACPMutex_Destroy(&pObject->ConnMutex);
}

/*================================================================================
CConObjSvr AddConnection: Adds a new connection to the ConnObjSvr.
================================================================================*/
boolean CConObjSvr_AddConnection(const CConObjSvr* pObject, const char* pSourceID)
{
    boolean Result = FALSE;
    int32 Index = -1;
    
    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);
    
    /* Find the index into the CLinkTable entry */
    Index = CLinkTable_GetEntryIndex(pObject->pLinkTable, pSourceID);
    if (Index != -1)
    {
        ConnObj *pConnObj = (ConnObj*)SYS_MALLOC(sizeof(ConnObj));
        if (pConnObj)
        {
            /* Setup the initial connection defaults from target.h */
            pConnObj->TxOptions = DEFAULT_CONN_SETTING;
            pConnObj->KeepAliveStatus = KEEP_ALIVE_STOPPED;
            pConnObj->TimeoutInterval = DEFAULT_TIMEOUT_INTERVAL;
            pConnObj->KeepAliveInterval = DEFAULT_KEEPALIVE_INTERVAL;

            /* Add the new object into the CLinkTable list */
            pObject->pLinkTable->Connection[Index]->pConnObj = pConnObj;
        }
    }

    CWACPMutex_Release(&pObject->ConnMutex);

    return (Result);
}

/*================================================================================
CConObjSvr RemoveConnection: Removes a connection from the ConnObjSvr.
================================================================================*/
boolean CConObjSvr_RemoveConnection(const CConObjSvr* pObject, const char* pSourceID)
{
    boolean Result = FALSE;
    int32 Index = 0;
    
    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);

    /* Find the index into the CLinkTable entry */
    Index = CLinkTable_GetEntryIndex(pObject->pLinkTable, pSourceID);
    if (Index != -1)
    {
        /* Clean up memory allocated in AddConnection */
        SYS_FREE(pObject->pLinkTable->Connection[Index]->pConnObj);
        pObject->pLinkTable->Connection[Index]->pConnObj = NULL;
    }

    CWACPMutex_Release(&pObject->ConnMutex);

    return (Result);
}


/*================================================================================
CConObjSvr GetConnObj: Returns a pointer to the ConnObj that matches the pSourceID,
    otherwise it returns NULL.
================================================================================*/
ConnObj* CConObjSvr_GetConnObj(const CConObjSvr* pObject, const char* pSourceID)
{
    ConnObj     *pConnObj = NULL;
    ConnArgs    *pConnArgs = NULL;

    /* Get a pointer to the ConnArgs for pSourceID */
    pConnArgs = CLinkTable_GetEntry(pObject->pLinkTable, pSourceID);
    if (pConnArgs)
    {
        pConnObj = pConnArgs->pConnObj;
    }

    return (pConnObj);
}
/*================================================================================
CConObjSvr SetTxOptions : Sets the Options for what the Conn server sends to keep 
         the connection alive.
         CONN_CFG_SpKEEPALIVEON - Turn on keep alive messages
         CONN_CFG_SpKEEPALIVEOFF - Turn off keep alive messages
         CONN_CFG_SpKEEPALIVECHECK - Turn on keep alive checks without messages.
================================================================================*/
boolean CConObjSvr_SetTxOptions(const CConObjSvr* pObject, const char* pSourceID, uint8 Options)
{
    boolean Result = FALSE;

    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);

    /* Check for a valid range of Options */
    if (Options < CONN_CFG_SpMAX)
    {
        ConnObj *pConnObj = CConObjSvr_GetConnObj(pObject, pSourceID);
        if (pConnObj)
        {
            /* Update the TxOption with the new option */
            pConnObj->TxOptions = Options;
            
            /* Reset the KeepAlive status and timer */
            CConObjSvr_StopKeepAlive(pObject, pSourceID);

            /* Start up the KeepAlive mechansim */
            CConObjSvr_StartKeepAlive(pObject, pSourceID);

#if defined(TRACE_SERVER_ON_STATUS)
            CTraceSvr_tprintf(&TraceServer, TRACE_Type_STATUS,
                    "CConObjSvr", "SetTxOptions", 0, 0, 0, 
                    "Synching TxOptions to %d\n", Options);
#endif

            /* Tx the new Options to the other end of the connection to sync up */
            Result = WACP_SendMsg(pSourceID, FmCONNECTION, GnCONFIG, Options, NULL, 0, FALSE);
        }
    }

    CWACPMutex_Release(&pObject->ConnMutex);

    return Result;
}

/*================================================================================
CConObjSvr GetTxOptions : Gets the Options for what the Conn server sends to keep 
         the connection alive.
         CONN_CFG_SpKEEPALIVEON - Turn on keep alive messages
         CONN_CFG_SpKEEPALIVEOFF - Turn off keep alive messages
         CONN_CFG_SpKEEPALIVECHECK - Turn on keep alive checks without messages.
================================================================================*/
uint8 CConObjSvr_GetTxOptions(const CConObjSvr* pObject, const char* pSourceID)
{
    uint8 Options = 0;
    ConnObj *pConnObj = NULL;

    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);

    pConnObj = CConObjSvr_GetConnObj(pObject, pSourceID);
    if (pConnObj)
    {
        Options = pConnObj->TxOptions;
    }

    CWACPMutex_Release(&pObject->ConnMutex);

    return Options;
}

/*================================================================================
CConObjSvr SetKeepAliveInterval : Sets the KeepAlive message interval
================================================================================*/
boolean CConObjSvr_SetKeepAliveInterval(const CConObjSvr* pObject, const char* pSourceID, time_t Interval)
{
    boolean Result = FALSE;

    ConnObj *pConnObj = NULL;

    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);

    pConnObj = CConObjSvr_GetConnObj(pObject, pSourceID);

    if (pConnObj)
    {
        /* Update the KeepAliveInterval with the new value */
        pConnObj->KeepAliveInterval = Interval;
        
        Result = TRUE;
#if defined(TRACE_SERVER_ON_STATUS)
        CTraceSvr_tprintf(&TraceServer, TRACE_Type_STATUS,
                "CConObjSvr", "SetTxOptions", 0, 0, 0, 
                "Setting KeepAliveInterval to %d\n", Interval);
#endif

    }

    CWACPMutex_Release(&pObject->ConnMutex);

    return Result;
}

/*================================================================================
CConObjSvr SetKeepAliveInterval : Gets the KeepAlive message interval
================================================================================*/
time_t CConObjSvr_GetKeepAliveInterval(const CConObjSvr* pObject, const char* pSourceID)
{
    time_t Interval = 0;

    ConnObj *pConnObj = NULL;

    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);

    pConnObj = CConObjSvr_GetConnObj(pObject, pSourceID);

    if (pConnObj)
    {
        Interval = pConnObj->KeepAliveInterval;
    }

    CWACPMutex_Release(&pObject->ConnMutex);

    return Interval;
}

/*================================================================================
CConObjSvr SetTimeoutInterval : Sets the KeepAlive timeout
================================================================================*/
boolean CConObjSvr_SetTimeoutInterval(const CConObjSvr* pObject, const char* pSourceID, time_t TimeOut)
{
    boolean Result = FALSE;

    ConnObj *pConnObj = NULL;

    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);

    pConnObj = CConObjSvr_GetConnObj(pObject, pSourceID);
    if (pConnObj)
    {
        /* Update the KeepAliveInterval with the new value */
        pConnObj->TimeoutInterval = TimeOut;

        Result = TRUE;

#if defined(TRACE_SERVER_ON_STATUS)
        CTraceSvr_tprintf(&TraceServer, TRACE_Type_STATUS,
                "CConObjSvr", "SetTxOptions", 0, 0, 0, 
                "Setting SetTimeoutInterval to %d\n", TimeOut);
#endif

    }

    CWACPMutex_Release(&pObject->ConnMutex);

    return Result;
}

/*================================================================================
CConObjSvr SetTimeoutInterval : Gets the KeepAlive timeout
================================================================================*/
time_t CConObjSvr_GetTimeoutInterval(const CConObjSvr* pObject, const char* pSourceID)
{
    time_t TimeOut = 0;
    ConnObj *pConnObj = NULL;

    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);

    pConnObj = CConObjSvr_GetConnObj(pObject, pSourceID);
    if (pConnObj)
    {
        TimeOut = pConnObj->TimeoutInterval;
    }

    CWACPMutex_Release(&pObject->ConnMutex);

    return TimeOut;
}

/*================================================================================
CConObjSvr Disconnect : 
================================================================================*/
boolean CConObjSvr_Disconnect(const CConObjSvr* pObject, const char* pSourceID)
{
    boolean Result = FALSE;
    ConnObj *pConnObj = NULL;

    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);

    pConnObj = CConObjSvr_GetConnObj(pObject, pSourceID);
    if (pConnObj)
    {
        ConnArgs    *pConnArgs = NULL;

        /* Get a pointer to the ConnArgs for pSourceID */
        pConnArgs = CLinkTable_GetEntry(pObject->pLinkTable, pSourceID);
        if (pConnArgs)
        {
            /* Only send ShutDown message if RNDZStatus is TRUE */
            if (pConnArgs->bRNDZStatus == TRUE)
            {
                /* Send a message to disconnect the current session */
                Result = WACP_SendMsg(pSourceID, FmCONNECTION, GnCOMMAND, CONN_CMD_SpSHUTDOWN, NULL, 0, FALSE);
            }
        }

        /* Turn off the Keep Alive */
        pConnObj->KeepAliveStatus = KEEP_ALIVE_STOPPED;

        CConObjSvr_RemoveConnection(pObject, pSourceID);
    }

    CWACPMutex_Release(&pObject->ConnMutex);

    return Result;
}

/*================================================================================
CConObjSvr EvaluateKeepAlive : This function is called once for every DataThread
                             Cycle in WACPPres when we have no normal messages 
                             waiting to be recieved. 
================================================================================*/
boolean CConObjSvr_EvaluateKeepAlive(const CConObjSvr* pObject, char* pSourceID)
{   
    boolean Result = FALSE;
    time_t  NewTime = SYS_TIME(NULL);
    ConnObj *pConnObj = NULL;
    
    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);

    pConnObj = CConObjSvr_GetConnObj(pObject, pSourceID);
    if (pConnObj)
    {
        if (pConnObj->TxOptions != CONN_CFG_SpKEEPALIVEOFF)
        {
            switch (pConnObj->KeepAliveStatus)
            {
                /* KeepAlive timers have been activated, so check how they are */ 
                case KEEP_ALIVE_RUNNING:
                {
                    /* Only send the keep alive message if we are not in CHECK mode */
                    if (pConnObj->TxOptions != CONN_CFG_SpKEEPALIVECHECK)
                    {
                        /* Check to see if our KeepAlive timer has expired*/
                        if(pConnObj->KeepAliveInterval <= (NewTime - pConnObj->KeepAliveTimer))
                        {
                            /* Its time to send another KeepAlive message, so reset
                            the timer and send a message. We reset the timer here
                            so we will continue to send messages every interval 
                            until we recieve a Timeout error*/
                            pConnObj->KeepAliveTimer = NewTime;
              
#if defined(TRACE_SERVER_ON_STATUS)
                            CTraceSvr_tprintf(&TraceServer, TRACE_Type_SEND,
                                    "CConObjSvr", "EvaluateKeepAlive", FmCONNECTION, GnREQUEST, (uint8)CONN_CRSP_SpKEEPALIVE, 
                                    "Sending KeepAlive request\n");
#endif
                            /* Set the Species to the right message type */
                            Result = WACP_SendMsg(pSourceID, FmCONNECTION, GnREQUEST, CONN_CRSP_SpKEEPALIVE, NULL, 0, FALSE);
                        }
                    }
                    /* Check to see if our Timeout timer has expired*/
                    if(pConnObj->TimeoutInterval <= (NewTime - pConnObj->TimeoutTimer))
                    {
                        /* We have gone too long without a KeepAlive response*/
                        pConnObj->KeepAliveStatus = KEEP_ALIVE_TIMEOUT;
                    }
                    break;  
                }
                case KEEP_ALIVE_TIMEOUT:
                {
#if defined(TRACE_SERVER_ON)
                    CTraceSvr_tprintf(&TraceServer, TRACE_Type_STATUS,
                            "CConObjSvr", "EvaluateKeepAlive", 0, 0, 0, 
                            "KEEP_ALIVE_TIMEOUT\n");
#endif
                    /* Handle the timeout */
                    WACP_RecvMsg(pSourceID, FmCONNECTION, GnCOMMAND, CONN_CMD_SpSHUTDOWN, NULL);

                    pConnObj->KeepAliveStatus = KEEP_ALIVE_END;
                    break;
                }
                case KEEP_ALIVE_STOPPED:
                    /* We have tried to evaluate while in a STOPPED state, so do
                    nothing and leave */
                case KEEP_ALIVE_END:
                default:
                    break;

            }/* switch (pConnObj->KeepAliveStatus) */

        }/* if (pConnObj->TxOptions != CONN_CFG_SpKEEPALIVEOFF) */

    }/* if (pConnObj) */

    CWACPMutex_Release(&pObject->ConnMutex);

    return Result;
}
/*================================================================================
CConObjSvr StartKeepAlive : Start the KeepAlive and Timeout timers
================================================================================*/
boolean CConObjSvr_StartKeepAlive(const CConObjSvr* pObject, const char* pSourceID)
{
    boolean Result = FALSE;
    ConnObj *pConnObj = NULL;
    
    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);

    pConnObj = CConObjSvr_GetConnObj(pObject, pSourceID);
    if (pConnObj)
    {
        if (pConnObj->TxOptions != CONN_CFG_SpKEEPALIVEOFF)
        {
            if(pConnObj->KeepAliveStatus == KEEP_ALIVE_STOPPED)
            {
                time_t NewTime = SYS_TIME(NULL);

                /* Reset the Timeout timer to activate on a normal interval */
                pConnObj->TimeoutTimer = NewTime;

                /* Reset the KeepAlive timer to activate on a normal interval */
                pConnObj->KeepAliveTimer = NewTime;

                /* Set the KeepAlive state to RUNNING */
                pConnObj->KeepAliveStatus = KEEP_ALIVE_RUNNING;
            }
            Result = TRUE;
        }
    }
    
    CWACPMutex_Release(&pObject->ConnMutex);
    
    return Result;
}
/*================================================================================
CConObjSvr StopKeepAlive : Stops the KeepAlive and Timeout timers
================================================================================*/
boolean CConObjSvr_StopKeepAlive(const CConObjSvr* pObject, const char* pSourceID)
{   
    boolean Result = TRUE;
    ConnObj *pConnObj = NULL;
    
    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);

    pConnObj = CConObjSvr_GetConnObj(pObject, pSourceID);
    if (pConnObj)
    {
        pConnObj->KeepAliveStatus = KEEP_ALIVE_STOPPED;
        pConnObj->KeepAliveTimer = 0;
    }

    CWACPMutex_Release(&pObject->ConnMutex);

    return Result;
}

/*================================================================================
CConObjSvr RecvMsgHandler: Decodes the Family, Genus, and Species tag and deserializes
     the messages with the appropriate object.
================================================================================*/
boolean CConObjSvr_RecvMsgHandler (CConObjSvr* pObject, char* pSourceID, uint8* Buffer)
{
    uint32  Result = FALSE;
    uint8   Genus;
    uint8   Species;
    boolean RetVal=TRUE;

    CWACPMutex_Lock(&pObject->ConnMutex, INFINITE);

    /* Extract the Genus */
    Genus = GetGenus(Buffer);

    /* Extract the Species */
    Species = GetSpecies(Buffer);

    /* Set the Genus and Species to the right message type */
    CMsgSvr_Init(&pObject->MsgSvr, FmCONNECTION, Genus, Species);

    /* Deserialize the data and check for errors */
    Result = CMsgSvr_Deserialize(&pObject->MsgSvr, pSourceID, Buffer);
    if (Result)
    {
        ConnObj *pConnObj = CConObjSvr_GetConnObj(pObject, pSourceID);
        
        if (pConnObj)
        {
            /* Species Router */
            switch (Genus)
            {           
                /*  If we are handling a request, then check for timeout, reset our timers,
                    *  and send back a Response message 
                    */
                case GnREQUEST:
                {
                    /*  We got a keepalive request, so the other end is alive. Update
                        our timer. */
                    pConnObj->TimeoutTimer = SYS_TIME(NULL);

#if defined(TRACE_SERVER_ON_STATUS)
                    CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                            "CConObjSvr", "RecvMsgHandler", FmCONNECTION, Genus, Species, 
                            "%s Received KeepAlive request\n", pSourceID);

                    CTraceSvr_tprintf(&TraceServer, TRACE_Type_SEND,
                            "CConObjSvr", "RecvMsgHandler", FmCONNECTION, GnRESPONSE, CONN_CRSP_SpKEEPALIVE, 
                            "%s Sending KeepAlive response\n", pSourceID);
#endif
                    /* Send the message */
                    Result = WACP_SendMsg(pSourceID, FmCONNECTION, GnRESPONSE, CONN_CRSP_SpKEEPALIVE, NULL, 0, FALSE);

                    break;
                }
                /*  If we get a Response, then we know our Request has been acknowledged, so just
                *  reset our timers. 
                */
                case GnRESPONSE:
                {
                    time_t NewTime = SYS_TIME(NULL);
#if defined(TRACE_SERVER_ON_STATUS)
                    CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                            "CConObjSvr", "RecvMsgHandler", FmCONNECTION, GnRESPONSE, Species, 
                            "%s Received KeepAlive response\n", pSourceID);
#endif
                    /* Check to see if our Timeout timer has expired*/
                    if(pConnObj->TimeoutInterval < (NewTime - pConnObj->TimeoutTimer))
                    {
                        /* We have gone too long without a KeepAlive response*/
                        pConnObj->KeepAliveStatus = KEEP_ALIVE_TIMEOUT;
                    }
                    else
                    {
                        /*  We got a keepalive request, so the other end is alive. Update
                            our timer. */
                        pConnObj->TimeoutTimer = NewTime;
                    }    
                    break;
                }
                /* We received a TxOption message, so sync our settings with the sender'S */
                case GnCONFIG:
                {
#if defined(TRACE_SERVER_ON_STATUS)
                    CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                            "CConObjSvr", "RecvMsgHandler", FmCONNECTION, GnCONFIG, Species, 
                            "%s Received new TxOption of %d\n", pSourceID, Species);
#endif
                    pConnObj->TxOptions = Species;
                    break;
                }
                case GnCOMMAND:
                {
                    /* Handle shutdown message */
#if defined(TRACE_SERVER_ON)
                    CTraceSvr_tprintf(&TraceServer, TRACE_Type_RECEIVE,
                            "CConObjSvr", "SetTxOptions", FmCONNECTION, GnCOMMAND, Species, 
                            "%s Received Shutdown command status\n", pSourceID);
#endif                
                    WACP_RecvMsg(pSourceID, FmCONNECTION, GnCOMMAND, Species, NULL);
                    break;
                }
                default:
                {
                    /* We have received an unknown connection type */
                    break;
                }
            }/* switch (Species)*/

        }/* if (pConnObj) */

    }/* if (Result) */

    /* Free MsgSvr.Data */                
    SYS_FREE(pObject->MsgSvr.Data);
    pObject->MsgSvr.Data = NULL;

    CWACPMutex_Release(&pObject->ConnMutex);

    return RetVal;
}

/*================================================================================
CConObjSvr KeepAliveThread: Main thread for handling KeepAlives for the
    ConnObjSvr.
================================================================================*/
uint32 WACPAPI CConObjSvr_KeepAliveThread(void* pData)
{
    uint32          Result;
    CConObjSvr      *pObject = (CConObjSvr*)pData;
    uint16          i;

    CWACPMutex_Lock(&pObject->ExitMutex, INFINITE);

#if defined(TRACE_SERVER_ON_STATUS)
    CTraceSvr_tprintf(&TraceServer, TRACE_Type_STATUS,
            "CConObjSvr", "KeepAliveThread", FmCONNECTION, 0, 0, 
            "Started \n");
#endif   

    /* Wait for the TimeOut semaphore to fire for processing a terminate or until we time out.*/
    Result = CWACPSemaphore_Wait(&pObject->TimeOutSem, 1000);

    while(pObject->ThreadStatus == TRUE)
    {
        char* pSourceList[MAX_CONNECTIONS];
        
        /* Get a list of all entries in linktable and shut down all of them individually */
        CLinkTable_GetSourceList(pObject->pLinkTable, pSourceList);
        for (i=0; pSourceList[i] != NULL; i++)
        {
            ConnArgs    *pConnArgs = NULL;

            /* Grab the ConnArgs from the CLinkTable to verify the connection exists */
            pConnArgs = CLinkTable_GetEntry(pObject->pLinkTable, pSourceList[i]);
            if (pConnArgs)
            {
                if ((!pConnArgs->bRNDZDisabled) && (pConnArgs->bConnected == TRUE))
                {
#if defined(DEBUG_PRINT) && defined(TRACE_SERVER_ON_STATUS)
                    CTraceSvr_tprintf(&TraceServer, TRACE_Type_STATUS,
                        "CConObjSvr", "KeepAliveThread", FmCONNECTION, 0, 0, 
                        "Checking %s Count %d\n", pSourceList[i], i);
#endif   
                    /* Check for keepalive timeout */
                    if (CConObjSvr_StartKeepAlive(pObject, pSourceList[i]))
                        CConObjSvr_EvaluateKeepAlive(pObject, pSourceList[i]);
                }
            }                
        }

        /* Wait for the TimeOut semaphore to fire for processing a terminate or until we time out.*/
        Result = CWACPSemaphore_Wait(&pObject->TimeOutSem, 1000);
    }

#if defined(TRACE_SERVER_ON)
    CTraceSvr_tprintf(&TraceServer, TRACE_Type_STATUS,
            "CConObjSvr", "KeepAliveThread", FmCONNECTION, 0, 0, 
            "End\n");
#endif   

    TOUCH(Result);

    CWACPMutex_Release(&pObject->ExitMutex);

    return TRUE;
}







