/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CEthCommSvr.c   :                                                 //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CEthCommSvr.h"
#include "CCommSvr.h"
#include "SysUtil.h"
#include "Mem.h"

void CEthCommSvr_Init(CEthCommSvr* pObject, 
                      boolean (*RecvMsg)(char* pSourceID, uint8* pBuf, uint32 MsgSize),
                      void (*Terminate)(char* pSourceID, uint16 Port))
{
    if (pObject)
    {
        /* Store the callback function to add a new message */
        pObject->pSetRecvMsg = RecvMsg;

        /* Store the callback function to terminate the connection */
        pObject->pTerminate = Terminate;

        /* Zero out structure */
        memset(&pObject->DestAddr, 0, sizeof(pObject->DestAddr));           
        memset((void*)&(pObject->SourceID[0]), 0, sizeof(pObject->SourceID));           
        
        /* Initialize the socket handles */
        pObject->TxSocket = INVALID_SOCKET;
        pObject->RxSocket = INVALID_SOCKET;

        /* Set the stack type */
        pObject->StackType = ETHERNET;

        /* Default Protocol is TCP */
        pObject->Protocol = IPPROTO_TCP;

        /* Default Port is 0 */
        pObject->Port = 0;

        /* Initialize the WIN32 dll flag */
        pObject->WSAStartFlag = FALSE;
    }
}

void CEthCommSvr_Destroy(CEthCommSvr* pObject)
{
    if (pObject)
    {
        if (pObject->TxSocket != INVALID_SOCKET)
        {
            closesocket(pObject->TxSocket);
            pObject->TxSocket = INVALID_SOCKET;
        }

        if (pObject->RxSocket != INVALID_SOCKET)
        {
            closesocket(pObject->RxSocket);
            pObject->RxSocket = INVALID_SOCKET;
        }

        if (pObject->WSAStartFlag == TRUE)
        {
            WSACleanup( );
        }
        SYS_SLEEP(WAIT_COUNT);
    }
}

/*================================================================================
CEthCommSvr SetSourceID: Sets the SourceID
================================================================================*/
boolean CEthCommSvr_SetSourceID(CEthCommSvr* pObject, const char* pSourceID)
{   
    boolean RetVal = FALSE;
    if (pObject)
    {
        if (pSourceID != NULL)
        {
            memset(pObject->SourceID, 0, sizeof(pObject->SourceID));
            memcpy(pObject->SourceID, pSourceID, strlen(pSourceID));
            RetVal = TRUE;
        }
    }
    return (RetVal);
}

/*================================================================================
CEthCommSvr SetPort: Sets the Port used by the object
================================================================================*/
boolean CEthCommSvr_SetPort(CEthCommSvr* pObject, uint16 NewPort)
{
    boolean Result = FALSE;

    if (pObject)
    {
        pObject->Port = NewPort;
        Result = TRUE;
    }
    return Result;
}

/*================================================================================
CEthCommSvr_SetConfig: Updates the addressing information of the object
================================================================================*/
void CEthCommSvr_SetConfig(CEthCommSvr* pObject, const EthAddr* pAddr)
{
    if (pObject)
    {
        if (pAddr != NULL)
        {
            memcpy(&pObject->Config, pAddr, sizeof(EthAddr));
            pObject->Protocol = pObject->Config.Protocol;
            pObject->TxSocket = pObject->Config.Socket;
            CEthCommSvr_SetPort(pObject, pObject->Config.Port);
            CEthCommSvr_SetSourceID(pObject, pObject->Config.pSourceID);
        }
    }
}

/*================================================================================
CEthCommSvr Connect: Creates a socket that connects to the passed in SourceID and Port.
================================================================================*/
boolean CEthCommSvr_Connect(CEthCommSvr* pObject, uint8* pSourceID, int PortNum)
{
    struct WSAData Data;
    boolean Result = FALSE;

    if (pObject)
    {
        if (WSAStartup(MAKEWORD(2,1),&Data) == 0)
        {
            pObject->WSAStartFlag = TRUE;
        }
#ifdef DEBUG_PRINT_STATUS
        else
        {
            SYS_PRINTF("Init_Server: WSAStartup failed: %d\n", WSAGetLastError());
        }
#endif

        /* Construct the server Address structure */
        memset(&pObject->DestAddr, 0, sizeof(pObject->DestAddr));       /* Zero out structure */
        pObject->DestAddr.sin_family      = AF_INET;                    /* Internet Address family */
        pObject->DestAddr.sin_addr.s_addr = inet_addr((char*)pSourceID);/* Server IP Address */
        pObject->DestAddr.sin_port        = htons((u_short)PortNum);    /* Server Port */

        if (pObject->Protocol == IPPROTO_TCP)
        {
            /* Create a reliable, Stream socket using TCP */
            pObject->TxSocket = socket(PF_INET, SOCK_STREAM, pObject->Protocol);
            if (pObject->TxSocket != INVALID_SOCKET)
            {
                /* Establish the connection to the server */
                if (connect(pObject->TxSocket, (struct sockaddr*)(&pObject->DestAddr), sizeof(pObject->DestAddr)) >= 0)
                {
                    Result = TRUE;
                }
#ifdef DEBUG_PRINT_STATUS
                else
                {
                    SYS_PRINTF("ERROR!!! Unable to connect %d\n", WSAGetLastError());
                }
#endif
            }/* if (TxSocket >= 0) */

#ifdef DEBUG_PRINT_STATUS
            else
            {
                SYS_PRINTF("ERROR!!! Unable to create a TCP socket %d\n", WSAGetLastError());
            }
#endif
        }
        else if (pObject->Protocol == IPPROTO_UDP)
        {
            /* Create a reliable, Stream socket using UDP */
            pObject->RxSocket = pObject->TxSocket = socket(PF_INET, SOCK_DGRAM, pObject->Protocol);

            if (pObject->TxSocket != INVALID_SOCKET)
            {
                pObject->Config.Port = (uint16)PortNum;
                pObject->Config.Protocol = pObject->Protocol;
                pObject->Config.Socket = pObject->TxSocket;
                pObject->Config.pSourceID = (char*)pSourceID;
                memcpy(&(pObject->Config.ClntAddr), &pObject->DestAddr, sizeof(pObject->DestAddr));
                Result = TRUE;
            }
#ifdef DEBUG_PRINT_STATUS
            else
            {
                SYS_PRINTF("ERROR!!! Unable to create UDP socket\n");
            }
#endif
        }  

        /* Update the connection data */
        CEthCommSvr_SetPort(pObject, (uint16)PortNum);

        /* Set the destination IP to be the source */
        CEthCommSvr_SetSourceID(pObject, (char*)pSourceID);
        
#ifdef DEBUG_PRINT
        if ( Result == TRUE)
        {
            SYS_PRINTF("Connection open to %s to Port %d\n", 
                    inet_ntoa(pObject->DestAddr.sin_addr), PortNum);
        }
        else
        {
            SYS_PRINTF("Unable to connect to %s to Port %d\n",
                    inet_ntoa(pObject->DestAddr.sin_addr), PortNum);
        }
#endif
    }
    return Result;

}

/*================================================================================
CEthCommSvr SendMessage: Uses the current socket descriptor to send a message.
================================================================================*/
boolean CEthCommSvr_SendMessage(const CEthCommSvr* pObject, const uint8* pBuf, uint32 pBufSize)
{
    boolean RetVal = TRUE;
    int Bytes;

    if (pObject)
    {
        /* Send the string to the server */
        if (pObject->TxSocket != INVALID_SOCKET)
        {
            if (pObject->Protocol == IPPROTO_TCP)
            {
                Bytes = send(pObject->TxSocket, (char*)pBuf, (int)pBufSize, 0);
            }
            else
            {
                Bytes = sendto(pObject->TxSocket, (char*)pBuf, (int)pBufSize, 0, 
                                (struct sockaddr *) &(pObject->Config.ClntAddr), 
                                sizeof(pObject->Config.ClntAddr));
            }

#ifdef DEBUG_PRINT
            SYS_PRINTF("SendMessage: Sent %d Bytes\n", Bytes);
            {
                uint32 i;
                for (i=0; i < pBufSize;i++)
                {
                    if ( i % 25 == 0)
                        SYS_PRINTF("\n");
                    SYS_PRINTF("%.2x ", (uint8)pBuf[i]);
                }
                SYS_PRINTF("\n\n");
            }
#endif  

            if (Bytes != (int)pBufSize)
            {
                RetVal = FALSE;
#ifdef DEBUG_PRINT
                SYS_PRINTF("\n\nERROR!!! SendMessage: %d Bytes were sent out of %d\n\n", 
                        Bytes, pBufSize);
#endif
            }
        }
        else
        {
            RetVal = FALSE;
        }
    }
    return RetVal;
}   

/*================================================================================
Static CEthCommSvr StartClient: Handles the start of the new client connection thread.
    The function loops until the full Packet length is received before adding the
    message into the link layer IN_queue. The loop ends when the call to recv() 
    returns a -1 to signify the end of the socket connection .
================================================================================*/
uint32 WACPAPI CEthCommSvr_StartClient(void* pData)
{
    CEthCommSvr* pCommSvr = (CEthCommSvr*)pData;
    char        Buffer[RCVBUFSIZE];
    int         MsgSize, PacketLen, BytesLeft;
    uint8*      pTemp;

#ifdef DEBUG_PRINT
    int CommMsgCount = 0;
#endif
    if (pCommSvr)
    {
        if (pCommSvr->Protocol == IPPROTO_TCP)
        {
            /* Loop while there is data available */
            do
            {
                /* Receive the next message from client */
                MsgSize = recv(pCommSvr->TxSocket, Buffer, MSG_HEADER, 0);

                if (MsgSize == (int)MSG_HEADER)
                {
                    /* Verify the Preamble */
                    if( (Buffer[0] == CTL_W) && 
                        (Buffer[1] == CTL_A) && 
                        (Buffer[2] == CTL_L) )
                    {
                        /* Get the packet length */
                        pTemp = (uint8*)&Buffer[PREAMBLE_SIZE];
                        PacketLen = (int)ReadInt32(&pTemp);
                        if (PacketLen < RCVBUFSIZE)
                        {
#ifdef DEBUG_PRINT
                            SYS_PRINTF("\n\nCommSvr %p: New Msg size = %d\n", pCommSvr, PacketLen);
#endif
                            /* Calculate the remaining Bytes to receive */
                            BytesLeft = PacketLen - (int)MSG_HEADER;

                            /* Receive the rest of the Packet */
                            MsgSize += recv(pCommSvr->TxSocket, &Buffer[MsgSize], BytesLeft, 0);
                            
                            /* Loop until we get the full Packet */
                            while ((MsgSize > 0) && (MsgSize < PacketLen))
                            {
                                int Bytes;

                                Bytes = recv(pCommSvr->TxSocket, &Buffer[MsgSize], PacketLen - MsgSize, 0);

                                /* If Bytes <= 0 then the connection has been closed */
                                if (Bytes > 0)
                                {
                                    MsgSize += Bytes;
                                }
                                else
                                {
                                    MsgSize = Bytes;
                                }
#ifdef DEBUG_PRINT
                                SYS_PRINTF("\n\n1.CommSvr %p: CommMsgCount = %d,MsgSize = %d\n\n", pCommSvr, CommMsgCount, MsgSize);
#endif
                            }

                            /* Check for a valid MsgSize before sending it to the link layer */
                            if (MsgSize > 0)
                            {
#ifdef DEBUG_PRINT
                                int i;
                                SYS_PRINTF("\n\nNew Msg:\n");
                                for (i=0; i < MsgSize;i++)
                                {
                                    if ( i % 25 == 0)
                                        SYS_PRINTF("\n");
                                    SYS_PRINTF("%.2x ", (uint8)Buffer[i]);
                                }
                                SYS_PRINTF("\n\n");
#endif
                                /* Update the Recv IN_queue with the new message */
                                (void)pCommSvr->pSetRecvMsg(pCommSvr->SourceID, (uint8*)Buffer, (uint32)MsgSize);
                            }
                        } /* if (PacketLen < RCVBUFSIZE) */
                    }/* if valid preamble */
                    else
                    {
#ifdef DEBUG_PRINT_STATUS
                        SYS_PRINTF("ERROR!!! Invalid Preamble\n");
#endif
                    }
                }

            }while (MsgSize > 0);

            SYS_SLEEP(1);

            /* Check to see if this was a normal termination or break in the communication */
            if (pCommSvr->TxSocket != INVALID_SOCKET)
            {
                pCommSvr->pTerminate(pCommSvr->SourceID, pCommSvr->Port);
            }

        }/* if (Protocol == IPPROTO_TCP) */
        else if (pCommSvr->Protocol == IPPROTO_UDP)
        {
            SYS_SLEEP(WAIT_COUNT);
            for(;;) 
            {
                if ((MsgSize = recvfrom(pCommSvr->TxSocket, Buffer, RCVBUFSIZE, 0,
                                        NULL, NULL)) > 0)
                {
                    /* Check for a valid PREAMBLE */
                    if( (Buffer[0] == CTL_W) && 
                        (Buffer[1] == CTL_A) && 
                        (Buffer[2] == CTL_L) )
                    {
                        /* Call the callback routine to create a new Comm Session */
                        (void)pCommSvr->pSetRecvMsg(pCommSvr->SourceID, (uint8*)Buffer, (uint32)MsgSize);
                    }
                }

                if (MsgSize < 0)
                {
                    if ((MsgSize == -1) && (GetLastError() == WSAEINVAL))
                    {
                        SYS_SLEEP(WAIT_COUNT);
                    }
                    else
                        break;
                }
            }

        }/*else if (Protocol == IPPROTO_UDP) */
    }
#ifdef DEBUG_PRINT
        SYS_PRINTF("CEthCommSvr HandleClient: Thread exiting\n");
#endif

    return TRUE;
}




