/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CLinkTable.h   :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CLinkTable.h"
#include "SysUtil.h"

/********************************************************************************************
CLinkTable_Init - Initializes the CLinkTable instance
********************************************************************************************/
boolean CLinkTable_Init(CLinkTable* pObject)
{
    boolean Result = TRUE;

    pObject->Count = 0;
    memset(pObject->Connection, 0, sizeof(ConnArgs*) * MAX_CONNECTIONS);

    return (Result);
}

/********************************************************************************************
CLinkTable_Destroy - Destroys the CLinkTable instance. This will only free up memory
    allocated for the encryption key if ENCRYPTION_ON is defined. The memory allocated for
    each ConnArgs structure will be free'd by the application that created them.
********************************************************************************************/
boolean CLinkTable_Destroy(CLinkTable* pObject)
{
    boolean Result = FALSE;
    uint16  i;

    for (i=0; i < MAX_CONNECTIONS; i++)
    {
        pObject->Connection[i] = NULL;
    }

    return (Result);
}

/********************************************************************************************
CLinkTable_AddEntry - Adds a pointer to a ConnArgs structure into the array of ConnArgs. If
    a matching SourceID is found, the pointer is overwritten only if the connection for that
    SourceID is not connected. If it is still in a connected state, then this returns FALSE.
********************************************************************************************/
boolean CLinkTable_AddEntry(CLinkTable* pObject, ConnArgs* pConnArgs)
{
    boolean     Result = FALSE;
    uint16      i;
    int32       Index = -1;

    /* Check for a valid SourceID */
    if (pConnArgs)
    {
        /* Check for an existing entry */
        Index = CLinkTable_GetEntryIndex(pObject, pConnArgs->SourceID);
        if (Index == -1)
        {
            for(i=0; i < MAX_CONNECTIONS; i++)
            {
                /* Look for an open spot */
                if (pObject->Connection[i] == NULL)
                {
                    /* Store the pointer to the ConnArgs*/
                    pObject->Connection[i] = pConnArgs;

                    /* Initialize the ConnArgs structure */
                    pObject->Connection[i]->bConnected = FALSE;
                    pObject->Connection[i]->bRNDZStatus = FALSE;
                    pObject->Connection[i]->ValidationState = 0;
                    pObject->Connection[i]->pKey = NULL;
                    pObject->Connection[i]->pGUIDStd = NULL;

                    /* Increment the count */
                    pObject->Count++;

#if defined(TRACE_SERVER_ON_STATUS)
                    SYS_PRINTF("CLinkTable_AddEntry: Added %s, Count = %d\n", pConnArgs->SourceID, pObject->Count);
#endif
                    Result = TRUE;
                    break;
                }
            }
#if defined(TRACE_SERVER_ON_STATUS)
            if (!Result)
                SYS_PRINTF("CLinkTable_AddEntry: ERROR MAX_CONNECTIONS %d\n", pObject->Count);
#endif
        }
        else
        {
            /* Store the pointer to the ConnArgs if the connection is closed 
             * otherwise return FALSE to indicate an error.
             */
            if (pObject->Connection[Index]->bConnected == FALSE)
            {
                pObject->Connection[Index] = pConnArgs;

                /* Initialize the ConnArgs structure */
                pObject->Connection[Index]->bRNDZStatus = FALSE;
                pObject->Connection[Index]->ValidationState = 0;
                pObject->Connection[Index]->pKey = NULL;
                pObject->Connection[Index]->pGUIDStd = NULL;

                Result = TRUE;
            }
#if defined(TRACE_SERVER_ON_STATUS)
            else
                SYS_PRINTF("CLinkTable_AddEntry: Existing Entry already open for %s\n", pConnArgs->SourceID);
#endif
        }
    }
#if defined(TRACE_SERVER_ON_STATUS)
    else
        SYS_PRINTF("CLinkTable_AddEntry: Invalid SourceID\n");
#endif

    return (Result);
}

/********************************************************************************************
CLinkTable_RemoveEntry - Searches the CLinkTable instance for a matching SourceID and 
    removes it from the list. It is expected that the application who allocated the memory 
    for the ConnArgs structure will free it.
********************************************************************************************/
boolean CLinkTable_RemoveEntry(CLinkTable* pObject, const ConnArgs* pConnArgs)
{
    boolean     Result = FALSE;

    /* Check for a valid SourceID */
    if (pConnArgs)
    {
        int32 Index = CLinkTable_GetEntryIndex(pObject, pConnArgs->SourceID);
        if (Index > -1)
        {
            /* Make sure we only look at valid instances */
            if (pObject->Connection[Index] != NULL)
            {
                SYS_FREE(pObject->Connection[Index]->pKey);
                pObject->Connection[Index]->pKey = NULL;
                SYS_FREE(pObject->Connection[Index]->pGUIDStd);
                pObject->Connection[Index]->pGUIDStd = NULL;
                pObject->Connection[Index] = NULL;
                pObject->Count--;
                Result = TRUE;
#if defined(TRACE_SERVER_ON_STATUS)
                SYS_PRINTF("CLinkTable_RemoveEntry: Removed %s, Count = %d\n", pConnArgs->SourceID, pObject->Count);
#endif
            }
        }
#if defined(TRACE_SERVER_ON_STATUS)
        if (!Result)
            SYS_PRINTF("CLinkTable_RemoveEntry: Unable to find %s for removal\n", pConnArgs->SourceID);
#endif
    }


    return (Result);
}

/********************************************************************************************
CLinkTable_GetEntryIndex - Helper routine to search the ConnArgs array for a matching 
    SourceID. This returns the 0 based index into the Connection array if an entry is found.
    It returns -1 if the entry is not found.
********************************************************************************************/
int32 CLinkTable_GetEntryIndex(const CLinkTable* pObject, const char* pSourceID)
{
    uint16       i;
    uint16       TestCount = 0;
    int32        RetVal = -1;

    /* Check for a valid SourceID */
    if (pSourceID)
    {
        /* Cycle through the list of ConnArgs pointers for a match */
        for(i=0; i < MAX_CONNECTIONS; i++)
        {
            /* Stop the search once we have tested all existing connections */
            if (TestCount >= pObject->Count)
                break;

            /* Make sure we only look at valid instances */
            if (pObject->Connection[i] != NULL) 
            {
                TestCount++;

                /* If we find a matching SourceID, return a pointer to it */
                if (strcmp(pSourceID, pObject->Connection[i]->SourceID) == 0)
                {
                    RetVal = i;

#if defined(DEBUG_PRINT)
                    SYS_PRINTF("CLinkTable_GetEntryIndex: Found %s at Index %d\n", pObject->Connection[i]->SourceID, RetVal);
#endif
                    break;
                }
            }
        }
    }

#if defined(TRACE_SERVER_ON_STATUS)
    if (RetVal == -1)
        SYS_PRINTF("CLinkTable_GetEntryIndex: %s not found\n", pSourceID);
#endif
    return (RetVal);
}

/********************************************************************************************
CLinkTable_GetEntry - Uses GetEntryIndex to return a pointer to the ConnArgs structure based
    on the SourceID passed in. This return NULL if an entry is not found.
********************************************************************************************/
ConnArgs* CLinkTable_GetEntry(const CLinkTable* pObject, const char* pSourceID)
{
    ConnArgs    *pConnArgs = NULL;
    int32       Index;

    if (pSourceID)
    {
        Index = CLinkTable_GetEntryIndex(pObject, pSourceID);
        if (Index > -1)
        {
            pConnArgs = pObject->Connection[Index];
        }
    }

#if defined(TRACE_SERVER_ON_STATUS)
    if (!pConnArgs)
        SYS_PRINTF("CLinkTable_GetEntry: %s not found\n", pSourceID);
#endif
    return (pConnArgs);
}

/********************************************************************************************
CLinkTable_GetSourceList - Returns a filled in array of SourceIDs of all existing entries in the 
    CLinkTable. The array passed in is expected to be a size of MAX_CONNECTION and will always  
    be cleared by this routine.

********************************************************************************************/
void CLinkTable_GetSourceList(const CLinkTable* pObject, char** pSourceList)
{
    uint16   i;
    uint16   TestCount = 0;

    if (pSourceList)
    {
        /* Always clear the passed in array */
        memset(pSourceList, 0, sizeof(char*) * MAX_CONNECTIONS);

        /* Check for any existing entries */
        if (pObject->Count)
        {
            /* Cycle through the list of ConnArgs pointers for a match */
            for(i=0; i < MAX_CONNECTIONS; i++)
            {
                /* Stop the search once we have tested all existing connections */
                if (TestCount >= pObject->Count)
                    break;

                /* Make sure we only look at valid instances */
                if (pObject->Connection[i] != NULL) 
                {
                    /* Save the SourceID */
                    pSourceList[TestCount] = pObject->Connection[i]->SourceID;

                    /* Increment the current count */
                    TestCount++;
                }
            }
        }
    }
}

#if defined(CLINKTABLE_TEST_MAIN)
/********************************************************************************************
Test_CLinkTable - Test function for exercising the CLinkTable routines.
********************************************************************************************/
void Test_CLinkTable(void)
{
    CLinkTable  Link;
    ConnArgs*   Config[11];
    uint16      i;
    boolean     Result;

    for(i=0; i<11; i++)
    {
        Config[i] = (ConnArgs*)SYS_CALLOC(sizeof(ConnArgs), 1);
        Config[i]->Config.EthConfig.Protocol = IPPROTO_TCP;
        sprintf(Config[i]->SourceID, "%s%d", "127.0.0.", i);
        Config[i]->bSvrFlag = i % 2;
        Config[i]->EncryptionType = WACP_ENCRYPTION_NONE;
        Config[i]->Timeout = DEFAULT_RNDZ_TIMEOUT;
    }

    CLinkTable_Init(&Link);

    /* Add all 10 + one extra to test max connections */
    for(i=0; i<11; i++)
    {
        Result = CLinkTable_AddEntry(&Link, Config[i]);
        SYS_PRINTF("AddEntry ret. %d\n", Result);
    }

    CLinkTable_RemoveEntry(&Link, Config[1]);
    CLinkTable_RemoveEntry(&Link, Config[2]);
    CLinkTable_RemoveEntry(&Link, Config[0]);
    CLinkTable_RemoveEntry(&Link, Config[0]);
    CLinkTable_RemoveEntry(&Link, NULL);

    /* Check for existing entries */
    CLinkTable_AddEntry(&Link, Config[9]);
    CLinkTable_AddEntry(&Link, Config[8]);

    /* Re-add the removed entries */
    CLinkTable_AddEntry(&Link, Config[2]);
    CLinkTable_AddEntry(&Link, Config[0]);
    CLinkTable_AddEntry(&Link, Config[1]);

    /* Test removing all connection plus an extra one that shouldn't exist*/
    for(i=0; i<11; i++)
    {
        Result = CLinkTable_RemoveEntry(&Link, Config[i]);
        SYS_PRINTF("RemoveEntry ret. %d\n", Result);
    }

    for(i=0; i<11; i++)
    {
        SYS_FREE(Config[i]);
    }
}
#endif
