/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CSerialCommSvr.c:                                                 //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CSerialCommSvr.h"
#include "CCommSvr.h"
#include "Mem.h"
#include "SysUtil.h"

/*********************************************************************************
  CSerialCommSvr initailizer
*********************************************************************************/
void CSerialCommSvr_Init(CSerialCommSvr* pObject, 
                         boolean (*RecvMsg)(char* pSourceID, uint8* pBuf, uint32 MsgSize),
                         void (*Terminate)(char* pSourceID, uint16 Port))
{
    if (pObject)
    {
        /* Zero out structure */
        memset(&(pObject->SourceID[0]), 0, sizeof(pObject->SourceID));           
        
        /* Store the callback function to add a pObject message */
        pObject->pSetRecvMsg = RecvMsg;

        /* Store the callback function to terminate the connection */
        pObject->pTerminate = Terminate;
        
        /* Set the stack type */
        pObject->StackType = SERIAL;
        pObject->hPort = INVALID_HANDLE_VALUE;
        pObject->OSWrite.Offset = 0 ;
        pObject->OSWrite.OffsetHigh = 0 ;
        pObject->OSRead.Offset = 0 ;
        pObject->OSRead.OffsetHigh = 0 ;
        pObject->BaudRate = DEFAULT_BAUD_RATE;

        /* Create the Read Event */
        pObject->OSRead.hEvent = CreateEvent(NULL,    /* no security */
                                    TRUE,    /* explicit reset req */
                                    FALSE,   /* initial event reset */
                                    NULL) ; /* no Name */

        /* Create the Write Event */
        pObject->OSWrite.hEvent = CreateEvent(NULL,    /* no security */
                                    TRUE,    /* explicit reset req */
                                    FALSE,   /* initial event reset */
                                    NULL) ; /* no Name */
        
        /* Create shutdown mutex */
        CWACPMutex_Init(&(pObject->MutexLock), NULL);
    }
}

/*================================================================================
CSerialCommSvr_Destroy: Closes the port and destroys the shutdown mutex.
================================================================================*/
void CSerialCommSvr_Destroy(CSerialCommSvr* pObject)
{
    if (pObject)
    {
        if (pObject->hPort != INVALID_HANDLE_VALUE)
        {
            /* Call SetCommMask to cancel any current threads waiting on WaitCommEvent() */
            SetCommMask (pObject->hPort, EV_DSR);

            /* Close the communication Port. */
            CloseHandle(pObject->hPort);

            pObject->hPort = INVALID_HANDLE_VALUE;

            /* Wait for shutdown */
            CWACPMutex_Lock(&pObject->MutexLock, INFINITE);
            CWACPMutex_Release(&pObject->MutexLock);
        }

        /* Destroy shutdown mutex */
        CWACPMutex_Destroy(&pObject->MutexLock);
    }
}

/*================================================================================
CSerialCommSvr_SetSourceID: Sets the SourceID
================================================================================*/
static boolean CSerialCommSvr_SetSourceID(CSerialCommSvr* pObject, const char* pSourceID)
{   
    boolean Result = FALSE;

    if (pObject)
    {
        if (pSourceID != NULL)
        {
            memset(pObject->SourceID, 0, sizeof(pObject->SourceID));
            memcpy(pObject->SourceID, pSourceID, strlen(pSourceID));
            Result = TRUE;
        }
    }
    return (Result);
}

/*================================================================================
CSerialCommSvr_SetPort: Sets the Port used by the object
================================================================================*/
boolean CSerialCommSvr_SetPort(CSerialCommSvr* pObject, uint16 NewPort)
{
    boolean Result = FALSE;
    if (pObject)
    {
        pObject->Port = NewPort;
        Result = TRUE;
    }
    return Result;
}

/*================================================================================
CSerialCommSvr_SetConfig: Updates the addressing information of the object
================================================================================*/
void CSerialCommSvr_SetConfig(CSerialCommSvr* pObject, const void* pAddr)
{
    if (pObject)
    {
        memcpy(&pObject->Config, pAddr, sizeof(SerialAddr));
        CSerialCommSvr_SetPort(pObject, pObject->Config.Port);
        CSerialCommSvr_SetSourceID(pObject, pObject->Config.pSourceID);
        pObject->BaudRate = pObject->Config.BaudRate;
    }
}

/*================================================================================
CSerialCommSvr_Connect: Opens a serial com port based on the pSourceID and PortNum
================================================================================*/
boolean CSerialCommSvr_Connect(CSerialCommSvr* pObject, const uint8* pSourceID, int PortNum)
{
    boolean Result = TRUE;
    DCB PortDCB;                    /* Port Control Settings Structure */
    COMMTIMEOUTS CommTimeouts;      /* Comm Port timeout structure */

    if (pObject)
    {
        pObject->Port = (uint16)PortNum;

        if (pObject->hPort == INVALID_HANDLE_VALUE)
        {
            char TempBuf[ADDR_BUF_SIZE];

            /* Clear the temporary buffer */
            memset(TempBuf, 0, ADDR_BUF_SIZE);

            /* Append "\\.\" to allow port number greater than 9 */
            sprintf(TempBuf, "\\\\.\\%s", pSourceID);

            /*  Open the serial Port. */
            pObject->hPort = CreateFile ((LPCSTR)TempBuf, /* Pointer to the Name of the Port */
                            GENERIC_READ | GENERIC_WRITE,
                                                            /*  Access (read-write) mode */
                            0,                              /* Share mode */
                            NULL,                           /* Pointer to the security attribute */
                            OPEN_EXISTING,                  /* How to open the serial Port */
                            FILE_ATTRIBUTE_NORMAL |
                            FILE_FLAG_OVERLAPPED,           /* Port attributes */
                            NULL);                          /* Handle to Port with attribute */
                                                            /* to copy */

            PortDCB.DCBlength = sizeof (DCB);     

            /* Get the default Port setting information. */
            GetCommState (pObject->hPort, &PortDCB);

            /* Change the DCB structure settings. */
            PortDCB.BaudRate = pObject->BaudRate;      /* Current baud */
            PortDCB.fBinary = TRUE;                 /* Binary mode; no EOF check */
            PortDCB.fParity = TRUE;                 /* Enable parity checking */
            PortDCB.fOutxCtsFlow = FALSE;           /* CTS Output flow control */
            PortDCB.fOutxDsrFlow = FALSE;           /* DSR Output flow control */
            PortDCB.fDtrControl = DTR_CONTROL_DISABLE; 
                                                    /* DTR flow control type */
            PortDCB.fDsrSensitivity = FALSE;        /* DSR sensitivity */
            PortDCB.fTXContinueOnXoff = FALSE;       /* XOFF continues Tx */
            PortDCB.fOutX = FALSE;                  /* No XON/XOFF out flow control */
            PortDCB.fInX = FALSE;                   /* No XON/XOFF in flow control */
            PortDCB.fErrorChar = FALSE;             /* Disable error replacement */
            PortDCB.fNull = FALSE;                  /* Disable null stripping */
            PortDCB.fRtsControl = RTS_CONTROL_DISABLE; 
                                                    /* RTS flow control */
            PortDCB.fAbortOnError = FALSE;          /* Do not abort reads/writes on */
                                                    /* error */
            PortDCB.ByteSize = 8;                   /* Number of bits/byte, 4-8 */
            PortDCB.Parity = NOPARITY;              /* 0-4=no,odd,even,mark,space */
            PortDCB.StopBits = ONESTOPBIT;          /* 0,1,2 = 1, 1.5, 2 */

            /* Configure the Port according to the specifications of the DCB  
            * structure. 
            */
            if (!SetCommState (pObject->hPort, &PortDCB))
            {
                Result = FALSE; 
#ifdef DEBUG_PRINT_STATUS
                SYS_PRINTF("Error %d!!! Unable to set the com Port settings\n", GetLastError ());
#endif
            }

            /* Retrieve the time-out parameters for all read and write operations
            * on the Port. 
            */
            GetCommTimeouts (pObject->hPort, &CommTimeouts);

            /* Change the COMMTIMEOUTS structure settings. */
            CommTimeouts.ReadIntervalTimeout = MAXDWORD;  
            CommTimeouts.ReadTotalTimeoutMultiplier = 0;  
            CommTimeouts.ReadTotalTimeoutConstant = 0;    
            CommTimeouts.WriteTotalTimeoutMultiplier = 10;  
            CommTimeouts.WriteTotalTimeoutConstant = 1000;    

            /* Set the time-out parameters for all read and write operations 
            * on the Port. 
            */
            if (!SetCommTimeouts (pObject->hPort, &CommTimeouts))
            {
                Result = FALSE; 
#ifdef DEBUG_PRINT_STATUS
                SYS_PRINTF("Error %d!!! Unable to set comm timeouts\n", GetLastError ());
#endif
            }

            /* Set the destination IP to be the source */
            CSerialCommSvr_SetSourceID(pObject, (char*)pSourceID);
        }
    }

    return Result;
}

/*================================================================================
CSerialCommSvr_SendMessage: Sends the passed in buffer through the com port.
================================================================================*/
boolean CSerialCommSvr_SendMessage(CSerialCommSvr* pObject, const uint8* pBuf, uint32 pBufSize)
{
    boolean Result = TRUE;
    DWORD DwNumBytesWritten,DwRes;

    if (pObject)
    {
        if (!WriteFile (pObject->hPort,            /* Port handle */
                        pBuf,                   /* Pointer to the data to write */
                        pBufSize,               /* Number of Bytes to write */
                        &DwNumBytesWritten,     /* Pointer to the number of Bytes */
                                                /* written */
                        &(pObject->OSWrite)))      /* Must be NULL for Windows CE */
        {
            if (GetLastError() != ERROR_IO_PENDING) 
            { 
                /* WriteFile failed, but it isn't delayed. Report error. */
                Result = FALSE;
            }
            else
            {
                /* Write is pending. */
                DwRes = WaitForSingleObject(pObject->OSWrite.hEvent, INFINITE);
                switch(DwRes)
                {
                    /* Overlapped event has been signaled. */
                    case WAIT_SUCCESS:
                        if (!GetOverlappedResult(pObject->hPort, &(pObject->OSWrite), &DwNumBytesWritten, FALSE))
                            Result = FALSE;
                        else 
                        {
                            if(DwNumBytesWritten != pBufSize) 
                            {
                                /* The write operation timed out. I now need to 
                                * decide if I want to abort or retry. If I retry, 
                                * I need to send only the Bytes that weren't sent. 
                                * If I want to abort, I would just set fRes to 
                                * FALSE and return.
                                */
                                Result = FALSE;
                            }
                            else
                                /* Write operation completed successfully. */
                                Result = TRUE;
                        }
                        break;
                    default:
                        /* An error has occurred in WaitForSingleObject. This usually */
                        /* indicates a problem with the overlapped event handle. */

                        Result = FALSE;
                        break;
                }
            }
        }

#ifdef DEBUG_PRINT
        {
            uint32 i;
            SYS_PRINTF("SendMessage: Sent %d of %d Bytes\n", DwNumBytesWritten, pBufSize);
            for (i=0; i < DwNumBytesWritten;i++)
            {
                if (i % 25 == 0)
                    SYS_PRINTF("\n");
                SYS_PRINTF("%.2x ", (uint8)pBuf[i]);
            }
            SYS_PRINTF("\n\n");
        }
#endif  
    }

    return Result;
}   


/*================================================================================
CSerialCommSvr_ReadData: Reads BytesToRead number of bytes of data from the serial 
    pipe. We will block until data is ready or the pipe is closed.
================================================================================*/
boolean CSerialCommSvr_ReadData(CSerialCommSvr* pObject, char* pBuf, uint32 BytesToRead, uint32* pBytesRead)
{
    boolean Result = FALSE;
    DWORD DwCommModemStatus;

    if (pObject)
    {
        if (pObject->hPort != INVALID_HANDLE_VALUE) 
        {
            /* Read the data from the serial Port. */
            Result = (boolean)ReadFile (pObject->hPort, pBuf, BytesToRead, pBytesRead, &pObject->OSRead);
            if (!Result)
            {
                if (GetLastError() == ERROR_IO_PENDING)
                {
                    while((Result = (boolean)GetOverlappedResult(pObject->hPort, &pObject->OSRead, pBytesRead, TRUE)) != TRUE)
                    {
                        if(GetLastError() == ERROR_IO_INCOMPLETE)
                            continue;
                        else
                            break;
                    }
                }
            }

            /* Get the latest modem status */
            if (GetCommModemStatus(pObject->hPort, &DwCommModemStatus))
            {
                /* Check to see if we are in a low DTR state */ 
                if(!(DwCommModemStatus & MS_DSR_ON))
                {
                    /* Wait CONNECTION_TIMEOUT milleseconds to see if the connection comes back */
                    SYS_SLEEP(CONNECTION_TIMEOUT);
                    if (GetCommModemStatus(pObject->hPort, &DwCommModemStatus))
                    {
                        /* If we are still low, then notify exchange to terminate */
                        if(!(DwCommModemStatus & MS_DSR_ON))
                        {
                            pObject->pTerminate(pObject->SourceID, pObject->Port);
                        }
                    }
                }
            }

            if (*pBytesRead == 0)
            {
                /* Wait for an event to occur for the Port. */
                Result = (boolean)WaitCommEvent(pObject->hPort, &DwCommModemStatus, 0);

                /* Specify a set of events to be monitored for the Port. */
                SetCommMask (pObject->hPort, EV_RXCHAR | EV_DSR);
                
                if (Result == TRUE)
                {
                    if (DwCommModemStatus & EV_RXCHAR) 
                    {
                        /* Read the data from the serial Port. */
                        Result = (boolean)ReadFile (pObject->hPort, pBuf, BytesToRead, pBytesRead, &pObject->OSRead);
                        if (!Result)
                        {
                            if (GetLastError() == ERROR_IO_PENDING)
                            {
                                while((Result = (boolean)GetOverlappedResult(pObject->hPort, &pObject->OSRead, pBytesRead, TRUE)) != TRUE)
                                {
                                    if(GetLastError() == ERROR_IO_INCOMPLETE)
                                        continue;
                                    else
                                        break;
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    return(Result);
}

/*================================================================================
CSerialCommSvr_CheckDTR(): Checks the serial connection for the DTR line to go high
    and creates a message to simulate the arrival of a RNDZ connect message.
    This routine blocks until the DTR line goes high or the pipe is closed.
================================================================================*/
boolean CSerialCommSvr_CheckDTR(CSerialCommSvr* pObject)
{
    DWORD   DwCommModemStatus;
    boolean Result = FALSE;

    static const uint8 RndzConnect[] = /* Faked RNDZCONNECT message */
    { 
        0x00, 0x1D, 0x0B, 0x01, 0x00, 0x00, 0x00, 0x12,
        0x00, 0x00, 0x00, 0x00, 0x0B, 0x52, 0x4E, 0x44,
        0x5A, 0x43, 0x4F, 0x4E, 0x4E, 0x45, 0x43, 0x54,
        0x57, 0xF3
    };

    if (pObject)
    {
        if (pObject->hPort != INVALID_HANDLE_VALUE) 
        {
            /* Get the status of the modem lines, so we can check the DSR line */
            GetCommModemStatus(pObject->hPort, &DwCommModemStatus);

            while(!(DwCommModemStatus & MS_DSR_ON) && (pObject->hPort != INVALID_HANDLE_VALUE))
            {       
                /* Specify a set of events to be monitored for the Port. */
                SetCommMask (pObject->hPort, EV_DSR);

                /*  Wait for an event to occur for the Port. */
                Result = (boolean)WaitCommEvent (pObject->hPort, &DwCommModemStatus, 0);

                /* Get the status of the modem lines, so we can check the DSR line */
                GetCommModemStatus(pObject->hPort, &DwCommModemStatus);
            }

            if (pObject->hPort != INVALID_HANDLE_VALUE)
            {
                uint32  PacketLen;    
                uint8   *pPacketBuf = NULL;
                
                pPacketBuf = GeneratePacket(&PacketLen,
                                            pObject->Port,
                                            (uint8*)RndzConnect,
                                            sizeof(RndzConnect));
                if (pPacketBuf)
                {
                    Result = pObject->pSetRecvMsg(pObject->SourceID,
                                                  pPacketBuf,
                                                  PacketLen);
            
                    /* Free up the memory allocated by GeneratePacket */
                    SYS_FREE(pPacketBuf);
                }
            }
        }
    }
    return Result;
}

/*================================================================================
Static CSerialCommSvr StartClient: Handles the start of the new client listener 
    thread. The function loops until the full Packet length is received before 
    passing the mesage through the callback supplied in the constructor. The 
    thread terminates when the ReadData returns FALSE or the Com Port is closed
    and set to INVALID_HANDLE_VALUE.
================================================================================*/
uint32 WACPAPI CSerialCommSvr_StartClient(void* pData)
{
    CSerialCommSvr* pCommSvr = (CSerialCommSvr*)pData;
    char    Buffer[RCVBUFSIZE];
    uint32  MsgSize, BytesLeft, PacketLen;
    int     Result = TRUE;
    uint8   Index = 0;
    uint8*  pTemp;

    if (pCommSvr)
    {
        CWACPMutex_Lock(&pCommSvr->MutexLock, INFINITE);

        PurgeComm(pCommSvr->hPort, PURGE_TXCLEAR | PURGE_RXCLEAR);

        /* SETDTR: Sends the DTR (data-terminal-ready) signal. */
        EscapeCommFunction (pCommSvr->hPort, SETDTR);

        if(pCommSvr->Config.bAutoDiscovery == TRUE)
        {
            CSerialCommSvr_CheckDTR(pCommSvr);
        }

        /* Specify a set of events to be monitored for the Port. */
        SetCommMask (pCommSvr->hPort, EV_RXCHAR | EV_DSR);

        while (pCommSvr->hPort != INVALID_HANDLE_VALUE) 
        {
            if(!Result)
                SYS_SLEEP(WAIT_COUNT);

            /* Loop until we get a valid preamble or an error */
            Index = 0;
            memset(Buffer, 0, RCVBUFSIZE);
            while(Index != PREAMBLE_SIZE)
            {
                Result = CSerialCommSvr_ReadData(pCommSvr, (char*)&Buffer[Index], 1, &MsgSize);
                if ((Result) && (MsgSize > 0))
                {
                    switch(Index)
                    {
                        case 0:
                            if (Buffer[Index] == CTL_W)
                                Index++;
                            break;
                        case 1:
                            if (Buffer[Index] == CTL_A)
                                Index++;
                            else
                                Index = 0;
                            break;
                        case 2:
                            if (Buffer[Index] == CTL_L)
                                Index++;
                            else
                                Index = 0;
                            break;
                        default:
                            break;
                    }
                }
                else
                {
                    if (!Result)
                        break;
                }
            }

            if (!Result)
                break;

            /* Read the remaining bytes of the message header */
            Result = CSerialCommSvr_ReadData(pCommSvr, (char*)&Buffer[PREAMBLE_SIZE], MSG_HEADER - PREAMBLE_SIZE, &MsgSize);
            if (Result)
            {
                BytesLeft = (MSG_HEADER - PREAMBLE_SIZE) - MsgSize;
                while (BytesLeft > 0)
                {
                    Result = CSerialCommSvr_ReadData(pCommSvr, (char*)&Buffer[MSG_HEADER - BytesLeft], BytesLeft, &MsgSize);
                    if (!Result)
                        break;
                    BytesLeft -= MsgSize;
                    SYS_SLEEP(1);
                }

                /* Check that we received a valid number of bytes for the message header */
                if (BytesLeft == 0)
                {
                    /* Get the packet length */
                    pTemp = (uint8*)&Buffer[PREAMBLE_SIZE];
                    PacketLen = ReadInt32(&pTemp);
                    if (PacketLen < RCVBUFSIZE)
                    {
                        /* Calculate the remaining Bytes to receive */
                        BytesLeft = PacketLen - MSG_HEADER;

                        /* Receive the rest of the Packet */
                        CSerialCommSvr_ReadData(pCommSvr, (char*)&Buffer[MSG_HEADER], BytesLeft, &MsgSize);
                        
                        BytesLeft -= MsgSize;

                        /* Loop until we get the full Packet */
                        while (BytesLeft > 0)
                        {
                            uint32 Bytes;

                            Result = CSerialCommSvr_ReadData(pCommSvr, (char*)&Buffer[PacketLen - BytesLeft], BytesLeft, &Bytes);
                            if (!Result)
                                break;
                            BytesLeft -= Bytes;
                            SYS_SLEEP(1);
                        }

                        /* Check for a valid MsgSize before sending it to the link layer */
                        if (BytesLeft == 0)
                        {
#if defined(DEBUG_PRINT)
                            uint32 i;
                            SYS_PRINTF("Receive Message: Received %d Bytes\n", PacketLen);
                            for (i=0; i < PacketLen;i++)
                            {
                                if (i % 25 == 0)
                                    SYS_PRINTF("\n");
                                SYS_PRINTF("%.2x ", (uint8)Buffer[i]);
                            }
                            SYS_PRINTF("\n\n");
#endif
                            /* Update the Recv IN_queue with the new message */
                            (void)pCommSvr->pSetRecvMsg(pCommSvr->SourceID, (uint8*)Buffer, PacketLen);
                        }
                    }/* if (PacketLen < MAX_DATA_SIZE) */
                
                }/* if (BytesLeft == 0) */

            }/* if (Result) */

        }/* while (pCommSvr->hPort != INVALID_HANDLE_VALUE)  */
    }
#ifdef DEBUG_PRINT
    SYS_PRINTF("CSerialCommSvr HandleClient: Thread exiting\n");
#endif

    if (pCommSvr)
        CWACPMutex_Release(&pCommSvr->MutexLock);

    return TRUE;
}/* CSerialCommSvr_StartClient() */




