/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CWACPQueue.c   :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "Target.h"
#include "SysUtil.h"
#include "CWACPQueue.h"
#include "String.h"

boolean CWACPQueue_Init(CWACPQueue* pObject, uint32 QueueSize)
{
    boolean Result = FALSE;
    uint32  MemSize = QueueSize * sizeof(QPACKET*);

    if (pObject)
    {
        pObject->pDataQueue = (QPACKET**)SYS_MALLOC(MemSize);
        if (pObject->pDataQueue)
        {
            memset(pObject->pDataQueue, 0, MemSize);
            pObject->Head = 0;
            pObject->Tail = 0;
            pObject->Count = 0;
            pObject->MaxSize = QueueSize;

            /* Initialize the mutex for accessing the queue */
            CWACPMutex_Init(&pObject->QueueMutex, NULL);

            Result = TRUE;
        }
    }

    return (Result);
}

boolean CWACPQueue_Destroy(CWACPQueue* pObject)
{
    boolean Result = FALSE;
    uint32 i;

    if (pObject)
    {
        CWACPMutex_Lock(&(pObject->QueueMutex), INFINITE);

        for (i=0; pObject->Head + i > pObject->Tail; i++)
        {
            SYS_FREE(&pObject->pDataQueue[pObject->Head + i]->pDataBuf);
            SYS_FREE(&pObject->pDataQueue[pObject->Head + i]);
        }

        SYS_FREE(pObject->pDataQueue);
        pObject->pDataQueue = NULL;

        CWACPMutex_Release(&(pObject->QueueMutex));

        /* Destroy the queue mutex */
        CWACPMutex_Destroy(&pObject->QueueMutex);

    }
    return (Result);
}

boolean CWACPQueue_Enqueue(CWACPQueue* pObject, char* pSourceID, uint8* pDataBuf, uint32 DataSize)
{
    boolean Result = FALSE;
    QPACKET* pQPacket = (QPACKET*)SYS_MALLOC(sizeof(QPACKET));

    if (pObject)
    {
        CWACPMutex_Lock(&(pObject->QueueMutex), INFINITE);
        if (pQPacket)
        {
            pQPacket->pSourceID = pSourceID;
            pQPacket->pDataBuf = pDataBuf;
            pQPacket->DataSize = DataSize;

            if (pObject->Count < MAX_QUEUE_SIZE)
            {
                pObject->pDataQueue[pObject->Tail]= pQPacket;
                pObject->Tail = (pObject->Tail + 1) % pObject->MaxSize;
                pObject->Count++;
                Result = TRUE;
            }
#if defined(TRACE_SERVER_ON)
            else
            {
                SYS_PRINTF("Queue full\n");
            }
#endif
        }
        CWACPMutex_Release(&(pObject->QueueMutex));
    }
    return (Result);
}

QPACKET* CWACPQueue_Dequeue(CWACPQueue* pObject)
{
    QPACKET* pQPacket = NULL;
    
    if (pObject)
    {
        CWACPMutex_Lock(&(pObject->QueueMutex), INFINITE);
        if (pObject->Count)
        {
            pQPacket = pObject->pDataQueue[pObject->Head];
            pObject->Head = (pObject->Head + 1) % pObject->MaxSize;
            pObject->Count--;
        }
        CWACPMutex_Release(&(pObject->QueueMutex));
    }
    return (pQPacket);
}

#if defined(QUEUE_TEST)

CWACPQueue_Test()
{
    QPACKET* Data = NULL;
    uint16 i;
    boolean Result;
    CWACPQueue  TestQueue;

    CWACPQueue_Init(&TestQueue, MAX_QUEUE_SIZE);

    /* Test Set/Get sequence */
    for (i=0; i < 30; i++)
    {
        Result = CWACPQueue_Enqueue(&TestQueue, "Source", (uint8*)"Foo", i);
        Data = CWACPQueue_Dequeue(&TestQueue);
        SYS_PRINTF("%s, %s, %d\n", Data->pSourceID, Data->pDataBuf, Data->DataSize);
    }

     /* Test group sequence */
    {
        for (i=0; i < MAX_QUEUE_SIZE; i++)
        {
            Result = CWACPQueue_Enqueue(&TestQueue, "Source", (uint8*)"Foo", i);
        }
        for (i=0; i < MAX_QUEUE_SIZE; i++)
        {
            Data = CWACPQueue_Dequeue(&TestQueue);
            SYS_PRINTF("%s, %s, %d\n", Data->pSourceID, Data->pDataBuf, Data->DataSize);
        }
    }

    /* Test overflow of queue */
    for (i=0; i < MAX_QUEUE_SIZE + 5; i++)
    {
        Result = CWACPQueue_Enqueue(&TestQueue, "Source", (uint8*)"Foo", i);
        SYS_PRINTF("Result = %d\n", Result);
    }

    /* Test Circular overflow */
    {
        for (i=0; i < MAX_QUEUE_SIZE - 5; i++)
        {
            Data = CWACPQueue_Dequeue(&TestQueue);
            SYS_PRINTF("%s, %s, %d\n", Data->pSourceID, Data->pDataBuf, Data->DataSize);
        }
        
        for (i=0; i < MAX_QUEUE_SIZE - 5; i++)
        {
            Result = CWACPQueue_Enqueue(&TestQueue, "Source", (uint8*)"Foo", i);
            SYS_PRINTF("Result = %d\n", Result);
        }

        for (i=0; i < MAX_QUEUE_SIZE; i++)
        {
            Data = CWACPQueue_Dequeue(&TestQueue);
            if (Data)
                SYS_PRINTF("%s, %s, %d\n", Data->pSourceID, Data->pDataBuf, Data->DataSize);
            else
                SYS_PRINTF("NULL\n");
        }
    }
}

int main(void)
{
    CWACPQueue_Test();
    return TRUE;
}
#endif /* #if defined(QUEUE_TEST) */


