/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2005-2006 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CDMOIBDictionary.h                                                //
'**  '// Version        :   1.00                                           //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CDMOIBDICTIONARY_H)
    #define  CDMOIBDICTIONARY_H

#include "DMOIBBase.h"

/*=============================================================================
    Types/Enums used by the Dictionary
=============================================================================*/
/*========================================================================*//**
 * @brief
 * Enum to define the search modes for getting definitions from a dictionary.
 *
 * The CDMOIBDicionary class manages dictionaries which are statically defined
 * for the system using them (with the macro usage in DMOIBDefinitions.h), as
 * well as ones that are dynamically acquired from an external source (loaded 
 * from a file or a network connection). This enumeration is used as a selector
 * for the class, to decide what definitions should be examined - only the 
 * static ones, only the dynamic ones, or all definitions.
 *
 *//*========================================================================*/
typedef enum SearchMode_e
{
    ALL_DEFINITIONS = 0,
    STATIC_DEFINITIONS_ONLY,
    DYNAMIC_DEFINITIONS_ONLY,

    UKNOWN_SEARCH_MODE

}SearchMode_t;

/*=============================================================================
    CDMOIBDictionary class definition
*//*======================================================================*//**
 * @brief
 * Class definition for the Dictionary interface used to manage definitions.
 *
 * This is the class to manage all definitions used by software projects. 
 * DMOIB definitions can exist by themselves as independent entities, and 
 * can be used fully without any knowledge of a CDMOIBDictionary object. 
 * But whenever a developer must resolve a ClassID toa known definition, 
 * then a dictionary is needed to do the translation. 
 *
 * The dictionary's primary role is to act as a lookup agent for managed 
 * definitions. Given a ClassID, it shall perform an internal search of all its
 * known definitions, and return a definition that matches (if available). It
 * also provides an interface for retrieving definitions anonymously, without
 * an initial ClassID, by requesting it using an index. This method can be 
 * used in an abstract way to get all the definitions known to a dictionary 
 * through an iterative process, without any previous knowledge of the 
 * associated ClassIDs.
 *
 *//*========================================================================*/
class CDMOIBDictionary
{
private:
    /*====================================================================*//**
     * @var     TableEntry_t* m_pEntryDefault
     * @brief
     * Reference to the first static entry of this dictionary.
     *
     * This is a reference to the first static definition in the default list 
     * provided to the constructor. This denotes the end of the dynamic 
     * definitions and the start of the static ones. If no default definitions
     * are provided to the constructor, then this will be set to NULL. 
     *
     *//*=================================================================*//**
     * @var     TableEntry_t* m_pEntryHead
     * @brief
     * Reference to the first dynamic entry of this dictionary.
     *
     * This is a reference to the first dynamic definition managed by this
     * dictionary. If no dynamic definitions are ever added to this dictionary, 
     * then this will reference the same thing as m_pEntryDefault.
     *
     *//*=================================================================*//**
     * @var     SearchMode_t m_SearchMode
     * @brief
     * The selected search mode for this dictionary (Static, Dynamic, or All).
     *
     * This is a configuration element for the dictionary which describes the
     * current search mode. This can equal any value from the SearchMode_e
     * set. Upon construction, this will default to a value of ALL_DEFINITIONS,
     * meaning the dictionary will include both static and dynamic definitions
     * when it performs a search/lookup. 
     *
     * @note
     * This setting also applies to the manner in which definitions are 
     * retrieved by their index. For example, if this is set to 
     * STATIC_DEFINITIONS_ONLY, and the definition at index 0 is requested, 
     * it will return the first static definition found, even if there are 
     * dynamic definitions available before it in the list.
     *
     *//*====================================================================*/

    TableEntry_t*   m_pEntryDefault;
    TableEntry_t*   m_pEntryHead;
    SearchMode_t    m_SearchMode;
    
public:
    /*=========================================================================
        Public Methods
    ===========================================================================
        Basic Constructor
    ======================================================================*//**
     * @brief
     * Basic constructor for a Dictionary from a default set of definitions.
     *
     * This is the only constructor available, and can be used to create a 
     * dictionary that will have a default set of definitions. If no default
     * set is provided, then this dictionary can only be used to manage
     * dynamic ones added after creation.
     *
     * @param pDefaultDictionary
     * Optional parameter which is a reference to a set of static definitions 
     * from the definition .h file (The variable name for this parameter 
     * should match the DictionaryName provided to the DMOIB_DICTIONARY() 
     * macro). If this parameter is not provided, it defaults to NULL, and
     * no static definitions will be available to the dictionary.
     *
     *//*====================================================================*/
    CDMOIBDictionary(void* pDefaultDictionary = NULL);
    /*====================================================================*//**
     * @brief
     * Standard destructor.
     *
     * Standard destructor that cleans up all allocated memory.
     *//*====================================================================*/
    ~CDMOIBDictionary();
    /*====================================================================*//**
     * @brief     
     * Accessor to determine the search mode for this dictionary.
     *
     * This method will report the current search mode configured for the 
     * dictionary. This allows the developer to determine the set of 
     * definitions used when performing a search, which are static, dynamic, 
     * or all definitions.
     *
     * @return
     * A uint8 value representing the current searching configuration of the
     * dictionary. This will be a value from the SearchMode_t set.
     * 
     *//*====================================================================*/
    SearchMode_t GetSearchMode();
    /*====================================================================*//**
     * @brief
     * Mutator to set the search mode for this dictionary.
     *
     * This method allows the developer to configure the dictionary to only 
     * search specific sets of definitions when performing a lookup. The 
     * dictionary's searching routine can be configured to check static 
     * definitions only, dynamic only, or all definitions. 
     *
     * @param SearchMode
     * A uint8 value representing the selected searching configuration of the
     * dictionary. This must be a value from the enum SearchMode_e set. If the
     * SearchMode value provided does not match a value from this set, the 
     * dictionary's configuration will not be modified.
     *
     *//*====================================================================*/
    void SetSearchMode(SearchMode_t SearchMode);
    /*====================================================================*//**
     * @brief
     * Reports the number of definitions available in this dictionary.
     *
     * This method will report the number of definitions available in this 
     * dictionary that can be used by the developer. If a definition is added
     * to this dictionary that overrides a previous definition, then this count
     * will not be modified. 
     *
     * @note
     * This method returns the number of definitions available for the currently
     * selected SearchMode. For example, if the dictionary has 10 static 
     * definitions available, 5 dynamic ones, and search mode is set to 
     * DYNAMIC_DEFINITIONS_ONLY, then this will return a value of 5.
     *
     * @return
     * A uint16 value representing the total number of definitions available in 
     * the dictionary, based on the current SearchMode.
     *
     *//*====================================================================*/
    uint16 GetDefinitionCount();
    /*====================================================================*//**
     * @brief
     * Method to access a definition at a specified index within the 
     * dictionary.
     *
     * This method provides access to managed definitions in an abstract way.
     * In cases where a developer may not know the ClassID of the definition
     * they wish to work with, this interface can be used in conjunction with
     * GetDefinitionCount() to iteratively access each definition in the 
     * dictionary.
     *
     * @param Index
     * A uint16 value representing the numeric index of the requested 
     * definition. The valid index values start at 0, and end at index = 
     * (GetDefinitionCount() - 1).
     *
     * @return
     * A reference to the definition at Index. If no definition exists at the 
     * specified index, or the index is out of bounds, this will return NULL.
     *
     *//*====================================================================*/
    void* GetDefinitionAt(uint16 Index);
    /*====================================================================*//**
     * @brief
     * Method to find a definition based on a ClassID
     *
     * Provided with a ClassID, this method will search the dictionary's 
     * contents for a definition that matches that ClassID. This allows the
     * developer to find a definition for a certain class at runtime, and in
     * turn create an object from that definition. 
     *
     * @param ClassID
     * The ClassID of the requested object definition.
     *
     * @return
     * A reference to the definition matching the ClassID. If no definition 
     * exists, this will return NULL.
     *
     *//*====================================================================*/
    void* GetDefinition(uint32 ClassID);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    void* GetDefinition(uint16 Family, uint8 Genus, uint8 Species);
    /*====================================================================*//**
     * @brief
     * Method to add new, dynamic definitions to this dictionary.
     *
     * This method is the only way to expand a dictionary to include more
     * definitions than those default ones provided to the constructor. Given
     * a reference to an existing definition, this method will check the 
     * currently managed definitions for conflicts, and then add the definition
     * to its collection if possible. 
     *
     * If a definition already exists in the dictionary with a matching ClassID
     * and version greater than or equal to the new definition, it will not be
     * added. Otherwise a copy will be made of the definition and added to the
     * dictionary. A copy must be made because the dictionary needs control of
     * the definition to delete it when necessary.
     *
     * @note
     * If a single CDMOIBDictionary instance is shared between multiple threads
     * within a system, then the developer should put thread-protection around
     * calls to this method. All other methods of this class are thread-safe,
     * but this method is volatile.
     *
     * @param pDefinition
     * A reference to an existing definition that will be added to the 
     * dictionary. 
     *
     * @return
     * A reference to the definition that was added to the dictionary
     * (This will not match the parameter passed in, because the dictionary 
     * creates a new copy). If the definition cannot be added, this will 
     * return NULL.
     *
     *//*====================================================================*/
    void* AddDefinition(void* pDefinition);

private:
    /*=========================================================================
        Private Methods
    ======================================================================*//**
     * @brief
     * Helper method to find an entry based on its ClassID.
     *
     * This method is used internally to find an entry based on a ClassID. The 
     * TableEntry_t structure is used to manage definitions on a 1:1 
     * basis. It includes some extra information that is specific to the 
     * dictionary's management techniques, but should not be exposed to the 
     * developer.
     *
     * @param ClassID
     * The ClassID of the requested dictionary entry.
     *
     * @param SearchMode
     * A uint8 value representing the searching mode for this method call. 
     * This must be a value from the SearchMode_t set, and will use this
     * value only for this call.
     *
     * @return
     * A reference to an entry related to a specific definition. If no entry 
     * exists for the ClassID, this will return NULL.
     *
     *//*====================================================================*/
    TableEntry_t* FindEntry(uint32 ClassID, SearchMode_t SearchMode);
    
};

/*=============================================================================
    Global functions to help access definition information
==========================================================================*//**
* Global function to extract the ClassID from a definition.
*
* @param pDefinition
* A reference to an existing definition that will be scanned to extract its 
* ClassID.
*
* @return
* A uint32 representing the ClassID of the definition.
*
*//*=========================================================================*/
uint32 DefinitionClassID(void* pDefinition);

/*========================================================================*//**
* Global function to extract the version from a definition.
*
* @param pDefinition
* A reference to an existing definition that will be scanned to extract its 
* version.
*
* @return
* A uint16 representing the version of the definition.
*
*//*=========================================================================*/
uint16 DefinitionVersion(void* pDefinition);

/*========================================================================*//**
* Global function to determine the size a definition.
*
* @param pDefinition
* A reference to an existing definition that will be scanned to determine its
* size.
*
* @return
* A uint16 representing the memory size of the definition, in bytes.
*
*//*=========================================================================*/
uint16 DefinitionSize(void* pDefinition);

/*========================================================================*//**
* Global function to determine the full size a definition (This includes
* the size of the XML text at the end of the definition, if it's present).
*
* @param pDefinition
* A reference to an existing definition that will be scanned to determine its
* size.
*
* @return
* A uint16 representing the memory size of the definition, in bytes.
*
*//*=========================================================================*/
uint16 DefinitionFullSize(void* pDefinition);

/*========================================================================*//**
* Global function to determine the XML Base size a definition.
*
* @param pDefinition
* A reference to an existing definition that will be scanned to determine its
* XML Base size.
*
* @return
* A uint16 representing the string length of the basic XML elements.
*
*//*=========================================================================*/
uint16 DefinitionXMLSize(void* pDefinition);

/*========================================================================*//**
* Global function to determine the number of members in a definition.
*
* @param pDefinition
* A reference to an existing definition that will be scanned to count the 
* number of members. 
*
* @return
* A uint16 representing the number of members included in the definition.
*
*//*=========================================================================*/
uint16 DefinitionMemberCount(void* pDefinition);

/*========================================================================*//**
* Global function to determine if a species is support by a definition.
*
* @param pDefinition
* A reference to an existing definition that will be scanned to check the 
* supported species. 
*
* @return
* A boolean representing whether the species exists in the definition or not.
* TRUE means it is supported, FALSE means it is not.
*
*//*=========================================================================*/
boolean IsSpeciesValid(void* pDefinition, uint8 Species);

/*===========================================================================*/

#endif /* #define CDMOIBDICTIONARY_H */
