/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2005-2006 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CDMOIBObject.h                                                    //
'**  '// Version        :   1.00                                           //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CDYNDMOIB_H)
    #define  CDYNDMOIB_H

#include "DMOIBBase.h"

/*
===============================================================================
    NOTE: Doxygen is used to provide automatic documentation of the Dynamic 
          DMOIB APIs. With this, all variables for internal structures or 
          classes are declared ahead of time, and separately from the location
          of the actual member declarations.
===============================================================================
*/

/*===========================================================================*/
/**
 * @brief
 * An enumeration to determine the max data member byte-size.
 *
 * This performs a compile-time comparison of all supported Dynamic DMOIB types 
 * to determine the largest sized type for the current system. This provides a 
 * system/compiler-independent method for finding the maximum memory space 
 * that each member will occupy.
 */
/*===========================================================================*/
typedef enum MemberSize_e
{
    MAX_MEMBER_SIZE =   MAX(sizeof(int8),
                        MAX(sizeof(int16), 
                        MAX(sizeof(int32), 
                        MAX(sizeof(uint8), 
                        MAX(sizeof(uint16), 
                        MAX(sizeof(uint32),                        
                        MAX(sizeof(float32), 
                        MAX(sizeof(float64), 
                        MAX(sizeof(byte), 
                        MAX(sizeof(word), 
                        MAX(sizeof(boolean), 
                        MAX(sizeof(ansichar),
                        MAX(sizeof(widechar),
                        MAX(sizeof(MOIB_TIME), 
                        MAX(sizeof(float), 
                        MAX(sizeof(double), 
                            sizeof(void*)
                            ))))))))))))))))

}MemberSize_t;

/*
===============================================================================
    Forward declaration of classes 
===============================================================================
*/
class CDMOIBObject;
class CDMOIBVector;
class CDMOIBDictionary;
class CDMOIBStringTable;

/*
===============================================================================
    CDMOIBData_t class definition
===============================================================================
*/
/**
 * @brief
 * The base DMOIB Data structure, used to represent all Dynamic DMOIB types with
 * a single class/data structure. 
 *
 * Since Dynamic DMOIB uses run-time typing, a single variant-type structure is
 * used to represent the actual data. This is declared as a union since each 
 * data member will only represent a single value at any point in time. For 
 * each data type supported, a separate union member is declared specifically
 * for that type. These individual members provide direct access to the data 
 * for each specific type.
 *
 * This union also provides cast operator overloads for each data type 
 * supported by Dynamic DMOIB, allowing for implicit type-casting when assigning
 * values directly from a CDMOIBData_t union to a simple data type (uint32, 
 * float32, etc.)
 */
/*===========================================================================*/
typedef union CDMOIBData_u
{
    friend class CDMOIBObject;
    friend class CDMOIBVector;

private:

    /*====================================================================*//**
     * @brief
     * A separate structure to store pointer-based information. 
     *
     * This structure includes the actual pointer value itself, an element 
     * count for the number of elements stored at that location, and a separate
     * memory size identifying the actual memory occupied by the reference. The
     * element count and memory size are separate values to accomodate data 
     * types where each element occupies more that a single byte of memory. 
     *
     * Also, the pointer value stored in this structure overlaps with the Ptr 
     * member of the parent CDMOIBData_t union (In all cases, the Pointer member
     * of this structure will be equivalent to the Ptr member of the 
     * CDMOIBData_t union). 
     *
     *//*====================================================================*/
    struct PointerInfo_s
    {
        /*================================================================*//**
         * @var void* m_pPointer
         * Generic pointer used for all pointer types.
         *//*=============================================================*//**
         * @var uint16 m_NumElements
         * Number of elements referenced by the Pointer member of this 
         * structure.
         *//*=============================================================*//**
         * @var uint16 m_MemSize
         * Memory space, in bytes, referenced by the Pointer member of this 
         * structure. 
         *//*================================================================*/

        void*   m_pPointer;
        uint16  m_NumElements;
        uint16  m_MemSize;

    }m_PointerInfo;

    /*====================================================================*//**
     * @brief
     * A separate structure to store type information. 
     *
     * This structure provides a padding area that occupies the same memory 
     * space as the largest type supported by Dynamic DMOIB, as well as an 
     * additional byte for type information. The padding is used to easily 
     * access and overwrite the CDMOIBData_t data, regardless of what type is 
     * represented.
     *
     *//*====================================================================*/
    struct MemberInfo_s
    {
        /*================================================================*//**
         * @var uint8 m_Padding[MAX_MEMBER_SIZE]
         * Generic memory space used to access any data type, regardless of the 
         * type information. 
         *//*=============================================================*//**
         * @var uint8 m_TypeInfo
         * This TypeInfo is a value from the DMOIBTypes_e set, and provides type
         * information at run-time. It is the primary source of type
         * information, and is used when deciding conversion routines and 
         * handling for each data member. 
         *
         * @see enum DMOIBTypes_e
         *//*================================================================*/

        uint8   m_Padding[MAX_MEMBER_SIZE]; 
        uint8   m_TypeInfo;                 

    }m_MemberInfo;

    /*====================================================================*//**
     * @brief
     * Generic function for accessing all pointer types.
     *
     * All pointer types are stored in the same manner, with the only 
     * distinguishing factors being the byte-width of individual elements,
     * and whether those elements are integral values or floating point. This
     * method provides a single interface for accessing all pointer types 
     * (arrays or objects). 
     *
     * @param BytesWide 
     * The byte-width of the elements for the member being accessed 
     * (Note: this is the byte-width stored in the type information, not
     * the exact byte-width, meaning it is minus 1 from the actual 
     * size. Ex: byte-width of a uint32 array is sizeof(uint32) - 1 = 
     * 4 - 1 = 3).
     *        
     * @param IsFP 
     * Optional parameter to specify whether the elements of this member
     * are floating point values or not. This defaults to 0, assuming
     * integral values.
     *
     * @return
     * A pointer to the referenced array/object, if all conditions are 
     * met (byte-width and floating point state match). Otherwise 
     * returns NULL.
     *
     *//*====================================================================*/
    void* GetPtr(uint8 BytesWide, uint8 IsFP = 0);

public:

    /*-------------------------------------------------------------------------
        Publically accessible union members (to allow direct developer 
        access, if needed)
    -------------------------------------------------------------------------*/
    int8        Int8;     /**< Member for int8 types (DMOIB_TYPE_INT8)        */
    int16       Int16;    /**< Member for int16 types (DMOIB_TYPE_INT8)       */
    int32       Int32;    /**< Member for int32 types (DMOIB_TYPE_INT8)       */
    uint8       Uint8;    /**< Member for uint8 types (DMOIB_TYPE_UINT8)      */
    uint16      Uint16;   /**< Member for uint16 types (DMOIB_TYPE_UINT16)    */
    uint32      Uint32;   /**< Member for uint32 types (DMOIB_TYPE_UINT32)    */
    float32     Float32;  /**< Member for float32 types (DMOIB_TYPE_FLOAT32)  */
    float64     Float64;  /**< Member for float64 types (DMOIB_TYPE_FLOAT64)  */
    ansichar    Ansichar; /**< Member for ansichar types (DMOIB_TYPE_ANSICHAR)*/
    widechar    Widechar; /**< Member for ansichar types (DMOIB_TYPE_WIDECHAR)*/
    MOIB_TIME   Time;     /**< Member for MOIB_TIME types (DMOIB_TYPE_TIME)   */
    void*       Ptr;      /**< Member for all array/object types.            */

    /*-------------------------------------------------------------------------
        Cast operators (Gets) for single members
    -------------------------------------------------------------------------*/
    operator int8       (); /**< Conversion operator to int8 types           */
    operator int16      (); /**< Conversion operator to int16 types          */
    operator int32      (); /**< Conversion operator to int32 types          */                            
    operator uint8      (); /**< Conversion operator to uint8 types          */
    operator uint16     (); /**< Conversion operator to uint16 types         */
    operator uint32     (); /**< Conversion operator to uint32 types         */                            
    operator float32    (); /**< Conversion operator to float32 types        */
    operator float64    (); /**< Conversion operator to float64 types        */                            
    operator ansichar   (); /**< Conversion operator to ansichar types       */
    operator MOIB_TIME  (); /**< Conversion operator to MOIB_TIME types      */

    /*-------------------------------------------------------------------------
        Cast operators (Gets) for Arrays/Dynamics
    -------------------------------------------------------------------------*/
    operator int8*      (); /**< Conversion operator to int8 array types     */
    operator int16*     (); /**< Conversion operator to int16 array types    */
    operator int32*     (); /**< Conversion operator to int32 array types    */                            
    operator uint8*     (); /**< Conversion operator to uint8 array types    */
    operator uint16*    (); /**< Conversion operator to uint16 array types   */
    operator uint32*    (); /**< Conversion operator to uint32 array types   */                           
    operator float32*   (); /**< Conversion operator to float32 array types  */
    operator float64*   (); /**< Conversion operator to float64 array types  */                            
    operator ansichar*  (); /**< Conversion operator to ansichar array types */
    operator MOIB_TIME* (); /**< Conversion operator to MOIB_TIME array types*/
    operator void*      (); /**< Conversion operator to generic void* types  */
    operator CDMOIBObject*();/**< Conversion operator to CDMOIBObject* types */
    operator CDMOIBVector*();/**< Conversion operator to CDMOIBVector* types */

}CDMOIBData_t;

#define DIRTY_FLAG_MASK 0x80 //  [1000 0000]
#define VALID_FLAG_MASK 0x40 //  [0100 0000]

#if ((TARGET_XML_SUPPORT==XML_DESERIALIZE) || (TARGET_XML_SUPPORT==XML_ALL))
/*========================================================================*//**
 * @brief
 * Custom user data structure used when parsing XML data
 *
 * This custom structure is used to maintain state based information for XML
 * parsing processes. As long as XML parsing is enable (as sepcified by the 
 * TARGET_XML_SUPPORT macro), CDMOIBObject and CDMOIBVector instances will 
 * include one of these structures to maintain parsing information..
 *//*========================================================================*/
typedef struct XMLUserData_s
{
    struct XMLUserData_s*   m_pXMLParent;
    void*                   m_pXMLObject;
    uint8                   m_XMLObjectType;
    uint16                  m_XMLCurrentIndex;
    boolean                 m_XMLContinueParsing;

}XMLUserData_t;
#endif

/*=============================================================================
    Dynamic DMOIB class definition
==========================================================================*//**
 * @brief
 * The base Dynamic DMOIB class used to store and manage various CDMOIBData_t 
 * members.
 *
 * This is the basic Dynamic DMOIB class used to manage DMOIB object 
 * instantiations. This class provides access to the standard DMOIB information
 * (ClassID, Version, etc.), and also provides generic interfaces for accessing
 * all types of data members. This acts as a collection of CDMOIBData_t objects
 * that is mutable to the point of instantiation. While an object can be 
 * created dynamically, and two objects can have separate formats based on their 
 * respective definitions, a single object cannot change it's internal layout
 * once it has been created. 
 *
 * The CDMOIBObject class also provides methods for serializing and 
 * deserializing data to binary streams, as well as proprietarily formatted
 * XML. While binary (de)serialization is always available, XML 
 * (de)serialization can be configured at compile time through the use of 
 * preprocessor definitions in target.h. This is because some embedded systems 
 * may not want the burden of including methods to create and parse XML 
 * strings.
 *
 *//*========================================================================*/
class CDMOIBObject
{    
    /*====================================================================*//**
     * @brief
     * Declaration to give CDMOIBVector instances private access.
     *
     * CDMOIBVector and CDMOIBObject instances interact frequently, and to 
     * make data access between them simpler, they are given access to 
     * private members.
     *//*====================================================================*/
    friend class CDMOIBVector;

#if ((TARGET_XML_SUPPORT==XML_DESERIALIZE) || (TARGET_XML_SUPPORT==XML_ALL))
    /*====================================================================*//**
     * @brief
     * Declaration to give XML parsing routines private access.
     *
     * During the parsing process, these methods need access to the private
     * members that store XML related information. So they are declared as 
     * friends here to allow that access.
     *//*====================================================================*/
    friend void XMLStartElement(void* pUserData, void* pTagName, 
                                void* pAttributes);
    friend void XMLEndElement(void* pUserData,void* pTagName);
    friend void XMLCharacterData(void* pUserData, void* pCDataString, 
                                 int CDataLength);
#endif

private:    
    /*====================================================================*//**
     * @var     CDMOIBData_t* m_pMemberArray
     * @brief
     * Head of the CDMOIBData_t member collection.
     *
     * This is a pointer to the head of the CDMOIBData_t member array for 
     * this object. This array is fixed in size for the lifetime of the object,
     * but is dynamically created during construction.
     *
     * @note
     * The data members contained in this array may be disparate from the
     * original definition. This is due to obsoleted data 
     * members that may not be available in the most recent version of the 
     * definition. Those members which are no longer in use are skipped 
     * during the parsing/construction phase of the object, and will not be
     * available in this array.
     *
     *//*=================================================================*//**
     * @var     CDMOIBData_t* m_pMemberIter
     * @brief
     * Iterator for keeping reference to the last accessed member in 
     * m_MemberArray.
     *
     * This is a CDMOIBData_t iterator used for traversing m_MemberArray. 
     * Certain methods need knowledge of the most recently accessed member, and
     * this variable is used to retain that reference. 
     *
     *//*=================================================================*//**
     * @var     CDMOIBData_t* m_pMemberEnd
     * @brief
     * Tail of the CDMOIBData_t member collection.
     *
     * This is a pointer to the tail of the CDMOIBData_t member array for 
     * this object. This pointer is set to an emtpy CDMOIBData_t member at the
     * final position of the collection, with a type of DMOIB_TYPE_UNKNOWN, and
     * does not change for the lifetime of the object. 
     * 
     * This is also used as a stopping/error condition for lookups, in 
     * conjunction with m_MemberIter. After a lookup routine has finished, if 
     * m_MemberIter and m_MemberEnd both reference the same member, then the 
     * lookup was unsuccessful.
     *
     *//*=================================================================*//**
     * @var     CDMOIBData_t* m_pMemberDynStart
     * @brief
     * Reference to the first dynamic member in the CDMOIBData_t collection 
     * (dynamic array or DMOIB sub-object/vector).
     *
     * This is a reference to the first CDMOIBData_t member that is categorized 
     * as a dynamic member. This includes any dynamic array type, DMOIB 
     * sub-objects, and DMOIB Vectors. This reference is set during construction 
     * of the object, and doesn't change for it's lifetime. It is used as a 
     * quick reference to the first dynamic member in the CDMOIBData_t 
     * collection when performing binary (de)serialization (Since dynamic 
     * members are treated differently during the (de)serialization process, 
     * and are positioned after all static members in the binary stream).
     *
     *//*=================================================================*//**
     * @var     uint16 m_MemberCount
     * @brief
     * Number of CDMOIBData_t members available in this object.
     *
     * This is a count of the total CDMOIBData_t members available in this 
     * object. This is set during construction and does not change for the 
     * lifetime of the object. 
     *
     * @note
     * This value may not match the number of members available in the 
     * original definition. This is due to obsoleted data 
     * members that may not be available in the most recent version of the 
     * definition. Those members which are no longer in use are skipped 
     * during the parsing/construction phase of the object, and will not be
     * counted towards the total number of members.
     *
     *//*=================================================================*//**
     * @var     uint16* m_pMemberTranslation
     * @brief
     * A lookup table to translate MemberIDs to offets in the CDMOIBData_t 
     * collection.
     *
     * This is an array of uint16 values used as a translation table to convert
     * MemberIDs from the original DMOIBDefinitions.h file to proper offsets
     * into m_MemberArray. Since m_MemberArray can be a disparate collection of
     * CDMOIBData_u members, which may not include all of those available in the
     * original definition, MemberIDs cannot always be assumed to have a direct
     * correlation in m_MemberArray. 
     *
     * m_MemberTranslation provides a way of 
     * determining the availablility of a member, along with it's offset. If a
     * member is not available, then the lookup offset matching it's MemberID 
     * will reference m_MemberEnd. Otherwise, the lookup offset matching it's 
     * MemberID can be trusted to reference the correct CDMOIBData_t member in 
     * m_MemberArray.
     *
     *//*=================================================================*//**
     * @var     const byte* m_pDefinition
     * @brief
     * Reference to the original definition used to create this object.
     *
     * This is a reference to the original definition used when this object 
     * was created. The DMOIB object needs to maintain a reference to the 
     * definition because there are values associated with it that may effect
     * access and management of the object after instantiation. In particular,
     * the min/max versions for each member are referenced when checking
     * the availability of that member the current instance of the object.
     *
     * @note
     * For all external interfaces, definitions are defined by the void*
     * type, while here, it is defined as a const byte*. This is because the
     * actual format of the defintion is a byte*, and when it is parsed, it
     * is easier to access it in that manner. The void* type is used to
     * provide an external layer of abstraction and dissuade developers from
     * accessing the Definition themselves.
     *
     *//*=================================================================*//**
     * @var     uint16 m_StaticMemberSize
     * @brief
     * The cummulative size, in bytes, of all static members.
     *
     * This value is the total size, in bytes, or all static data members for
     * this object. This value is determined at construction and remains the 
     * same for the lifetime of the object. Since static members will never
     * change size, this only needs to be calculated once and can then be used
     * as-is by any method that needs to determine the serialized size of the
     * object.
     *
     *//*=================================================================*//**
     * @var     uint32 m_ClassID
     * @brief
     * The ClassID of this object (Family | Genus | Species).
     *
     * This ClassID for this object, which follows the standard DMOIB 
     * convention; Family being the most significant 16 bits of this value, 
     * Genus being the next 8 bits, and Species being the least significant 
     * 8 bits.
     *
     *//*=================================================================*//**
     * @var     uint16 m_Version
     * @brief
     * The Version of this object.
     *
     * The Version of this object, with the one's and ten's positions 
     * accounting for the minor portion of the version, and the rest of the 
     * value accounting for the major portion.
     *
     * @note
     * This value may or may not match the version of the definition used to 
     * create an instance of this class. The developer can specify the version
     * of the object they want to instantiate in the constructor, and as long
     * as that version is less than or equal to the highest known version in 
     * the dictionary, then an object of that particular version will be 
     * created.
     *
     *//*=================================================================*//**
     * @var     uint8 m_DataBitField
     * @brief
     * An 8-bit field used to store binary settings.
     *
     * An 8-bit field that stores various state-based information for an 
     * object, including its dirty state, its validity, and the presence of
     * encryption or compression. Each bit in the member is used as a flag to
     * represent the binary state of these settings.
     *
     *//*=================================================================*//**
     * @var     CDMOIBDictionary* m_pDictionary
     * @brief
     * A reference to a CDMOIBDictionary object.
     *
     * A reference to an existing CDMOIBDictionary object that can be used for
     * looking up additional definitions based on ClassID. This is necessary
     * during construction of an object with sub-objects, as sub-objects are 
     * denoted by ClassID in definitions. The dictionary is used to resolve 
     * the sub-object ClassIDs to full definitions that are subsequently 
     * instantiated. 
     *
     *//*=================================================================*//**
     * @var     CDMOIBStringTable* m_pStringTable
     * @brief
     * A reference to a CDMOIBStringTable object.
     *
     * A reference to an existing CDMOIBStringTable object that can be used for
     * looking up textual descriptions. This is used to implicitly link 
     * members and enumerations to strings that represent them.
     *
     *//*====================================================================*/

    CDMOIBData_t*       m_pMemberArray;    
    CDMOIBData_t*       m_pMemberIter;
    CDMOIBData_t*       m_pMemberEnd;
    CDMOIBData_t*       m_pMemberDynStart;
    uint16*             m_pMemberTranslation;
    uint16              m_MemberCount;    

    const byte*         m_pDefinition;
    const byte*         m_pDefinitionRef;

    uint16              m_StaticMemberSize;
    uint32              m_ClassID;
    uint16              m_Version;
    uint8               m_DataBitField;    

    CDMOIBDictionary*   m_pDictionary;
    CDMOIBStringTable*  m_pStringTable;

#if ((TARGET_XML_SUPPORT==XML_DESERIALIZE) || (TARGET_XML_SUPPORT==XML_ALL))
    /*====================================================================*//**
     * @var     XMLUserData_t m_XMLUserData;
     * @brief
     * A custom data structure for XML parsing
     *
     * This is a custom structure for maintaining state information when 
     * parsing an XML stream. It is only included in the class definition if
     * XML_DESERIALIZE is enabled.
     *//*====================================================================*/
    XMLUserData_t       m_XMLUserData;
#endif

    /*=========================================================================
        Private helper methods
    ===========================================================================
        Member Lookup methods
    ======================================================================*//**
     * @brief
     * Method to find a member based on its MemberID.
     *
     * This method will resolve a numeric MemberID to a CDMOIBData_t union that
     * represents the member associated with that ID. This method performs 
     * bounds checking to make sure the requested member is available.
     *
     * @note
     * This method does not return any value, but instead modifies the class
     * member m_MemberIter. Once this method has completed, then m_MemberIter 
     * will reference the proper DMOIB member, if it was found. If no member 
     * could be found, then m_MemberIter will be equivalent to m_MemberEnd.
     *
     * @param MemberID
     * A uint16 numeric value that is a unique identifier for a specific 
     * member, within the scope of this object's definition. An enumerated
     * value is automatically generated for each member listed in the 
     * DMOIBDefinitions.h, and is labeled with the name of the member.
     *
     *//*====================================================================*/
    void FindMember(const uint16 MemberID);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    void FindMemberAt(uint16 Offset);

    /*========================================================================
        Private helper methods for setting members
    ======================================================================*//**
     * @brief
     * Method to set the value of the member referenced by m_MemberIter.
     *
     * This is a helper method to assign values to any supported DMOIB type. 
     * All publicly exposed SetMember() methods that handle single element 
     * types use one of these methods internally for modifying the actual 
     * member values. All unsigned integer values are passed-through as uint32
     * since that type has the ability to handle any smaller unsigned integer 
     * data types. Similar is true for signed and floating point values.
     *
     * @param Value
     * A value for a supported MOIB type that will be assigned to m_MemberIter.
     *
     * @return
     * The status of the assignment. TRUE denotes success, while FALSE denotes
     * failure.
     *//*====================================================================*/
    boolean SetMember(const uint32 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(const int32 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(const float64 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(const MOIB_TIME& Value);

    /*====================================================================*//**
     * @brief
     * Method to set the value of ANY member that is an array type.
     *
     * This is a helper method to assign values to any supported DMOIB array 
     * type. All publicly exposed SetMember() methods that handle array types
     * use this method internally for modifying the actual arrays. It treats
     * all data types generically, distinguishing them only by their byte-size 
     * and floating point status. It assumes that m_MemberIter will reference
     * the proper member when this method is called, and makes appropriate 
     * type checks on that member to ensure its compatibility. In the case of
     * a static array, the size of the data being assigned is checked to make 
     * sure it won't overflow the array. The method then memcpy()'s the data 
     * passed to it into the internal array, performing resizes and allocations
     * as necessary for dynamic arrays.
     *
     * @param pValue
     * A pointer to an array of values of supported DMOIB types that will be 
     * assigned to the specified member. 
     *
     * @param ValueSize
     * A uint8 value representing the byte-size of individual elements in the
     * array referenced by Value.
     *
     * @param ValueLength
     * A uin16 value representing the number of elements in the array 
     * referenced by Value.
     *
     * @param IsFP
     * A uint8 value that represents the floating point status of the array 
     * referenced by Value. A value of 0 means it does not contain floating
     * point values, while a value of DMOIB_TYPE_FLAG_FP means it does.
     *
     * @return
     * The status of the assignment. TRUE denotes success, while FALSE denotes
     * failure.
     *//*====================================================================*/
    boolean SetBuffer(void* pValue, uint8 ValueSize, uint16 ValueLength, 
                      uint8 IsFP = 0);

    /*========================================================================
        Private helper methods for constructing and destructing an object
    ======================================================================*//**
     * @brief
     * Helper method for constructing the object.
     *
     * This method is a helper used by all public constructors to instatiate 
     * a DMOIB object. All of the public constructors have different interfaces,
     * but end up handling the same data and performing the same parsing 
     * process when creating an object, so this is provided as a pass-through 
     * to them. 
     *
     * @param pDictionary
     * Reference to an existing dictionary that can translate ClassIDs to 
     * definitons.
     *
     * @param pDefinition
     * An existing definition that will be used directly to construct the 
     * DMOIB object. If this definition has no sub-objects, then no lookup
     * is performed with the provided dictionary.
     *
     * @param ClassID
     * A uint32 value representing the ClassID of this object. This is 
     * necessary for definitions that have multiple Species assigned to them.
     * In that case, the default Family|Genus|Species combination cannot be 
     * used, because the Species is different. This explicit ClassID is 
     * assigned before the definition is parsed, and then compared against the
     * list of valid Species for that definition (instead of the reverse where
     * the ClassID would be assigned directly from the default available in the
     * definition).
     *
     * @param pBuffer
     * An optional parameter that specifies a serialized byte stream the object
     * should be created from. This will override any definition, classID, or
     * version values passed in, and use the values extracted from the stream.
     *
     * @param Version
     * A uint16 value that specifies the version of this object. This parameter
     * must be less than or equal to the version of the provided definition.
     *
     *//*====================================================================*/
    void Construct(CDMOIBDictionary* pDictionary, void* pDefinition, 
                   uint32 ClassID, uint16 Version, uint8* pBuffer = NULL);
    /*====================================================================*//**
     * @brief
     * Method to perform clean-up and deallocations for the object.
     *
     * Similar to the Construct() method, this is provided as a pass-through
     * for the destructor, to maintain consistency. All work to clean up the 
     * instantiated MOIB object is done here. This deletes any allocated 
     * members, and in turn their allocated space, along with lookup values.
     *
     *//*====================================================================*/
    void Destruct();

public:    
    /*=========================================================================
        Public Methods
    ===========================================================================
        Basic Constructors
    ======================================================================*//**
     * @brief
     * Basic constructor to build an object from a Dictionary and Definition.
     *
     * This basic constructor takes a reference to an existing dictionary, and 
     * a known definition to create an object that follows the data structure
     * described in that definition. This requires a reference to an existing
     * dictionary in the case that the provided definition has members that 
     * are sub-objects. A sub-object member only stores the ClassID for that 
     * member in the definition, so the dictionary is required to find the
     * archived definition related to that ClassID. 
     *
     * The other constructors follow a similar routine, differing only by 
     * using a ClassID or Family/Genus/Species combination directly for an
     * initial definition lookup using the dictionary. 
     *
     * @note
     * If the ClassID or Family|Genus|Species can't be resolved in the provided
     * Dictionary, then the object will still be created, but will effectively
     * be empty (contain no members - MemberCount() == 0).
     *
     * @param pDictionary
     * Reference to an existing dictionary that can translate ClassIDs to 
     * definitons.
     *
     * @param pDefinition
     * An existing definition that will be used directly to construct the 
     * DMOIB object. If this definition has no sub-objects, then no lookup
     * is performed with the provided dictionary.
     *
     * @param Version
     * Optional argument that allows the developer to instantiate an object 
     * with a specific version. This parameter must be less than or equal to 
     * the version of the provided definition. If this parameter is greater
     * than the version of the definition, it will default to the same 
     * version as the definition.
     *//*====================================================================*/
    CDMOIBObject(CDMOIBDictionary* pDictionary, void* pDefinition, 
                 uint16 Version = HIGHEST_KNOWN_VERSION);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    CDMOIBObject(CDMOIBDictionary* pDictionary, uint32 ClassID, 
                 uint16 Version = HIGHEST_KNOWN_VERSION);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    CDMOIBObject(CDMOIBDictionary* pDictionary, 
                 uint16 Family, uint8 Genus, uint8 Species, 
                 uint16 Version = HIGHEST_KNOWN_VERSION);

    /*=========================================================================
        Deserialize constructor
    ======================================================================*//**
     * @brief
     * Constructor to create an object from a stream.
     *
     * This constructor takes an existing serialized binary stream, extracts
     * class information from it, constructs a new object based on that 
     * information, then deserializes the content of the stream into the 
     * object. The result is an object that matches the content of the stream
     * exactly, including version.
     *
     * @param pDictionary
     * Reference to an existing dictionary that can translate ClassIDs to 
     * definitons.
     *
     * @param pBuffer
     * Reference to a binary stream containing the serilialized contents of
     * a MOIB object.
     *//*====================================================================*/
    CDMOIBObject(CDMOIBDictionary* pDictionary, uint8* pBuffer);

    /*=========================================================================
        Copy Constructors
    ======================================================================*//**
     * @brief
     * Standard copy constructor.
     *
     * Standard copy constructor that creates an object using the same 
     * definition used by ObjectCopy, with the same version. Once the 
     * construction is complete, the public Copy method is called to copy all
     * data from the parameter object to the new instantiation. 
     *
     * @note
     * This copy method recreates all data from the original object, 
     * particularily for dynamic arrays. It does not merely copy references 
     * to existing data, it creates entirely new dynamic arrays where the data
     * of those arrays is copied from the originals.
     *
     * @param pObjectCopy
     * Pointer to an existing DMOIB object that will have all of it's data 
     * copied to the new object.
     *//*====================================================================*/
    CDMOIBObject(CDMOIBObject* pObjectCopy);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    CDMOIBObject(CDMOIBObject& ObjectCopy);

    /*====================================================================*//**
     * @brief
     * Standard destructor.
     *
     * Standard destructor that cleans up all allocated memory.
     *//*====================================================================*/
    ~CDMOIBObject();

    /*====================================================================*//**
     * @brief
     * Mutator to set the CDMOIBStringTable object.
     *
     * This mutator allows the developer to link this DMOIB object with an
     * existing string table. The DMOIB object can then use that string table 
     * as a source of textual descriptions for members and enumerations. This
     * allows the developer to directly query the DMOIB object for textual
     * descriptions instead of needing to maintain a reference to the string
     * table themselves. 
     *
     * @note
     * An DMOIB object can only be linked to a single string table at any point
     * in time, and successive calls to this method will overwrite the previous
     * reference to a string table.
     *
     * @param pStringTable
     * Pointer to an existing string table that has textual descriptions 
     * related to this object. 
     *
     * @return
     * The status of the assignment. TRUE denotes success, while FALSE denotes
     * failure.
     *//*====================================================================*/
    boolean SetStringTable(CDMOIBStringTable* pStringTable);

    /*====================================================================*//**
     * @brief
     * Accessor to retrieve the CDMOIBStringTable object.
     *
     * This provides access to the linked string table that has been 
     * previously assigned with SetStringTable(). This reference can be used
     * by the developer to retrieve extra textual descriptions that might not
     * be directly available through an DMOIB object's interface, and also 
     * provides a means of determining if the object is actually linked to
     * a string table.
     *
     * @return
     * A pointer to the linked string table. A value of NULL means that no 
     * string table is available for this object.
     *//*====================================================================*/
    CDMOIBStringTable* GetStringTable();

    
    /*========================================================================
        Standard DMOIB methods
    ======================================================================*//**
     * @brief     
     * Method to retrieve the ClassID
     *
     * This is a standard DMOIB method to get the ClassID of the current object.
     * This ClassID follows the normal format, with the most significant 16 
     * bits representing Family, the next significant 8 bits representing the 
     * Genus, and the least significant 8 bits representing the species.
     * 
     * @return
     * The uint32 value of the ClassID.
     *//*====================================================================*/
    uint32 ClassID();

    /*====================================================================*//**
     * @brief     
     * Method to retrieve the Family
     *
     * This is a standard DMOIB method to get the Family of the current object.
     * This is the most significant 16 bits of the ClassID for the object.
     *
     * @return
     * The uint16 value of the Family.
     *//*====================================================================*/
    uint16 Family();

    /*====================================================================*//**
     * @brief     
     * Method to retrieve the Genus
     *
     * This is a standard DMOIB method to get the Genus of the current object.
     * This is the most significant 8 bits of the low order 16 bits of the 
     * ClassID for the object.
     *
     * @return
     * The uint8 value of the Genus.
     *//*====================================================================*/
    uint8 Genus();

    /*====================================================================*//**
     * @brief     
     * Method to retrieve the Species
     *
     * This is a standard DMOIB method to get the Species of the current object.
     * This is the least significant 8 bits of the ClassID for the object.
     *
     * @return
     * The uint8 value of the Species.
     *//*====================================================================*/
    uint8 Species();

    /*====================================================================*//**
     * @brief     
     * Method to retrieve the Version
     *
     * This is a standard DMOIB method to get the version of the current object.
     * This is the version of the instantiated object, which may or may not
     * match the version of the original definition used to create it. Since 
     * object definitions contain full history information for all previous 
     * versions, an object can be instantiated with any version less than or
     * equal to the version of the definition.
     *
     * @return
     * The uint16 value of the object's version (see the description of 
     * m_Version).
     *//*====================================================================*/
    uint16 Version();

    /*====================================================================*//**
     * @brief
     * Method to determine the dirty state of the object.
     *
     * This method is used to determine the dirty state of the object. There
     * is an associated valid bit in m_DataBitField within the DMOIB object 
     * that represents this value. The dirty bit is never set by an internal 
     * DMOIB method and is only cleared on object instantiation or a call to 
     * Reset(). This bit is only modifiable by the SetDirty() method, and  
     * maintenance of this flag is left entirely to the developer. 
     *
     * @return
     * A boolean representing the dirty state of the object. The intended 
     * use of this flag is to show that some internal values of the object 
     * have changed from a previously known base state.
     *
     *//*====================================================================*/
    boolean IsDirty();

    /*====================================================================*//**
     * @brief
     * Method to set the dirty state of the object.
     *
     * This method is used to set the dirty state of the object. This
     * will set the dirty bit of m_DataBitField to flag the object as dirty.
     * The bit is only modifiable by this method and is provided as an 
     * interface for the developer to maintain state information about an 
     * object.
     *
     * @param Dirty
     * A boolean value representing the new dirty state of the object. There
     * is no pre-condition to meet for this value to be accepted, and the 
     * object's dirty state will always match this value upon return.
     *
     * @return
     * A boolean representing the dirty state of the object, which will match
     * the boolean value passed to this method as an argument.
     *
     *//*====================================================================*/
    boolean SetDirty(boolean Dirty);

    /*====================================================================*//**
     * @brief     
     * Method to determine the validity of the object.
     *
     * This method is used to determine the validity of the object, which means
     * that the object has been modified since its initial state. There
     * is an associated valid bit in m_DataBitField within the DMOIB object 
     * that represents this value. The valid bit is never set by an internal 
     * DMOIB method and is only cleared on object instantiation or a call to 
     * Reset(). This bit is only modifiable by the SetValid() method, and  
     * maintenance of this flag is left entirely to the developer. 
     *
     * @return
     * A boolean representing the validity of the object. The intended 
     * use of this flag is to show that some internal values of the object 
     * have changed from the object's initial state.
     *
     *//*====================================================================*/
    boolean IsValid();

    /*====================================================================*//**
     * @brief     
     * Method to set the valid state of the object.
     *
     * This method is used to set the valid state of the object. This
     * will set the valid bit of m_DataBitField to flag the object as valid.
     * The bit is only modifiable by this method and is provided as an 
     * interface for the developer to maintain state information about an 
     * object.
     *
     * @param Valid
     * A boolean value representing the new valid state of the object. There
     * is no pre-condition to meet for this value to be accepted, and the 
     * object's valid state will always match this value upon return.
     *
     * @return
     * A boolean representing the valid state of the object, which will match
     * the boolean value passed to this method as an argument.
     *//*====================================================================*/
    boolean SetValid(boolean Valid);

    /*====================================================================*//**
     * @brief
     * Method to reset the entire object.
     *
     * This method will return the object to an initial state, as if it had 
     * just been constructed. This means all values for all members will be 
     * reset to zero, any dynamic arrays will be deallocated, and 
     * m_DataBitField will return all flags to their initial state.
     *
     * @param ResetAll
     * A boolean value specifying whether this reset operation should be 
     * performed recursively on any sub-objects that may exist within this
     * object. TRUE means to reset all sub-objects recursively, while FALSE
     * means to only reset the root object that this method was invoked on.
     *
     *//*====================================================================*/
    void Reset(boolean ResetAll = TRUE);

    /*====================================================================*//**
     * @brief
     * Method to determine the serialized size of this object.
     *
     * This method will evaluate the members of this object, and determine how
     * much space, in bytes, this object will occupy in a serialized form. This
     * is provided for developers to query the object before actually perfoming
     * a serialization to tell if they have enough space available for the
     * resultant stream, or if they should allocate more space. 
     *
     * @note 
     * This method does not allocate any memory itself, or write any values. 
     * It is merely a prescan to determine the size without modifying any 
     * memory.
     *
     * @return
     * A uint32 value representing the space, in bytes, that this object will
     * occupy in its serialized form.
     *//*====================================================================*/
    uint32 SizeAll();

    /*====================================================================*//**
     * @brief     
     * Method to produced a serialized stream representation of the current
     * object.
     *
     * This method converts the current object into a tightly packed, serial 
     * form. All data that was contained in the object itself is written to the
     * buffer provided, in Big-Endian form. This is a convention of all DMOIB 
     * objects, and it is important to note that regardless of system 
     * endianess, the serialized stream will always be Big-Endian. 
     *
     * @param pBuffer
     * A pointer to a uint8 buffer that the method will write the serialized 
     * object to. This buffer is assumed to be large enough to contain the 
     * entire stream, and it is the responsibility of the developer to ensure
     * this by performing a preliminary check with the SizeAll() method.
     *
     * @return
     * A uint32 value representing the number of bytes written to the buffer
     * argument. This should match the value returned from SizeAll() 
     * exactly.
     *//*====================================================================*/
    uint32 Serialize(uint8* pBuffer);

    /*====================================================================*//**
     * @brief
     * Method to parse a serialized stream and fill an object with its data.
     *
     * This method will take the output stream from a previously serialized 
     * object and parse the data from that stream. The stream is first 
     * checked for validity by performing a CRC check, comparing ClassIDs of 
     * the stream and the object to make sure they match, and comparing 
     * versions to make sure the stream is of a lesser or equal version to the
     * object. If all these pass, then the stream is parsed, and the data 
     * from it collected. 
     *
     * @param pBuffer
     * A pointer to a uint8 buffer containing a serialized object. Serialized
     * objects themselves include a size identifier directly in the stream, so
     * the size of the buffer can be determine from that.
     *
     * @return
     * A uint32 value representing the number of bytes read from the buffer
     * during the parsing process. On a CRC failure or version error (stream 
     * version is greater than the object's version), 0 is returned to denote
     * failure.
     *//*====================================================================*/
    uint32 Deserialize(uint8* pBuffer);

    /*====================================================================*//**
     * @brief
     * A standard copy method.
     *
     * This method will copy the entire contents from one object to this 
     * object. This will only succeed if the ClassID and Version from both 
     * objects match exactly.
     *
     * @note
     * If an error occurs during the copy process, this object will be returned
     * to an initial state with the Reset() method. This is because the 
     * validity of the data in the object can not be ensured, since the process
     * was halted prematurely. 
     *
     * @param pObjectCopy
     * A pointer to an existing DMOIB object, whose contents will be copied into
     * this object.
     *
     * @return
     * A boolean value representing the success of the copy. TRUE means that
     * the copy was completed, while FALSE means that the copy attempt was a 
     * failure.
     *//*====================================================================*/
    boolean Copy(CDMOIBObject* pObjectCopy);    

    /*========================================================================
        Ancillary information methods
    ======================================================================*//**
     * @brief
     * Method to determine the number of members in this object.
     *
     * This method allows the developer to determine the number of members in 
     * this object at run-time. Since an DMOIB object can be created from any
     * definition at run-time, it could conceivably have any number of data 
     * members. This provides a way to get the member count without having any
     * predetermined knowledge of the object or its definition.
     *
     * @return
     * A uint16 value representing the number of members in the object. This 
     * may or may not match the number of members in the original definition, 
     * since some members may have been obsoleted in the version of this 
     * object instance.
     *//*====================================================================*/
    uint16 MemberCount();

    /*====================================================================*//**
     * @brief  
     * Method to detemine if the requested member is available in this version
     * of the object.
     *
     * This method allows the developer to determine the availablity of a 
     * specific data member at run-time. Since definitions evolve over time, 
     * and data members that existed in past versions can be removed in future
     * versions, in certain cases a developer can't be sure if a member is
     * available in the current version of an object. 
     *
     * @note
     * In the case where definitions used by a device/system are loaded at 
     * run-time, or acquired from a network transmission, the developer should 
     * always check to make sure a member is available before accessing it.
     *
     * @param MemberID
     * A uint16 value representing the numeric MemberID assigned to a member
     * during the definition and compilation phase. 
     *
     * @return
     * A boolean value representing the availablity of the specified member. 
     * TRUE means the member is available, while FALSE means it is not.
     *//*====================================================================*/
    boolean MemberAvailable(const uint16 MemberID);

    /*====================================================================*//**
     * @brief
     * Method to get the textual description of the the definition used to
     * create this object.
     *
     * This method is used to retrieve a textual description of the original 
     * definition used to instantiate this object. This is a string literal 
     * matching the actual ClassName used for the original definition. 
     *
     * @note
     * This string is only available if the Object has been linked to a 
     * CDMOIBStringTable object using SetStringTable(), and only if that string
     * table has XML strings loaded. This should be noted in particular for 
     * cases where definitions are retrieved from a network transmission. In
     * that case, if this information is required, then the developer must 
     * take the steps to retrieve the additional data needed - being the XML
     * language string table.
     *
     * @return
     * A pointer to the constant string contained in the linked string table.
     * (This string MUST NOT be directly written to or modified.) NULL is 
     * returned if no string table is available, or if one is available, but 
     * does not have XML strings in it. 
     *//*====================================================================*/
    const ansichar* GetClassName();
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    const ansichar* GetClassName(uint8 LanguageID);

    /*====================================================================*//**
     * @brief
     * Method to retrieve the original definition used to create this object.
     *
     * This method allows the developer to retrieve a reference to the original
     * definition used when creating the object. In most cases, the developer 
     * shouldn't need to access the definition itself, and should be able to
     * accomplish everything with the interfaces provided through the
     * CDMOIBDictionary and CDMOIBObject classes. 
     *
     * @return
     * A reference to the original definition. This should never be NULL, as 
     * an object cannot be created without a definition.
     *//*====================================================================*/
    void* GetDefinition();    
    
    /*========================================================================
        Gets for all member types 
    ======================================================================*//**
     * @brief
     * Method to access a member based on it's MemberID.
     *
     * This method provides an interface for retrieving a member's value based
     * on its MemberID. The MemberID is an enumerated value, whose name should
     * match the MemberID specified in DMOIBDefinitions.h. 
     *
     * @param MemberID
     * A uint16 numeric value that is a unique identifier for a specific 
     * member, within the scope of this object's definition. An enumerated
     * value is automatically generated for each member listed in the 
     * DMOIBDefinitions.h, and is labeled with the name of the member.
     *
     * @param pNumElements
     * An optional parameter that is a pointer to an existing uint16 that 
     * will be filled with a value on return from the method. This is used 
     * when accessing array type members, to return the size of the array in 
     * a single method call. This value represents the number of elements for 
     * the member, not the size in bytes.
     *
     * @return
     * A reference to a CDMOIBData_t instance which contains the data for this
     * member. The CDMOIBData_t union has a number of cast operator overloads
     * which allows this reference to be assigned to a simple data type, and 
     * implicitly converted to match that type.
     *//*====================================================================*/
    CDMOIBData_t& GetMember(const uint16 MemberID, uint16* pNumElements = NULL);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    CDMOIBData_t& GetMemberAt(uint16 Index, uint16* pNumElements = NULL);

    /*====================================================================*//**
     * @brief
     * Method to retrieve the size of a member, in terms of elements
     *
     * This method provides an interface for querying the size of a data 
     * member, in terms of the number of elements it contains (NOT the
     * size of the member in bytes). For data members defined as single 
     * elements, this will always be 1. But for arrays of data types, this will
     * report the number of elements contained in that array.
     *
     * @param MemberID
     * A uint16 numeric value that is a unique identifier for a specific 
     * member, within the scope of this object's definition. An enumerated
     * value is automatically generated for each member listed in the 
     * DMOIBDefinitions.h, and is labeled with the name of the member.
     *
     * @return
     * A uint16 value representing the number of elements stored by this data
     * member.
     *//*====================================================================*/
    uint16 GetMemberNumElements(const uint16 MemberID);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    uint16 GetMemberAtNumElements(uint16 Index);

    /*====================================================================*//**
     * @brief
     * Method to retrive the type of a member.
     *
     * This method will return the DMOIB data type of the specified member, as
     * an encoded value. This should match a value from the enumerated set
     * enum DMOIBTypes_e. This provides a way to determine the type of each
     * data member at run-time.
     *
     * @param MemberID
     * A uint16 numeric value that is a unique identifier for a specific 
     * member, within the scope of this object's definition. An enumerated
     * value is automatically generated for each member listed in the 
     * DMOIBDefinitions.h, and is labeled with the name of the member.
     *
     * @return
     * A uint8 value that matches a value from the enumerated set enum 
     * DMOIBTypes_e. 
     *//*====================================================================*/
    uint8 GetMemberType(const uint16 MemberID);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    uint8 GetMemberAtType(uint16 Index);

    /*====================================================================*//**
     * @brief
     * Method to retrive the size of a member.
     *
     * This method will return the byte-size of the specified member. In the 
     * case of array types, this will return the byte-size of the elements 
     * stored within the array. This provides a way to determine the size of
     * each data member at run-time.
     *
     * @param MemberID
     * A uint16 numeric value that is a unique identifier for a specific 
     * member, within the scope of this object's definition. An enumerated
     * value is automatically generated for each member listed in the 
     * DMOIBDefinitions.h, and is labeled with the name of the member.
     *
     * @return
     * A uint16 value representing the byte-size of the member.
     *//*====================================================================*/
    uint16 GetMemberSize(const uint16 MemberID);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    uint16 GetMemberAtSize(uint16 Index);

    /*====================================================================*//**
     * @brief
     * Method to retrieve the alias of a member.
     *
     * This method will return a universal alias for the specified member that
     * can be used to associate this member with members from other objects
     * without prior knowledge. This allows members to be substituted in
     * algorithms or displays based on alias, as long as it matches the 
     * expected alias that is applied to that algorithm or display.
     *
     * @param MemberID
     * A uint16 numeric value that is a unique identifier for a specific 
     * member, within the scope of this object's definition. An enumerated
     * value is automatically generated for each member listed in the 
     * DMOIBDefinitions.h, and is labeled with the name of the member.
     *
     * @return
     * A uint32 value that represents the universal alias of the specified 
     * member
     *//*====================================================================*/
    uint32 GetMemberAlias(const uint16 MemberID);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    uint32 GetMemberAtAlias(uint16 Index);

#if defined(CRA383_SUPPORT)
    /*====================================================================*//**
     * @brief
     * Method to retrieve the units of a member.
     *
     * This method will return a numerical representation of the units 
     * associated with a member. This numerical value aligns with a globally
     * known enumeration that details all known units for all supported data
     * sets.
     *
     * @param MemberID
     * A uint16 numeric value that is a unique identifier for a specific 
     * member, within the scope of this object's definition. An enumerated
     * value is automatically generated for each member listed in the 
     * DMOIBDefinitions.h, and is labeled with the name of the member.
     *
     * @return
     * A uint16 value that represents the units associated with the specified 
     * member 
     *//*====================================================================*/
    uint16 GetMemberUnits(const uint16 MemberID);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    uint16 GetMemberAtUnits(uint16 Index);

    /*====================================================================*//**
     * @brief
     * Method to retrive the scale of a member.
     *
     * This method will return a numerical representation of the scale 
     * associated with a member. This numerical value aligns with a globally
     * known enumeration that details all known scales for all supported data
     * sets.
     *
     * @param MemberID
     * A uint16 numeric value that is a unique identifier for a specific 
     * member, within the scope of this object's definition. An enumerated
     * value is automatically generated for each member listed in the 
     * DMOIBDefinitions.h, and is labeled with the name of the member.
     *
     * @return
     * A uint16 value that represents the scale associated with the specified 
     * member
     *//*====================================================================*/
    uint16 GetMemberScale(const uint16 MemberID);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    uint16 GetMemberAtScale(uint16 Index);
#endif

    /*====================================================================*//**
     * @brief
     * Method to get the descriptive text for a member.
     *
     * This method allows the developer to retrieve the descriptive text for 
     * a member directly from the DMOIB object. This is only possible if a 
     * string table has been linked to this object with the SetStringTable() 
     * method. 
     *
     * @param MemberID
     * A uint16 numeric value that is a unique identifier for a specific 
     * member, within the scope of this object's definition. An enumerated
     * value is automatically generated for each member listed in the 
     * DMOIBDefinitions.h, and is labeled with the name of the member.
     *
     * @param LanguageID
     * An optional argument specifying the language of the requested text. If
     * no language is specified, it uses the value LANGUAGE_DEFAULT, which in 
     * turn defaults to whatever language the linked string table is currently 
     * configured for.
     *
     * @return
     * A pointer to the constant string contained in the linked string table.
     * (This string MUST NOT be directly written to or modified.) NULL is 
     * returned if no string table is available.
     *//*====================================================================*/
    const ansichar* GetMemberText(const uint16 MemberID, 
                                  uint8 LanguageID = LANGUAGE_DEFAULT);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    const ansichar* GetMemberAtText(const uint16 Index, 
                                    uint8 LanguageID = LANGUAGE_DEFAULT);

    /*========================================================================
        Sets for single members    
    ======================================================================*//**
     * @brief
     * Method to set the value of a member with a single element.
     *
     * This is the method used to modify the value of data members. This 
     * applies only to members which are single elements (not array types, and 
     * not sub-object types). It is overloaded for each data type available to 
     * DMOIB, and performs implicit conversions between data types when 
     * assigning values.
     *
     * @note
     * Certain types such as byte, word, and dword do not have their own 
     * explicitly overloaded SetMember() methods. This is because the typedef
     * for those types matches an already defined typedef exactly (byte = 
     * uint8, word = uint16, etc.). Because these typedefs match exactly, the 
     * compiler makes no destinction between them, and a method defined which 
     * takes a uint8 Value argument will also apply to byte Value arguments.
     * Any attempt to define an explicit SetMember() method for the byte type
     * will result in an ambiguity conflict, and will not compile.
     *
     * @param MemberID
     * A uint16 numeric value that is a unique identifier for a specific 
     * member, within the scope of this object's definition. An enumerated
     * value is automatically generated for each member listed in the 
     * DMOIBDefinitions.h, and is labeled with the name of the member.
     *
     * @param Value
     * A value for a supported DMOIB type that will be assigned to the specified
     * member. 
     *
     * @return
     * The status of the assignment. TRUE denotes success, while FALSE denotes
     * failure.
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const int8 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const int16 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const int32 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const uint8 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const uint16 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const uint32 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const float32 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const float64 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const ansichar Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const MOIB_TIME& Value);

    /*========================================================================
        Sets for single members (using Offsets)
    ======================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const int8 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const int16 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const int32 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const uint8 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const uint16 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const uint32 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const float32 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const float64 Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const ansichar Value);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const MOIB_TIME& Value);


    /*========================================================================
        Sets for Static/Dynamic Arrays
    ======================================================================*//**
     * @brief
     * Method to set the value of a member that is an array type.
     *
     * This is the method used to modify the value of data members. This 
     * applies only to members which are array types (not single elements, and 
     * not sub-object types). It is overloaded for each data type available to 
     * DMOIB.
     *
     * @note
     * Certain types such as byte, word, and dword do not have their own 
     * explicitly overloaded SetMember() methods. This is because the typedef
     * for those types matches an already defined typedef exactly (byte = 
     * uint8, word = uint16, etc.). Because these typedefs match exactly, the 
     * compiler makes no destinction between them, and a method defined which 
     * takes a uint8 Value argument will also apply to byte Value arguments.
     * Any attempt to define an explicit SetMember() method for the byte type
     * will result in an ambiguity conflict, and will not compile.
     *
     * @param MemberID
     * A uint16 numeric value that is a unique identifier for a specific 
     * member, within the scope of this object's definition. An enumerated
     * value is automatically generated for each member listed in the 
     * DMOIBDefinitions.h, and is labeled with the name of the member.
     *
     * @param pValue
     * A pointer to an array of values of supported DMOIB types that will be 
     * assigned to the specified member. If this argument is specified as NULL,
     * and NumElements is > 0, then the array will be resized accordingly if it 
     * is a dynamic array, and static arrays will maintain their current size. 
     * The data of the array will then be overwritten with all zeroes. 
     *
     * @param NumElements
     * A uint16 value representing the number of elements, not the size in 
     * bytes, of the array being copied. If this argument is specified as 0,
     * then it will force the array's memory allocation to be freed, if it is
     * a dynamically sized array. For static arrays, a value of 0 will have the
     * same effect as specifying pValue as NULL.
     *
     * @note
     * When dealing with text type arrays (ansichar and widechar), do not 
     * include the NULL-terminator for the string in the NumElements 
     * argument. A NULL-terminator will be added to the string automatically
     * when it is copied.
     *
     * @return
     * The status of the assignment. TRUE denotes success, while FALSE denotes
     * failure. If the pValue array fails to copy, then the values in the 
     * current array will all be set to 0.
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const int8* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const int16* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const int32* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const uint8* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const uint16* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const uint32* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const float32* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const float64* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const ansichar* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMember(uint16 MemberID, const MOIB_TIME* pValue, uint16 NumElements);
    
    /*========================================================================
        Sets for Static/Dynamic Arrays (using Offsets)
    ======================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const int8* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const int16* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const int32* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const uint8* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const uint16* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const uint32* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const float32* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const float64* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const ansichar* pValue, uint16 NumElements);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    boolean SetMemberAt(uint16 Offset, const MOIB_TIME* pValue, uint16 NumElements);

    
#if ((TARGET_XML_SUPPORT == XML_SERIALIZE) || (TARGET_XML_SUPPORT == XML_ALL))
    /*========================================================================
        XML Serialization methods
    ======================================================================*//**
     * @brief
     * Method to determine the serialized XML string size of this object.
     *
     * This method will evaluate the members of this object, and determine how
     * much space, in bytes, this object will occupy in a serialized XML 
     * string. This is provided for developers to query the object before 
     * actually perfoming a serialization to tell if they have enough space 
     * available for the resultant string, or if they should allocate more 
     * space.
     *
     * @note 
     * This method does not allocate any memory itself, or write any values. 
     * It is merely a prescan to determine the size without modifying any 
     * memory.
     *
     * @return
     * A uint32 value representing the space, in bytes, that this object will
     * occupy in a serialized XML string.
     *//*====================================================================*/
    uint32 XMLSizeAll();

    /*====================================================================*//**
     * @brief     
     * Method to produced a serialized string representation of the current
     * object, in XML.
     *
     * This method converts the data of a DMOIB object to a well-formatted XML
     * string. This string can then be written to a file or parsed by some 
     * external system for display to a user.
     *
     * @param pXMLBuffer
     * A pointer to an ansichar buffer that the method will write the 
     * serialized object to. This buffer is assumed to be large enough to 
     * contain the entire stream, and it is the responsibility of the developer
     * to ensure this by performing a preliminary check with the 
     * XMLSizeAll() method.
     *
     * @return
     * A uint32 value representing the number of bytes written to pXMLBuffer
     * argument. This should match the value returned from XMLSizeAll() 
     * exactly.
     *//*====================================================================*/
    uint32 XMLSerialize(ansichar* pXMLBuffer);

#endif
#if ((TARGET_XML_SUPPORT==XML_DESERIALIZE) || (TARGET_XML_SUPPORT==XML_ALL))
    /*========================================================================
        XML Deserialization method
    ======================================================================*//**
     * @brief
     * Method to parse a serialized XML string and fill an object with the 
     * data it contains.
     *
     * This method will take the output XML string from a previously serialized
     * object and absorb that data into this object. The XML string is parsed
     * and checked to make sure the data follows the DMOIB format, and values 
     * are extracted throughout the process as elements are encountered. The 
     * data is then available for direct access through the object.
     *
     * @param pXMLBuffer
     * A pointer to an ansichar buffer containing a XML string of a serialized 
     * object. The buffer is parsed until a null terminating character
     * is reached, signifying the end of the string.
     *
     * @return
     * A uint32 value representing the number of bytes read from pXMLBuffer
     * during the parsing process.
     *//*====================================================================*/
    uint32  XMLDeserialize(ansichar* pXMLBuffer);
#endif

};

/*=============================================================================
    Dynamic DMOIB object Vector class (this is included with the CDMOIBObject 
    definition because it can only ever be used with it)
==========================================================================*//**
 * @brief
 * A complementary class used with CDMOIBObject to represent collections of 
 * DMOIB objects.
 *
 * This class is used to manage collections of CDMOIBObject instances, which 
 * are inside a parent CDMOIBObject instance. This collection can contain 
 * homogenous DMOIB object types, or disparate types, and can contain a fixed 
 * number of elements, or a dynamic number of them. These two options are 
 * specified in the DMOIBDefinitions.h file for a data member declared as a 
 * DMOIBVector type, and they are specified independently of each other. 
 *
 * The interface for the CDMOIBVector class was designed to simulate the 
 * interface for the standard library's STL::Vector template class as closely
 * as possible. It is not a template class itself, but since it handles 
 * CDMOIBObject instances, it is in a way mutable in the type of structures it
 * manages. 
 *
 * CDMOIBVector instances can only be instantiated by parent CDMOIBObject 
 * instances during construction. The constructors for this class were 
 * intentionally made private to prevent developers from trying to create
 * CDMOIBVector objects for their own usage in code. All references to 
 * CDMOIBVector should only come from the members of parent DMOIB objects.
 * 
 *
 *//*========================================================================*/
class CDMOIBVector
{
    /*====================================================================*//**
     * @brief
     * Declaration to give CDMOIBObject instances private access.
     *
     * CDMOIBObject instances are the only things allowed to create CDMOIBVector
     * instances, so the constructors for this class are made private, and the
     * CDMOIBObject class is declared as a friend to allow private access.
     *//*====================================================================*/
    friend class CDMOIBObject;

#if ((TARGET_XML_SUPPORT==XML_DESERIALIZE) || (TARGET_XML_SUPPORT==XML_ALL))
    /*====================================================================*//**
     * @brief
     * Declaration to give XML parsing routines private access.
     *
     * During the parsing process, these methods need access to the private
     * members that store XML related information. So they are declared as 
     * friends here to allow that access.
     *//*====================================================================*/
    friend void XMLStartElement(void* pUserData, void* pTagName, 
                                void* pAttributes);
    friend void XMLEndElement(void* pUserData, void* pTagName);
    friend void XMLCharacterData(void* pUserData, void* pCDataString, 
                                 int CDataLength);
#endif

private:

    /*====================================================================*//**
     * @var     CDMOIBDictionary* m_pDictionary
     * @brief
     * A reference to a CDMOIBDictionary object.
     *
     * A reference to an existing CDMOIBDictionary object that can be used for
     * looking up additional definitions based on ClassID. This is necessary
     * during construction of an object with sub-objects, as sub-objects are 
     * denoted by ClassID in definitions. The dictionary is used to resolve 
     * the sub-object ClassIDs to full definitions that are subsequently 
     * instantiated. This could be the case for a CDMOIBVector for a collection
     * of specified DMOIB object types, where each element of the collection 
     * needs to be instantiated at the point when the vector is. In this case,
     * the vector needs to provide the Dictionary reference to each of the 
     * collection elements for lookups.
     *
     *//*=================================================================*//**
     * @var     CDMOIBObject** m_ppObjectArray
     * @brief
     * The collection of CDMOIBObject instances
     *
     * This is a statically or dynamically declared array of CDMOIBObject 
     * instances. Depending on the declaration for this vector, this array 
     * may grow/shrink with the addition/removal of elements.
     *
     *//*=================================================================*//**
     * @var     uint16 m_NumElements
     * @brief
     * The number of currently managed elements in the collection.
     *
     * This is the number of elements currently in the collection, 
     * not the size in bytes. This value will always represent the number
     * of items in the array, but may not represent the full slots
     * available for elements in the array. 
     *
     *//*=================================================================*//**
     * @var     uint16 m_NumElementsMax
     * @brief
     * The total available space for elements in the collection
     *
     * This is the number of elements currently in the collection, 
     * not the size in bytes. To minimize memory reallocations, m_ObjectArray 
     * is only resized when it needs to grow larger, and never shrinks (until 
     * it is deleted of course). Because of this, the actual number of elements
     * in the array may not fill it completely. This separate count specifies
     * how much space is available in the collection before it needs to be 
     * resized again.
     *
     * @note
     * m_NumElements will always be less than or equal to m_NumElementsMax,
     * and at any point where the previous would exceed the latter, 
     * m_ObjectArray is resized.
     *
     *//*=================================================================*//**
     * @var     uint32 m_FixedClassID
     * @brief
     * A restrictor on the type of DMOIB objects that can be collected by this
     * vector.
     *
     * This value can either be a specific ClassID, or the special case value 
     * ANY_CLASS::CLASSID. If this is set to ANY_CLASS::CLASSID, then it means 
     * the collection can contain any type of CDMOIBObject instance. If it is 
     * set to any other value, then it is used as a filter to only allow 
     * objects with a matching ClassID to enter the collection.
     *
     *//*=================================================================*//**
     * @var     uint16 m_FixedSize
     * @brief
     * A restrictor on the number of DMOIB objects that can be collected by this
     * vector.
     *
     * This value can either be a specific value, or the special case value
     * UNBOUNDED (equivalent to 0). If this is set to UNBOUNDED, then it means 
     * the collection can contain any number of CDMOIBObject instances, and will
     * change size to fit them accordingly. If it is set to any other value, 
     * then the collection will be configured during construction to store that 
     * many elements, and it will not be allowed to resize after that point.
     *
     *//*====================================================================*/

    CDMOIBDictionary*   m_pDictionary;
    CDMOIBObject**      m_ppObjectArray;
    uint16              m_NumElements;
    uint16              m_NumElementsMax;
    uint32              m_FixedClassID;
    uint16              m_FixedSize;
	uint16				m_FixedVersion;

#if ((TARGET_XML_SUPPORT==XML_DESERIALIZE) || (TARGET_XML_SUPPORT==XML_ALL))
    /*====================================================================*//**
     * @var     XMLUserData_t m_XMLUserData;
     * @brief
     * A custom data structure for XML parsing
     *
     * This is a custom structure for maintaining state information when 
     * parsing an XML stream. It is only included in the class definition if
     * XML_DESERIALIZE is enabled.
     *//*====================================================================*/
    XMLUserData_t       m_XMLUserData;
#endif
    /*
    ===========================================================================
        Private methods
    ===========================================================================
        Private constructors
    ======================================================================*//**
     * @brief
     * Basic constructor for building a DMOIB object vector.
     *
     * This is the only constructor available for instantiating DMOIB vector 
     * objects, and it is declared as private to limit its usage. The only 
     * thing that should be able to actually create a CDMOIBVector instance is 
     * a CDMOIBObject instance that has a vector member. A developer should 
     * never be allowed to instantiate a CDMOIBVector object themselves, as the 
     * vector requires some level of interaction with an existing CDMOIBObject 
     * instance. The CDMOIBObject class is declared as a friend of this class, 
     * allowing it access to the privately declared members/methods. 
     *
     * @param pDictionary
     * Reference to an instantiated CDMOIBDictionary that can translate 
     * ClassIDs to definitons.
     * 
     * @param FixedClassID
     * An optional parameter that can specify the ClassID that will be used to 
     * filter the objects that are inserted/appended to this vector. This 
     * defaults to ANY_CLASS::CLASSID, which allows the vector to accept 
     * objects with any ClassID.
     * 
     * @param FixedSize
     * An optional parameter that can specify a limit on the number of elements
     * available in this vector. This defaults to 0, which means the vector has
     * no size limitation and will be resized when appropriate. 
     * 
     * @param FixedVersion
     * An optional parameter that can specify the version of objects to be 
     * created in the vector. This only applies if both FixedClassID and 
     * FixedSize are provided. If FixedVersion is not provided, the highest
     * known version in the dictionary is used.
     * 
     *//*====================================================================*/
    CDMOIBVector(CDMOIBDictionary* pDictionary, 
                 uint32 FixedClassID = ANY_CLASS::CLASSID, 
                 uint16 FixedSize = 0,
				 uint16 FixedVersion = HIGHEST_KNOWN_VERSION);

    /*====================================================================*//**
     * @brief
     * Standard copy constructor
     *
     * Standard copy constructor that will create a new vector matching the 
     * structure of the referenceprovided, then copy all of the data from that
     * reference to the new vector. This is only ever used by the CDMOIBObject 
     * copy constructor when an instance contains a vector member.
     *
     * @param pVectorCopy
     * Reference to an existing CDMOIBVector instance. This object will acquire
     * all data/settings from that reference.
     *
     *//*====================================================================*/
    CDMOIBVector(CDMOIBVector* pVectorCopy);

    /*====================================================================*//**
     * @brief
     * Standard destructor.
     *
     * Standard destructor that cleans up all allocated memory.
     *//*====================================================================*/
    ~CDMOIBVector();

    /*====================================================================*//**
     * @brief
     * Helper method for constructing the object.
     *
     * This method is a helper used by the private constructor to instatiate 
     * a DMOIB vector. This effectively takes all of the same parameters as the
     * constructor, and is available to maintain consistency with CDMOIBObject
     * constructor implementations. It also provides a way of configuring a 
     * vector without using the constructor directly.
     *
     * @param pDictionary
     * Reference to an existing dictionary that can translate ClassIDs to 
     * definitons.
     * 
     * @param FixedClassID
     * An optional parameter that can specify the ClassID that will be used to 
     * filter the objects that are inserted/appended to this vector. This 
     * defaults to ANY_CLASS::CLASSID, which allows the vector to accept 
     * objects with any ClassID.
     * 
     * @param FixedSize
     * An optional parameter that can specify a limit on the number of elements
     * available in this vector. This defaults to 0, which means the vector has
     * no size limitation and will be resized when appropriate. 
     *
     * @param FixedVersion
     * An optional parameter that can specify the version of objects to be 
     * created in the vector. This only applies if both FixedClassID and 
     * FixedSize are provided. If FixedVersion is not provided, the highest
     * known version in the dictionary is used.
     *
     *//*====================================================================*/
    void Construct(CDMOIBDictionary* pDictionary, 
                   uint32 FixedClassID, 
                   uint16 FixedSize,
				   uint16 FixedVersion = HIGHEST_KNOWN_VERSION);

    /*====================================================================*//**
     * @brief
     * Method to perform clean-up and deallocations for the object.
     *
     * This is provided as a pass-through for the destructor, and does all of
     * the cleanup operations. It releases any memory that has been allocated
     * for the Vector's internal array, as well as any memory allocated for
     * new objects in the array.
     * 
     *//*====================================================================*/
    void Destruct();

    /*====================================================================*//**
     * @brief
     * Private helper method to consolidate all insert calls to the vector.
     *
     * This is a helper method that is basically a pass-through for all public
     * insert/push methods. This is to consolidate the routine for actually
     * performing an insert into one method. All of the publically declared 
     * methods really only differ in the arguments they accept, but accomplish
     * the same task by useing this private method.
     *
     * @param Index
     * The numerical position in the vector where the new object is to be 
     * inserted. Index 0 represents the first position in the vector, and 
     * inserts will shift all elements in the vector with higher indices up by
     * one.
     *
     * @param pDMOIBObject
     * A reference to an existing CDMOIBObject instance that will be copied and
     * added to the vector. 
     *
     * @param pDefinition
     * This is a definition used to create a new object at the specified index.
     * If this is provided, a new empty object will be created at that index
     * instead of copying the data from an existing object. This is used as a 
     * complementary argument to the pDMOIBObject parameter. In cases were the 
     * developer may not have an existing object to copy, or simply does not 
     * want to copy an object but rather use a new empty one, this option is
     * available.
     *
     * @param ClassID
     * The ClassID related to an empty object that will be placed at Index.
     *
     * @param Version
     * The specific version of the related empty object. If this is empty, the
     * highest known version for the object in the dictionary will be used.
     *
     * @return
     * A pointer to the new DMOIBObject instance that was created by this 
     * method. If this value is NULL, then a new object could not be created. 
     *
     *//*====================================================================*/
    CDMOIBObject* Insert(uint16 Index, CDMOIBObject* pDMOIBObject, 
                         void* pDefinition, uint32 ClassID, uint16 Version = HIGHEST_KNOWN_VERSION);

    
    /*====================================================================*//**
     * @brief
     * Method to determine the serialized size of this vector.
     *
     * This method will evaluate all elements of this vector, and determine how
     * much space, in bytes, they will occupy in a serialized form. 
     *
     * @note 
     * This method does not allocate any memory itself, or write any values. 
     * It is a prescan to determine the size without modifying any memory.
     *
     * @return
     * A uint32 value representing the space, in bytes, that all the elements
     * of this vector will occupy in a serialized form.
     *
     *//*====================================================================*/
    uint32 SizeAll();

    /*====================================================================*//**
     * @brief     
     * Method to produced a serialized stream representation of the current
     * vector.
     *
     * This method converts the all the elements of the current vector into a 
     * tightly packed, serial form, by calling the Serialize() method of each
     * object in an iterative process. All data that was contained in the 
     * objects are written to the buffer provided, in Big-Endian form. 
     *
     * @param pBuffer
     * A pointer to a uint8 buffer that the method will write the serialized 
     * vector to. This buffer is assumed to be large enough to contain the 
     * entire stream.
     *
     * @return
     * A uint32 value representing the number of bytes written to the buffer
     * argument. This should match the value returned from SizeAll() 
     * exactly.
     *//*====================================================================*/
    uint32 Serialize(uint8* pBuffer);

    /*====================================================================*//**
     * @brief
     * Method to parse a serialized stream and fill a vector with its data.
     *
     * This method will take the output stream from a previously serialized 
     * vector and absorb the data from that stream. This vector is first 
     * returned to an empty state, and then appends elements as it traverses
     * the stream. Each entry in the stream is checked against m_FixedClassID
     * to make sure it is acceptable, and the vector ensures that it does not
     * extend past the m_FixedSize bounds.
     *
     * @param pBuffer
     * A pointer to a uint8 buffer containing a serialized vector. 
     *
     * @return
     * A uint32 value representing the number of bytes read from the buffer
     * during the parsing process.
     *//*====================================================================*/
    uint32 Deserialize(uint8* pBuffer);

    
#if ((TARGET_XML_SUPPORT == XML_SERIALIZE) || (TARGET_XML_SUPPORT == XML_ALL))
    /*========================================================================
        XML Serialization methods
    ======================================================================*//**
     * @brief
     * Method to determine the serialized XML string size of this vector.
     *
     * This method will evaluate the elements of this vector, and determine how
     * much space, in bytes, they will occupy in a serialized XML string. 
     *
     * @note 
     * This method does not allocate any memory itself, or write any values. 
     * It is a prescan to determine the size without modifying any memory.
     *
     * @return
     * A uint32 value representing the space, in bytes, that this object will
     * occupy in a serialized XML string.
     *//*====================================================================*/
    uint32  XMLSizeAll();

    /*====================================================================*//**
     * @brief     
     * Method to produced a serialized string representation of the current
     * vector, in XML.
     *
     * This method converts the data of a DMOIB vector to a well-formatted XML
     * string. This string can then be written to a file or parsed by some 
     * external system for display to a user.
     *
     * @param pXMLBuffer
     * A pointer to an ansichar buffer that the method will write the 
     * serialized vector to. This buffer is assumed to be large enough to 
     * contain the entire stream.
     *
     * @return
     * A uint32 value representing the number of bytes written to the XMLBuffer
     * argument. This should match the value returned from XMLSizeAll() 
     * exactly.
     *//*====================================================================*/
    uint32  XMLSerialize(ansichar* pXMLBuffer);
#endif
#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE) || (TARGET_XML_SUPPORT == XML_ALL))
    // This method is the only symmetrical (de)serialization method not 
    // supported by the CDMOIBVector class. The parsing process for the parent
    // CDMOIBObject handles all parsing for each of it's members, so a Vector
    // member is handled in that process. It was too difficult to remove Vector
    // parsing from that process and put it into its own method, especially 
    // since it was an unecessary feature to support this method directly 
    // through the CDMOIBVector interface, so it was left unimplemented.
    //uint32  XMLDeserialize(ansichar* XMLBuffer);
#endif

public:
    /*=========================================================================
        Public methods
    ======================================================================*//**
     * @brief
     * Method to retrieve the DMOIB object at the specified index.
     *
     * This method provides a way to retrieve a pointer to an existing 
     * CDMOIBObject instance, at a specified index. The vector is tightly 
     * packed, and object indices are adjusted after insertiongs/removals.
     * Therefore, any index value between 0 and Size() - 1, inclusive, will 
     * return a valid pointer.
     *
     * @param Index
     * A uint16 value representing the numeric index of the requested object.
     * The vector starts at index 0, and ends at index = (Size() - 1).
     *
     * @return
     * A pointer to the object at the index. If this value is NULL, then there
     * is no object available at that index, which in turn means that the index
     * was outside the bounds of the vector. Any other value represents success.
     *
     *//*====================================================================*/
    CDMOIBObject* At(uint16 Index);

    /*====================================================================*//**
     * @brief
     * Method to retrieve the first object in the vector.
     *
     * This method returns a pointer to the first CDMOIBObject in the vector.
     * (This acts the same as using the At() method with an Index of 0.)
     *
     * @return
     * A pointer to the the first object in the vector. If this value is NULL, 
     * then there are no elements available in the vector. Any other value 
     * represents success.
     *
     *//*====================================================================*/
    CDMOIBObject* Front();

    /*====================================================================*//**
     * @brief
     * Method to retrieve the last object in the vector.
     *
     * This method returns a pointer to the last CDMOIBObject in the vector.
     * (This acts the same as using the At() method with an Index of Size() - 
     * 1.)
     *
     * @return
     * A pointer to the the last object in the vector. If this value is NULL, 
     * then there are no elements available in the vector. Any other value 
     * represents success.
     *
     *//*====================================================================*/
    CDMOIBObject* Back();

    /*====================================================================*//**
     * @brief
     * Method to insert an object at the end of the vector.
     *
     * This method will append a single CDMOIBObject instance to the end of the
     * vector. (This acts the same as using the Insert() method with an Index 
     * of Size() - 1.)
     *
     * @param pDMOIBObject
     * Reference to an existing CDMOIBObject instance, whose data will be copied
     * into a newly created object at the last position in the vector.
     *
     * @return
     * A pointer to the new DMOIBObject instance that was created by this 
     * method. If this value is NULL, then a new object could not be created.
     *
     *//*====================================================================*/
    CDMOIBObject* Push(CDMOIBObject* pDMOIBObject);

    /*====================================================================*//**
     * @brief
     * Method to insert an empty object at the end of the vector.
     *
     * This method will append a single CDMOIBObject instance to the end of the
     * vector. However, this object will be left in an initialized state, as 
     * it is not being copied from an existing object. (This acts the same as 
     * using the Insert() method with an Index of Size() - 1.)
     *
     * @param pDefinition
     * A definition to use in the construction of a new, empty object, which 
     * will be inserted at the end of the vector.
     *
     * @param Version
     * The specific version of the related empty object. If this is empty, the
     * highest known version for the object in the dictionary will be used.
     *
     * @return
     * A pointer to the new DMOIBObject instance that was created by this 
     * method. If this value is NULL, then a new object could not be created.
     *
     *//*====================================================================*/
    CDMOIBObject* Push(void* pDefinition);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    CDMOIBObject* Push(void* pDefinition, uint16 Version);
	/*====================================================================*//**
     * @overload
     *//*====================================================================*/
    CDMOIBObject* Push(uint32 ClassID);
	/*====================================================================*//**
     * @overload
     *//*====================================================================*/
    CDMOIBObject* Push(uint32 ClassID, uint16 Version);

    /*====================================================================*//**
     * @brief
     * Method to remove the last element of the vector.
     *
     * This method will remove the last element of the vector, performing any
     * memory clean-up necessary.
     *
     * @note
     * Unlike the STL:Vector, this does not return a pointer to the object 
     * that is being removed. This is because management of that object is done
     * by the vector class itself, and once it is removed from the internal 
     * array, it must be destroyed. Since it is destroyed in the removal 
     * process, there is no value to return from this method.
     *
     *//*====================================================================*/
    void Pop();

    /*====================================================================*//**
     * @brief
     * Method to insert an object at the specified index in the vector.
     *
     * This method will insert a single CDMOIBObject instance at the specified 
     * index of the vector. If the index is less than index of the last 
     * element, the new object will be inserted at that index, and all existing
     * elementes at that index and greater will be shifted one position higher in
     * the vector.
     *
     * @param pDMOIBObject
     * Reference to an existing CDMOIBObject instance, whose data will be copied
     * into a newly created object at the last position in the vector.
     *
     * @param Index
     * The numerical position in the vector where the new object is to be 
     * inserted. Index 0 represents the first position in the vector, and 
     * an index which is greater than or equal to the Size() will result in a 
     * failure.
     *
     * @return
     * A pointer to the new DMOIBObject instance that was created by this 
     * method. If this value is NULL, then a new object could not be created.
     *
     *//*====================================================================*/
    CDMOIBObject* Insert(CDMOIBObject* pDMOIBObject, uint16 Index);

    /*====================================================================*//**
     * @brief
     * Method to insert an empty object at the specified index in the vector.
     *
     * This method will insert a single CDMOIBObject instance at the specified 
     * index of the vector. However, this object will be left in an initialized
     * state, as it is not being copied from an existing object. If the index 
     * is less than index of the last element, the new object will be inserted 
     * at that index, and all existing elementes at that index and greater will
     * be shifted one position higher in the vector.
     *
     * @param pDefinition
     * A definition to use in the construction of a new, empty object, which 
     * will be inserted at the end of the vector.
     *
     * @param Version
     * The specific version of the related empty object. If this is empty, the
     * highest known version for the object in the dictionary will be used.
     *
     * @param Index
     * The numerical position in the vector where the new object is to be 
     * inserted. Index 0 represents the first position in the vector, and 
     * an index which is greater than or equal to the Size() will result in a 
     * failure.
     *
     * @return
     * A pointer to the new DMOIBObject instance that was created by this 
     * method. If this value is NULL, then a new object could not be created.
     *
     *//*====================================================================*/
    CDMOIBObject* Insert(void* pDefinition, uint16 Index);
    /*====================================================================*//**
     * @overload
     *//*====================================================================*/
    CDMOIBObject* Insert(void* pDefinition, uint16 Version, uint16 Index);
	/*====================================================================*//**
     * @overload
     *//*====================================================================*/
    CDMOIBObject* Insert(uint32 ClassID, uint16 Index);
	/*====================================================================*//**
     * @overload
     *//*====================================================================*/
    CDMOIBObject* Insert(uint32 ClassID, uint16 Version, uint16 Index);

    /*====================================================================*//**
     * @brief
     * Method to remove elements from the vector.
     *
     * This method allows the developer to remove existing elements from the
     * vector. Since all instances referenced by the vector are created and
     * managed by this class, a call to erase will destroy those objects
     * specified. 
     *
     * @param Index
     * The numerical position in the vector of the first element to be removed. 
     * Index 0 represents the first position in the vector, and an index which 
     * is greater than or equal to the Size() will result in no objects being 
     * erased.
     *
     * @param Count
     * An optional parameter that specifies how many elements to remove, 
     * starting and Index an moving forward through the vector. This defaults
     * to 1, so a normal call to Erase() will only remove a single element.
     *
     *//*====================================================================*/
    void Erase(uint16 Index, uint16 Count = 1);

    /*====================================================================*//**
     * @brief
     * Method to remove all objects from the vector.
     *
     * This method will remove all objects from the vector, and effectively 
     * return it to an initial state. (This acts the same as calling 
     * Erase(0, Size() - 1)).
     *
     *//*====================================================================*/
    void Clear();

    /*====================================================================*//**
     * @brief
     * Method to get the number of elements in the vector. 
     *
     * This method reports the number of elements, not the size in bytes, of
     * the vector.
     *
     * @return
     * A uint16 value representing the number of elements in the vector.
     *
     *//*====================================================================*/
    uint16 Size();

    /*====================================================================*//**
     * @brief
     * Method to tell if the vector is empty.
     *
     * This method reports whether the vector is empty (0 elements) or not. If
     * a vector is empty, then it is effectively in an intial state.
     *
     * @return
     * A boolean value representing the empty state of the vector. TRUE means 
     * there are no elements in the vector, while FALSE means there is at least
     * 1 element available.
     *
     *//*====================================================================*/
    boolean Empty();

    /*====================================================================*//**
     * @brief
     * A standard copy method.
     *
     * This method will copy the entire contents from one vector to this 
     * vector. This will only succeed if the FixedClassID and FixedSize 
     * settings from both objects match.
     *
     * @param pVectorCopy
     * A pointer to an existing DMOIB vector, whose contents will be copied into
     * this object.
     *
     * @return
     * A boolean value representing the success of the copy. TRUE means that
     * the copy was completed, while FALSE means that the copy attempt was a 
     * failure.
     *//*====================================================================*/
    boolean Copy(CDMOIBVector* pVectorCopy);
};


#endif // CDYNDMOIB_H


