/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2005-2006 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CDMOIBStringTable.h                                               //
'**  '// Version        :   1.00                                           //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CDMOIBSTRINGTABLE_H)
    #define  CDMOIBSTRINGTABLE_H

#include "DMOIBBase.h"

class CDMOIBDictionary;

/*=============================================================================
    CDMOIBTableManager base-class definition
*//*======================================================================*//**
 * @brief
 * Helper class to manage tables stored in linked-list form.
 *
 * This is a small helper class to managed generic binary tables in a linked 
 * list. This is an abstract class definition with pure virtual methods. 
 * Inheriting classes must override the TableID() and TableSize() methods to
 * provide implmentation specific methods for comparing tables to each other
 * and determine their size for memory allocation.
 *
 *//*========================================================================*/
class CDMOIBTableManager
{
protected:
    /*====================================================================*//**
     * @var     TableEntry_t*   m_pHead
     * @brief
     * Reference to the first dynamic entry of this linked-list.
     *
     * This is a reference to the first dynamic table managed by this
     * linked-list. If no dynamic tables are ever added to this list, 
     * then this will reference the same thing as m_pDefault.     
     *
     *//*=================================================================*//**
     * @var     TableEntry_t*   m_pDefault
     * @brief
     * Reference to the first static entry of the linked-list.
     *
     * This is a reference to the first static table in the default list 
     * provided to the constructor. This denotes the end of the dynamic 
     * tables and the start of the static ones. If no default tables
     * are provided to the constructor, then this will be set to NULL.
     *
     *//*=================================================================*//**
     * @var     TableEntry_t*   m_pSelected
     * @brief
     * Reference to the currently selected entry of the linked-list.
     *
     * This is a reference an entry in the linked-list, dynamic or static, that
     * is currently selected. This allows the table manager to keep reference 
     * to a commonly accessed table, as specified by the developer.
     *
     *//*====================================================================*/
    TableEntry_t*   m_pHead;
    TableEntry_t*   m_pDefault;
    TableEntry_t*   m_pSelected;


    /*====================================================================*//**
     * @brief
     * Constructor to instantiate the manager with a default list.
     *
     * This is the only constructor available for the class, and it creates
     * linked-list using the default list head provided.
     *
     * @param pDefaultTable
     * Reference to an existing static set of list entries. This static list
     * is assumed to be NULL terminated. If this value is specified as NULL,
     * then no default entries are added to the list.
     *
     *//*====================================================================*/
    CDMOIBTableManager(TableEntry_t* pDefaultTable);
    /*====================================================================*//**
     * @brief
     * Standard destructor.
     *
     * Standard destructor that cleans up all allocated memory.
     *//*====================================================================*/
    ~CDMOIBTableManager();

    /*====================================================================*//**
     * @brief
     * Method to add a new table to the linked-list
     *
     * This method will add a new entry to the linked-list, and copy the 
     * contents of the generic table provided to that entry. The memory space 
     * for the new entry is allocated and managed by this class. This method 
     * uses the TableID() to check for conflicts within the list, and 
     * TableSize() to determine the memory size allocated for the new entry.
     *
     * @param pTable
     * Reference to an existing generic table whose contents will be copied
     * into a new entry within the linked-list.
     *
     * @return
     * A reference to the newly created internal copy of pTable. If a new entry
     * could not be created, then this will return NULL.
     *
     *//*====================================================================*/
    byte* AddTable(byte* pTable);
    /*====================================================================*//**
     * @brief
     * Method to get a table from the list based on its ID.
     *
     * This method will do a linear search of the linked-list, and return a 
     * pointer to a table entry that matches the identifier provided. This 
     * method uses the TableID() to check for ID matches.
     *
     * @param Identifier
     * The numerical ID associated with the table being requested.
     *
     * @return
     * A reference to the linked-list entry matching Identifier. If no entry
     * could be found, then this will return NULL. 
     *
     *//*====================================================================*/
    byte* GetTable(uint32 Identifier);
    /*====================================================================*//**
     * @brief
     * Method to set a list entry as the selected entry.
     *
     * This method will set a linked-list entry as the selected default. This
     * is used to keep a reference to a commonly accessed table within the 
     * list.
     *
     * @param Identifier
     * The numerical ID associated with the table being set as the selected
     * default.
     *
     * @return
     * A reference to the linked-list entry matching Identifier. If no entry
     * could be found, then this will return NULL. 
     *
     *//*====================================================================*/
    byte* SetTable(uint32 Identifier);
    /*====================================================================*//**
     * @brief
     * Method to remove all dynamically added entries from the list.
     *
     * This method will remove all entries from the linked-list, and free any
     * memory allocated for dynamically added tables.
     *
     *//*====================================================================*/
    void  Clear();

    /*====================================================================*//**
     * @brief
     * Method to determine the ID of a table.
     *
     * Method to determine the ID of a table. This method must be implemented
     * in an inherting class.
     *
     * @param pTable
     * Reference to an existing table whose ID shall be determined.
     *
     * @return
     * A unint32 value representing the numerical ID of the table referenced
     * by pTable.
     *
     *//*====================================================================*/
    virtual uint32  TableID(byte* pTable) = 0;
    /*====================================================================*//**
     * @brief
     * Method to determine the Size of a table.
     *
     * Method to determine the Size of a table. This method must be implemented
     * in an inherting class.
     *
     * @param pTable
     * Reference to an existing table whose size shall be determined.
     *
     * @return
     * A unint32 value representing the size, in bytes, of the table referenced
     * by pTable.
     *
     *//*====================================================================*/
    virtual uint32  TableSize(byte* pTable) = 0;
};

/*=============================================================================
    CDMOIBLinkTableManager class definition
*//*======================================================================*//**
 * @brief
 * Helper class to manage link tables
 *
 * This class inherits from CDMOIBTableManager to implement a linked-list 
 * specifically for handling link table entries.
 *//*========================================================================*/
class CDMOIBLinkTableManager : CDMOIBTableManager
{
    friend class CDMOIBStringTable;

private:    
    /*====================================================================*//**
     * @brief
     * Constructor to instantiate the manager with a default list.
     *
     * This is the only constructor available for the class, and it creates
     * linked-list using the default list head provided.
     *
     * @param pDefaultTable
     * Reference to an existing static set of list entries. This static list
     * is assumed to be NULL terminated. If this value is specified as NULL,
     * then no default entries are added to the list.
     *
     *//*====================================================================*/
    CDMOIBLinkTableManager(TableEntry_t* pDefaultTable = NULL);

    /*====================================================================*//**
     * @brief
     * Method to determine the ID of a link table.
     *
     * This method implements the pure virtual method of the base class, and 
     * uses the global function LinkTableID() to extract the numerical ID from
     * a managed link table entry.
     *
     * @param pTable
     * Reference to an existing table whose ID shall be determined.
     *
     * @return
     * A unint32 value representing the numerical ID of the table referenced
     * by pTable.
     *
     *//*====================================================================*/
    uint32  TableID(byte* pTable);
    /*====================================================================*//**
     * @brief
     * Method to determine the Size of a link table.
     *
     * This method implements the pure virtual method of the base class, and 
     * uses the global function LinkTableSize() to determine the memory size, 
     * in bytes, of a managed link table entry.
     *
     * @param pTable
     * Reference to an existing table whose size shall be determined.
     *
     * @return
     * A unint32 value representing the size, in bytes, of the table referenced
     * by pTable.
     *
     *//*====================================================================*/
    uint32  TableSize(byte* pTable);
};

/*=============================================================================
    CDMOIBLinkTableManager class definition
*//*======================================================================*//**
 * @brief
 * Helper class to manage language tables
 *
 * This class inherits from CDMOIBTableManager to implement a linked-list 
 * specifically for handling language table entries.
 *//*========================================================================*/
class CDMOIBLanguageTableManager : CDMOIBTableManager
{
    friend class CDMOIBStringTable;

private:    
    /*====================================================================*//**
     * @brief
     * Constructor to instantiate the manager with a default list.
     *
     * This is the only constructor available for the class, and it creates
     * linked-list using the default list head provided.
     *
     * @param pDefaultTable
     * Reference to an existing static set of list entries. This static list
     * is assumed to be NULL terminated. If this value is specified as NULL,
     * then no default entries are added to the list.
     *
     *//*====================================================================*/
    CDMOIBLanguageTableManager(TableEntry_t* pDefaultTable = NULL);

    /*====================================================================*//**
     * @brief
     * Method to determine the ID of a language table.
     *
     * This method implements the pure virtual method of the base class, and 
     * uses the global function LanguageTableID() to extract the numerical ID 
     * from a managed language table entry.
     *
     * @param pTable
     * Reference to an existing table whose ID shall be determined.
     *
     * @return
     * A unint32 value representing the numerical ID of the table referenced
     * by pTable.
     *
     *//*====================================================================*/
    uint32  TableID(byte* pTable);
    /*====================================================================*//**
     * @brief
     * Method to determine the Size of a language table.
     *
     * This method implements the pure virtual method of the base class, and 
     * uses the global function LanguageTableSize() to determine the memory 
     * size, in bytes, of a managed language table entry.
     *
     * @param pTable
     * Reference to an existing table whose size shall be determined.
     *
     * @return
     * A unint32 value representing the size, in bytes, of the table referenced
     * by pTable.
     *
     *//*====================================================================*/
    uint32  TableSize(byte* pTable);
};

/*========================================================================*//**
 * @brief
 * Enum to define the status of adding multiple definitions.
 *
 * When adding multiple tables using the AddLink/LanguageTables() APIs, there 
 * is a chance that none of the tabls, all of the tables, or some of the
 * tables may be added to the stringtable. Because of these variable
 * states, an error code is used to determine the status of the method call.
 *
 *//*========================================================================*/
typedef enum AddTablesStatus_e
{
    ALL_TABLES_ADDED = UINT16_MAX,
    NO_TABLES_ADDED = 0

}AddTablesStatus_t;

/*=============================================================================
    CDMOIBStringTable class definition
*//*======================================================================*//**
 * @brief
 * Class definition for the string table interface used to manage 
 * multi-lingual strings.
 *
 * This class can be used in an independent fashion that has no knowledge of 
 * DMOIB objects, or it can be used conjuntively with them. Both modes offer
 * the same generic usage model, while the conjunctive option merely adds 
 * extra support for DMOIB objects. 
 *
 * In the independent mode, the string table acts generically, translating 
 * symbolic links into associated strings of the selected language. This is 
 * intended to mimic much of the functionality found in other string table 
 * implmentations, as well as providing a very simple and intuitive interface
 * that can be used in projects that might not even be using DMOIB definitions.
 *
 * In the conjunctive mode, a separate LinkTable entity is created as a 
 * companion to the string table. This is used as a translation sheet that 
 * will take a DMOIB object's ClassID and MemberID and convert that to a 
 * symbolic link. That symbolic link is then used in a normal lookup process 
 * to return a language-specific string. Both string tables and link tables 
 * can be updated at run time to allow dynamic behavior similar to that of 
 * DMOIB definitions.
 *
 *//*========================================================================*/
class CDMOIBStringTable
{
private:
    /*====================================================================*//**
     * @var     CDMOIBTableManager m_LanguageTables
     * @brief
     * Linked-list for languages that are added to the string table.
     *
     * This helper object organizes the language tables managed by the string
     * table. It provides a simple interface for adding and retrieving language
     * tables from a collection of them.
     *
     *//*=================================================================*//**
     * @var     CDMOIBTableManager m_LinkTables
     * @brief
     * Linked-list for LinkTables that are added to the string table.
     *
     * This helper object organizes the link tables managed by the string
     * table. It provides a simple interface for adding and retrieving link
     * tables from a collection of them.
     *
     *//*====================================================================*/

    CDMOIBLanguageTableManager  m_LanguageTables;
    CDMOIBLinkTableManager      m_LinkTables;
    CDMOIBDictionary*           m_pDictionary;

    /*====================================================================*//**
     * @brief     
     * Method to retrieve the real MemberID for a requested member.
     *
     * This method finds the real MemberID for a member, which may be an
     * adjusted value due to inheritence. When an object inherits members from
     * another object, all the MemberIDs in the child object are shifted by the
     * number of members in the parent object(s), so this method rectifies that
     * shift.
     *
     * @param pMemberIDSeed
     * A uint16 value used as both an "in" and "out" parameter. This is used as
     * the initial MemberID seed when traversing the inheritence tree, and will
     * contain the adjusted MemberID when the method returns. The out value of 
     * this parameter is only valid when the method returns a value or TRUE.
     *
     * @param pClassIDSeed
     * A uint32 value used as both an "in" and "out" parameter. This is used as
     * the initial ClassID seed when traversing the inheritence tree, and will
     * contain the adjusted ClassID when the method returns. The out value of 
     * this parameter is only valid when the method returns a value or TRUE.
     *
     * @return
     * A boolean value representing if the MemberID was actually found in the 
     * inheritence tree.
     * 
     *//*====================================================================*/
    boolean RealMemberID(uint16* pMemberIDSeed, uint32* pClassIDSeed);

public:
    /*=========================================================================
        Public Methods
    ===========================================================================
        Basic Constructor
    ======================================================================*//**
     * @brief
     * Basic constructor for a string table, with optional defaults.
     *
     * This is the only constructor available, and is used to create an empty
     * string table, or one with default values. If a default string table is
     * specified, then this string table will acquire references to all the 
     * language tables in that default. If a default link table is provided as
     * well, then this string table will also acquire information to translate
     * certain ClassID and MemberID combinations to symbolic links.
     *
     * @param pDefaultStringTable
     * Optional parameter which is a reference to a set of static language 
     * tables from the DMOIBDefinitions.h file (The variable name for this 
     * parameter should match the DictionaryName provided to the 
     * DMOIB_STRING_TABLE() macro). If this parameter is not provided, it 
     * defaults to NULL, and no initial strings will be available.
     *
     * @param pDefaultLinkTable
     * Optional parameter which is a reference to a link table for translating
     * DMOIB IDs to symbolic links. This linkage is also created from a set of
     * optional macros in the DMOIBDefinitions.h file (The variable name for 
     * this parameter should match the DictionaryName provided to the 
     * DMOIB_LINK_TABLE() macro). If this parameter is not provided, it defaults
     * to NULL, and no automatic linkage will be available.
     *
     * @param pDictionary
     * Optional parameter which is a reference to a CDMOIBDictionary for 
     * resolving ClassIDs, MemberIDs, and Enumerations when using LinkTable
     * functionality. If this parameter is not provided, it defaults to NULL, 
     * and no automatic linkage will be available.
     *
     *//*====================================================================*/
    CDMOIBStringTable(void* pDefaultStringTable     = NULL, 
                      void* pDefaultLinkTable       = NULL,
                      CDMOIBDictionary* pDictionary = NULL);

    /*====================================================================*//**
     * @brief
     * Standard destructor.
     *
     * Standard destructor that cleans up all allocated memory.
     *//*====================================================================*/
    ~CDMOIBStringTable();

    /*====================================================================*//**
     * @brief     
     * Accessor to determine the selected language for this string table.
     *
     * This method provides an interface for the developer to determine the 
     * currently selected language for this string table. This ID relates to 
     * the language table that will be used for all subsequent string lookups.
     *
     * @return
     * A uint8 value representing the currently selected language.
     * 
     *//*====================================================================*/
    uint8 GetLanguageID();

    /*====================================================================*//**
     * @brief     
     * Mutator to set the selected language for this string table.
     *
     * This method allows the developer to change the language used for string
     * table lookups.
     *
     * @param LanguageID
     * A uint8 value representing the new language ID to be used when 
     * performing string lookups.
     *
     * @return
     * A boolean value representing the success of the LanguageID assignment.
     * If there is a language table available matching the ID, this will 
     * return TRUE and accept the new ID. Otherwise this will return FALSE and 
     * the ID will remain unchanged.
     *//*====================================================================*/
    boolean SetLanguageID(uint8 LanguageID);

    /*====================================================================*//**
     * @brief     
     * Method to change the link table used for DMOIB/symbolic link 
     * translations.
     *
     * If no LinkTables are available when this string table is constructed, or 
     * there is a need to change that LinkTables after creation, this method 
     * can be used to update the tables. This method will check the currently
     * managed link tables for ClassID conflicts before adding the new table to
     * the internal list. If there is a ClassID match with an existing table,
     * the new table will overwrite the existing one.
     *
     * @param pLinkTable
     * A reference to an existing LinkTable that will be adopted by this string
     * table for DMOIB ID conversions. This LinkTable will not be referenced 
     * once this method has returned, as the data from the LinkTable is copied
     * to an internal structure.
     *
     * @return
     * A boolean value representing the state of the update. TRUE means the
     * copy was successful, while FALSE means it failed. A failure means that 
     * a new LinkTable could not be allocated, and the previous LinkTable will 
     * not be overwritten. 
     * 
     *//*====================================================================*/
    void* AddLinkTable(void* pLinkTable);

    /*====================================================================*//**
     * @brief     
     * Method to change the link tables used for DMOIB/symbolic link 
     * translations.
     *
     * This method works in the same way as AddLinkTable(), except that it works
     * on packed list of link tables instead of a single one. This method will
     * iterate through that list of link tables, calling AddLinkTable() for each
     * one it encounters until it has reached the end.
     *
     * @param pLinkTables
     * A reference to a series of existing LinkTables that will be adopted by 
     * this string table for DMOIB ID conversions. This iterates through the 
     * provided reference and copies the packed link tables one by one.
     *
     * @param Size
     * The total byte size of the pLinkTables reference. This is used to track
     * the progress when iterating through the packed link table list.
     *
     * @return
     * This method will return a variable status value representing the success
     * of the addition. The known return values are defined in 
     * AddTablesStatus_t. If all tables are successfully accepted,
     * this method will return ALL_TABLES_ADDED. If none are added, it
     * will return NO_TABLES_ADDED. If some of the tables in the 
     * buffer are added, but not all of them, it will return a non-zero value
     * representing the number of tables added until an error was 
     * encountered.
     * 
     *//*====================================================================*/
    uint16 AddLinkTables(void* pLinkTables, uint32 Size);

    /*====================================================================*//**
     * @brief     
     * Method to get a reference to the internal LinkTable.
     *
     * This method is a way of retrieving a reference to a LinkTable managed
     * by the string table. This is used to transfer a LinkTable from one
     * string table to another, possibly across a network connection. 
     *
     * @param ClassID
     * The ClassID associated with the requested link table.
     *
     * @return
     * A reference to the internal LinkTable. If this value is NULL, then it
     * means there is no LinkTable available matching the requested ClassID.
     * 
     *//*====================================================================*/
    void* GetLinkTable(uint32 ClassID);

    /*====================================================================*//**
     * @brief     
     * Method to add a language table to this string table.
     *
     * This method is used to add language tables to this string table after
     * creation. This is useful for loading only those language tables needed
     * by the application, and can be extended to gather language language 
     * tables from a network connection and add them dynamically. There can be
     * many language tables managed by this CDMOIBStringTable instance, but only
     * one per specific language ID. The LanguageTable itself is embedded with
     * a language ID, and if it conflicts with an already exisiting language
     * table, then the new one will overwrite the old. Also, the language 
     * tables for a string table are managed by the string table itself, so 
     * any language tables which are added are copied fully.
     *
     * @param pLanguageTable
     * A reference to an existing LanguageTable which is copied and added to
     * this string table. This LanguageTable will not be referenced once this 
     * method has returned, as the data from the LanguageTable is copied to an
     * internal structure.
     *
     * @return
     * A reference to the newly added language table. If this value is NULL,
     * then the addition was a failure. A failure means that a new language
     * table could not be allocated, and no previous tables will not be 
     * overwritten. 
     *
     *//*====================================================================*/
    void* AddLanguageTable(void* pLanguageTable);

    /*====================================================================*//**
     * @brief     
     * Method to add language tables to this string table.
     *
     * This method works in the same way as AddLanguageTable(), except that it 
     * works on packed list of language tables instead of a single one. This 
     * method will iterate through that list of language tables, calling 
     * AddLanguageTable() for each one it encounters until it has reached the 
     * end.
     *
     * @param pLanguageTables
     * A reference to existing LanguageTables which is copied and added to this 
     * string table. This iterates through the provided reference and copies 
     * the packed link tables one by one.
     *
     * @param Size
     * The total byte size of the pLanguageTables reference. This is used to 
     * track the progress when iterating through the packed language table list.
     *
     * @return
     * This method will return a variable status value representing the success
     * of the addition. The known return values are defined in 
     * AddTablesStatus_t. If all tables are successfully accepted,
     * this method will return ALL_TABLES_ADDED. If none are added, it
     * will return NO_TABLES_ADDED. If some of the tables in the 
     * buffer are added, but not all of them, it will return a non-zero value
     * representing the number of tables added until an error was 
     * encountered.
     * 
     *//*====================================================================*/
    uint16 AddLanguageTables(void* pLanguageTables, uint32 Size);

    /*====================================================================*//**
     * @brief
     * Method to retrieve a full language table based on its language ID.
     *
     * This method is a way of retrieving a reference to a specific languge 
     * table managed by the string table. This is used to transfer language 
     * tables from one string table to another, possibly across a network 
     * connection. 
     *
     * @param LanguageID
     * A uint8 representing a unique numerical identifier for the requested 
     * language.
     *
     * @return
     * A reference to a specific language table. If this value is NULL, then it
     * means there is no language table available matching the specified ID.
     * 
     *//*====================================================================*/
    void* GetLanguageTable(uint8 LanguageID);
        
    /*====================================================================*//**
     * @brief     
     * Method to set the dictionary used by this string table.
     *
     * This method is used to add a CDMOIBDictionary reference to the string
     * table. To properly link string tables to object definitions, and use
     * the LinkTable structures, the StringTable needs a reference to a
     * dictionary so it can resolve ClassID, MemberID and Enumeration 
     * references.
     *
     * @param pDictionary
     * A reference to an existing CDMOIBDictionary that can be used to resolve
     * IDs when requesting strings from the StringTable with the LinkTable.
     *
     *//*====================================================================*/
    void SetDictionary(CDMOIBDictionary* pDictionary);

    /*====================================================================*//**
     * @brief
     * Method to retrieve the dictionary used by this string table.
     *
     * This method is a way of retrieving a reference to the CDMOIBDictionary
     * provided to the SetDictionary() method, or the constructor. 
     *
     * @return
     * A reference to the CDMOIBDictionary object used for resolving ClassIDs, 
     * MemberIDs, and Enumerations. If no Dictionary has been set for this 
     * object, this will return NULL.
     * 
     *//*====================================================================*/
    CDMOIBDictionary* GetDictionary();

    /*====================================================================*//**
     * @brief
     * Method to retrieve a string associated with the specified symbolic link.
     *
     * This method retrieves the string associated with the specified symbolic 
     * link, for the selected language. This is a standard string table 
     * interface that can translate a numerically unique ID to a 
     * language-specific string.
     *
     * @param SymbolicLink
     * A uint16 value representing a numerically unique ID linked to the 
     * requested string.
     *
     * @param LanguageID
     * An optional parameter to override the selected language ID for the 
     * string table. This is useful if the developer wants to get a single 
     * string for a specific language, but doesn't want to effect the current
     * language configuration of the string table.
     *
     * @return
     * A reference to the string identified by SymbolicLink. This reference is
     * read-only and should not be modified by any external method. If this
     * value is NULL, then no string was available for SymbolicLink.
     * 
     *//*====================================================================*/
    const ansichar* GetString(uint16 SymbolicLink, 
                              uint8 LanguageID = LANGUAGE_DEFAULT);

    /*====================================================================*//**
     * @brief     
     * Method to retrieve a class name based on a ClassID.
     *
     * This method retrieves the textual description of a class definition.
     * This string is only available if there is at least one language table 
     * available, and a link table available related to the requested ClassID.
     *
     * @param ClassID
     * The ClassID for the requested class name.
     *
     * @return
     * A reference to a textual representation of the class name identified by
     * ClassID. This reference is read-only and should not be modified by any 
     * external method. If this value is NULL, then no string was available for
     * the ClassID.
     * 
     *//*====================================================================*/
    const ansichar* GetClassText(uint32 ClassID, 
                                 uint8 LanguageID = LANGUAGE_DEFAULT);

    /*====================================================================*//**
     * @brief
     * Method to retrive the textual description for a Species
     *
     * This method can be used to retrieve the textual description for a 
     * specific Species, based on a definition's base ClassID. This is useful
     * to get the strings for definitions with multiple Species defined.
     *
     * @param ClassID
     * The base ClassID for the definition with the requested Species.
     *
     * @param Species
     * A unint8 value for the requested Species.
     *
     * @param LanguageID
     * An optional parameter to override the selected language ID for the 
     * string table. This is useful if the developer wants to get a single 
     * string for a specific language, but doesn't want to effect the current
     * language configuration of the string table.
     *
     * @return
     * A reference to a textual representation of the Species identified. This 
     * reference is read-only and should not be modified by any external 
     * method. If this value is NULL, then no string was available for the 
     * Species.
     * 
     *//*====================================================================*/
    const ansichar* GetSpeciesString(uint32 ClassID, uint8 Species,
                                     uint8 LanguageID = LANGUAGE_DEFAULT);

    /*====================================================================*//**
     * @brief
     * Method to get the description for a specific member. 
     *
     * This method provides the developer with a means to extract the 
     * description for a specific member from the string table, based on the 
     * ClassID of the definition where the member resides, and the member's
     * unique ID. 
     *
     * @param ClassID
     * The base ClassID for the definition with the requested member.
     *
     * @param MemberID
     * A uint16 value representing the numerical ID for the requested member.
     * This is generated by the macros in DMOIBDefinitions.h, and is unique 
     * among all members of the parent definition. (This is the same value 
     * used to access a member with the CDMOIBObject::GetMember() methods)
     *
     * @param LanguageID
     * An optional parameter to override the selected language ID for the 
     * string table. This is useful if the developer wants to get a single 
     * string for a specific language, but doesn't want to effect the current
     * language configuration of the string table.
     * 
     * @return
     * A reference to a textual representation of the member identified. This 
     * reference is read-only and should not be modified by any external 
     * method. If this value is NULL, then no string was available for the 
     * member.
     * 
     *//*====================================================================*/
    const ansichar* GetMemberString(uint32 ClassID, uint16 MemberID,
                                    uint8 LanguageID = LANGUAGE_DEFAULT);

    /*====================================================================*//**
     * @brief     
     * Method to get the description for a specific enumeration.
     *
     * This method provides the developer with a means to extract the 
     * description for a specific enumeration from the string table, based on 
     * the ClassID of the definition where the member resides, and the 
     * enumeration's unique ID. This ID is very similar to that used for 
     * uniquely identifying members within a definition, with regards to its
     * creation process in DMOIBDefinitions.h.
     *
     * @param ClassID
     * The base ClassID for the definition with the requested enumeration.
     *
     * @param Enumeration
     * A uint16 value representing the numerical ID for the requested member.
     * This is generated by the macros in DMOIBDefinitions.h, and is unique 
     * among all enumerations of the parent definition. 
     *
     * @param LanguageID
     * An optional parameter to override the selected language ID for the 
     * string table. This is useful if the developer wants to get a single 
     * string for a specific language, but doesn't want to effect the current
     * language configuration of the string table.
     *
     * @return
     * A reference to a textual representation of the enumeration identified. 
     * This reference is read-only and should not be modified by any external 
     * method. If this value is NULL, then no string was available for the 
     * enumeration.
     * 
     *//*====================================================================*/
    const ansichar* GetEnumerationString(uint32 ClassID, uint16 Enumeration,
                                         uint8 LanguageID = LANGUAGE_DEFAULT);

    /*====================================================================*//**
     * @brief     
     * Method to get the description for a specific enumerator.
     *
     * This method provides the developer with a means to extract the 
     * description for a specific enumerated value from the string table, based
     * on the ClassID of the definition where the member resides, the parent
     * enumeration's unique ID, and the value of the specific enumerator from 
     * that set.
     *
     * @param ClassID
     * The base ClassID for the definition with the requested 
     * enumeration/enumerator combination.
     *
     * @param Enumeration
     * A uint16 value representing the numerical ID for the requested 
     * enumeration. (This is the same type of value provided to
     * GetEnumerationString()) 
     *
     * @param Enumerator
     * A uint16 value representing the numerical value for the requested 
     * enumerator.
     *
     * @param LanguageID
     * An optional parameter to override the selected language ID for the 
     * string table. This is useful if the developer wants to get a single 
     * string for a specific language, but doesn't want to effect the current
     * language configuration of the string table.
     *
     * @return
     * A reference to a textual representation of the enumerator identified. 
     * This reference is read-only and should not be modified by any external 
     * method. If this value is NULL, then no string was available for the 
     * enumerator.
     * 
     *//*====================================================================*/
    const ansichar* GetEnumeratorString(uint32 ClassID, uint16 Enumeration, 
                                        uint16 Enumerator, 
                                        uint8 LanguageID = LANGUAGE_DEFAULT);
};

/*=============================================================================
    Global functions to help access definition information
==========================================================================*//**
* Global function to determine the size of a LinkTable.
*
* @param pLinkTable
* A reference to an existing LinkTable whose size will be determined (without
* editing its contents).
*
* @return
* A uint32 representing the size, in bytes, of the LinkTable.
*
*//*=========================================================================*/
uint32 LinkTableSize(void* pLinkTable);

/*========================================================================*//**
* Global function to determine the ClassID of a Link Table.
*
* @param pLinkTable
* A reference to an existing pLinkTable whose ClassID will be extracted
* (without editing its contents).
*
* @return
* A uint32 representing the ClassID associated with of the Link Table.
*
*//*=========================================================================*/
uint32 LinkTableID(void* pLinkTable);

/*========================================================================*//**
* Global function to determine the size of a Language Table.
*
* @param pLanguageTable
* A reference to an existing LanguageTable whose size will be determined 
* (without editing its contents).
*
* @return
* A uint32 representing the size, in bytes, of the LanguageTable.
*
*//*=========================================================================*/
uint32 LanguageTableSize(void* pLanguageTable);

/*========================================================================*//**
* Global function to determine the Language ID of a Language Table.
*
* @param pLinkTable
* A reference to an existing pLanguageTable whose Language ID will be extracted
* (without editing its contents).
*
* @return
* A uint32 representing the Language ID associated with of the Language Table.
*
*//*=========================================================================*/
uint32 LanguageTableID(void* pLanguageTable);

/*===========================================================================*/

#endif /* CDMOIBSTRINGTABLE_H */
