/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// DMOIBBase.h    :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#if !defined(DMOIBBASE_H)
    #define  DMOIBBASE_H

#include "StdVarTyp.h"
#include "target.h"

/*
===============================================================================
    This file includes all of the shared macros, enums, typedefs, and basic 
    functions used across multiple elements of the DMOIB library.
===============================================================================
    All known DMOIB data types
==========================================================================*//**
 * @brief
 * An enumeration of all valid types for Dynamic DMOIB.
 *
 * All of the types used by Dynamic DMOIB must be deterministic at run-time,
 * so Dynamic MOIB uses these enumerated values to flag each member and 
 * describe them explicitly. 
 *
 *//*========================================================================*/
enum DMOIBTypes_e
{
    /*
        BitField Masking
        -----------------
        0 0 0 0   0 0 0 0
        | | | |   | | | |
        | | | |   | | | +-> -\
        | | | |   | | +---> --|--> Byte-size of the member, minus 1
        | | | |   | +-----> -/
        | | | |   |
        | | | |   +-------> FLAG: Text Types (For NULL Terminator in SetBuffer)
        | | | +-----------> FLAG: Unsigned Values
        | | +-------------> FLAG: Floating Point values
        | +---------------> FLAG: Pointers
        +-----------------> FLAG: "Static" sized Arrays
    */

    DMOIB_TYPE_FLAG_TXT   = 0x08,    // Flag -> 0000 1000
    DMOIB_TYPE_FLAG_UNS   = 0x10,    // Flag -> 0001 0000
    DMOIB_TYPE_FLAG_FP    = 0x20,    // Flag -> 0010 0000
    DMOIB_TYPE_FLAG_PTR   = 0x40,    // Flag -> 0100 0000
    DMOIB_TYPE_FLAG_ARR   = 0x80,    // Flag -> 1000 0000

    DMOIB_TYPE_FLAG_OBJ   = 0x06,    // Size -> 0000 0110
    DMOIB_TYPE_FLAG_TIME  = 0x02,    // Size -> 0000 0010 (3 - 1)

    DMOIB_TYPE_MASK_BW    = 0x07,    // Mask -> 0000 0111 
    DMOIB_TYPE_UNKNOWN    = 0xF8,

    // Normal Data Types
    DMOIB_TYPE_INT8          = (sizeof(int8)  - 1),
    DMOIB_TYPE_INT16         = (sizeof(int16) - 1),
    DMOIB_TYPE_INT32         = (sizeof(int32) - 1),
                                                     
    DMOIB_TYPE_UINT8         = (DMOIB_TYPE_FLAG_UNS) | (sizeof(uint8)    - 1),
    DMOIB_TYPE_UINT16        = (DMOIB_TYPE_FLAG_UNS) | (sizeof(uint16)   - 1),
    DMOIB_TYPE_UINT32        = (DMOIB_TYPE_FLAG_UNS) | (sizeof(uint32)   - 1),
                                                     
    DMOIB_TYPE_FLOAT32       = (DMOIB_TYPE_FLAG_FP)  | (sizeof(float32)  - 1),
    DMOIB_TYPE_FLOAT64       = (DMOIB_TYPE_FLAG_FP)  | (sizeof(float64)  - 1),
                                                     
    DMOIB_TYPE_BYTE          = (DMOIB_TYPE_FLAG_UNS) | (sizeof(byte)     - 1),
    DMOIB_TYPE_WORD          = (DMOIB_TYPE_FLAG_UNS) | (sizeof(word)     - 1),
    DMOIB_TYPE_BOOLEAN       = (DMOIB_TYPE_FLAG_UNS) | (sizeof(boolean)  - 1),
                                                     
    DMOIB_TYPE_ANSICHAR      = (DMOIB_TYPE_FLAG_TXT) | (sizeof(ansichar) - 1),
    DMOIB_TYPE_WIDECHAR      = (DMOIB_TYPE_FLAG_TXT) | (sizeof(widechar) - 1),

    // Pointer Data Types
    DMOIB_TYPE_INT8_PTR      = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_INT8),
    DMOIB_TYPE_INT16_PTR     = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_INT16),
    DMOIB_TYPE_INT32_PTR     = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_INT32),
    
    DMOIB_TYPE_UINT8_PTR     = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_UINT8),
    DMOIB_TYPE_UINT16_PTR    = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_UINT16),
    DMOIB_TYPE_UINT32_PTR    = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_UINT32),
    
    DMOIB_TYPE_FLOAT32_PTR   = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_FLOAT32),    
    DMOIB_TYPE_FLOAT64_PTR   = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_FLOAT64),
    
    DMOIB_TYPE_BYTE_PTR      = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_BYTE),
    DMOIB_TYPE_WORD_PTR      = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_WORD),
    DMOIB_TYPE_BOOLEAN_PTR   = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_BOOLEAN),
    
    DMOIB_TYPE_ANSICHAR_PTR  = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_ANSICHAR),
    DMOIB_TYPE_WIDECHAR_PTR  = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_WIDECHAR),

    // Static Array Data Types
    DMOIB_TYPE_INT8_ARR      = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_INT8),
    DMOIB_TYPE_INT16_ARR     = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_INT16),
    DMOIB_TYPE_INT32_ARR     = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_INT32),
    
    DMOIB_TYPE_UINT8_ARR     = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_UINT8),
    DMOIB_TYPE_UINT16_ARR    = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_UINT16),
    DMOIB_TYPE_UINT32_ARR    = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_UINT32),
    
    DMOIB_TYPE_FLOAT32_ARR   = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_FLOAT32),    
    DMOIB_TYPE_FLOAT64_ARR   = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_FLOAT64),
    
    DMOIB_TYPE_BYTE_ARR      = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_BYTE),
    DMOIB_TYPE_WORD_ARR      = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_WORD),
    DMOIB_TYPE_BOOLEAN_ARR   = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_BOOLEAN),
    
    DMOIB_TYPE_ANSICHAR_ARR  = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_ANSICHAR),
    DMOIB_TYPE_WIDECHAR_ARR  = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_WIDECHAR),
    
    // Struct data types    
    DMOIB_TYPE_TIME          =  DMOIB_TYPE_FLAG_TIME,
    DMOIB_TYPE_TIME_PTR      = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_FLAG_TIME),
    DMOIB_TYPE_TIME_ARR      = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_FLAG_TIME),

    // Object Data Types
    DMOIB_TYPE_OBJECT        =  DMOIB_TYPE_FLAG_OBJ,
    DMOIB_TYPE_BASE_CLASS    = (DMOIB_TYPE_FLAG_UNS | DMOIB_TYPE_FLAG_OBJ),
    DMOIB_TYPE_VECTOR_PTR    = (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_FLAG_OBJ),
    DMOIB_TYPE_VECTOR_ARR    = (DMOIB_TYPE_FLAG_ARR | DMOIB_TYPE_FLAG_OBJ)

};

/*========================================================================*//**
 * @brief
 * Constant values for predetermined sizes of definition elements
 *
 *//*========================================================================*/
#define OMITTED UINT32_MAX

#define SIZEOF_FAMILY               sizeof(uint16)
#define SIZEOF_GENUS                sizeof(uint8)
#define SIZEOF_SPECIES              sizeof(uint8)
#define SIZEOF_CLASSID              (SIZEOF_FAMILY + SIZEOF_GENUS + SIZEOF_SPECIES)

#define SIZEOF_SPECIES_COUNT        sizeof(uint8)
#define SIZEOF_VERSION              sizeof(uint16)
#define SIZEOF_DEFINITION           sizeof(uint16)
#define SIZEOF_XML_CONTENT          sizeof(uint16)
#define SIZEOF_MEMBER_COUNT         sizeof(uint16)

#define SIZEOF_TYPE_INFO            sizeof(uint8)
#define SIZEOF_MEMBER_ID            sizeof(uint16)
#define SIZEOF_ARRAY_SIZE           sizeof(uint16)

#define SIZEOF_ALIAS                sizeof(uint32)

#define SIZEOF_UNITS                sizeof(uint8)
#define SIZEOF_SCALE_COEFFICIENT    sizeof(uint16)
#define SIZEOF_SCALE_EXPONENT       sizeof(int8)

#define SIZEOF_LANGUAGE_ID          sizeof(uint8)
#define SIZEOF_SYMBOLIC_LINK_COUNT  sizeof(uint16)
#define SIZEOF_SYMBOLIC_LINK        sizeof(uint32)

#define SIZEOF_STRING_TABLE_SIZE    sizeof(uint32)

#define SIZEOF_LINK_TABLE_SIZE      sizeof(uint16)
#define SIZEOF_LINK_TABLE_COUNT     sizeof(uint16)
#define SIZEOF_LINK_TABLE_ENTRY     sizeof(uint16)

#define SIZEOF_CRC                  sizeof(uint16)

 
/*========================================================================*//**
 * @brief
 * Constant strings used for XML processing
 *
 *//*========================================================================*/
#define XML_TAG_DEFINITION         "DEFINITION"
#define XML_TAG_MEMBER             "MEMBER"
#define XML_TAG_MEMBERS            "MEMBERS"
#define XML_TAG_VALUE              "VALUE"
#define XML_TAG_ENUMERATION        "ENUMERATION"
#define XML_TAG_ENUM               "ENUM"
#define XML_TAG_CDATA_OPEN         "![CDATA["
#define XML_TAG_CDATA_CLOSE        "]]"

#define XML_ATT_FAMILY             "family="
#define XML_ATT_GENUS              "genus="
#define XML_ATT_SPECIES            "species="
#define XML_ATT_VERSION            "version="
#define XML_ATT_TYPE               "type="
#define XML_ATT_NAME               "name="
#define XML_ATT_MINVERSION         "minversion="
#define XML_ATT_MAXVERSION         "maxversion="
#define XML_ATT_SIZE               "size="
#define XML_ATT_CLASS              "class="
#define XML_ATT_DESCRIPTION        "description="
#define XML_ATT_SELECTED           "selected="
#define XML_ATT_DEFAULT_VALUE      "defaultvalue="
#define XML_ATT_ALIAS              "alias="

#define XML_ATT_UNITS              "units="
#define XML_ATT_SCALE_COEFFICIENT  "scalecoefficient="
#define XML_ATT_SCALE_EXPONENT     "scaleexponent="

#define XML_ATT_INHERITED          "inherit="

#define XML_CHAR_QUOTE             "\""
#define XML_CHAR_OPENPAREN         "("
#define XML_CHAR_CLOSEPAREN        ")"
#define XML_CHAR_LESSTHAN          "<"
#define XML_CHAR_GREATERTHAN       ">"
#define XML_CHAR_SLASH             "/"
#define XML_CHAR_SPACE             " "
#define XML_CHAR_RETURN            "\n"
#define XML_CHAR_ARRAY_DELIM       ";"
#define XML_CHAR_TERMINATOR        "\0"

#define XML_VAL_UNBOUNDED          "UNBOUNDED"
#define XML_VAL_ANY_CLASS          "ANY_CLASS"
#define XML_VAL_CURRENT_VERSION    "CURRENT_VERSION"
#define XML_VAL_YES                "YES"
#define XML_VAL_OBJECT             "OBJECT"

/*========================================================================*//**
 * @namespace ANY_CLASS
 * @brief
 * Predefined namespace used for a default override in some macro expansions.
 *
 * The macros provided in DMOIBDefiner.h generate namespaces for the 
 * definitions in DMOIBDefinitions.h. In certain instances, if specific 
 * elements are not defined, values will default to those defined in the 
 * ANY_CLASS namespace. Also, for defining DMOIBVectors that will accept any
 * type of MOIB object, the keyword ANY_CLASS can be used, and it will resolve
 * to this namespace for constant values.
 *
 *//*========================================================================*/
 /*========================================================================*//**
 * @enum ClassConstants_e
 * @brief
 * Enumeration to define default Class values.
 *
 * The values of this enumeration are used to define default cases for classes.
 * In particular, for DMOIBVector types that are defined to allow ANY_CLASS,
 * the values of this enumeration describe the filtering properties of allowing
 * any type of MOIB class into the DMOIBVector.
 *
 *//*========================================================================*/
 /*========================================================================*//**
 * @enum ArrayConstants_e
 * @brief
 * Enumeration to define special values for array declaration.
 *
 * This enumeration provides a set of special values that are used as keywords
 * when defining dynamic arrays and dynamically sized vectors. 
 *
 *//*========================================================================*/
 /*========================================================================*//**
 * @enum VersionConstants_e
 * @brief
 * Enumeration to define special values for object versions.
 *
 * This enumeration provides a set of special values that are used as keywords
 * for identifying special version contraints. These are used as versioning
 * boundaries for members declared in a DMOIB object, and for default values
 * used when constructing a new DMOIB object.
 *
 *//*========================================================================*/
 /*========================================================================*//**
 * @enum EnumerationConstants_e
 * @brief
 * Enumeration to define special values for enumeration identifiers.
 *
 * This enumeration defines special identifiers for specific enumeration sets
 * that apply to all DMOIB objects. Since these sets of enumerations are always
 * available in a DMOIB definition, a constant predefined value can be assigned
 * as an identifier for that enumeration.
 *
 *//*========================================================================*/
 /*========================================================================*//**
 * @enum LanguageConstants_e
 * @brief
 * Enum to identify all special language table values.
 *
 * This enumeration provides a set of special values applying to the languages 
 * available to a string table. These values represent boundary and default 
 * conditions for the language tables managed by a CDMOIBStringTable object.
 *
 *//*========================================================================*/
namespace ANY_CLASS
{
    enum ClassConstants_e
    {
        CLASSID             = UINT32_MAX,
        MEMORY_SIZE         = 0,
        DMOIB_MEMBER_COUNT  = 0
    };
};
enum ClassConstants_e
{
    BASE_CLASS_MEMBER_COUNT = 0
};
enum ArrayConstants_e
{
    UNBOUNDED = 0
};
enum VersionConstants_e
{
    HIGHEST_KNOWN_VERSION   = UINT16_MAX,
    CURRENT_VERSION         = UINT16_MAX
};
enum EnumerationConstants_e
{
    SPECIES_ENUMERATION_ID  = UINT16_MAX
};
enum LanguageConstants_e
{    
    LANGUAGE_DEFAULT    = 0xFF,
    LANGUAGE_UNKNOWN    = 0xFF,

    // Boundary/special conditions
    LANGUAGE_MAX        = LANGUAGE_DEFAULT,
    LANGUAGE_SIZE       = 0,
    LANGUAGE_END        = 0,
};

/*
===============================================================================
    General Macros for helping out in code
===============================================================================
*/
#define MAKE_CLASSID(Family, Genus, Species) \
        ((uint32)((Family << 16) | (Genus << 8) | Species))

#define GET_FAMILY(ClassID)     ((uint16)((((uint32)ClassID) >> 16) & 0x0FFFF))
#define GET_GENUS(ClassID)      ((uint8) ((((uint32)ClassID) >>  8) & 0x0FF  ))
#define GET_SPECIES(ClassID)    ((uint8) ( ((uint32)ClassID)        & 0x0FF  ))

#ifndef MIN
#define MIN(x,y)        (((x) < (y)) ? (x) : (y))
#endif
#ifndef MAX
#define MAX(x,y)        (((x) > (y)) ? (x) : (y))
#endif
#define ALIGN(x, y)             ((((x) + ((y)-1))/(y))*(y))
#define ALIGN_TO_BOUNDARY(x)    ALIGN(x, sizeof(uint32))

/*========================================================================*//**
 * @brief
 * Structure to organize tables used by dictionaries and string tables.
 *
 * This structure is used to organize all the binary tables which represent 
 * definitions, string tables, and link tables, into a linked-list format. 
 * This allows binary tables to be easily added and removed from the list, 
 * and allows static ones to be treated in the same way as dynamically added 
 * ones. Also, Since all lookups for a binary tables of any type involve a 
 * linear search to find a corresponding identifier, there is no performance 
 * loss from using this implementation instead of a normal array.
 *
 *//*========================================================================*/
typedef struct TableEntry_s
{
    byte*                   m_pTable;
    struct TableEntry_s*    m_pNext;
}TableEntry_t;

/*========================================================================*//**
 * @brief
 * Function to copy a value, from LSBs to MSBs
 *
 * This function is similar to the standard memcpy() funtion, as it copys the
 * values from one buffer to another, byte by byte. However, this function is
 * different because it follows the endianess of the target system, and copies
 * values starting at the LSBs, working towards the MSBs.
 *
 * @param pDest
 * Pointer to the destination memory space where the values will be copied to.
 *
 * @param DestSize
 * The maximum number of bytes to be copied to pDest. Once this limit is 
 * reached, or SourceSize is reached, the function will return.
 * 
 * @param pSource
 * Pointer to the source memory space where the values will be copied from.
 *
 * @param SourceSize
 * The maximum number of bytes to be copied from pSource. Once this limit is 
 * reached, or DestSize is reached, the function will return.
 *
 *//*========================================================================*/
void    ByteCopy(byte* pDest,   uint16 DestSize, 
                 byte* pSource, uint16 SourceSize);
#define BYTE_COPY(pDest, DestSize, pSource, SourceSize) \
        ByteCopy((byte*)(pDest),   (uint16)(DestSize), \
                 (byte*)(pSource), (uint16)(SourceSize))
#define BYTE_COPY_FP (ByteCopy)

/*========================================================================*//**
 * @brief
 * Function to copy a value, performing endianess swapping if necessary.
 *
 * This function is similar to the ByteCopy() function, as it copys the
 * values from one buffer to another, byte by byte. However, this function
 * only applies to copying values from a media source to a target destination,
 * or vice versa. If the endianesses of the 2 buffers are different, then the
 * function will swap the bytes as they are copied.
 *
 * @param pDest
 * Pointer to the destination memory space where the values will be copied to.
 *
 * @param DestSize
 * The maximum number of bytes to be copied to pDest. Once this limit is 
 * reached, or SourceSize is reached, the function will return.
 * 
 * @param pSource
 * Pointer to the source memory space where the values will be copied from.
 *
 * @param SourceSize
 * The maximum number of bytes to be copied from pSource. Once this limit is 
 * reached, or DestSize is reached, the function will return.
 *
 *//*========================================================================*/
#if(((TARGET_ENDIANNESS==LITTLE_ENDIAN) && (MEDIA_ENDIANNES==BIG_ENDIAN)) || \
    ((TARGET_ENDIANNESS==BIG_ENDIAN) && (MEDIA_ENDIANNES==LITTLE_ENDIAN)))
void    LineCopy(byte* pDest,   uint16 DestSize, 
                 byte* pSource, uint16 SourceSize);
#define LINE_COPY(pDest, pSource, Size) \
        LineCopy((byte*)(pDest),   (uint16)(Size), \
                 (byte*)(pSource), (uint16)(Size))
#define LINE_COPY_FP (LineCopy)
#else
#define LINE_COPY(pDest, pSource, Size) BYTE_COPY(pDest, Size, pSource, Size)
#define LINE_COPY_FP BYTE_COPY_FP
#endif

/*========================================================================*//**
 * @brief
 * Function to compare a media endianess value to a target endianess value.
 *
 * This function will compare 2 values, byte by byte, starting at the MSBs and
 * progressing to the LSBs of the values. If their is a mismatch in this 
 * process, then the function will report the number of bytes left in the 
 * values when the mismatch occured. This function will also take the 
 * appropriate measures to swap byte orders if the 2 values' endianesses are
 * different (If the media is big endian and the target is little endian for
 * example).
 *
 * @param pMediaSource
 * Pointer to the memory space of the value from the media.
 *
 * @param pTargetSource
 * Pointer to the memory space of the value from the target.
 *
 * @param Size
 * The maximum number of bytes to be compared between the 2 values.
 *
 * @return
 * The number of bytes remaining when the first mismatch was encountered. If
 * both values match exactly, this will return 0.
 *
 *//*========================================================================*/
uint16  LineCheck(byte* pMediaSource, byte* pTargetSource, uint16 Size);
#define LINE_CHECK(pMediaSource, pTargetSource, Size) \
      (!LineCheck((byte*)(pMediaSource),(byte*)(pTargetSource),(uint16)(Size)))
#define LINE_CHECKN(pMediaSource, pTargetSource, Size) \
        LineCheck((byte*)(pMediaSource),(byte*)(pTargetSource),(uint16)(Size))


#endif /* #if !defined(DMOIBBASE_H) */
