/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CCommSvr.cpp   :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CCommSvr.h"
#include "SysUtil.h"

#if defined(STACK_SERIAL)
#include "CSerialCommSvr.h"
#endif

#if defined(STACK_ETHERNET)
#include "CUDPCommSvr.h"
#include "CTCPCommSvr.h"
#endif

#if defined(STACK_USB)
#include "CUSBCommSvr.h"
#endif

#if defined(STACK_VIRTUAL_CHANNEL)
#include "CVCCommSvr.h"
#endif

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*================================================================================
  Default constructor
================================================================================*/
CCommSvr::CCommSvr(SetRecvBufPtr RecvMsgFunc, 
                   DevArrPtr DeviceArrivalFunc, 
                   DevRemPtr DeviceRemovalFunc, 
                   char* pAppName)
{
    pRecvMsg = RecvMsgFunc;
    pDeviceRemoval =  DeviceRemovalFunc;
    pDeviceArrival =  DeviceArrivalFunc;
    pSemName = pAppName;
}

/*================================================================================
  Default destructor
================================================================================*/
CCommSvr::~CCommSvr(void)
{
    pRecvMsg = NULL;
    pDeviceArrival =  NULL;
    pDeviceRemoval =  NULL;
    pSemName = NULL;
}

/*================================================================================
CCommSvr GetSourceID: Returns the SourceID of the object
Parameters:
    pConnArgs: Pointer to a ConnArgs structure.

Returns: 
    SourceID, if the pConnArgs is valid, pConnArgs->pCommSvr is valid, and the 
        call to GetSourceID returns TRUE.
    NULL, if the pConnArgs is invalid, pConnArgs->pCommSvr is invalid, or the call 
        to GetSourceID returns NULL.
================================================================================*/
char* CCommSvr::GetSourceID(ConnArgs* pConnArgs)
{
    char* pSourceID = NULL;
    if (pConnArgs)
    {
        if (pConnArgs->pCommSvr)
        {
            pSourceID = ((CBaseCommSvr*)(pConnArgs->pCommSvr))->GetSourceID();
        }
    }
    return(pSourceID);
}

/*================================================================================
CCommSvr GetPortNum: Returns the Port used by the object
Parameters:
    pConnArgs: Pointer to a ConnArgs structure.

Returns: 
    Port Value, if the pConnArgs is valid, pConnArgs->pCommSvr is valid, and the 
        call to GetPortNum returns TRUE.
    0, if the pConnArgs is invalid, pConnArgs->pCommSvr is invalid, or the call 
        to GetPortNum returns 0.
================================================================================*/
uint16 CCommSvr::GetPortNum(ConnArgs* pConnArgs)
{
    uint16 Port = 0;
    
    if (pConnArgs)
    {        
        if (pConnArgs->pCommSvr)
        {
            Port = ((CBaseCommSvr*)(pConnArgs->pCommSvr))->GetPortNum();
        }
    }

    return(Port);
}

/*================================================================================
CCommSvr SetPortNum: Changes the current configuration to use the WACPPort. 
Parameters:
    pConnArgs: Pointer to a ConnArgs structure.

Returns: 
    TRUE, if the pConnArgs is valid and the call to SetPortNum returns TRUE.
    FALSE, if the pConnArgs is invalid or the call to SetPortNum returns FALSE.
================================================================================*/
boolean CCommSvr::SetPortNum(ConnArgs* pConnArgs)
{
    boolean Result = FALSE;

    if (pConnArgs)
    {
        if (pConnArgs->pCommSvr)
        {
            /* Update the port number of the comm server */
            Result = ((CBaseCommSvr*)(pConnArgs->pCommSvr))->SetPortNum(pConnArgs->WACPPort);
        }
    }

    return (Result);
}

/*================================================================================
CCommSvr Connect: Creates a new specific commsvr instance and calls to connect to 
    the passed in SourceID and Port.

Parameters:
    pConnArgs: Pointer to a ConnArgs structure.

Returns: 
    TRUE, if the pConnArgs is valid, a new stack instance is created, Connect() returns
        TRUE, and the StartReader() method returns TRUE.
    FALSE, if the pConnArgs is invalid, new () fails, Connect fails, or StartReader
        fails.
================================================================================*/
boolean CCommSvr::Connect(ConnArgs* pConnArgs)
{
    boolean Result = FALSE;
    CBaseCommSvr* pClient = NULL;

    if (pConnArgs)
    {
		switch (pConnArgs->StackType)
		{
#if defined(STACK_SERIAL)
		    case SERIAL:
            {
			    pClient = new CSerialCommSvr();
			    break;
            }
#endif
#if defined(STACK_ETHERNET)
		    case ETHERNET:
            {
			    if (0 == strncmp(pConnArgs->SourceID, UDP_SOURCE_PREFIX, strlen(UDP_SOURCE_PREFIX)))
			    {
				    pClient = new CUDPCommSvr();
			    }
			    else
			    {
				    pClient = new CTCPCommSvr();
			    }
			    break;
            }
#endif
#if defined(STACK_VIRTUAL_CHANNEL)
		    case VIRTUAL_CHANNEL:
            {
			    pClient = new CVCCommSvr();
			    break;
            }
#endif

#if defined(STACK_USB)
		    case USB:
            {
			    pClient = new CUSBCommSvr();
			    break;
            }
#endif
		}
		if (pClient)
		{
			pClient->Init(pRecvMsg, pDeviceArrival, pDeviceRemoval, pSemName);
			pClient->SetSourceID(pConnArgs->SourceID);
			pClient->SetPortNum(pConnArgs->RNDZPort);
			pClient->SetConfig(pConnArgs->pConfig);
			Result = pConnArgs->bConnected = pClient->Connect((uint8*)pConnArgs->SourceID, pConnArgs->RNDZPort);
			if (Result)
			{
				pConnArgs->pCommSvr = pClient;
			}
			else
			{
				delete pClient;
				pConnArgs->pCommSvr = NULL;
			}
		}
    }
    return Result;
}

/*================================================================================
CCommSvr Disconnect: terminates the current connection pointed to pConnArgs.
Parameters:
    pConnArgs: Pointer to a ConnArgs structure.

Returns: 
    TRUE, if the pConnArgs is valid and the pConnArgs->pCommSvr is valid.
    FALSE, if the pConnArgs is invalid or the pConnArgs->pCommSvr is NULL.
================================================================================*/
boolean CCommSvr::Disconnect(ConnArgs* pConnArgs)
{
    boolean Result = FALSE;
    CBaseCommSvr* pBaseClass = NULL;

    if (pConnArgs)
    {
        pBaseClass = (CBaseCommSvr*)pConnArgs->pCommSvr;
        if (pBaseClass)
        {
            /* Clean up the commsvr instance */
            delete(pBaseClass);
            pConnArgs->pCommSvr = NULL;
            
            Result = TRUE;
        }
    }

    return Result;
}

/*================================================================================
CCommSvr SendMsg: Sends the message through the BaseCommSvr instance.
Parameters:
    pConnArgs: Pointer to a ConnArgs structure.
    pBuf:      Pointer to the data buffer to send.

Returns: 
    TRUE, if the pConnArgs is valid, stacktype is valid, and the listener thread is 
        started.
    FALSE, if the pConnArgs is invalid, stacktype is invalid, or the listener 
        thread fails to start.
================================================================================*/
boolean CCommSvr::SendMsg(ConnArgs* pConnArgs, uint8* pBuf, uint32 BufSize)
{
    boolean Result = FALSE;

    if (pConnArgs)
    {
        if (pConnArgs->pCommSvr)
        {
            /* Send the message */
            Result = ((CBaseCommSvr*)(pConnArgs->pCommSvr))->SendMsg(pBuf, BufSize);
        }
    }
    return Result;
}   

/*================================================================================
CCommSvr StartServer: Starts up a server to listen for connections. The class 
    instance is deleted in CCommSvr:Disconnect().

Parameters:
    pConnList:      Array of ConnArgs structures.
    NumConnections: Number of connections in the array.

Returns: 
    TRUE, if the stacktype is valid and the listener thread is started.
    FALSE, if the stacktype is invalid or the listener thread fails to start.
================================================================================*/
boolean CCommSvr::StartServer(ConnArgs** pConnList, uint8 NumConnections)
{
    boolean Result = FALSE;
    uint16 i;
	CBaseCommSvr* pServer = NULL;

	for (i=0; i < NumConnections; i++)
	{
		switch (pConnList[i]->StackType)
		{
#if defined(STACK_ETHERNET)
		    case ETHERNET:
            {
			    if (0 == strncmp(pConnList[i]->SourceID, UDP_SOURCE_PREFIX, strlen(UDP_SOURCE_PREFIX)))
			    {
				    pServer = new CUDPCommSvr(true);
			    }
			    else
			    {
				    pServer = new CTCPCommSvr(true);
			    }
			    break;
            }
#endif
#if defined(STACK_VIRTUAL_CHANNEL)
		    case VIRTUAL_CHANNEL:
            {
			    pServer = new CVCCommSvr(true);
			    break;
            }
#endif

#if defined(STACK_USB)
		    case USB:
            {
			    pServer = new CUSBCommSvr();
			    break;
            }
#endif
		}
		if (pServer)
		{
			pServer->Init(pRecvMsg, pDeviceArrival, pDeviceRemoval, pSemName);
			pServer->SetSourceID(pConnList[i]->SourceID);
			pServer->SetPortNum(pConnList[i]->RNDZPort);
			pServer->SetConfig(pConnList[i]->pConfig);
			Result = !pServer->StartServer(); //start listener
			if (Result)
			{
				pConnList[i]->pCommSvr = pServer;
			}
			else
			{
				delete pServer;
			}
		}

		/* Exit the for loop if we are unable to start up a listener thread */
        if (!Result)
            break;

    } /* for (i=0; i < NumConnections; i++) */

    return Result;
}

/* End of File */


