/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CConObjSvr.cpp :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CConObjSvr.h"
#include "CWACPThread.h"
#include "CWACPSemaphore.h"
#if defined(TRACE_SERVER_ON)
    #include "CTraceSvr.h"
#endif
#include "CRNDZDStd.h"
#include "CConDStd.h"

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor
    CConObjSvr Start up: 

*********************************************************************************/
CConObjSvr::CConObjSvr()
{
    /* Initialize the buffer used for sending data via the ConnObjSvr. */
    pOutbuf = NULL;
}

CConObjSvr::~CConObjSvr()
{
    ThreadStatus = FALSE;

    /* Notify the KeepAlive thread to check the ThreadStatus */
    pTimeoutSem->Raise();

    /* Allow the KeepAliveThread to terminate */
    SYS_SLEEP(1);

    /* Wait for the Thread to terminate */
    ExitMutex.Lock(INFINITE);
    ExitMutex.Release();

    if (pTimeoutSem != NULL)
    {
        delete(pTimeoutSem);
        pTimeoutSem = NULL;
    }

    SYS_FREE(pOutbuf);
    pOutbuf = NULL;
}

void CConObjSvr::Init(boolean (*SendPacket)(char*, char*, uint8*, uint32),
                      boolean (*MsgFunc)(char*, uint16, void*, uint16),
                      uint32 (*RxConn)(char*, uint32, uint8*, void* ), 
                      void* pDefaultRouter,
                      LinkStruct* pLinkStruct)
{
    /* Set the function pointers for sending data and connection handling */
    pSendPacket = SendPacket;
    pMsgFunc = MsgFunc;

    pConnHandler = RxConn;
    pDefaultRtr = pDefaultRouter;

    /* Set the pointer to the CLinkTable instance for use by the KeepAliveThread */
    pCLinkTable = (CLinkTable*)pLinkStruct->pCLinkTable;

    /* Initialize the generic message server used for all message types. */
    MsgSvr.Init(FmCONNECTION, GnDATA, pLinkStruct);

    /* Set the SemaphoreName */
    pSemName = pLinkStruct->pSemName;
    
    /* Start up the Connection KeepAliveThread */
    ThreadStatus = TRUE;

    /* Create the Timeout semaphore used by the KeepAliveThread */
    pTimeoutSem = new CWACPSemaphore(NULL, 0, 1);

    /* Start up the KeepAliveThread */
    ConnObjThread.StartThread(KeepAliveThread, this, pLinkStruct->ThreadPriority, 
        "CConnObjSvr::KeepAliveThread");
}

/*================================================================================
CConObjSvr AddConnection: Adds a new connection to the CConObjSvr.
================================================================================*/
boolean CConObjSvr::AddConnection(char* pSourceID)
{
    boolean     Result = FALSE;
    int32       Index = -1;
    
    ConnMutex.Lock(INFINITE);
    
    /* Find the index into the CLinkTable entry */
    Index = pCLinkTable->GetEntryIndex(pSourceID);
    if (Index != -1)
    {
        if (pCLinkTable->Connection[Index]->pConnObj == NULL)
        {
            ConnObj* pConnObj = (ConnObj*)SYS_MALLOC(sizeof(ConnObj));
            if (pConnObj)
            {
                /* Setup the initial connection defaults from target.h */
                pConnObj->TxOptions = DEFAULT_CONN_SETTING;
                pConnObj->KeepAliveStatus = KEEP_ALIVE_STOPPED;
                pConnObj->TimeoutInterval = DEFAULT_TIMEOUT_INTERVAL;
                pConnObj->KeepAliveInterval = DEFAULT_KEEPALIVE_INTERVAL;

                /* Add the new object into the CLinkTable list */
                pCLinkTable->Connection[Index]->pConnObj = pConnObj;
            }
        }
    }

    ConnMutex.Release();

    return (Result);
}

/*================================================================================
CConObjSvr RemoveConnection: Removes a connection from the CConObjSvr.
================================================================================*/
boolean CConObjSvr::RemoveConnection(char* pSourceID)
{
    boolean Result = FALSE;
    int32 Index = -1;
    
    ConnMutex.Lock(INFINITE);

    /* Find the index into the CLinkTable entry */
    Index = pCLinkTable->GetEntryIndex(pSourceID);
    if (Index != -1)
    {
        /* Clean up memory allocated in AddConnection */
        SYS_FREE(pCLinkTable->Connection[Index]->pConnObj);
        pCLinkTable->Connection[Index]->pConnObj = NULL;
    }

    ConnMutex.Release();

    return (Result);
}

/*================================================================================
CConObjSvr GetConnObj: Returns a pointer to the ConnObj that matches the pSourceID,
    otherwise it returns NULL.
================================================================================*/
ConnObj* CConObjSvr::GetConnObj(char* pSourceID)
{
    ConnObj*    pConnObj = NULL;
    ConnArgs*   pConnArgs = NULL;

    /* Get a pointer to the ConnArgs for pSourceID */
    pConnArgs = pCLinkTable->GetEntry(pSourceID);
    if (pConnArgs)
    {
        pConnObj = (ConnObj*)pConnArgs->pConnObj;
    }

    return (pConnObj);
}
/*================================================================================
CConObjSvr SetTxOptions : Sets the Options for what the Conn server sends to keep 
         the connection alive.
         CONN_CFG_SpKEEPALIVEON - Turn on keep alive messages
         CONN_CFG_SpKEEPALIVEOFF - Turn off keep alive messages
================================================================================*/
boolean CConObjSvr::SetTxOptions(char* pSourceID, uint8 Options, boolean bSyncFlag)
{
    boolean Result = FALSE;

    ConnMutex.Lock(INFINITE);

    /* Check for a valid range of Options */
    if (Options < CONN_CFG_SpMAX)
    {
        ConnObj* pConnObj = GetConnObj(pSourceID);
        if (pConnObj)
        {

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
            SYS_TRACE(TRACE_Type_STATUS, "CConObjSvr","SetTxOptions",
                FmCONNECTION, GnRESPONSE, CONN_CRST_SpKEEPALIVE,
                "Setting TxOptions to %d\n", Options);
#endif
            /* Update the TxOption with the new option */
            pConnObj->TxOptions = Options;

            /* Reset the KeepAlive status and timer */
            StopKeepAlive(pSourceID);

            /* Start up the KeepAlive mechansim */
            StartKeepAlive(pSourceID);

            /* Check for a valid range of Options */
            if (bSyncFlag)
            {
                /* Tx the new Options to the other end of the connection to sync up */
                Result = TxMsg(pSourceID, &MsgSvr, GnCONFIG, Options, NULL, 0);
            }
            else
                Result = TRUE;
        }
    }

    ConnMutex.Release();

    return Result;
}

/*================================================================================
CConObjSvr GetTxOptions : Gets the current Connection Object Server option.
         CONN_CFG_SpKEEPALIVEON - Turn on keep alive messages
         CONN_CFG_SpKEEPALIVEOFF - Turn off keep alive messages
================================================================================*/
uint8 CConObjSvr::GetTxOptions(char* pSourceID)
{
    uint8    Options = 0;
    ConnObj* pConnObj = NULL;

    ConnMutex.Lock(INFINITE);

    pConnObj = GetConnObj(pSourceID);
    if (pConnObj)
    {
        Options = pConnObj->TxOptions;
    }

    ConnMutex.Release();

    return Options;
}

/*================================================================================
CConObjSvr SetTimeoutInterval : 
================================================================================*/
boolean CConObjSvr::SetTimeoutInterval(char* pSourceID, time_t Count)
{
    boolean  Result = FALSE;
    ConnObj* pConnObj = NULL;

    ConnMutex.Lock(INFINITE);

    pConnObj = GetConnObj(pSourceID);
    if (pConnObj)
    {
        pConnObj->TimeoutInterval = Count;
        Result = TRUE;
    }

    ConnMutex.Release();

    return Result;
}
/*================================================================================
CConObjSvr GetTimeoutInterval : 
================================================================================*/
time_t CConObjSvr::GetTimeoutInterval(char* pSourceID)
{
    time_t Timeout = 0;
    ConnObj* pConnObj = NULL;

    ConnMutex.Lock(INFINITE);
    pConnObj = GetConnObj(pSourceID);
    if (pConnObj)
    {
        Timeout = pConnObj->TimeoutInterval;
    }

    ConnMutex.Release();

    return Timeout;
}
/*================================================================================
CConObjSvr SetKeepAliveInterval : 
================================================================================*/
boolean CConObjSvr::SetKeepAliveInterval(char* pSourceID, time_t Count)
{
    boolean Result = FALSE;
    ConnObj* pConnObj = NULL;

    ConnMutex.Lock(INFINITE);

    pConnObj = GetConnObj(pSourceID);
    if (pConnObj)
    {
        pConnObj->KeepAliveInterval = Count;
        Result = TRUE;
    }
    
    ConnMutex.Release();
    
    return Result;
}
/*================================================================================
CConObjSvr GetKeepAliveInterval : 
================================================================================*/
time_t CConObjSvr::GetKeepAliveInterval(char* pSourceID)
{
    time_t Interval = 0;
    ConnObj* pConnObj = NULL;

    ConnMutex.Lock(INFINITE);

    pConnObj = GetConnObj(pSourceID);
    if (pConnObj)
    {
        Interval = pConnObj->KeepAliveInterval;
    }

    ConnMutex.Release();
    
    return Interval;
}
/*================================================================================
CConObjSvr Connect: Pass through function to start up a connection.
================================================================================*/
boolean CConObjSvr::Connect(ConnArgs** pConnArgs, uint16 NumConnections)
{
    boolean Result = FALSE;

    Result = pMsgFunc(pSemName, WACP_CONNECT, (void*)pConnArgs, NumConnections);

    return Result;
}

/*================================================================================
CConObjSvr Close: Calls Shutdown to terminate the connection and then pMsgFunc to 
    disconnect the connection. 
================================================================================*/
boolean CConObjSvr::Close(char* pSourceID)
{
    boolean Result = FALSE;

    /* For WACP session you must send a Shutdown message. RNDZ sessions 
     * incorporate this shutdown message as part of the session creation
     */
    Shutdown(pSourceID);

    /* RNDZ sessions... For servers this is a no op since
     * the session is automatically disconnected.
     * For WACP. Client sessions*/
    Result = pMsgFunc(pSemName, WACP_DISCONNECT, pSourceID, 0);

    return Result;
}

/*================================================================================
CConObjSvr UpdateStatus : Pass through function to update the RNDZ/WACP port 
    status when RNDZ completes successfully.
================================================================================*/
boolean CConObjSvr::UpdateStatus(char* pSourceID, boolean bStatus)
{
    boolean Result = FALSE;

    /* Use the SemName to distinguish the context of the callback */
    Result = pMsgFunc(pSemName, WACP_STATUS, pSourceID, bStatus);

    return Result;
}


/*================================================================================
CConObjSvr Shutdown : Sends a CONN_CMD_SpSHUTDOWN message to terminate the 
    connection and removes the connection structure from the linktable to stop
    the Keepalive checks.
================================================================================*/
boolean CConObjSvr::Shutdown(char* pSourceID)
{
    boolean   Result = FALSE;
    ConnArgs* pConnArgs = NULL;

    /* Get a pointer to the ConnArgs for pSourceID */
    pConnArgs = pCLinkTable->GetEntry(pSourceID);
    if (pConnArgs)
    {
        /* Only send out a Shutdown message if we RNDZ has connected */
        if (pConnArgs->bRNDZStatus == TRUE)
        {
            /* Send the message */
            Result = TxMsg(pSourceID, &MsgSvr, GnCOMMAND, CONN_CMD_SpSHUTDOWN, NULL, 0);
        }
    }

    /* Remove the connection structure from the CLinkTable */
    RemoveConnection(pSourceID);

    return Result;
}
/*================================================================================
CConObjSvr TxMsg : Helper routine that transmits the contents of a CMsgSvr object
================================================================================*/
boolean CConObjSvr::TxMsg(char* pSourceID, CMsgSvr* pMsgSvr, uint8 Genus, uint8 Species, 
                          uint8* pData, uint32 Size)
{
    uint32  OutBufSz;
    boolean Result = FALSE;

    /* Set the Genus */
    pMsgSvr->set_Genus(Genus);

    /* Set the Species */
    pMsgSvr->set_Species(Species);

    /* Set the Data */
    pMsgSvr->SetData(pData, Size);

    /* Get the Size of the serialized byte Stream */
    OutBufSz = MsgSvr.size_all();

    /* (Re)Allocate space for the serialized Buffer */
    if ((pOutbuf = (uint8*)SYS_REALLOC(pOutbuf, OutBufSz))!= NULL)
    {
        /* Serialize the CMsgSvr object */ 
        pMsgSvr->serialize_all(pOutbuf);
#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
        char tempString[] = {"Message Data:\n%A"};
        SYS_TRACE(TRACE_Type_SEND, "CConObjSvr","TxMsg",
            pMsgSvr->family(),pMsgSvr->genus(),pMsgSvr->species(),
            tempString, pOutbuf, OutBufSz);
#endif
        /* Send the message */
        Result = pSendPacket(pSemName, pSourceID, pOutbuf, OutBufSz);
    }

    return Result;
}

/*================================================================================
CConObjSvr EvaluateKeepAlive : This function is called once for every DataThread
                             Cycle in WACPApp when we have no normal messages 
                             waiting to be recieved. 
================================================================================*/
boolean CConObjSvr::EvaluateKeepAlive(char* pSourceID)
{   
    time_t NewTime = SYS_TIME(NULL);
    boolean Result = FALSE;
    ConnObj* pConnObj = NULL;

    ConnMutex.Lock(INFINITE);

    pConnObj = GetConnObj(pSourceID);
    if (pConnObj)
    {
        if (pConnObj->TxOptions != CONN_CFG_SpKEEPALIVEOFF)
        {
            switch (pConnObj->KeepAliveStatus)
            {
                /* KeepAlive timers have been activated, so check how they are */ 
                case KEEP_ALIVE_RUNNING:
                {
                    /* Do not send a keep alive message when in the CHECK mode */
                    if (pConnObj->TxOptions != CONN_CFG_SpKEEPALIVECHECK)
                    {
                        /* Check to see if our KeepAlive timer has expired*/
                        if(pConnObj->KeepAliveInterval <= (NewTime - pConnObj->KeepAliveTimer))
                        {
                            /* Its time to send another KeepAlive message, so reset
                            the timer and send a message. We reset the timer here
                            so we will continue to send messages every interval 
                            until we recieve a Timeout error*/
                            pConnObj->KeepAliveTimer = NewTime;
#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)         
                            SYS_TRACE(TRACE_Type_SEND, "CConObjSvr","EvaluateKeepAlive",
                                FmCONNECTION, GnRESPONSE, CONN_CRST_SpKEEPALIVE,
                                "%s Sending KeepAlive Message", pSourceID);
#endif
                            /* Send the message */
                            Result = TxMsg(pSourceID, &MsgSvr, GnREQUEST, CONN_CRST_SpKEEPALIVE, NULL, 0);
                        }
                    } /*if (TxOptions != CONN_CFG_SpKEEPALIVECHECK)*/

                    /* Check to see if our Timeout timer has expired*/
                    if(pConnObj->TimeoutInterval <= (NewTime - pConnObj->TimeoutTimer))
                    {
                        /* We have gone too long without a KeepAlive response*/
                        pConnObj->KeepAliveStatus = KEEP_ALIVE_TIMEOUT;
                    }
                    break;  
                }
                case KEEP_ALIVE_TIMEOUT:
                {   
                    /* We handle TIMEOUTs here after the state has changed. This means
                    that in case a TIMEOUT message was missed, subsequent 
                    EvaluateKeepAlive calls will trigger another TIMEOUT message here */
#if defined(TRACE_SERVER_ON)
                    SYS_TRACE(TRACE_Type_ERROR, "CConObjSvr","EvaluateKeepAlive",
                        FmCONNECTION, GnRESPONSE, CONN_CRST_SpKEEPALIVE,
                        "%s KeepAlive TIMEOUT", pSourceID);
#endif
                    pConnObj->KeepAliveStatus = KEEP_ALIVE_END;
                    if(pConnHandler != NULL)
                    {
                        /* Send a callback message to the application layer for TIMEOUT */
                        Result = pConnHandler(pSourceID, MakeClassID(FmCONNECTION, GnCOMMAND, CONN_CMD_SpSHUTDOWN),
                                              NULL, pDefaultRtr) ? TRUE:FALSE;
                    }

                }
                case KEEP_ALIVE_STOPPED:
                    /* We have tried to evaluate while in a STOPPED state, so do
                    nothing and leave */
                default:
                    break;

            }
        }
    }
    ConnMutex.Release();    
    
    return Result;
}
/*================================================================================
CConObjSvr StartKeepAlive : Starts the KeepAlive and Timeout timers
================================================================================*/
boolean CConObjSvr::StartKeepAlive(char* pSourceID)
{
    boolean Result = FALSE;
    ConnObj* pConnObj = NULL;

    ConnMutex.Lock(INFINITE);

    pConnObj = GetConnObj(pSourceID);
    if (pConnObj)
    {
        if (pConnObj->TxOptions != CONN_CFG_SpKEEPALIVEOFF)
        {
            if(pConnObj->KeepAliveStatus == KEEP_ALIVE_STOPPED)
            {
                time_t NewTime = SYS_TIME(NULL);
                
                /* Reset the Timeout timer to activate on a normal interval */
                pConnObj->TimeoutTimer = NewTime;

                pConnObj->KeepAliveTimer = NewTime;

                /* Set the KeepAlive state to RUNNING */
                pConnObj->KeepAliveStatus = KEEP_ALIVE_RUNNING;
            }

            Result = TRUE;
        }
    }

    ConnMutex.Release();

    return Result;
}
/*================================================================================
CConObjSvr StopKeepAlive : Stops the KeepAlive and Timeout timers
================================================================================*/
boolean CConObjSvr::StopKeepAlive(char* pSourceID)
{   
    boolean Result = TRUE;
    ConnObj* pConnObj = NULL;

    ConnMutex.Lock(INFINITE);

    pConnObj = GetConnObj(pSourceID);
    if (pConnObj)
    {
        pConnObj->KeepAliveStatus = KEEP_ALIVE_STOPPED;
        pConnObj->KeepAliveTimer = 0;
    }

    ConnMutex.Release();
    
    return Result;
}

/*================================================================================
CConObjSvr SendMsgHandler: Sends a Connection message. 
================================================================================*/
boolean CConObjSvr::SendMsgHandler(char* pSourceID, uint16 Family, uint8 Genus, uint8 Species, 
                            uint8 *pBuffer, uint32 Size, boolean EncryptionOn)
{   
    boolean Result = FALSE;

    TOUCH(EncryptionOn);

    if (Family == FmCONNECTION)
    {
        Result = TxMsg(pSourceID, &MsgSvr, Genus, Species, pBuffer, Size);
    }

        
    return Result;
}
/*================================================================================
CTrapObjSvr RecvMsgHandler: Decodes the Family, Genus, and Species tag and deserializes
     the messages with the appropriate object.
================================================================================*/
boolean CConObjSvr::RecvMsgHandler(char* pSourceID, uint8 *pBuffer, void* pRouter)
{
    uint16  Family;
    uint8   Genus;
    uint8   Species;
    uint32  Result;
    uint32  Size;
    boolean RetVal=TRUE;
    time_t  NewTime = SYS_TIME(NULL);
    ConnObj* pConnObj = NULL;

    /* Extract the Family */
    Family = GetFamily(pBuffer);

    /* Extract the Genus */
    Genus = GetGenus(pBuffer);

    /* Extract the Species */
    Species = GetSpecies(pBuffer);

    /* Set the Genus */
    MsgSvr.set_Genus(Genus);

    /* Deserialize the data and check for errors */
    Result = MsgSvr.deserialize_all(pBuffer);


    pConnObj = GetConnObj(pSourceID);
    if (pConnObj)
    {
        if (Result)
        {
            /* Species Router */
            switch (Genus)
            {           
                /* If we are handling a request, then check for timeout, reset our timers,
                * and send back a Response message 
                */
                case GnREQUEST:
                {
                    switch(Species)
                    {
                        case CONN_CRST_SpKEEPALIVE:
                        {
#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
                            SYS_TRACE(TRACE_Type_RECEIVE, "CConObjSvr","RecvMsgHandler",
                                Family,Genus,Species,
                                "%s Received KeepAlive Request", pSourceID);
#endif
                            ConnMutex.Lock(INFINITE);
                            /*  We got a keepalive request, so the other end is alive. Update
                                our timer. */
                            pConnObj->TimeoutTimer = NewTime;
                            ConnMutex.Release();
                            /* Send the message */
                            Result = TxMsg(pSourceID, &MsgSvr, GnRESPONSE, CONN_CRSP_SpKEEPALIVE, NULL, 0);
                            break;
                        }
                        case CONN_CRST_SpGETCONNLIST:
                        {
#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
                            SYS_TRACE(TRACE_Type_RECEIVE, "CConObjSvr","RecvMsgHandler",
                                Family,Genus,Species,
                                "%s Received GetConnList Request", pSourceID);
#endif
                            ConnMutex.Lock(INFINITE);
                            SendConnList(pSourceID);
                            ConnMutex.Release();
                            break;
                        }
                    }/* switch(Species) */
                    break;
                }/* case GnREQUEST: */
                /* If we get a Response, then we know our Request has been acknowledged, so just
                * reset our timers. 
                */
                case GnRESPONSE:
                {
                    switch(Species)
                    {
                        case CONN_CRSP_SpKEEPALIVE:
                        {
#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
                            SYS_TRACE(TRACE_Type_RECEIVE, "CConObjSvr","RecvMsgHandler",
                                Family,Genus,Species,
                                "%s Received KeepAlive Response", pSourceID);
#endif
                            ConnMutex.Lock(INFINITE);
                            /* Check to see if our Timeout timer has expired*/
                            if(pConnObj->TimeoutInterval < (NewTime - pConnObj->TimeoutTimer))
                            {
                                /* We have gone too long without a KeepAlive response*/
                                pConnObj->KeepAliveStatus = KEEP_ALIVE_TIMEOUT;
                            }
                            else
                            {
                                /*  We got a keepalive request, so the other end is alive. Update
                                    our timer. */
                                pConnObj->TimeoutTimer = NewTime;
                            }    
                            ConnMutex.Release();
                            break;
                        }
                        case CONN_CRSP_SpPUTCONNLIST:
                        {
#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
                            SYS_TRACE(TRACE_Type_RECEIVE, "CConObjSvr","RecvMsgHandler",
                                Family,Genus,Species,
                                "%s Received ConnList Response", pSourceID);
#endif
                            /* Pass up to the Application */
                            if(pConnHandler != NULL)
                            {
                                /* Send a callback message to the application layer for handling connection list */
                                Result = pConnHandler(pSourceID, MakeClassID(Family, Genus, Species), MsgSvr.GetData(&Size), pRouter);
                            }

                            break;
                        }
                    }/* switch(Species) */
                    break;
                }/* case GnRESPONSE: */
                /* We received a TxOption message, so sync our settings with the sender'S */
                case GnCONFIG:
                {
                    ConnMutex.Lock(INFINITE);
                    pConnObj->TxOptions = Species;

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
                    /* Log the TxOptions we have received */
                    SYS_TRACE(TRACE_Type_STATUS, "CConObjSvr","RecvMsgHandler",
                        Family, Genus, Species,
                        "%s Syncing TxOptions to %d\n", pSourceID, pConnObj->TxOptions);
#endif
                    ConnMutex.Release();
                    break;
                }/* case GnCONFIG: */
                case GnCOMMAND:
                {
                    /* Handle shutdown message */
                    if(pConnHandler != NULL)
                    {
                        /* Send a callback message to the application layer for TIMEOUT */
                        Result = pConnHandler(pSourceID, MakeClassID(Family, Genus, Species), MsgSvr.GetData(&Size), pRouter);
                    }
                    break;
                }/* case GnCOMMAND: */
                default:
                {
                    /* We have received an unknown connection type */
#if defined(TRACE_SERVER_ON)
                SYS_TRACE(TRACE_Type_ERROR, "CConObjSvr", "RecvMsgHandler",
                    Family, Genus, Species, 
                    "%s Invalid Genus type", pSourceID);
#endif
                    break;
                }
            }/* switch (Species) */

        }/* if (Result) */
#if defined(TRACE_SERVER_ON)
        else
        {
            SYS_TRACE(TRACE_Type_ERROR, "CConObjSvr","RecvMsgHandler",
                Family, Genus, Species,
                "%s Deserialize_all() Failed. Could Not Sync TxOptions\n", pSourceID);
        }
#endif
    }/* if (pConnObj) */


    return RetVal;
}

/*================================================================================
CConObjSvr KeepAliveThread: Main thread for handling KeepAlives for the
    ConnObjSvr.
================================================================================*/
uint32 WACPAPI CConObjSvr::KeepAliveThread(void* pData)
{
    CConObjSvr*     pObject = (CConObjSvr*)pData;
    uint32          i;

    pObject->ExitMutex.Lock(INFINITE);

    /* Wait for the DataSem to fire for processing a message or until we time out.*/
    pObject->pTimeoutSem->Wait(CONNOBJSVR_POLLTIME);

    while(pObject->ThreadStatus == TRUE)
    {
        ConnArgs*   pConnArgs = NULL;
        char*       pSourceList[MAX_CONNECTIONS];

        /* Get a list of all entries in CLinkTable and check all of them individually */
        pObject->pCLinkTable->GetSourceList(pSourceList);
        for (i=0; i<MAX_CONNECTIONS; i++)
        {
            if (pSourceList[i] == NULL)
                break;

            /* Grab the ConnArgs from the CLinkTable to verify the connection exists */
            pConnArgs = pObject->pCLinkTable->GetEntry(pSourceList[i]);
            if (pConnArgs)
            {
                if (pConnArgs->bConnected == TRUE && pConnArgs->bSvrFlag == FALSE)
                {
                    /* Check for keepalive timeout */
                    if (pObject->StartKeepAlive(pSourceList[i]))
                    {
                        pObject->EvaluateKeepAlive(pSourceList[i]);
                    }
                }
            }
        }                

        /* Wait for the DataSem to fire for processing a message or until we time out.*/
        pObject->pTimeoutSem->Wait(CONNOBJSVR_POLLTIME);
    }
#if defined(DEBUG_PRINT)
    SYS_PRINTF("CConObjSvr::KeepAliveThread Ending\n");
#endif
    pObject->ExitMutex.Release();

    return TRUE;
}


/*================================================================================
CConObjSvr SendConnList: Creates a list of connection GUIDs and SourceIDs.
    ConnObjSvr.
================================================================================*/
boolean CConObjSvr::SendConnList(char* pSourceID) 
{
    boolean Result = TRUE;

    CConDExtLst ConnList;
    CConDStd    ConDStd;
    CRNDZDStd*  pRNDZDStd = NULL;
    uint8*      pGUID = NULL;
    uint16      GUIDSize = 0;
    uint8*      pObjBuffer = NULL;
    uint32      ObjBufSize = 0;
    char*       pSourceList[MAX_CONNECTIONS];
    uint32      i;
    ConnArgs*   pConnArgs = NULL;

    ConnList.InitConnectionList(0);

    /* Get a list of all entries in CLinkTable and check all of them individually */
    pCLinkTable->GetSourceList(pSourceList);
    for (i=0; i<MAX_CONNECTIONS; i++)
    {
        if (pSourceList[i] == NULL)
            break;
        
        /* Grab the ConnArgs from the CLinkTable to verify the connection exists */
        pConnArgs = pCLinkTable->GetEntry(pSourceList[i]);
        if (pConnArgs)
        {
            /* Check for valid connections */
            if ((pConnArgs->bConnected == TRUE) && (pConnArgs->bSvrFlag == FALSE) &&
                (strcmp(pConnArgs->SourceID, pSourceID) != 0))
            {
                /* Grab the CRNDZDStd object pointer to get the GUID */
                pRNDZDStd = (CRNDZDStd*)pConnArgs->pGUIDStd;
                if (pRNDZDStd)
                {
                    /* Grab the GUID */
                    pGUID = pRNDZDStd->GetDeviceGUID(&GUIDSize);

                    /* Set the GUID */
                    ConDStd.SetDeviceGUID(pGUID, GUIDSize);
                }

                /* Set the SourceID */
                ConDStd.SetSourceID(pConnArgs->SourceID, ADDR_BUF_SIZE);
                
                /*Prepare a buffer for the CConDStd object stream */
                ObjBufSize = ConDStd.size_all();
                pObjBuffer = (uint8*)SYS_REALLOC(pObjBuffer, ObjBufSize);
                if (pObjBuffer)
                {
                    /* Serialize the object */
                    if (ConDStd.serialize_all(pObjBuffer))
                    {
                        /* Add to Container */
                        Result = ConnList.AppendConnectionList(pObjBuffer, ObjBufSize, 1);
                    }
                    else
                    {
                        Result = FALSE;
                        break;
                    }
                }
                else
                {
                    Result = FALSE;
                    break;
                }
            }/* if (pConnArgs->bConnected == TRUE) */
        }/* if (pConnArgs) */
    }/* for (i=0; pSourceList[i] != NULL; i++) */     

    /* If we found valid connections and created a Connection list, send it out */
    if (Result)
    {
        ObjBufSize = ConnList.size_all();
        pObjBuffer = (uint8*)SYS_REALLOC(pObjBuffer, ObjBufSize);
        if (pObjBuffer)
        {
            /* Serialize the object */
            if (ConnList.serialize_all(pObjBuffer))
            {
                Result = TxMsg(pSourceID, &MsgSvr, GnRESPONSE, CONN_CRSP_SpPUTCONNLIST, pObjBuffer, ObjBufSize);
            }
        }
    }

    /* Free up the allocated buffer */
    if (pObjBuffer)
    {
        SYS_FREE(pObjBuffer);
    }

    return Result;
}

/* End of File */






