/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CDataObjSvr.cpp                                                   //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'****************************************************************************
'**
'**
'****************************************************************************
'****************************************************************************
'**
'*-E************************************************************************/

#include "CDataObjSvr.h"
#if defined(TRACE_SERVER_ON)
    #include "CTraceSvr.h"
#endif

/*================================================================================
CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
Default constructor
    MODULE Start up: All available servers are instantiated here. i.e Command Servers,
    Request Servers, Response Servers, etc.
*********************************************************************************/
CDataObjSvr::CDataObjSvr()
{
    /* Initialize the pointers */
    pMessageHandler = NULL;
    pOutbuf = NULL;
}
CDataObjSvr::~CDataObjSvr()
{
    SYS_FREE(pOutbuf);
    pOutbuf = NULL;
}

void CDataObjSvr::Init(CTrapObjSvr* pTrapObjSvr,
                       boolean (*SendPacket)(char*, char*, uint8*, uint32),
                       uint32 (*RxMessage)(char*, uint32, uint8*, void*),
                       LinkStruct* pLinkStruct)
{

    /* Initialize the generic message server used for incoming messages. */
    MsgSvrIn.Init(FmWACP, GnDATA, pLinkStruct);

    /* Initialize the generic message server used for outgoing messages. */
    MsgSvrOut.Init(FmWACP, GnDATA, pLinkStruct);

    /* Set the pointer to the Trap Object Server*/
    pTrapServer = pTrapObjSvr;

    /* Set the function pointers for sending data and connection handling */
    pSendMsg = SendPacket;
    pMessageHandler = RxMessage;

    /* Set the SemaphoreName */
    pSemName = pLinkStruct->pSemName;
    
    /* Set the pointer to the CLinkTable instance */
    pCLinkTable = (CLinkTable*)pLinkStruct->pCLinkTable;
}
/*================================================================================
CDataObjSvr TxMsg : Helper routine that transmits the contents of a CMsgSvr object
================================================================================*/
boolean CDataObjSvr::TxMsg(char* pSourceID, CMsgSvr* pMsgSvr, uint16 Family, 
                           uint8 Genus, uint8 Species, uint8* Payload, 
                           uint32 Size, boolean bEncryptionOn)
{
    uint32 OutBufSz;
    boolean Result = FALSE;

    /* Set the Family */
    pMsgSvr->set_Family(Family);

    /* Set the Genus */
    pMsgSvr->set_Genus(Genus);

    /* Set the Species */
    pMsgSvr->set_Species(Species);

#if defined(ENCRYPTION_ON)
    /* Set the Encryption status */
    pMsgSvr->SetEncryption(pSourceID, bEncryptionOn);
#else
    TOUCH(bEncryptionOn);
#endif

    /* Set the passed in Payload into the dynamic Data member variable */
    pMsgSvr->SetData(Payload, Size);

    /* Get the Size of the serialized byte Stream */
    OutBufSz = pMsgSvr->size_all();

    /* (Re)Allocate space for the serialized Buffer */
    pOutbuf = (uint8*)SYS_REALLOC(pOutbuf, OutBufSz);
    
    /* Serialize the message */
    if(pOutbuf != NULL)
    {
        /* Serialize the CErrSvr object */
        pMsgSvr->serialize_all(pOutbuf);

        /* Send the message */
        Result = pSendMsg(pSemName, pSourceID, pOutbuf, OutBufSz);
    }

    return(Result);
}
/*================================================================================
MODULE SendMsgHandler: Decodes the Family, Genus, and Species tag and deserializes
    the messages with the appropriate object.
================================================================================*/
boolean CDataObjSvr::SendMsgHandler(char* pSourceID, uint16 Family, uint8 Genus, uint8 Species, 
                uint8* pBuffer, uint32 Size, boolean bEncryptionOn)
{
    boolean RetVal = FALSE;

#if defined(TRACE_SERVER_ON_STATUS)
    SYS_TRACE(TRACE_Type_SEND, "CDataObjSvr", "SendMsgHandler",
                          Family, Genus, Species, "");
#endif

    if(Genus != GnDATA)
    {
        RetVal = TxMsg(pSourceID, &MsgSvrOut, Family, Genus, Species, pBuffer, Size, bEncryptionOn);
    }
    else
    {
        RetVal = pSendMsg(pSemName, pSourceID, pBuffer, Size);
    }

    return (RetVal);
}
/*================================================================================
MODULE RecvMsgHandler: Decodes the Family, Genus, and Species tag and deserializes
    the messages with the appropriate object.
================================================================================*/
boolean CDataObjSvr::RecvMsgHandler(char* pSourceID, uint8* pBuffer, void* pWACPApp)
{
    uint16  Family;
    uint8   Genus;
    uint8   Species;
    uint32  Result = TRUE;
    uint32  Size;
    uint32  ClassID;

    /* Extract the Family */
    Family = GetFamily(pBuffer);

    /* Extract the Genus */
    Genus = GetGenus(pBuffer);

    /* Extract the Species */
    Species = GetSpecies(pBuffer);

    ClassID = MakeClassID(Family, Genus, Species);

    if(Genus != GnDATA)
    {
        /* Set the MsgSvr object to the incoming Genus tag for deserialization */
        MsgSvrIn.set_Family(Family);
        MsgSvrIn.set_Genus(Genus);

#if defined(ENCRYPTION_ON)
        /* Update the CMsgSvr with the proper SourceID. The Encryption flag is ignored
         * for deserialization
         */
        MsgSvrIn.SetEncryption(pSourceID, TRUE);
#endif
        /* Deserialize the data and check for errors */
        Result = MsgSvrIn.deserialize_all(pBuffer);
        if(!Result)
        {
#if defined(TRACE_SERVER_ON)
            SYS_TRACE(TRACE_Type_ERROR, "CDataObjSvr", "RecvMsgHandler",
                      Family, Genus, Species, "MsgSvr deserialization failed");
#endif

            /* Call the trapserver with the error code stored in Result */
            pTrapServer->SendError(pSourceID, ClassID, CERR_CRC_MISMATCH, pWACPApp);
        }
    }

    if(Result)
    {
        /*Genus Router*/
        switch (Genus)
        {
            case GnDATA:
            {
                Result = pMessageHandler(pSourceID, ClassID, pBuffer, pWACPApp);
                break;
            }
            default:
            {
                Result = pMessageHandler(pSourceID, ClassID, MsgSvrIn.GetData(&Size), pWACPApp);
                break;
            }
        }/* switch (Genus)*/
    }

    return (Result ? TRUE : FALSE);
}
/* End of File */
