/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CLinkTable.cpp :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include <string.h>
#include "CLinkTable.h"
#include "SysUtil.h"

#if defined(TRACE_SERVER_ON)
    #include "CTraceSvr.h"
#endif

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor
    CLinkTable Start up: 
*********************************************************************************/
CLinkTable::CLinkTable()
{
    Count = 0;
    memset(Connection, 0, sizeof(ConnArgs*) * MAX_CONNECTIONS);
}


/*********************************************************************************
  Default destructor
    CLinkTable termination: 
*********************************************************************************/
CLinkTable::~CLinkTable()
{
    uint16   i;

    for (i=0; i < MAX_CONNECTIONS; i++)
    {
        Connection[i] = NULL;
    }
}

/*================================================================================
CLinkTable AddEntry: Adds the pointer to a ConnArgs into the static array if a
    free slot is found and the pSourceID does not match an existing pointer.
================================================================================*/
boolean CLinkTable::AddEntry(ConnArgs* pConnArgs)
{
    boolean     Result = FALSE;

    /* Check for a valid SourceID */
    if ((pConnArgs) && (pConnArgs->SourceID))
    {
        uint16      i;
        int32       Index;

        LinkMutex.Lock(INFINITE);

        /* Check for an existing entry */
        Index = GetEntryIndex(pConnArgs->SourceID);
        if (Index == -1)
        {
            /* Cycle through the list of ConnArgs pointers for an empty spot */
            for(i=0; i < MAX_CONNECTIONS; i++)
            {
                /* Look for an open spot */
                if (Connection[i] == NULL)
                {
                    /* Store a copy of the ConnArgs. */
                    Connection[i] = CopyConnArgs(pConnArgs);

                    /* Initialize the ConnArgs structure */
                    Connection[i]->bConnected = FALSE;
                    Connection[i]->bRNDZStatus = FALSE;
                    Connection[i]->ValidationState = 0;
                    Connection[i]->pKey = NULL;
                    Connection[i]->pGUIDStd = NULL;
                    Connection[i]->pByPassSrc = NULL;

                    /* Increment the count */
                    Count++;

#if defined(TRACE_SERVER_ON_STATUS)
                    SYS_PRINTF("CLinkTable::AddEntry: Added %s, Count = %d\n", Connection[i]->SourceID, Count);
#endif
                    Result = TRUE;
                    break;
                }
            }
#if defined(TRACE_SERVER_ON_STATUS)
            if (!Result)
                SYS_PRINTF("CLinkTable::AddEntry: ERROR MAX_CONNECTIONS %d\n", Count);
#endif
        }
        LinkMutex.Release();
    }
#if defined(TRACE_SERVER_ON_STATUS)
    else
        SYS_PRINTF("CLinkTable::AddEntry: Invalid SourceID\n");
#endif

    return (Result);
}

/*================================================================================
CLinkTable RemoveEntry: Searches the CLinkTable instance for a matching SourceID 
    and removes it from the list. 
================================================================================*/
boolean CLinkTable::RemoveEntry(char* pSourceID)
{
    boolean     Result = FALSE;

    LinkMutex.Lock(INFINITE);
    /* Check for a valid SourceID */
    int32 Index = GetEntryIndex(pSourceID);
    if (Index > -1)
    {
        /* Make sure we only look at valid instances */
        if (Connection[Index] != NULL)
        {
#if defined(TRACE_SERVER_ON_STATUS)
            SYS_PRINTF("CLinkTable::RemoveEntry: Removing %s, Count = %d\n", pSourceID, Count - 1);
#endif

            SYS_FREE(Connection[Index]->pKey);
            Connection[Index]->pKey = NULL;

            if (Connection[Index]->pGUIDStd)
            {
                delete(Connection[Index]->pGUIDStd);
                Connection[Index]->pGUIDStd = NULL;
            }

            Connection[Index]->pCommSvr = NULL;

            SYS_FREE(Connection[Index]->pConfig);
            SYS_FREE(Connection[Index]);
            Connection[Index] = NULL;
            Count--;
            Result = TRUE;

        }
    }
#if defined(TRACE_SERVER_ON_STATUS)

    if (!Result)
        SYS_PRINTF("CLinkTable::RemoveEntry: Unable to find %s for removal\n", pSourceID);
#endif
    LinkMutex.Release();


    return (Result);
}


/********************************************************************************************
CLinkTable GetEntryIndex - Helper routine to search the ConnArgs array for a matching 
    SourceID. This returns the 0 based index into the Connection array if an entry is found.
    It returns -1 if the entry is not found.
********************************************************************************************/
int32 CLinkTable::GetEntryIndex(char* pSourceID)
{
    uint16       i;
    uint16       TestCount = 0;
    int32        RetVal = -1;

    /* Check for a valid SourceID */
    if (pSourceID)
    {
        /* Cycle through the list of ConnArgs pointers for a match */
        for(i=0; i < MAX_CONNECTIONS; i++)
        {
            /* Stop the search once we have tested all existing connections */
            if (TestCount >= Count)
                break;

            /* Make sure we only look at valid instances */
            if (Connection[i] != NULL) 
            {
                TestCount++;

                /* If we find a matching SourceID, return a pointer to it */
                if (strcmp(pSourceID, Connection[i]->SourceID) == 0)
                {
                    RetVal = i;
                    break;
                }
            }
        }
    }

    return (RetVal);
}

/********************************************************************************************
CLinkTable GetEntry - Uses GetEntryIndex to return a pointer to the ConnArgs structure based
    on the SourceID passed in. This return NULL if an entry is not found.
********************************************************************************************/
ConnArgs* CLinkTable::GetEntry(char* pSourceID)
{
    ConnArgs*   pConnArgs = NULL;
    int32       Index = -1;

    /* Check for a valid SourceID */
    if (pSourceID)
    {
        Index = GetEntryIndex(pSourceID);
        if (Index != -1)
        {
            pConnArgs = Connection[Index];
        }
    }

    return (pConnArgs);
}

/********************************************************************************************
CLinkTable::GetSourceList - Returns a filled in array of SourceIDs of all existing entries in the 
    CLinkTable. The array passed in is expected to be a size of MAX_CONNECTION and will always  
    be cleared by this routine.
********************************************************************************************/
void CLinkTable::GetSourceList(char** pSourceList)
{
    uint16   i;
    uint16   TestCount = 0;

    if (pSourceList)
    {
        /* Always clear the passed in array */
        memset(pSourceList, 0, sizeof(char*) * MAX_CONNECTIONS);

        /* Check for any existing entries */
        if (Count)
        {
            /* Cycle through the list of ConnArgs pointers for a match */
            for(i=0; i < MAX_CONNECTIONS; i++)
            {
                /* Stop the search once we have tested all existing connections */
                if (TestCount >= Count)
                    break;

                /* Make sure we only look at valid instances */
                if (Connection[i] != NULL) 
                {
                    /* Save the SourceID */
                    pSourceList[TestCount] = Connection[i]->SourceID;

                    /* Increment the current count */
                    TestCount++;
                }
            }
        }
    }
}

/*================================================================================
CLinkTable CopyConnArg: Helper function that creates a new copy of the ConnArgs 
    structure.
================================================================================*/
ConnArgs* CLinkTable::CopyConnArgs(ConnArgs* pSrcConnArgs)
{
    ConnArgs* pDestConnArgs = NULL;

    if (pSrcConnArgs)
    {
        pDestConnArgs = (ConnArgs*)SYS_CALLOC(sizeof(ConnArgs), 1);
        if (pDestConnArgs)
        {
            /* Copy over the ConnArg structure */
            memmove(pDestConnArgs, pSrcConnArgs, sizeof(ConnArgs));
            
            if (pSrcConnArgs->ConfigSize)
            {
                pDestConnArgs->pConfig = (void*)SYS_CALLOC(pSrcConnArgs->ConfigSize, 1);
            }
            else
            {
                pDestConnArgs->pConfig = NULL;
            }

            /* Insure we have a valid pConfig pointer for both Src and Dest */
            if ((pSrcConnArgs->pConfig) && (pDestConnArgs->pConfig))
            {
                memmove(pDestConnArgs->pConfig, pSrcConnArgs->pConfig, pSrcConnArgs->ConfigSize);
            }
        }/* switch(pSrcConnArgs->StackType) */
    }/* if (pSrcConnArgs) */

    return(pDestConnArgs);
}

/* End of File */





