/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// SysUtil.cpp    :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "SysUtil.h"
#include "Target.h"


#if defined (WIN32)
#include <windows.h>
#include <process.h>
#include <time.h>

#if defined(TRACE_SERVER_ON)
    #include "CTraceSvr.h"
    
    CTraceSvr       TraceSvr;
#endif


#if defined(MEMORY_DEBUG)
#include "CWACPMutex.h"

#define MEMORY_TXT  "memory.txt"

static uint32   MallocCount;
FILE*           pMemFile;
CWACPMutex      FileMutex;

void* sys_malloc(MEMORY_POOL Pool, size_t Size, char* pName, int Line )
{   
    void* pTemp;
    TOUCH(Pool);
    
    pTemp = malloc(Size);
    if (!pMemFile)
    {
        pMemFile = fopen(MEMORY_TXT, "w+");
    }

    FileMutex.Lock(INFINITE);

    if (pMemFile)
    {
        fprintf(pMemFile, "sys_malloc\t %p\t %d\t %d\t %s\t Line %d\n", pTemp, ++MallocCount, Size, pName, Line);
        fflush(pMemFile);
    }
    FileMutex.Release();

    return pTemp;
}

void* sys_calloc(MEMORY_POOL Pool, size_t num, size_t Size, char* pName, int Line  )
{
    void* pTemp;
    TOUCH(Pool);
    
    if (!pMemFile)
    {
        pMemFile = fopen(MEMORY_TXT, "w+");
    }

    FileMutex.Lock(INFINITE);

    pTemp = calloc(num,Size);

    if (pMemFile)
    {
        fprintf(pMemFile, "sys_calloc\t %p\t %d\t %d\t %s\t Line %d\n", pTemp, ++MallocCount, Size, pName, Line);
        fflush(pMemFile);
    }
    FileMutex.Release();

    return pTemp;
}

void sys_free( MEMORY_POOL Pool, void* pMemBlock , char* pName, int Line )
{
    TOUCH(Pool);
    if (!pMemFile)
    {
        pMemFile = fopen(MEMORY_TXT, "w+");
    }

    FileMutex.Lock(INFINITE);

    if ((pMemFile) && (pMemBlock))
    {
        fprintf(pMemFile, "sys_free\t %p\t %d\t \t %s\t Line %d\n", pMemBlock, --MallocCount, pName, Line);
        fflush(pMemFile);
        free(pMemBlock);
    }
    FileMutex.Release();
}

void* sys_realloc( MEMORY_POOL Pool, void* pMemBlock, size_t Size, char* pName, int Line   )
{
    void* pTemp;

    TOUCH(Pool);

    if (!pMemFile)
    {
        pMemFile = fopen(MEMORY_TXT, "w+");
    }

    FileMutex.Lock(INFINITE);

    if ((pMemBlock) && (pMemFile))
    {
        fprintf(pMemFile, "clr_free\t %p\t %d\t \t %s\t Line %d\n", pMemBlock, MallocCount, pName, Line);
    }
    else
    {
        MallocCount++;
    }

    pTemp = realloc(pMemBlock, Size);

    if (pMemFile) 
    {
        fprintf(pMemFile, "sys_realloc\t %p\t %d\t %d\t %s\t Line %d\n", pTemp, MallocCount, Size, pName, Line);
        fflush(pMemFile);
    }
    FileMutex.Release();

    return pTemp;
}
#else
void* sys_malloc( MEMORY_POOL Pool, size_t Size)
{   
    TOUCH(Pool);
    return malloc(Size);
}

void* sys_calloc( MEMORY_POOL Pool, size_t num, size_t Size)
{
    TOUCH(Pool);
    return calloc(num,Size);
}

void sys_free( MEMORY_POOL Pool, void* pMemBlock)
{
    TOUCH(Pool);
    if (pMemBlock)
    {
       free(pMemBlock);
       pMemBlock = NULL;
    }
}

void* sys_realloc(MEMORY_POOL Pool, void* pMemBlock, size_t Size)
{
    TOUCH(Pool);
    return realloc(pMemBlock, Size);
}

#endif

void  sys_sleep(uint32 Timeout)
{
    Sleep(Timeout);
}

int sys_getpid(void)
{
    return (_getpid());
}

time_t sys_time(time_t* pTimer)
{
    return (time(pTimer));
}

/*
 * Abstractions for C pseudorandom number functions.
 *
 * void srand( unsigned int seed );
 * int rand( void );
 */

/* Set a random starting point. */
void sys_srand(uint32 Seed)
{
    srand(Seed);
}

/* Generate and return a pseudorandom number. */
int sys_rand(void)
{
    return(rand());
}

/* Abstraction for printf function*/
int sys_printf(const char* pFormat, ...)
{
    va_list(vargs);
    va_start(vargs, pFormat);

    return vprintf(pFormat, vargs);
}

#if defined(TRACE_SERVER_ON)
/* Abstraction for trace function*/
boolean sys_trace(uint16 TraceType, 
              ansichar* ClassName, ansichar* FunctionName,
              uint16 Family, uint8 Genus, uint8 Species, 
              ansichar* DescriptionFormat, ...)
{
    va_list(vList);

    va_start(vList, DescriptionFormat);

    TraceSvr.tprintfEx((TRACE_TYPES)TraceType, ClassName, FunctionName, Family, Genus, Species,
                     DescriptionFormat, vList);

    va_end(vList);
    return TRUE;

}
#endif

/* Abstraction for initialization function*/
boolean sys_init()
{
#if defined(TRACE_SERVER_ON)
    //================================================================
    /* Open the Trace file here so we can start writing to it */
    TraceSvr.OpenFile(TRACE_WACP_LOG, "");
    TraceSvr.GenerateXSLFile();
    TraceSvr.EnableLogging(TRUE, TRUE);
    //================================================================
#endif

    return TRUE;
}

/* Abstraction for destroy function*/
boolean sys_destroy()
{
    return TRUE;
}

#endif
/* End of File */





