/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CIOCP.cpp      :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CIOCP.h"
#include <assert.h>

/*-D*************************************************************************
*
*  CIOCP constructor
*
*  PURPOSE: Create an I/O completion port that is not associated with a file handle
*
*  ARGUMENTS:
*      [IN] NumberOfConcurrentThreads = Maximum number of threads that the
*           operating system can allow to concurrently process I/O completion
*           packets for the I/O completion port.
*
*  RETURNS: none
*
**-E************************************************************************/
CIOCP::CIOCP(DWORD NumberOfConcurrentThreads)
{
	m_hIOcp = CreateIoCompletionPort(INVALID_HANDLE_VALUE, NULL, 0, NumberOfConcurrentThreads);
	assert(m_hIOcp);
}

/*-D*************************************************************************
*
*  CIOCP destructor
*
*  PURPOSE: Close the handle to free the completion port
*
*  ARGUMENTS: none
*
*  RETURNS: none
*
**-E************************************************************************/
CIOCP::~CIOCP()
{
	CloseHandle(m_hIOcp);
}

/*-D*************************************************************************
*
*  CIOCP::Associate
*
*  PURPOSE: Associates an input/output (I/O) completion port with a file handle
*
*  ARGUMENTS:
*      [IN] FileHandle = Handle to a file opened for overlapped I/O completion.
*           The file must be opened with the overlapped flag specified.
*      [IN] CompletionKey = Per-file completion key that is included in every
*           I/O completion packet for the specified file.
*
*  RETURNS: ERROR_SUCCESS, or standard WIN32 API error.
*
*  NOTES:  When a file/socket handle is associated to an I/O completion port,
*          the I/O system sends a completion notification packet to the
*          completion port when the I/O operation completes. It is normal
*          for the read or write function to return while the operation is
*          still outstanding with an error of ERROR_IO_PENDING.
**-E************************************************************************/
DWORD CIOCP::Associate(HANDLE FileHandle, ULONG_PTR CompletionKey)
{
	return CreateIoCompletionPort(FileHandle, m_hIOcp, CompletionKey, 0) ? ERROR_SUCCESS : GetLastError();
}

/*-D*************************************************************************
*
*  CIOCP::GetHandle
*
*  PURPOSE: Read-only accessor to internal completion port handle
*
*  ARGUMENTS: none
*
*  RETURNS: HANDLE to completion port
*
**-E************************************************************************/
HANDLE CIOCP::GetHandle()
{
	return m_hIOcp;
}

/*-D*************************************************************************
*
*  CIOCP::Enqueue
*
*  PURPOSE: Posts an I/O completion packet to an I/O completion port
*
*  ARGUMENTS:
*      [IN] NumberOfBytesTransferred = Value to be returned through the
*           pNumberOfBytesTransferred parameter of the Dequeue function.
*      [IN] CompletionKey = Value to be returned through the pCompletionKey
*           parameter of the Dequeue function.
*      [IN] pOverlapped = Value to be returned through the ppOverlapped
*           parameter of the GetQueuedCompletionStatus function.
*
*  RETURNS: ERROR_SUCCESS, or standard WIN32 API error.
*
**-E************************************************************************/
DWORD CIOCP::Enqueue(DWORD NumberOfBytesTransferred, ULONG_PTR CompletionKey, LPOVERLAPPED pOverlapped)
{
	return PostQueuedCompletionStatus(m_hIOcp, NumberOfBytesTransferred, CompletionKey, pOverlapped) ? ERROR_SUCCESS : GetLastError();
}


/*-D*************************************************************************
*
*  CIOCP::Dequeue
*
*  PURPOSE: Dequeue an I/O completion packet from the specified I/O completion port
*
*  ARGUMENTS:
*     [OUT] pNumberOfBytesTransferred = Pointer to a variable that receives the
*           number of bytes transferred during an I/O operation that has completed.
*     [OUT] pCompletionKey = Pointer to a variable that receives the completion
*           key value associated with the file handle whose I/O operation has completed.
*     [OUT] ppOverlapped = Pointer to a variable that receives the address of
*           the OVERLAPPED structure that was specified when the completed I/O
*           operation was started.
*      [IN] Timeout = Number of milliseconds that the caller is willing to wait
*           for an completion packet to appear at the completion port.
*
*  RETURNS: ERROR_SUCCESS, or standard WIN32 API error.
*
**-E************************************************************************/
DWORD CIOCP::Dequeue(LPDWORD pNumberOfBytesTransferred, PULONG_PTR pCompletionKey, LPOVERLAPPED* ppOverlapped, DWORD Timeout)
{
	return GetQueuedCompletionStatus(m_hIOcp, pNumberOfBytesTransferred, pCompletionKey, ppOverlapped, Timeout) ? ERROR_SUCCESS : GetLastError();
}
