/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*	Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// COverlappedTransport.cpp:                                         //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include <assert.h>
#include <process.h>
#include "COverlappedTransport.h"
#include "SysUtil.h"

//globals
const HANDLE g_hHeap = GetProcessHeap();

//locals
static CIOCP *l_pIoCp = NULL;
static int l_iNumThreads = 0;
static PHANDLE l_phThreads = NULL;
static volatile LONG l_OLRefCount = 0;

/*-D*************************************************************************
*
*  COverlappedTransport default constructor
*
*  PURPOSE: Initialize: On first reference, creates completion port and starts threads.
*
*  ARGUMENTS: none
*
*  RETURNS: none
*
**-E************************************************************************/
COverlappedTransport::COverlappedTransport()
{
	SYSTEM_INFO si;

	/* only create when no references */
	if (0 == l_OLRefCount && NULL == l_pIoCp)
	{
		//Create completion port
		l_pIoCp = new CIOCP(NUM_OVERLAPPED_THREADS);
		assert(l_pIoCp);

		l_iNumThreads = NUM_OVERLAPPED_THREADS;
		if (l_iNumThreads == 0)
		{	//Use thread per processor
			GetSystemInfo(&si);
			l_iNumThreads = si.dwNumberOfProcessors;
		}
		l_phThreads = new HANDLE[l_iNumThreads];
		assert(l_phThreads);
		//Start threads for completion port
		for (int i=0; i<l_iNumThreads; i++)
		{
			l_phThreads[i] = (HANDLE)_beginthreadex(NULL, 0, CompletionPortThreadProc, this, 0, NULL);
		}
	}
	InterlockedIncrement(&l_OLRefCount);
}

/*-D*************************************************************************
*
*  COverlappedTransport destructor
*
*  PURPOSE: On last reference provide ordered shutdown and thorough cleanup
*
*  ARGUMENTS: none
*
*  RETURNS: none
*
**-E************************************************************************/
COverlappedTransport::~COverlappedTransport()
{
	if (InterlockedDecrement(&l_OLRefCount) == 0 && l_pIoCp) //destroy on last reference
	{
		//Signal all threads on completion port to stop
		for (int i=0; i<l_iNumThreads; i++)
		{
			l_pIoCp->Enqueue(0, (ULONG_PTR)OVERLAPPED_TRANSPORT_STOP, NULL);
		}
		//Wait for all threads to complete
		WaitForMultipleObjects(l_iNumThreads, l_phThreads, TRUE, INFINITE);
		//Close completion port
		delete l_pIoCp;
		l_pIoCp = NULL;
		//Free thread handles
		for (int i=0; i<l_iNumThreads; i++)
		{
			if (l_phThreads[i])
			{
				CloseHandle(l_phThreads[i]);
				l_phThreads[i] = NULL;
			}
		}
		delete l_phThreads;
		l_phThreads = NULL;
	}
}

/*-D*************************************************************************
*
*  COverlappedTransport::CompletionPortThreadProc
*
*  PURPOSE: Worker thread for processing network client requests on completion port
*
*  ARGUMENTS:
*      [IN] pArgs = Reserved for future use.
*
*  RETURNS: ERROR_SUCCESS, or standard WIN32 API error.
*
*  NOTES:   Serve the IOCP for processing client data. Each worker thread processes
*           independently per overlapped, using GetQueuedCompletionStatus()
*           to signal client requests. This function waits in the kernel for
*           overlapped I/O that has been queued to the IOCP. When the I/O is
*           dequeued, it allows for further processing. The thread is exited
*           upon receiving a stop cookie for the completion key.
**-E************************************************************************/
unsigned int COverlappedTransport::CompletionPortThreadProc(void* pArgs)
{
	DWORD Result = ERROR_SUCCESS;
	DWORD Numbytes;
	ULONG_PTR Key;
	LPOVERLAPPED pOL;
	COverlappedTransport *pOT;

	for(;;)
	{
		Result = l_pIoCp->Dequeue(&Numbytes, &Key, &pOL, INFINITE);
		pOT = (COverlappedTransport*)Key;
		if (Result == ERROR_SUCCESS)
		{
			if (OVERLAPPED_TRANSPORT_STOP == Key)
			{
				break;
			}
			if (pOT)
			{
				pOT->ProcessCP(Numbytes, pOL);
			}
		}
		else
		{
			pOT->RecvCPError(Result, pOL);
		}
	}
	return Result;
	UNREFERENCED_PARAMETER(pArgs);
}

/*-D*************************************************************************
*
*  COverlappedTransport::Assign
*
*  PURPOSE: Associate overlapped handle with completion port
*
*  ARGUMENTS:
*      [IN] OverlappedFileHandle = any handle opened as overlapped.
*      [IN] Cookie = per file key that is included in every I/O completion packet
*
*  RETURNS: ERROR_SUCCESS, or standard WIN32 API error.
*
**-E************************************************************************/
DWORD COverlappedTransport::Assign(HANDLE OverlappedFileHandle, ULONG_PTR Cookie)
{
	return l_pIoCp->Associate(OverlappedFileHandle, Cookie);
}
