/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CTCPCommSvr.cpp:                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include <assert.h>
#include "CTCPCommSvr.h"

#define DELIM ":"
#define EX_ADDRLEN (sizeof(SOCKADDR_IN) + 16) //required for MS-extended Winsock functions

/*-D*************************************************************************
*
*  CTCPCommSvr default constructor
*
*  PURPOSE: Initialize
*
*  ARGUMENTS: none
*
*  RETURNS: none
*
**-E************************************************************************/
CTCPCommSvr::CTCPCommSvr()
{
    Construct();
}

/*-D*************************************************************************
*
*  CTCPCommSvr overloaded constructor
*
*  PURPOSE: Initialize with option to specify whether server instance.
*
*  ARGUMENTS:
*      [IN] Server = If true, a server listening instance will be created.
*                    When false, a stand-alone client or client session on a
*                    server will be created.
*
*  RETURNS: none
*
**-E************************************************************************/
CTCPCommSvr::CTCPCommSvr(bool Server)
{
    Construct(Server);
}

/*-D*************************************************************************
*
*  CTCPCommSvr destructor
*
*  PURPOSE: Provide ordered shutdown and thorough cleanup of transport instance
*
*  ARGUMENTS: none
*
*  RETURNS: none
*
**-E************************************************************************/
CTCPCommSvr::~CTCPCommSvr()
{
    while(m_NumOverlappedWrites)
    {
        Sleep(0);
    }
    if (m_Socket)
    {
        closesocket(m_Socket);
        m_Socket = NULL;
    }
    //Wait for any outstanding reads to complete:
    // They will acceptably fail after closing the handle with Error 995:
    // The I/O operation has been aborted because of either a thread exit or an application request.
    while (m_NumOverlappedReads)
    {
        Sleep(0);
    }
    WSACleanup();
    if (!m_bServer)
    {
        DeleteCriticalSection(&m_CS);
    }
}

/*-D*************************************************************************
*
*  CTCPCommSvr::Construct
*
*  PURPOSE: Helper function for constructors. Initializes Winsock and all
*           private members.
*
*  ARGUMENTS:
*      [IN] Server = If true, a server listening instance will be created.
*                    When false, a stand-alone client or client session on a
*                    server will be created.
*
*  RETURNS: none
*
**-E************************************************************************/
void CTCPCommSvr::Construct(bool Server)
{
    WSADATA WSAData;

    WSAStartup(MAKEWORD(2, 2), &WSAData);
    m_bServer = Server;
    m_bClientMode = false;
    m_bSetConfig = false;
    m_Socket = NULL;
    m_NumOverlappedReads = 0;
    m_NumOverlappedWrites = 0;
    m_SocketClosed = FALSE;
    if (!Server)
    {
        InitializeCriticalSection(&m_CS);
    }
    this->StackType = ETHERNET;
}

/*-D*************************************************************************
*
*  CTCPCommSvr::SetConfig
*
*  PURPOSE: Updates addressing information for socket
*
*  ARGUMENTS:
*      [IN] pAddr = A TCPAddr structure with session information.
*           Required for internal session instances. Must be NULL for server
*           or dedicated client instance.
*
*  RETURNS: none
*
*  NOTES:  As this function cannot return a value, an error is stored in
*          the instance and later checked in Connect or StartServer function.
*
*          The format of the SourceID must be: "[Transport:]IP:port"
*          IP must be a Null-terminated character string representing a number
*          expressed in the IPv4 standard "." (dotted) notation. Server instance
*          may specify "*" for the IP address to listen on any interface,
*          which may be useful on a multi-homed host. IP is required for client
*          instance, but may be blank for server listener. A blank IP will
*          result in any appropriate network address being used, resulting in
*          the same behavior as if specifying "*".
*          Port must be in the range of 1-65535. If port is not specified,
*          the default rendezvous port is used.
*          Transport must be "TCP". For server instance, Transport specifier
*          is required.
**-E************************************************************************/
void CTCPCommSvr::SetConfig(void *pAddr)
{
    TCPAddr* pConfig = (TCPAddr*)pAddr;
    char ConnectionString[ADDR_BUF_SIZE];
    char *pTmp;
    int Port;

    if (!m_bServer)
    {
        if (pConfig)
        {
            m_Socket = pConfig->hSocket;
            if (NULL == m_Socket)
            {
                m_bSetConfig = false;
                return;
            }
        }
        else
        {
            m_bClientMode = true;
        }
    }
    if (m_bServer || m_bClientMode) //server or true client mode (not client session of server)
    {
        //Initialize default parameters
        memset(&m_addr, 0, sizeof(m_addr));
        m_addr.sin_family = AF_INET;
        m_addr.sin_addr.S_un.S_addr = htonl(INADDR_ANY);
        m_addr.sin_port = htons(this->Port);

        if (this->SourceID)
        {
            strcpy(ConnectionString, this->SourceID);
            pTmp = strtok(ConnectionString, DELIM);
            if (pTmp)
            {
                if (0 == _stricmp(pTmp, TCP_SOURCE_PREFIX))
                {
                    pTmp = strtok(NULL, DELIM);
                    if (NULL == pTmp && m_bClientMode)
                    {
                        m_bSetConfig = false;
                        return;
                    }
                }
                if (pTmp)
                {
                    m_addr.sin_addr.S_un.S_addr = inet_addr(pTmp);
                    if (INADDR_NONE == m_addr.sin_addr.S_un.S_addr)
                    {
                        if (0 == strcmp(pTmp, "*") && m_bServer)
                        {
                            m_addr.sin_addr.S_un.S_addr = htonl(INADDR_ANY);
                        }
                        else
                        {
                            m_bSetConfig = false;
                            return;
                        }
                    }
                    pTmp = strtok(NULL, DELIM);
                    if (pTmp)
                    {
                        Port = atoi(pTmp);
                        if (Port > 0 && Port <= USHRT_MAX)
                        {
                            m_addr.sin_port = htons((u_short)Port);
                        }
                        else
                        {
                            m_bSetConfig = false;
                            return;
                        }
                    }
                }
            }
            else if (!m_bServer)
            {
                m_bSetConfig = false;
                return;
            }
        }
    }
    m_bSetConfig = true;
}

/*-D*************************************************************************
*
*  CTCPCommSvr::SendMsg
*
*  PURPOSE: Sends data from the current transport instance to the remote end
*
*  ARGUMENTS:
*      [IN] pBuf = Pointer to a buffer that contains the data to be sent.
*      [IN] BufSize = Size of the buffer pointed to by pBuf.
*
*  RETURNS: TRUE if successful, otherwise FALSE.
*
**-E************************************************************************/
boolean CTCPCommSvr::SendMsg(uint8 *pBuf, uint32 BufSize)
{
    boolean bRet = FALSE;
    int Result;
    PTCP_PACKET pPacket = NULL;
    DWORD dwBytes;

    if (NULL == pBuf) //validate params
    {
        return FALSE;
    }
    pPacket = (PTCP_PACKET)HeapAlloc(g_hHeap, HEAP_ZERO_MEMORY, sizeof(TCP_PACKET));
    if (NULL == pPacket)
    {
        return FALSE;
    }
    pPacket->Wsa.buf = (char*)HeapAlloc(g_hHeap, 0, BufSize);
    if (NULL == pPacket->Wsa.buf)
    {
        HeapFree(g_hHeap, 0, pPacket);
        return FALSE;
    }
    memcpy(pPacket->Wsa.buf, pBuf, BufSize);
    pPacket->Wsa.len = BufSize;
    pPacket->State = OVERLAPPED_WRITE;
    InterlockedIncrement(&m_NumOverlappedWrites);
    Result = WSASend(m_Socket, &pPacket->Wsa, 1, &dwBytes, 0, &pPacket->Ol, NULL);
    if (SOCKET_ERROR == Result)
    {
        switch (WSAGetLastError())
        {
            case ERROR_IO_PENDING:
            {
                bRet = TRUE; //reset return boolean (this is expected).
                break;
            }
            default:
            {
                HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
                HeapFree(g_hHeap, 0, pPacket);
                InterlockedDecrement(&m_NumOverlappedWrites);
                break;
            }
        }
    }
    else if (ERROR_SUCCESS == Result)
    {
        bRet = TRUE;
    }

    return bRet;
}

/*-D*************************************************************************
*
*  CTCPCommSvr::Connect
*
*  PURPOSE: Creates a socket for clients that connects to the IP passed in the
*            connection string.
*           Assigns socket to the completion port, and posts receives.
*           Updates the WACP Port and SourceID once a connection is created.
*
*  ARGUMENTS:
*      [IN] pSourceID = String identifier(IP address) of the connection.
*      [IN] PortNum = Port value for the socket connection.
*
*  RETURNS: TRUE if successful, otherwise FALSE.
*
**-E************************************************************************/
boolean CTCPCommSvr::Connect(uint8 *pSourceID, uint16 PortNum)
{
    boolean bRet = FALSE;

    //Don't allow servers to Connect
    if (m_bServer || !m_bSetConfig)
    {
        return FALSE;
    }
    if (m_bClientMode)
    {
        m_Socket = WSASocket(AF_INET, SOCK_STREAM, IPPROTO_TCP, NULL, 0, WSA_FLAG_OVERLAPPED);
        if (INVALID_SOCKET != m_Socket)
        {
            if (SOCKET_ERROR == connect(m_Socket, (LPSOCKADDR)&m_addr, sizeof(m_addr)))
            {
                closesocket(m_Socket);
                m_Socket = NULL;
                return FALSE;
            }
        }
    }
    if (ERROR_SUCCESS == this->Assign((HANDLE)m_Socket, (ULONG_PTR)this))
    {
        if (ERROR_SUCCESS == PostReceive(NULL, TCP_MAX_OVERLAPPED_READS))
        {
            bRet = TRUE;
        }
    }
    if (!bRet)
    {
        closesocket(m_Socket);
        m_Socket = NULL;
    }

    return bRet;
    UNREFERENCED_PARAMETER(PortNum);
    UNREFERENCED_PARAMETER(pSourceID);
}

/*-D*************************************************************************
*
*  CTCPCommSvr::StartServer
*
*  PURPOSE: Creates a listener socket for server that binds to the IP passed
*            in the connection string.
*           Assigns socket to the completion port, and builds accept sockets.
*
*  ARGUMENTS: none
*
*  RETURNS: ERROR_SUCCESS, or standard Winsock API error.
*
**-E************************************************************************/
uint32 CTCPCommSvr::StartServer()
{
    uint32 Result = ERROR_SUCCESS;

    //Don't allow clients to listen
    if (!m_bServer)
    {
        return ERROR_INVALID_FUNCTION;
    }
    //Check previous failure
    if (!m_bSetConfig)
    {
        return ERROR_BAD_ARGUMENTS;
    }
    m_Socket = WSASocket(AF_INET, SOCK_STREAM, IPPROTO_TCP, NULL, 0, WSA_FLAG_OVERLAPPED);
    if (INVALID_SOCKET == m_Socket)
    {
        m_Socket = NULL;
        return WSAGetLastError();
    }
    Result = bind(m_Socket, (LPSOCKADDR)&m_addr, sizeof(m_addr));
    if (SOCKET_ERROR == Result)
    {
        closesocket(m_Socket);
        m_Socket = NULL;
        return WSAGetLastError();
    }
    Result = listen(m_Socket, SOMAXCONN);
    if (SOCKET_ERROR == Result)
    {
        closesocket(m_Socket);
        m_Socket = NULL;
        return WSAGetLastError();
    }
    Result = this->Assign((HANDLE)m_Socket, (ULONG_PTR)this);
    if (ERROR_SUCCESS == Result)
    {
        Result = BuildAccepts(m_Socket, TCP_ACCEPT_SOCKETS);
    }
    else
    {
        closesocket(m_Socket);
        m_Socket = NULL;
    }

    return Result;
}

/*-D*************************************************************************
*
*  CTCPCommSvr::ProcessCP
*
*  PURPOSE: Callback to process queued I/O from completion port
*
*  ARGUMENTS:
*      [IN] dwBytes = number of bytes processed from the overlapped handle.
*      [IN] lpOl = Pointer to overlapped plus structure for network data.
*
*  RETURNS: none
*
**-E************************************************************************/
void CTCPCommSvr::ProcessCP(DWORD dwBytes, LPOVERLAPPED lpOl)
{
    PTCP_PACKET pPacket = (PTCP_PACKET)lpOl;
    TCPAddr addr;
    char cSourceID[ADDR_BUF_SIZE];
    LPSOCKADDR_IN pLocalAddr, pRemoteAddr;
    INT LocalAddrLen, RemoteAddrLen;
    LPFN_GETACCEPTEXSOCKADDRS GetAcceptExSockaddrs = NULL;
    GUID GuidGetAcceptExSockaddrs = WSAID_GETACCEPTEXSOCKADDRS;

    if (pPacket)
    {
        if (m_bServer)
        {
            switch (pPacket->State)
            {
                case OVERLAPPED_INIT:
                {
                    if (ERROR_SUCCESS == WSAIoctl(m_Socket, SIO_GET_EXTENSION_FUNCTION_POINTER, &GuidGetAcceptExSockaddrs, sizeof(GuidGetAcceptExSockaddrs), &GetAcceptExSockaddrs, sizeof(GetAcceptExSockaddrs), &dwBytes, NULL, NULL))
                    {
                        GetAcceptExSockaddrs(pPacket->Wsa.buf, 0, EX_ADDRLEN, EX_ADDRLEN, (LPSOCKADDR*)&pLocalAddr, &LocalAddrLen, (LPSOCKADDR*)&pRemoteAddr, &RemoteAddrLen);
                        sprintf(cSourceID, "%s%s%s%s%hu", TCP_SOURCE_PREFIX, DELIM, inet_ntoa(pRemoteAddr->sin_addr), DELIM, ntohs(pRemoteAddr->sin_port));
                        addr.hSocket = pPacket->hSocket;
                        this->Arrival(cSourceID, ETHERNET, &addr, sizeof(addr));
                    }
                    BuildAccepts(m_Socket, 1);
                    InterlockedDecrement(&m_NumOverlappedReads);
                    break;
                }
                default:
                {
                    assert(0); //should not get here
                    break;
                }
            }
            HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
            HeapFree(g_hHeap, 0, pPacket);
        }
        else
        {
            switch (pPacket->State)
            {
                case OVERLAPPED_WRITE:
                {
                    HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
                    HeapFree(g_hHeap, 0, pPacket);
                    InterlockedDecrement(&m_NumOverlappedWrites);
                    break;
                }
                case OVERLAPPED_READ:
                {
                    if (dwBytes > 0)
                    {
                        EnterCriticalSection(&m_CS);
                        this->WACPParserEx((uint8*)pPacket->Wsa.buf, dwBytes, NULL);
                        LeaveCriticalSection(&m_CS);
                        PostReceive(pPacket);
                    }
                    else
                    {// If the connection has been closed, shutdown this instance
                        HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
                        HeapFree(g_hHeap, 0, pPacket);
                        if (!InterlockedExchange(&m_SocketClosed, TRUE))
                        {
                            this->Removal();
                        }
                    }
                    InterlockedDecrement(&m_NumOverlappedReads);
                    break;
                }
                default:
                {
                    assert(0); //should not get here
                    HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
                    HeapFree(g_hHeap, 0, pPacket);
                    break;
                }
            }
        }
    }
    return;
}

/*-D*************************************************************************
*
*  CTCPCommSvr::RecvCPError
*
*  PURPOSE: Receive error callback from I/O completion port
*
*  ARGUMENTS:
*      [IN] LastError = standard WIN32 error of reason for overlapped failure.
*      [IN] lpOl = Pointer to overlapped plus structure for network data.
*
*  RETURNS: none
*
**-E************************************************************************/
void CTCPCommSvr::RecvCPError(DWORD LastError, LPOVERLAPPED lpOl)
{
    PTCP_PACKET pPacket = (PTCP_PACKET)lpOl;
    int state;

    if (pPacket)
    {
        state = pPacket->State;
        HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
        HeapFree(g_hHeap, 0, pPacket);
        switch (state)
        {
            case OVERLAPPED_WRITE:
            {
                InterlockedDecrement(&m_NumOverlappedWrites);
                break;
            }
            case OVERLAPPED_READ:
            case OVERLAPPED_INIT:
            {
                InterlockedDecrement(&m_NumOverlappedReads);
                break;
            }
            default:
            {
                assert(0);
                break;
            }
        }
    }
    return;
    UNREFERENCED_PARAMETER(LastError);
}

/*-D*************************************************************************
*
*  CTCPCommSvr::PostReceive
*
*  PURPOSE: Helper function for posting receives for network client that will
*           complete when incoming data is available.
*
*  ARGUMENTS:
*      [IN] pPacket = Pointer to overlapped plus structure for network data.
*      [IN] Count = number of receives to post.
*
*  RETURNS: ERROR_SUCCESS, or standard Winsock API error.
*
**-E************************************************************************/
DWORD CTCPCommSvr::PostReceive(PTCP_PACKET pPacket, UINT Count)
{
    DWORD Result = ERROR_SUCCESS;
    PTCP_PACKET pTempPacket = pPacket;
    DWORD dwBytes;
    DWORD dwFlags = 0;

    if (pPacket && Count > 1)
    {
        return ERROR_INVALID_FUNCTION;
    }
    for (UINT i=0; i<Count; i++)
    {
        if (NULL == pTempPacket)
        {
            pPacket = (PTCP_PACKET)HeapAlloc(g_hHeap, HEAP_ZERO_MEMORY, sizeof(TCP_PACKET));
            if (NULL == pPacket)
            {
                Result = ERROR_OUTOFMEMORY;
                break;
            }
            pPacket->Wsa.buf = (char*)HeapAlloc(g_hHeap, 0, TCP_RCVBUFSIZE);
            if (NULL == pPacket->Wsa.buf)
            {
                HeapFree(g_hHeap, 0, pPacket);
                Result = ERROR_OUTOFMEMORY;
                break;
            }
            pPacket->Wsa.len = TCP_RCVBUFSIZE;
        }
        else if (pPacket->Wsa.len != TCP_RCVBUFSIZE || NULL == pPacket->Wsa.buf)
        {	//should never get here, fail safe
            assert(0);
            memset(&pPacket->Ol, 0, sizeof(pPacket->Ol));
            pPacket->Wsa.buf = (char*)HeapReAlloc(g_hHeap, 0, pPacket->Wsa.buf, TCP_RCVBUFSIZE);
            if (NULL == pPacket->Wsa.buf)
            {
                HeapFree(g_hHeap, 0, pPacket);
                Result = ERROR_OUTOFMEMORY;
                break;
            }
            pPacket->Wsa.len = TCP_RCVBUFSIZE;
        }
        pPacket->State = OVERLAPPED_READ;
        InterlockedIncrement(&m_NumOverlappedReads);
        Result = WSARecv(m_Socket, &pPacket->Wsa, 1, &dwBytes, &dwFlags, &pPacket->Ol, NULL);
        if (SOCKET_ERROR == Result)
        {
            Result = WSAGetLastError();
            switch (Result)
            {
            case WSA_IO_PENDING: //reset this error OK
                Result = ERROR_SUCCESS;
                break;
            default:
#if defined(TRACE_SERVER_ON)
                SYS_PRINTF("Socket Error: %u\n", Result);
#endif
                HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
                HeapFree(g_hHeap, 0, pPacket);
                InterlockedDecrement(&m_NumOverlappedReads);
                break;
            }
        }
        if (ERROR_SUCCESS != Result)
        {
            break;
        }
    }
    pPacket = pTempPacket;

    return Result;
}

/*-D*************************************************************************
*
*  CTCPCommSvr::BuildAccepts
*
*  PURPOSE: Helper function for building sockets on accepts for client connections.
*
*  ARGUMENTS:
*      [IN] ListenSocket = socket for server transport.
*      [IN] Count = number of look-ahead sockets to build.
*
*  RETURNS: ERROR_SUCCESS, or standard Winsock API error.
*
**-E************************************************************************/
int CTCPCommSvr::BuildAccepts(SOCKET ListenSocket, BYTE Count)
{
    int Result = ERROR_SUCCESS;
    LPFN_ACCEPTEX AcceptEx = NULL;
    GUID GuidAcceptEx = WSAID_ACCEPTEX;
    DWORD dwBytes;
    PTCP_PACKET pPacket;

    //Per MSDN, load the AcceptEx function into memory using WSAIoctl.
    //Calls the AcceptEx function directly, rather than refer to the Mswsock.lib library.
    Result = WSAIoctl(ListenSocket, SIO_GET_EXTENSION_FUNCTION_POINTER, &GuidAcceptEx, sizeof(GuidAcceptEx), &AcceptEx, sizeof(AcceptEx), &dwBytes, NULL, NULL);
    if (ERROR_SUCCESS == Result)
    {
        for (int i=0; i<Count; i++)
        {
            pPacket = (PTCP_PACKET)HeapAlloc(g_hHeap, HEAP_ZERO_MEMORY, sizeof(TCP_PACKET));
            if (NULL == pPacket)
            {
                Result = ERROR_OUTOFMEMORY;
                break;
            }
            //Allocate enough room for the local address of the server and remote address of the client.
            pPacket->Wsa.len = EX_ADDRLEN * 2;
            pPacket->Wsa.buf = (char*)HeapAlloc(g_hHeap, 0, pPacket->Wsa.len);
            if (NULL == pPacket->Wsa.buf)
            {
                HeapFree(g_hHeap, 0, pPacket);
                Result = ERROR_OUTOFMEMORY;
                break;
            }
            pPacket->hSocket = WSASocket(AF_INET, SOCK_STREAM, IPPROTO_TCP, NULL, 0, WSA_FLAG_OVERLAPPED);
            if (INVALID_SOCKET == pPacket->hSocket)
            {
                HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
                HeapFree(g_hHeap, 0, pPacket);
                Result = WSAGetLastError();
                break;
            }
            pPacket->State = OVERLAPPED_INIT;
            InterlockedIncrement(&m_NumOverlappedReads);
            //Since not expecting initial data from client, specify 0 so function does not wait for any data from a receive.
            if (!AcceptEx(ListenSocket, pPacket->hSocket, pPacket->Wsa.buf, 0, EX_ADDRLEN, EX_ADDRLEN, &dwBytes, &pPacket->Ol))
            {
                switch (Result = WSAGetLastError())
                {
                    case ERROR_IO_PENDING:
                    {
                        Result = ERROR_SUCCESS; //reset, this is OK
                        break;
                    }
                    default:
                    {
                        assert(0);
                        closesocket(pPacket->hSocket);
                        HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
                        HeapFree(g_hHeap, 0, pPacket);
                        InterlockedDecrement(&m_NumOverlappedReads);
                        break;
                    }
                }
            }
        }
    }

    return Result;
}

#undef EX_ADDRLEN
#undef DELIM
