/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CUDPCommSvr.cpp:                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CUDPCommSvr.h"
#include "assert.h" //remove

#define DELIM ":"

/*-D*************************************************************************
*
*  CUDPCommSvr default constructor
*
*  PURPOSE: Initialize
*
*  ARGUMENTS: none
*
*  RETURNS: none
*
**-E************************************************************************/
CUDPCommSvr::CUDPCommSvr(void)
{
    Construct();
}

/*-D*************************************************************************
*
*  CUDPCommSvr overloaded constructor
*
*  PURPOSE: Initialize with option to specify whether server instance.
*
*  ARGUMENTS:
*      [IN] Server = If true, a server listening instance will be created.
*                    When false, a stand-alone client or client session on a
*                    server will be created.
*
*  RETURNS: none
*
**-E************************************************************************/
CUDPCommSvr::CUDPCommSvr(bool Server)
{
    Construct(Server);
}

/*-D*************************************************************************
*
*  CUDPCommSvr destructor
*
*  PURPOSE: Provide ordered shutdown and thorough cleanup of transport instance
*
*  ARGUMENTS: none
*
*  RETURNS: none
*
**-E************************************************************************/
CUDPCommSvr::~CUDPCommSvr(void)
{
    //Let outstanding writes complete
    while (m_NumOverlappedWrites)
    {
        Sleep(0);
    }
    if (m_Socket)
    {
        closesocket(m_Socket);
        m_Socket = NULL;
    }
    //Wait for any outstanding reads to complete:
    // They will acceptably fail after closing the handle with Error 995:
    // The I/O operation has been aborted because of either a thread exit or an application request.
    while (m_NumOverlappedReads)
    {
        Sleep(0);
    }
    WSACleanup();
    if (!m_bServer)
    {
        DeleteCriticalSection(&m_CS);
    }
}

/*-D*************************************************************************
*
*  CUDPCommSvr::Construct
*
*  PURPOSE: Helper function for constructors. Initializes Winsock and all
*           private members.
*
*  ARGUMENTS:
*      [IN] Server = If true, a server listening instance will be created.
*                    When false, a stand-alone client or client session on a
*                    server will be created.
*
*  RETURNS: none
*
**-E************************************************************************/
void CUDPCommSvr::Construct(bool Server)
{
    WSADATA WSAData;

    WSAStartup(MAKEWORD(2, 2), &WSAData);
    m_bServer = Server;
    m_bClientMode = false;
    m_bSetConfig = false;
    m_PastFirstSend = FALSE;
    m_PastFirstRecv = FALSE;
    m_Socket = NULL;
    m_NumOverlappedReads = 0;
    m_NumOverlappedWrites = 0;
    if (!Server)
    {
        InitializeCriticalSection(&m_CS);
    }
    this->StackType = ETHERNET;
}

/*-D*************************************************************************
*
*  CUDPCommSvr::SetConfig
*
*  PURPOSE: Updates addressing information for socket
*
*  ARGUMENTS:
*      [IN] pAddr = A UDPAddr structure with session information.
*           Required for internal session instances. Must be NULL for server
*           or dedicated client instance.
*
*  RETURNS: none
*
*  NOTES:  As this function cannot return a value, an error is stored in
*          the instance and later checked in Connect or StartServer function.
*
*          The format of the SourceID must be: "Transport:IP:port"
*          IP must be a Null-terminated character string representing a number
*          expressed in the IPv4 standard "." (dotted) notation. Server instance
*          may specify "*" for the IP address to listen on any interface,
*          which may be useful on a multi-homed host. IP is required for client
*          instance, but may be blank for server listener. A blank IP will
*          result in any appropriate network address being used, resulting in
*          the same behavior as if specifying "*".
*          Port must be in the range of 1-65535. If port is not specified,
*          the default rendezvous port is used.
*          Transport specifier (any unique string) is required for  all instances.
**-E************************************************************************/
void CUDPCommSvr::SetConfig(void *pAddr)
{
    UDPAddr* pConfig = (UDPAddr*)pAddr;
    char ConnectionString[ADDR_BUF_SIZE];
    char *pTmp;
    int Port;

    if (!m_bServer)
    {
        if (pConfig) //Device arrival already has addr figured out
        {
            memcpy(&m_Addr, &pConfig->ClntAddr, sizeof(SOCKADDR_IN));
        }
        else //operating as a client
        {
            m_bClientMode = true;
        }
    }
    if (m_bServer || m_bClientMode) //server or true client mode (not client session of server)
    {
        //Initialize default parameters
        memset(&m_Addr, 0, sizeof(m_Addr));
        m_Addr.sin_family = AF_INET;
        m_Addr.sin_addr.S_un.S_addr = htonl(INADDR_ANY);
        m_Addr.sin_port = htons(this->Port);

        if (this->SourceID)
        {
            strcpy(ConnectionString, this->SourceID);
            strtok(ConnectionString, DELIM); //ignore prefix
            pTmp = strtok(NULL, DELIM);
            if (pTmp)
            {
                m_Addr.sin_addr.S_un.S_addr = inet_addr(pTmp);
                if (INADDR_NONE == m_Addr.sin_addr.S_un.S_addr)
                {
                    if (0 == strcmp(pTmp, "*") && m_bServer)
                    {
                        m_Addr.sin_addr.S_un.S_addr = htonl(INADDR_ANY);
                    }
                    else
                    {
                        m_bSetConfig = false;
                        return;
                    }
                }
                pTmp = strtok(NULL, DELIM);
                if (pTmp)
                {
                    Port = atoi(pTmp);
                    if (Port > 0 && Port <= USHRT_MAX)
                    {
                        m_Addr.sin_port = htons((u_short)Port);
                    }
                    else
                    {
                        m_bSetConfig = false;
                        return;
                    }
                }
            }
            else if (!m_bServer)
            {
                m_bSetConfig = false;
                return;
            }
        }
    }
    m_bSetConfig = true;
}

/*-D*************************************************************************
*
*  CUDPCommSvr::SendMsg
*
*  PURPOSE: Sends data from the current transport instance to the remote end
*
*  ARGUMENTS:
*      [IN] pBuf = Pointer to a buffer that contains the data to be sent.
*      [IN] BufSize = Size of the buffer pointed to by pBuf.
*
*  RETURNS: TRUE if successful, otherwise FALSE.
*
**-E************************************************************************/
boolean CUDPCommSvr::SendMsg(uint8 *pBuf, uint32 BufSize)
{
    boolean bRet = FALSE;
    int r;
    PUDP_PACKET pPacket = NULL;
    DWORD dwBytes;

    if (NULL == pBuf) //validate params
    {
        return FALSE;
    }
    pPacket = (PUDP_PACKET)HeapAlloc(g_hHeap, HEAP_ZERO_MEMORY, sizeof(UDP_PACKET));
    if (NULL == pPacket)
    {
        return FALSE;
    }
    pPacket->Wsa.buf = (char*)HeapAlloc(g_hHeap, 0, BufSize);
    if (NULL == pPacket->Wsa.buf)
    {
        HeapFree(g_hHeap, 0, pPacket);
        return FALSE;
    }
    memcpy(pPacket->Wsa.buf, pBuf, BufSize);
    pPacket->Wsa.len = BufSize;
    pPacket->SinLen = sizeof(pPacket->Sin);
    pPacket->State = OVERLAPPED_WRITE;
    InterlockedIncrement(&m_NumOverlappedWrites);
    r = WSASendTo(m_Socket, &pPacket->Wsa, 1, &dwBytes, 0, (LPSOCKADDR)&(m_Addr), sizeof(m_Addr), &pPacket->Ol, NULL);
    if (SOCKET_ERROR == r)
    {
        switch (WSAGetLastError())
        {
            case ERROR_IO_PENDING:
            {
                bRet = TRUE; //reset return boolean (this is expected).
                break;
            }
            default:
            {
                HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
                HeapFree(g_hHeap, 0, pPacket);
                InterlockedDecrement(&m_NumOverlappedWrites);
                break;
            }
        }
    }
    else if (ERROR_SUCCESS == r)
    {
        bRet = TRUE;
    }

    return bRet;
}

/*-D*************************************************************************
*
*  CUDPCommSvr::Connect
*
*  PURPOSE: Creates a socket for clients that connects to the IP passed in the
*            connection string.
*           Assigns socket to the completion port, and posts receives.
*           Updates the WACP Port and SourceID once a connection is created.
*
*  ARGUMENTS:
*      [IN] pSourceID = String identifier(IP address) of the connection.
*      [IN] PortNum = Port value for the socket connection.
*
*  RETURNS: TRUE if successful, otherwise FALSE.
*
**-E************************************************************************/
boolean CUDPCommSvr::Connect(uint8* pSourceID, uint16 PortNum)
{
    boolean bRet = FALSE;

    //Don't allow servers to Connect
    if (m_bServer || !m_bSetConfig)
    {
        return FALSE;
    }
    if (SetupUDP(&m_Socket, NULL))
    {
        if (ERROR_SUCCESS == this->Assign((HANDLE)m_Socket, (ULONG_PTR)this))
        {
            bRet = TRUE;
        }
    }

    return bRet;
    UNREFERENCED_PARAMETER(PortNum);
    UNREFERENCED_PARAMETER(pSourceID);
}

/*-D*************************************************************************
*
*  CUDPCommSvr::StartServer
*
*  PURPOSE: Creates a listener socket for server that binds to the IP passed
*            in the connection string.
*           Assigns socket to the completion port, and builds accept sockets.
*
*  ARGUMENTS: none
*
*  RETURNS: ERROR_SUCCESS, or standard Winsock API error.
*
**-E************************************************************************/
uint32 CUDPCommSvr::StartServer()
{
    uint32 r = ERROR_SUCCESS;

    //Don't allow clients to listen
    if (!m_bServer)
    {
        return ERROR_INVALID_FUNCTION;
    }
    //Check previous failure
    if (!m_bSetConfig)
    {
        return ERROR_BAD_ARGUMENTS;
    }
    //Create socket for UDP
    if (!SetupUDP(&m_Socket, &m_Addr))
    {
        return WSAGetLastError();
    }
    //Associate the socket handle to the completion port
    r = this->Assign((HANDLE)m_Socket, (ULONG_PTR)this);
    if (ERROR_SUCCESS == r)
    {
        r = PostReceive(NULL, UDP_MAX_OVERLAPPED_READS);
    }

    return r;
}

/*-D*************************************************************************
*
*  CUDPCommSvr::ProcessCP
*
*  PURPOSE: Callback to process queued I/O from completion port
*
*  ARGUMENTS:
*      [IN] dwBytes = number of bytes processed from the overlapped handle.
*      [IN] lpOl = Pointer to overlapped plus structure for network data.
*
*  RETURNS: none
*
**-E************************************************************************/
void CUDPCommSvr::ProcessCP(DWORD dwBytes, LPOVERLAPPED lpOl)
{
    PUDP_PACKET pPacket = (PUDP_PACKET)lpOl;
    UDPAddr addr;
    char cSourceID[ADDR_BUF_SIZE];

    if (pPacket)
    {
        if (m_bServer)
        {
            switch (pPacket->State)
            {
                case OVERLAPPED_INIT:
                {
                    sprintf(cSourceID, "%s%s%s%s%hu", UDP_SOURCE_PREFIX, DELIM, inet_ntoa(pPacket->Sin.sin_addr), DELIM, ntohs(pPacket->Sin.sin_port));
                    memset(&addr, 0, sizeof(addr));
                    memcpy(&addr.ClntAddr, &pPacket->Sin, pPacket->SinLen);
                    this->Arrival(cSourceID, ETHERNET, &addr, sizeof(addr));
                    PostReceive(pPacket);
                    InterlockedDecrement(&m_NumOverlappedReads);
                    break;
                }
                default:
                {
                    assert(0); //should not get here
                    HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
                    HeapFree(g_hHeap, 0, pPacket);
                    break;
                }
            }
        }
        else //client session instance
        {
            switch (pPacket->State)
            {
                case OVERLAPPED_WRITE:
                {
                    HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
                    HeapFree(g_hHeap, 0, pPacket);
                    if (!InterlockedExchange(&m_PastFirstSend, TRUE))
                    {	//Now that the socket has been implicitly bound by the inital send, post receives
                        PostReceive(NULL, UDP_MAX_OVERLAPPED_READS);
                    }
                    InterlockedDecrement(&m_NumOverlappedWrites);
                    break;
                }
                case OVERLAPPED_READ:
                {
                    if (m_bClientMode && !InterlockedExchange(&m_PastFirstRecv, TRUE))
                    {
                        memcpy(&m_Addr, &pPacket->Sin, sizeof(SOCKADDR_IN));
                    }
                    //Drop packets outside of client session
                    if (pPacket->Sin.sin_addr.S_un.S_addr == m_Addr.sin_addr.S_un.S_addr && pPacket->Sin.sin_port == m_Addr.sin_port)
                    {
                        EnterCriticalSection(&m_CS);
                        this->WACPParserEx((uint8*)pPacket->Wsa.buf, dwBytes, NULL);
                        LeaveCriticalSection(&m_CS);
                    }
                    PostReceive(pPacket);
                    InterlockedDecrement(&m_NumOverlappedReads);
                    break;
                }
                default:
                {
                    assert(0); //should not get here
                    HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
                    HeapFree(g_hHeap, 0, pPacket);
                    break;
                }
            }
        }
    }
    return;
    UNREFERENCED_PARAMETER(dwBytes);
}

/*-D*************************************************************************
*
*  CUDPCommSvr::RecvCPError
*
*  PURPOSE: Receive error callback from I/O completion port
*
*  ARGUMENTS:
*      [IN] LastError = standard WIN32 error of reason for overlapped failure.
*      [IN] lpOl = Pointer to overlapped plus structure for network data.
*
*  RETURNS: none
*
**-E************************************************************************/
void CUDPCommSvr::RecvCPError(DWORD LastError, LPOVERLAPPED lpOl)
{
    PUDP_PACKET pPacket = (PUDP_PACKET)lpOl;
    int state;

    if (pPacket)
    {
        state = pPacket->State;
        HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
        HeapFree(g_hHeap, 0, pPacket);
        switch (state)
        {
            case OVERLAPPED_WRITE:
            {
                InterlockedDecrement(&m_NumOverlappedWrites);
                break;
            }
            case OVERLAPPED_READ:
            case OVERLAPPED_INIT:
            {
                InterlockedDecrement(&m_NumOverlappedReads);
                break;
            }
            default:
            {
                assert(0);
                break;
            }
        }
    }
    return;
    UNREFERENCED_PARAMETER(LastError);
}

/*-D*************************************************************************
*
*  SetupUDP
*
*  PURPOSE: Create an overlapped (asynchronous) UDP socket.
*
*  ARGUMENTS:
*      [OUT] hSocket = UDP socket
*      [IN] addr = for server binding, contains port that socket should listen on.
*                  Otherwise specify NULL.
*
*  RETURNS: BOOL - TRUE if successful, FALSE on failure.
*
*  NOTES:   Use WSAGetLastError() to get extended error information.
*
**-E************************************************************************/
BOOL CUDPCommSvr::SetupUDP(SOCKET *hSocket, LPSOCKADDR_IN addr)
{
    DWORD bVal = TRUE;

    //UDP socket creation
    *hSocket = INVALID_SOCKET;
    //create a datagram socket
    if ((*hSocket = WSASocket(AF_INET, SOCK_DGRAM, IPPROTO_UDP, NULL, 0, WSA_FLAG_OVERLAPPED)) == INVALID_SOCKET)
    {
        return (FALSE);
    }
    //Don't allow socket hijack
    if (SOCKET_ERROR == setsockopt(*hSocket, SOL_SOCKET, SO_EXCLUSIVEADDRUSE, (const char*)&bVal, sizeof(bVal)))
    {
        return (FALSE);
    }
    //Servers should bind
    if (addr)
    {
        if (SOCKET_ERROR == bind(*hSocket, (LPSOCKADDR)addr, sizeof(SOCKADDR)))
        {
            return (FALSE);
        }
    }

    return (TRUE);
}


/*-D*************************************************************************
*
*  CUDPCommSvr::PostReceive
*
*  PURPOSE: Helper function for posting receives for network client that will
*           complete when incoming data is available.
*
*  ARGUMENTS:
*      [IN] pPacket = Pointer to overlapped plus structure for network data.
*      [IN] Count = number of receives to post.
*
*  RETURNS: ERROR_SUCCESS, or standard Winsock API error.
*
**-E************************************************************************/
DWORD CUDPCommSvr::PostReceive(PUDP_PACKET pPacket, UINT Count)
{
    DWORD r = ERROR_SUCCESS;
    PUDP_PACKET pTempPacket = pPacket;
    DWORD dwBytes;
    DWORD dwFlags = 0;

    if (pPacket && Count > 1)
    {
        return ERROR_INVALID_FUNCTION;
    }
    for (UINT i=0; i<Count; i++)
    {
        if (NULL == pTempPacket)
        {
            pPacket = (PUDP_PACKET)HeapAlloc(g_hHeap, HEAP_ZERO_MEMORY, sizeof(UDP_PACKET));
            if (NULL == pPacket)
            {
                r = ERROR_OUTOFMEMORY;
                break;
            }
            pPacket->Wsa.buf = (char*)HeapAlloc(g_hHeap, 0, UDP_RCVBUFSIZE);
            if (NULL == pPacket->Wsa.buf)
            {
                HeapFree(g_hHeap, 0, pPacket);
                r = ERROR_OUTOFMEMORY;
                break;
            }
            pPacket->Wsa.len = UDP_RCVBUFSIZE;
            pPacket->SinLen = sizeof(pPacket->Sin);
        }
        else if (pPacket->Wsa.len != UDP_RCVBUFSIZE || NULL == pPacket->Wsa.buf)
        {	//should never get here, fail safe
            assert(0);
            memset(&pPacket->Ol, 0, sizeof(pPacket->Ol));
            pPacket->Wsa.buf = (char*)HeapReAlloc(g_hHeap, 0, pPacket->Wsa.buf, UDP_RCVBUFSIZE);
            if (NULL == pPacket->Wsa.buf)
            {
                HeapFree(g_hHeap, 0, pPacket);
                r = ERROR_OUTOFMEMORY;
                break;
            }
            pPacket->Wsa.len = UDP_RCVBUFSIZE;
        }
        pPacket->State = m_bServer ? OVERLAPPED_INIT : OVERLAPPED_READ;
        InterlockedIncrement(&m_NumOverlappedReads);
        r = WSARecvFrom(m_Socket, &pPacket->Wsa, 1, &dwBytes, &dwFlags, (LPSOCKADDR)&pPacket->Sin, &pPacket->SinLen, &pPacket->Ol, NULL);
        if (SOCKET_ERROR == r)
        {
            switch (r = WSAGetLastError())
            {
                case WSA_IO_PENDING: //reset this error OK
                {    
                    r = ERROR_SUCCESS;
                    break;
                }
                default:
                {    
#if defined(TRACE_SERVER_ON)
                    SYS_PRINTF("Socket Error: %u\n", r);
#endif
                    HeapFree(g_hHeap, 0, pPacket->Wsa.buf);
                    HeapFree(g_hHeap, 0, pPacket);
                    InterlockedDecrement(&m_NumOverlappedReads);
                    break;
                }
            }
        }
        if (ERROR_SUCCESS != r)
        {
            break;
        }
    }
    pPacket = pTempPacket;

    return r;
}

#undef DELIM