/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CDMOIBObject.cpp                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CDMOIBObject.h"
#include "CDMOIBDictionary.h"
#include "CDMOIBStringTable.h"

#include "DtSupport.h"
/* DtSupport needed for:
 - Family and Genus enums (used ONLY in DMOIBSerialSizeWidth() function)
*/
#include "MiscUtil.h" 
/* MiscUtil needed for: 
 - CalculateCRC()
*/

#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE) || (TARGET_XML_SUPPORT == XML_ALL))
#include "xmlparse.h"
#endif

// Macro to determine if a member is a basic type (Not array, and not an 
// object)
#define IS_BASIC_TYPE(TypeInfo) ( ((TypeInfo) < DMOIB_TYPE_FLAG_PTR) && \
                                 (((TypeInfo) & 0x01) || \
                                 !((TypeInfo) & DMOIB_TYPE_MASK_BW)))

#define IS_NEGATIVE(Value, ByteWidth) ((Value) & (0x80 << ((ByteWidth) << 3)))
#define SIGN_EXTEND(Value, ByteWidth) ((0xFFFFFF00 << ((ByteWidth) << 3)) | (Value))

// Macro to skip the header portion of a definition, and point to the first
// member of the def. (This macro returns a byte* value)
#define SKIP_DEF_HEADER(pDefinition) \
   (((byte*)pDefinition)    /* Pointer to Start of Def  */ \
    + sizeof(uint8)         /* skip Species Count       */ \
    + sizeof(uint16)        /* skip Family              */ \
    + sizeof(uint8)         /* skip Genus               */ \
    + (pDefinition == NULL ? 0 : *((byte*)pDefinition)) /* skip all the Species     */ \
    + sizeof(uint16)        /* skip Version             */ \
    + sizeof(uint16)        /* skip DefSize             */ \
    + sizeof(uint16)        /* skip Full DefSize        */ \
    + sizeof(uint16)        /* skip XML Base Size       */ \
    + sizeof(uint16))       /* skip Member Count        */ 

/*
===============================================================================
    Global function to determine the amount of bits needed for representing 
    a serialized object's size (32 or 16 - based on IsAggregate())
===============================================================================
*/
uint8 DMOIBSerialSizeWidth(uint32 ClassID)
{
    if(  (GET_FAMILY(ClassID) == FmSESSION && GET_GENUS(ClassID) != GnPARAMETER)
       || GET_FAMILY(ClassID) == FmDTCOLLECTION
       || GET_GENUS (ClassID) == GnDATAEXT
       || ClassID == MAKE_CLASSID(FmSPIRO,          GnDATA, 0)
       || ClassID == MAKE_CLASSID(FmECG,            GnDATA, 0) 
       || ClassID == MAKE_CLASSID(FmCALIBRATION,    GnDATA, 128)
       || ClassID == MAKE_CLASSID(FmTEST,           GnDATA, 255))
    {
        return sizeof(uint32);
    }
    else
    {
        return sizeof(uint16);
    }
}

/*
===============================================================================
    Global function to determine the size of a serialized stream, taking into
    account whether it uses a 16 or 32 bit size.
===============================================================================
*/
uint32 DMOIBSerialSize(byte* pBuffer)
{
    uint32  l_Value = 0;
    uint8   l_SizeWidth;

    if(pBuffer != NULL)
    {
        // Grab the ClassID.
        LINE_COPY(&l_Value, pBuffer, sizeof(uint32));
        pBuffer += sizeof(uint32);

        // Get the length of the size ID for that ClassID.
        l_SizeWidth = DMOIBSerialSizeWidth(l_Value);

        // Copy that number of bytes from the buffer to get the size value.
        l_Value = 0;
        LINE_COPY(&l_Value, pBuffer, l_SizeWidth);

        // Increase our return value by the size width, and the size of the
        // ClassID
        l_Value += (l_SizeWidth + sizeof(uint32));
    }
    
    return l_Value;
}

/*
===============================================================================
    Global functions for writing/reading MOIB_TIME data to/from a serialized 
    stream
===============================================================================
*/
void TimeToBuffer(byte* pBuffer, uint16 BufferSize, byte* pTime, uint16 TimeSize)
{
    if(pTime != NULL && pBuffer != NULL && 
       BufferSize == MOIB_TIME_SIZE && 
       TimeSize == sizeof(MOIB_TIME))
    {
        LINE_COPY(pBuffer, &((MOIB_TIME*)pTime)->year, sizeof(uint16));
        pBuffer[2] = ((MOIB_TIME*)pTime)->month;
        pBuffer[3] = ((MOIB_TIME*)pTime)->day;
        pBuffer[4] = ((MOIB_TIME*)pTime)->hour;
        pBuffer[5] = ((MOIB_TIME*)pTime)->minute;
        pBuffer[6] = ((MOIB_TIME*)pTime)->second;
        pBuffer[7] = ((MOIB_TIME*)pTime)->format;
    }
}
void BufferToTime(byte* pTime, uint16 TimeSize, byte* pBuffer, uint16 BufferSize)
{
    if(pTime != NULL && pBuffer != NULL && 
       BufferSize == MOIB_TIME_SIZE && 
       TimeSize == sizeof(MOIB_TIME))
    {
        LINE_COPY(&((MOIB_TIME*)pTime)->year, pBuffer, sizeof(uint16));
        ((MOIB_TIME*)pTime)->month  = pBuffer[2];
        ((MOIB_TIME*)pTime)->day    = pBuffer[3];
        ((MOIB_TIME*)pTime)->hour   = pBuffer[4];
        ((MOIB_TIME*)pTime)->minute = pBuffer[5];
        ((MOIB_TIME*)pTime)->second = pBuffer[6];
        ((MOIB_TIME*)pTime)->format = pBuffer[7];
    }
}

/*
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    CDMOIBData_t structure Methods    
    (The basic struct used to represent all data types.)
    
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
===============================================================================
    Operator casting methods for implicit conversions
===============================================================================
    Signed Integer casts (All go through int32)
===============================================================================
*/
CDMOIBData_t::CDMOIBData_u()
{
    memset(&m_MemberInfo, 0, sizeof(m_MemberInfo));
}

CDMOIBData_t::operator int8()
{
    return (int8)(this->operator int32());
}
CDMOIBData_t::operator int16()
{
    return (int16)(this->operator int32());
}
CDMOIBData_t::operator int32()
{
    if(IS_BASIC_TYPE(m_MemberInfo.m_TypeInfo)) 
    {
        if((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP)) 
        { 
            if((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) > 4) 
                return (int32)Float64; 
            else 
                return (int32)Float32; 
        } 
        else 
        { 
            if(!(m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_UNS) && 
                IS_NEGATIVE(Int32, m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW))
            {
                return SIGN_EXTEND(Int32, m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW);
            }
            else 
            {
                return Int32;
            }        
        }
    }
    else
    { 
        return 0; 
    } 
}
/*
===============================================================================
    Unsigned Integer casts (All go through uint32)
===============================================================================
*/
CDMOIBData_t::operator uint8()
{
    return (uint8)(this->operator uint32());
}
CDMOIBData_t::operator uint16()
{
    return (uint16)(this->operator uint32());
}
CDMOIBData_t::operator uint32()
{
    if(IS_BASIC_TYPE(m_MemberInfo.m_TypeInfo)) 
    {
        if((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP)) 
        { 
            if((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) > 4) 
                return (uint32)Float64; 
            else 
                return (uint32)Float32; 
        } 
        else 
        { 
            return Uint32; 
        }
    }
    else
    {
        return 0; 
    } 
}
/*
===============================================================================
    Floating Point casts (All go through float64)
===============================================================================
*/
CDMOIBData_t::operator float32()
{
    return (float32)(this->operator float64());
}
CDMOIBData_t::operator float64() 
{ 
    if(IS_BASIC_TYPE(m_MemberInfo.m_TypeInfo)) 
    {    
        if((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP)) 
        { 
            if((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) > 4) 
                return Float64; 
            else 
                return (float64)Float32; 
        } 
        else if(!(m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_UNS) && 
                 IS_NEGATIVE(Int32, m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW))
        {
            return (float64)((int32)SIGN_EXTEND(Int32, m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW));
        }
        else
        { 
            return (float64)Uint32; 
        } 
    }
    else
    { 
        return 0; 
    } 
}
/*
===============================================================================
    Textual character casts (All go through uint32)
===============================================================================
*/
CDMOIBData_t::operator ansichar()
{
    return (ansichar)(this->operator uint32());
}
/*
===============================================================================
    Custom MOIB_TIME structure cast
===============================================================================
*/
CDMOIBData_t::operator MOIB_TIME()
{
    if(m_MemberInfo.m_TypeInfo == DMOIB_TYPE_TIME)
    {
        return Time;
    }
    else
    {
        MOIB_TIME l_EmptyTime = {0,0,0,0,0,0,0};
        return l_EmptyTime;
    }
}
/*
===============================================================================
    Pointer casts (All go through GetPtr() method)
===============================================================================
*/
CDMOIBData_t::operator int8*()
{ 
    return (int8*)GetPtr(sizeof(int8));
}
CDMOIBData_t::operator int16*()
{ 
    return (int16*)GetPtr(sizeof(int16));
}
CDMOIBData_t::operator int32*()
{ 
    return (int32*)GetPtr(sizeof(int32));
}
CDMOIBData_t::operator uint8*()
{ 
    return (uint8*)GetPtr(sizeof(uint8));
}
CDMOIBData_t::operator uint16*()
{ 
    return (uint16*)GetPtr(sizeof(uint16));
}
CDMOIBData_t::operator uint32*()
{ 
    return (uint32*)GetPtr(sizeof(uint32));
}
CDMOIBData_t::operator float32*()
{ 
    return (float32*)GetPtr(sizeof(float32), DMOIB_TYPE_FLAG_FP);
}
CDMOIBData_t::operator float64*()
{ 
    return (float64*)GetPtr(sizeof(float64), DMOIB_TYPE_FLAG_FP);
}
CDMOIBData_t::operator ansichar*()
{ 
    return (ansichar*)GetPtr(sizeof(ansichar));
}
CDMOIBData_t::operator MOIB_TIME*()
{ 
    return (MOIB_TIME*)GetPtr(DMOIB_TYPE_FLAG_TIME + 1);
}
CDMOIBData_t::operator void*() 
{ 
    if((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR) || 
       (m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
       (m_MemberInfo.m_TypeInfo == DMOIB_TYPE_OBJECT)  || 
       (m_MemberInfo.m_TypeInfo == DMOIB_TYPE_BASE_CLASS)) 
    {
        return Ptr;
    }
    else
    {
        return NULL;
    }
}
CDMOIBData_t::operator CDMOIBObject*()
{ 
    if(m_MemberInfo.m_TypeInfo == DMOIB_TYPE_OBJECT || 
       m_MemberInfo.m_TypeInfo == DMOIB_TYPE_BASE_CLASS)
    { 
        return (CDMOIBObject*)Ptr; 
    } 
    else 
    { 
        return NULL; 
    }
}

CDMOIBData_t::operator CDMOIBVector*()
{   
    if(m_MemberInfo.m_TypeInfo == DMOIB_TYPE_VECTOR_PTR || 
       m_MemberInfo.m_TypeInfo == DMOIB_TYPE_VECTOR_ARR )
    {
        return (CDMOIBVector*)Ptr;
    }
    else
    { 
        return NULL; 
    }
}

/*
===============================================================================
    Method to return a generic pointer to array/object member.
===============================================================================
*/
void* CDMOIBData_t::GetPtr(uint8 BytesWide, uint8 IsFP)
{ 
    if(((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR) || 
        (m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR)) && 
       ((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP) == IsFP) && 
       ((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW)+1) == BytesWide) 
    { 
        return Ptr; 
    } 
    else 
    { 
        return NULL; 
    } 
}

/*
===============================================================================
    Assigment operators
===============================================================================
*/
CDMOIBData_t& CDMOIBData_t::operator=(uint32 Value)
{
    if(Available() && IS_BASIC_TYPE(m_MemberInfo.m_TypeInfo))
    {
        memset(m_MemberInfo.m_Padding, 0, MAX_MEMBER_SIZE);

        if((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP)) 
        { 
            if((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) > 4) 
                Float64 = (float64)Value; 
            else 
                Float32 = (float32)Value; 
        } 
        else 
        { 
            // We just copy over the integer data for the size of the data 
            // member. We'll worry about sign extending it later in the Get()
            // method
            BYTE_COPY(&Uint32, (m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW)+1, &Value, sizeof(Value)); 
        }
    }
    
    return *this;
}

/*
===============================================================================
    Method to copy a generic buffer of data to the selected member
    [PRIVATE]

    Possible argument cases:
        ValueSize and IsFP are determined automatically, so pValue (pV) and
        ValueLength (VL) are the only variables from outside sources.

                Inputs                      Result
        -----------------------------------------------------------------------
        pV == NULL | VL == 0 | Reset buffer to NULL and 0 (static buffer reset)
        pV == NULL | VL  > 0 | Reset buffer to empty array of VL length ( '' )
        pV != NULL | VL == 0 | FAILURE
        pV != NULL | VL  > 0 | Copy pV to buffer by VL length (FAILURE if VL
                   |         | is larger than static buffer size)
===============================================================================
*/
boolean CDMOIBData_t::Set(void* pValue, uint8 ValueSize, uint16 ValueLength, uint8 IsFP)
{
    boolean l_RetVal = FALSE;

    if(((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP) == IsFP) &&
       ((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW)+1) == ValueSize &&
       !(pValue != NULL && ValueLength == 0))
    { 
        // Check for special types
        switch(ValueSize)
        {
            case (DMOIB_TYPE_FLAG_TIME + 1):
                // This is a MOIB_TIME struct, so use a specific copy 
                // function and change the ByteWidth.
                ValueSize = sizeof(MOIB_TIME);
                break;
            default:
                break;
        }            

        // We are dealing with a buffer that has the same bytewidth as our
        // member, so we can copy the data properly.
                
        // ARRAY TYPE
        if( (m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) && 
            Ptr != NULL && 
            m_PointerInfo.m_MemSize > 0)
        {
            // If we're dealing with text, then we have to leave the last
            // element of the array as 0 for the NULL terminator. So check if
            // it is text, and set our copy length to one less than the actual
            // length to account for the extra element.
            uint16 Length = ((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_TXT) ? ValueLength+1 : ValueLength);

            // The only failure condition here is if the copy length is too
            // large for the static buffer. Check to make sure it's within
            // limits.
            if(Length <= m_PointerInfo.m_NumElements)
            {
                l_RetVal = TRUE;

                // We are an array type, so zero our buffer by default
                memset(Ptr, 0, m_PointerInfo.m_MemSize);
             
                // If we have enough space available to copy the entire buffer, 
                // then copy it. Otherwise it will remain zeroed out.
                if(pValue != NULL && ValueLength > 0)
                {
                    memcpy(Ptr, pValue, (ValueLength * ValueSize));
                }
            }            
        }
        // POINTER TYPE
        else if(m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR)
        {
            // If we are dealing with text, add an extra element for the NULL 
            // terminator.
            uint16 Length = ((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_TXT) ? ValueLength+1 : ValueLength);
            
#if (defined(DMOIB_FORCE_ARRAY_RESIZE) && (DMOIB_FORCE_ARRAY_RESIZE == 1))
            if(Ptr != NULL)
#else
            if(Ptr != NULL &&
              (m_PointerInfo.m_MemSize < (Length * ValueSize) || 
               ValueLength == 0))               
#endif       
            {
                // If we have memory allocated already, but it isn't large 
                // enough to fit the pValue buffer, then delete it and set the
                // pointer to NULL.
                delete [] (byte*)Ptr;
                Ptr = NULL;
                m_PointerInfo.m_MemSize = 0;
                m_PointerInfo.m_NumElements = 0;
            }

            // In the case where the array is reset to empty (NULL), the proper
            // return value is TRUE. Check to see if the arguments specifed
            // are NULL and 0 and the internal array matches it as NULL and 0.
            if(Ptr == NULL && pValue == NULL &&
               m_PointerInfo.m_MemSize == 0 && ValueLength == 0)
            {
                l_RetVal = TRUE;
            }
            else if(ValueLength > 0)
            {
                if(Ptr == NULL)
                {
                    // Either we haven't allocated any memory for this member 
                    // yet, or the memory was freed above. Either way, we 
                    // allocate new memory.
                    m_PointerInfo.m_MemSize = (Length * ValueSize);
                    Ptr = new byte[m_PointerInfo.m_MemSize];
                }
                
                if(Ptr != NULL)
                {
                    l_RetVal = TRUE;

                    // Either we are reusing the memory allocated earlier, or we have 
                    // new memory from the call above. Zero out the buffer and save
                    // the NumElements.                    
                    m_PointerInfo.m_NumElements = Length;
                    memset(Ptr, 0, m_PointerInfo.m_MemSize);
                    
                    // If we were passed NULL but ValueLength > 0, then the 
                    // intention was to create a new, empty buffer, so don't
                    // do a memcpy.
                    if(pValue != NULL)
                    {
                        memcpy(Ptr, pValue, (ValueLength * ValueSize));
                    }
                }
                else
                {
                    m_PointerInfo.m_MemSize = 0;
                }
            }
        }
    }
    
    return l_RetVal;
}

boolean CDMOIBData_t::Set(int8 *pValue, uint16 Elements)
{
    return Set((void*)pValue, sizeof(*pValue), Elements);
}

boolean CDMOIBData_t::Set(int16 *pValue, uint16 Elements)
{
    return Set((void*)pValue, sizeof(*pValue), Elements);
}

boolean CDMOIBData_t::Set(int32 *pValue, uint16 Elements)
{
    return Set((void*)pValue, sizeof(*pValue), Elements);
}

boolean CDMOIBData_t::Set(uint8 *pValue, uint16 Elements)
{
    return Set((void*)pValue, sizeof(*pValue), Elements);
}

boolean CDMOIBData_t::Set(uint16 *pValue, uint16 Elements)
{
    return Set((void*)pValue, sizeof(*pValue), Elements);
}

boolean CDMOIBData_t::Set(uint32 *pValue, uint16 Elements)
{
    return Set((void*)pValue, sizeof(*pValue), Elements);
}

boolean CDMOIBData_t::Set(float32 *pValue, uint16 Elements)
{
    return Set((void*)pValue, sizeof(*pValue), Elements, DMOIB_TYPE_FLAG_FP);
}

boolean CDMOIBData_t::Set(float64 *pValue, uint16 Elements)
{
    return Set((void*)pValue, sizeof(*pValue), Elements, DMOIB_TYPE_FLAG_FP);
}

boolean CDMOIBData_t::Set(ansichar *pValue, uint16 Elements)
{
    return Set((void*)pValue, sizeof(*pValue), Elements);
}

boolean CDMOIBData_t::Set(MOIB_TIME *pValue, uint16 Elements)
{
    return Set((void*)pValue, DMOIB_TYPE_FLAG_TIME + 1, Elements);
}

/*
===============================================================================
    Member Information Methods
===============================================================================
*/
/*
===============================================================================
    Method to determine if a the member specified by MemberID is available in
    this version of the object.
===============================================================================
*/
boolean CDMOIBData_t::Available()
{
    return (m_MemberInfo.m_TypeInfo != DMOIB_TYPE_UNKNOWN);
}

/*
===============================================================================
    Method to get the type information for a member
===============================================================================
*/
uint8 CDMOIBData_t::Type()
{
    return m_MemberInfo.m_TypeInfo;
}

/*
===============================================================================
    Method to determine the number of elements contained in a member.
===============================================================================
*/
uint16 CDMOIBData_t::Elements()
{
    uint16 l_Elements = 0;

    if((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
       (m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR) )
    {
        if((m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
        {
            l_Elements = ((CDMOIBVector*)Ptr)->Size();
        }
        else
        {
            l_Elements = m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_TXT && 
                         m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR &&
                         m_PointerInfo.m_NumElements > 0 ? 
                         m_PointerInfo.m_NumElements - 1 : 
                         m_PointerInfo.m_NumElements;
        }
    }
    else if(Available())
    {
        l_Elements = 1;
    }

    return l_Elements;
}

/*
===============================================================================
    Method to get the byte size information for a member
===============================================================================
*/
uint8 CDMOIBData_t::Size()
{
    uint8 l_MemberSize = (uint8)(m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW);

    switch(l_MemberSize)
    {
        // Check for the 2 special cases of objects or TIME structs.
        case DMOIB_TYPE_FLAG_OBJ:
            l_MemberSize = sizeof(void*);
            break;
        case DMOIB_TYPE_FLAG_TIME:
            l_MemberSize = sizeof(MOIB_TIME);
            break;
        // If it's not a special case, then the byte-size is encoded, so decode
        // it by incrementing the masked value by 1.
        default:
            l_MemberSize++;
            break;
    }

    return l_MemberSize;
}

/*
===============================================================================
    Methods to get the alias information for a member
===============================================================================
*/
uint32 CDMOIBData_t::Alias()
{
    uint32 l_Alias = OMITTED;
        
    if(m_MemberInfo.m_ExtendedMemberInfo.m_BlockSize >= SIZEOF_ALIAS)
    {
        LINE_COPY(&l_Alias, 
                   m_MemberInfo.m_ExtendedMemberInfo.m_pBlock, 
                   SIZEOF_ALIAS);
    }

    return l_Alias;
}

/*
===============================================================================
    Methods to get the alias information for a member
===============================================================================
*/
uint8 CDMOIBData_t::Units()
{
    uint8 l_Units = (uint8)(OMITTED & 0x0FF);
        
    if(m_MemberInfo.m_ExtendedMemberInfo.m_BlockSize >= (SIZEOF_ALIAS + 
                                                         SIZEOF_UNITS))
    {
        LINE_COPY(&l_Units, 
                   m_MemberInfo.m_ExtendedMemberInfo.m_pBlock + 
                   SIZEOF_ALIAS, 
                   SIZEOF_UNITS);
    }

    return l_Units;
}

/*
===============================================================================
    Methods to get the alias information for a member
===============================================================================
*/
uint16 CDMOIBData_t::ScaleCoefficient()
{
    uint16 l_ScaleCoefficient = 1;
        
    if(m_MemberInfo.m_ExtendedMemberInfo.m_BlockSize >= (SIZEOF_ALIAS + 
                                                         SIZEOF_UNITS + 
                                                         SIZEOF_SCALE_COEFFICIENT))
    {
        LINE_COPY(&l_ScaleCoefficient, 
                   m_MemberInfo.m_ExtendedMemberInfo.m_pBlock + 
                   SIZEOF_ALIAS + 
                   SIZEOF_UNITS, 
                   SIZEOF_SCALE_COEFFICIENT);
    }

    return l_ScaleCoefficient;
}

/*
===============================================================================
    Methods to get the alias information for a member
===============================================================================
*/
int8 CDMOIBData_t::ScaleExponent()
{
    int8 l_ScaleExponent = 0;
        
    if(m_MemberInfo.m_ExtendedMemberInfo.m_BlockSize >= (SIZEOF_ALIAS + 
                                                         SIZEOF_UNITS + 
                                                         SIZEOF_SCALE_COEFFICIENT + 
                                                         SIZEOF_SCALE_EXPONENT))
    {
        LINE_COPY(&l_ScaleExponent, 
                   m_MemberInfo.m_ExtendedMemberInfo.m_pBlock + 
                   SIZEOF_ALIAS + 
                   SIZEOF_UNITS + 
                   SIZEOF_SCALE_COEFFICIENT, 
                   SIZEOF_SCALE_EXPONENT);
    }

    return l_ScaleExponent;
}

/*
===============================================================================
    Methods to get the alias information for a member
===============================================================================
*/
const ansichar* CDMOIBData_t::Text(uint8 LanguageID)
{
    const ansichar* l_Text = NULL;

    if(m_MemberInfo.m_ExtendedMemberInfo.m_pParent != NULL)
    {
        uint16 l_Offset = (uint16)(this - m_MemberInfo.m_ExtendedMemberInfo.m_pParent->m_pMemberArray);
        l_Text = m_MemberInfo.m_ExtendedMemberInfo.m_pParent->GetMemberAtText(l_Offset, LanguageID);
    }

    return l_Text;
}

/*
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    CDMOIBObjectPtr Class Methods

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
===============================================================================
    Small Accessor class for dealing with pointers to CDMOIBObjects. It is not
    required to use this class for CDMOIBObject* access, but is recommended.
===============================================================================
*/
CDMOIBObjectPtr::CDMOIBObjectPtr(CDMOIBObject *pDMOIBObject) 
: m_pDMOIBObject(pDMOIBObject) 
{
}
CDMOIBObject& CDMOIBObjectPtr::operator*() const 
{
    return *m_pDMOIBObject;
}
CDMOIBObject* CDMOIBObjectPtr::operator->() const 
{
    return m_pDMOIBObject;
}
CDMOIBObjectPtr::operator CDMOIBObject*()
{
    return m_pDMOIBObject;
}
CDMOIBData_t& CDMOIBObjectPtr::operator()(const uint16 MemberID)
{
    return (*m_pDMOIBObject)(MemberID);
}
CDMOIBData_t& CDMOIBObjectPtr::operator[](uint16 Offset)
{
    return (*m_pDMOIBObject)[Offset];
}

/*
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    CDMOIBObject Class Methods

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
===============================================================================
    Constructors
===============================================================================
*/

CDMOIBObject::CDMOIBObject(CDMOIBDictionary* pDictionary, 
                           void* pDefinition, 
                           uint16 Version)
{    
    Construct(pDictionary, 
              pDefinition, 
              DefinitionClassID(pDefinition), 
              Version);
}
CDMOIBObject::CDMOIBObject(CDMOIBDictionary* pDictionary, 
                           uint32 ClassID, 
                           uint16 Version)
{   
    Construct(pDictionary, 
              pDictionary == NULL ? NULL : pDictionary->GetDefinition(ClassID), 
              ClassID, 
              Version);
}
CDMOIBObject::CDMOIBObject(CDMOIBDictionary* pDictionary, 
                           uint16 Family, uint8 Genus, uint8 Species,
                           uint16 Version)
{   
    Construct(pDictionary, 
              pDictionary == NULL ? NULL : pDictionary->GetDefinition(Family, Genus, Species), 
              MAKE_CLASSID(Family, Genus, Species), Version);
}

CDMOIBObject::CDMOIBObject(CDMOIBDictionary* pDictionary, uint8* pBuffer)
{   	
	Construct(pDictionary, 
              NULL, 
			  ANY_CLASS::CLASSID, 
			  HIGHEST_KNOWN_VERSION,
			  pBuffer);

	Deserialize(pBuffer);
}

CDMOIBObject::CDMOIBObject(CDMOIBObject* pObjectCopy)
{    
    if(pObjectCopy != NULL)
    {
        Construct(pObjectCopy->m_pDictionary, 
           (void*)pObjectCopy->m_pDefinition, 
                  pObjectCopy->m_ClassID, 
                  pObjectCopy->m_Version);

        Copy(pObjectCopy);
    }
    else
    {
        // If they gave us a bogus pointer, then create an empty object.
        Construct(NULL, NULL, (uint32)ANY_CLASS::CLASSID, HIGHEST_KNOWN_VERSION);
    }
}

CDMOIBObject::CDMOIBObject(CDMOIBObject& ObjectCopy)
{
    Construct(ObjectCopy.m_pDictionary, 
       (void*)ObjectCopy.m_pDefinition, 
              ObjectCopy.m_ClassID, 
              ObjectCopy.m_Version);

    Copy(&ObjectCopy);
}
/*
===============================================================================
    Destructor
===============================================================================
*/
CDMOIBObject::~CDMOIBObject()
{   
    Destruct();
}

/*
===============================================================================
    Method to Initialize internal members and objects
    [PRIVATE]
===============================================================================
*/
void CDMOIBObject::Construct(CDMOIBDictionary* pDictionary, 
                             void* pDefinition, 
                             uint32 ClassID, 
                             uint16 Version,
							 uint8* pBuffer)
{    
    m_pDictionary           = pDictionary;
    m_pDefinition           = (const byte*)pDefinition;
    m_pDefinitionRef        = m_pDefinition;    
    m_ClassID               = ClassID;
    m_Version               = Version;

    m_pMemberArray          = NULL;
    m_pMemberIter           = NULL;
    m_pMemberEnd            = NULL;
    m_pMemberDynStart       = NULL;
    m_pMemberTranslation    = NULL;
    m_pStringTable          = NULL;
    m_pBaseClass            = NULL;
    m_BaseClassMemberCount  = 0;
    m_MemberCount           = 0;
    m_MemberCountTotal      = 0;
    m_StaticMemberSize      = 0;
    m_DataBitField          = 0;

#if ((TARGET_XML_SUPPORT==XML_DESERIALIZE) || (TARGET_XML_SUPPORT==XML_ALL))
    m_XMLUserData.m_pXMLParent          = NULL;
    m_XMLUserData.m_pXMLObject          = this;
    m_XMLUserData.m_XMLObjectType       = DMOIB_TYPE_OBJECT;    
    m_XMLUserData.m_XMLCurrentIndex     = 0;
    m_XMLUserData.m_XMLContinueParsing  = TRUE;
    m_XMLUserData.m_XMLIsValue          = FALSE;
#endif
    
	uint8*		l_pBufferIter = pBuffer;
	uint8*		l_pDynamicIter = pBuffer;

	// Check to see if we are constructing an object from a binary stream.
	if(pBuffer != NULL && m_pDictionary != NULL)
	{		
        uint8  l_ByteWidth = 0;
		uint32 l_SerialSize = 0;
        uint16 l_StaticMemberSize = 0;
		uint16 l_CRC;		

		// Grab the ClassID from the stream and assign it to this object
		LINE_COPY(&m_ClassID, l_pBufferIter, SIZEOF_CLASSID);

        l_ByteWidth = DMOIBSerialSizeWidth(m_ClassID);
        l_SerialSize = DMOIBSerialSize(pBuffer);

        // Step past the ClassID we just grabbed and the size of the stream...
        l_pBufferIter += SIZEOF_CLASSID + l_ByteWidth; 
        
		// Check the CRC to make sure data is not corrupted.
        l_CRC = CalculateCRC(pBuffer, l_SerialSize - SIZEOF_CRC);
        if(LINE_CHECK(pBuffer + l_SerialSize - SIZEOF_CRC, &l_CRC, SIZEOF_CRC) == FALSE)
        {
            return;
        }

		// Grab the version of the object, and get a matching definition from
		// the dictionary.
        LINE_COPY(&m_Version, l_pBufferIter, SIZEOF_VERSION);

		// Step past the Version we just checked...
        l_pBufferIter += SIZEOF_VERSION; 
        l_pBufferIter += sizeof(uint8); // m_DataBitField
        LINE_COPY(&l_StaticMemberSize, l_pBufferIter, sizeof(uint16));
        l_pBufferIter += sizeof(l_StaticMemberSize);

        l_pDynamicIter = l_pBufferIter + l_StaticMemberSize;

		m_pDefinition = (const byte*)m_pDictionary->GetDefinition(m_ClassID);
	}

    const byte* l_pDefPtr = m_pDefinition;
    const byte* l_pDefEnd = m_pDefinition;
    uint32      l_DefSize = DefinitionSize((void*)m_pDefinition);
    uint16      l_MinVersion;
    uint16      l_MaxVersion;
    uint16      l_Value = 0;
    uint16      l_MemberIndex = 0;
    uint16      l_MemberCount = 0;
    uint8       l_ByteWidth;
    
    // If the Definition doesn't exist, we can't parse it for our structure, 
    // so just skip it.
    if(m_pDefinition != NULL)
    {
        uint32  l_Offset = DefinitionFullSize((void*)m_pDefinition) - sizeof(uint16);
        uint16  l_CRC = CalculateCRC((uint8*)m_pDefinition, l_Offset);

        if(LINE_CHECK(m_pDefinition + l_Offset, &l_CRC, sizeof(uint16)))
        {
            l_pDefPtr += sizeof(uint8)     // Skip Species Count
                       + sizeof(uint16)    // Skip Family
                       + sizeof(uint8)     // Skip Genus
                       + (*l_pDefPtr);     // Skip all the Species

            LINE_COPY(&l_MaxVersion, l_pDefPtr, sizeof(uint16));
            l_pDefPtr += sizeof(uint16);

            // Check to see if the version the developer requested is higher than
            // our greatest known version. If it is, then we force them to use the
            // highest known version. We can't return an error from the 
            // constructor to let them know the version isn't supported, so the 
            // best we can do is create the closest thing to what they want.
            if(m_Version > l_MaxVersion)
                m_Version = l_MaxVersion;
            
            
            l_pDefPtr += sizeof(uint16) // Skip the DefSize
                       + sizeof(uint16) // Skip the Full DefSize
                       + sizeof(uint16);// Skip the Base XML Size
            
            // Grab the total member count (may not match our MemberCount because
            // of version differences)
            LINE_COPY(&m_MemberCountTotal, l_pDefPtr, sizeof(uint16));
            l_pDefPtr += sizeof(uint16);

            // We have to parse the full definition once to see what data members
            // we will be using in this version of the object...
            for(l_MemberCount = 0; l_MemberCount < m_MemberCountTotal; l_MemberCount++)
            {            
                // Check to see if we are greater than the Min Version
                LINE_COPY(&l_MinVersion, l_pDefPtr, sizeof(uint16));
                l_pDefPtr += sizeof(uint16);
                // Check to see if we are less than the Max Version
                LINE_COPY(&l_MaxVersion, l_pDefPtr, sizeof(uint16));
                l_pDefPtr += sizeof(uint16);

                // This member is compatible, so increase our count.
                if(l_MinVersion <= m_Version && m_Version <= l_MaxVersion)
                    m_MemberCount++;

                l_pDefEnd = l_pDefPtr;

                if((*l_pDefEnd & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
                    l_pDefPtr += sizeof(uint32);

                if(*l_pDefEnd & DMOIB_TYPE_FLAG_ARR)
                    l_pDefPtr += sizeof(uint16);

                l_pDefPtr++;
            }

            // Keep track of where the member information ends. This represents
            // the start of Alias/Units/Scale information.
            m_pDefinitionRef = l_pDefPtr;
        }
        else
        {
            // The CRC check failed, so set the definition pointer to NULL
            // so we don't try to use it later.
            m_pDefinition = NULL;
        }
    }

    // If we parsed the definition, then all our values should be accurate for
    // the specified version. If no Definition was available, all the variables
    // will still be in their default states. Either way, we can create our 
    // object. (If no Definition was available, it will just be empty, which is
    // the best we can do since we can't return an error code from the 
    // constructor)

    // Allocate space for the MemberTranslation table and all the members
    // (+1 for the EmptyMember that we'll use for error conditions.)        
    m_pMemberTranslation = new uint16[m_MemberCountTotal];
    m_pMemberArray = new CDMOIBData_t[m_MemberCount + 1];
    
    if(m_pMemberTranslation != NULL && m_pMemberArray != NULL)
    {
        memset(m_pMemberTranslation, 0, m_MemberCountTotal * sizeof(uint16));

        m_pMemberIter   = m_pMemberArray;
        m_pMemberEnd    = m_pMemberArray + m_MemberCount;
        m_pMemberEnd->m_MemberInfo.m_TypeInfo = DMOIB_TYPE_UNKNOWN;        
    }
    else
    {
        // If we couldn't allocate any memory for the most basic and necessary 
        // elements of the class, then the best we can do is revert the 
        // member count to 0 and create an empty MOIB object.
        m_MemberCount = 0;
        l_MemberCount = 0;
    }

    // ...Then parse the full definition a second time to actually fill in
    // our data members. If no Definition is available, this will immediately
    // exit as both l_Value and l_MemberCount will be zero.

    for(l_pDefPtr = SKIP_DEF_HEADER(m_pDefinition),
        l_Value = 0;
        l_Value < l_MemberCount;
        l_Value++)
    {
        // Check to see if we are greater than the Min Version
        LINE_COPY(&l_MinVersion, l_pDefPtr, sizeof(uint16));
        l_pDefPtr += sizeof(uint16);
        // Check to see if we are less than the Max Version
        LINE_COPY(&l_MaxVersion, l_pDefPtr, sizeof(uint16));
        l_pDefPtr += sizeof(uint16);            

        if(l_MinVersion <= m_Version && m_Version <= l_MaxVersion)
        {
            // Index into the padding using it's size. This will push us one past
            // the end of it, putting us right on top of the Type bit-field.
            m_pMemberIter->m_MemberInfo.m_TypeInfo = *l_pDefPtr++;
            l_ByteWidth = (uint8)(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW);

            m_pMemberIter->m_MemberInfo.m_ExtendedMemberInfo.m_BlockSize = (uint16)(((m_pDefinition + l_DefSize) - m_pDefinitionRef) / l_MemberCount);
            m_pMemberIter->m_MemberInfo.m_ExtendedMemberInfo.m_pBlock = (uint8*)(m_pDefinitionRef + (l_Value * m_pMemberIter->m_MemberInfo.m_ExtendedMemberInfo.m_BlockSize));
            m_pMemberIter->m_MemberInfo.m_ExtendedMemberInfo.m_pParent = this;
            
            // Check for special types
            switch(l_ByteWidth)
            {
                case DMOIB_TYPE_FLAG_TIME:
                    // This is a MOIB_TIME struct, so use a specific copy 
                    // function and change the ByteWidth.
                    l_ByteWidth = MOIB_TIME_SIZE-1;
                    break;
                default:
                    break;
            }                    

            if(l_ByteWidth++ == DMOIB_TYPE_FLAG_OBJ)
            {
                uint32 l_ClassID;
				uint16 l_Version = HIGHEST_KNOWN_VERSION;

                LINE_COPY(&l_ClassID, l_pDefPtr, sizeof(uint32));
                l_pDefPtr += sizeof(uint32);

                if(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR || 
                   m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR)
                {
                    if(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR)
                    {
                        LINE_COPY(&m_pMemberIter->m_PointerInfo.m_NumElements, l_pDefPtr, sizeof(uint16));
                        l_pDefPtr += sizeof(uint16);
                    }

					if(l_pBufferIter != NULL)
					{
						// Check to see if the ClassID is a specific one, and
                        // the vector size is dynamic. If both are true, then
                        // we're dealing with a legacy Collection type, and
                        // we have an extra uint16 value before each object
                        // element in the stream we need to handle.
                        boolean l_bSkipCollectionSize = (l_ClassID != (uint32)ANY_CLASS::CLASSID && 
														(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR));
                        uint16	l_SerialSize = 0;
						uint32  l_CurrentClassID = ANY_CLASS::CLASSID;

                        // Grab the number of vector elements in the stream.
                        LINE_COPY(&l_SerialSize, l_pBufferIter, sizeof(uint16));
                        l_pBufferIter += sizeof(uint16);

                        // Iterate through the objects in the stream.
                        while(l_SerialSize-- > 0)
                        {
                            // If this boolean is TRUE, then we are dealing
                            // with a Collection-style vector, and we need
                            // to skip the uint16 value before each object.
                            // (This value is effectively the size of the
                            // following object, but we don't use/need it
                            // because there is a size ID in the object 
                            // itself.
                            if(l_bSkipCollectionSize == TRUE)
                                l_pBufferIter += sizeof(uint16);

							LINE_COPY(&l_CurrentClassID, l_pBufferIter, SIZEOF_CLASSID);
							LINE_COPY(&l_Version, l_pBufferIter + SIZEOF_CLASSID + DMOIBSerialSizeWidth(l_CurrentClassID), SIZEOF_VERSION);

                            // Extract the size from the object and move
                            // ahead that many bytes.
                            l_pBufferIter += DMOIBSerialSize(l_pBufferIter);
                        }
					}
                
                    m_pMemberIter->Ptr = new CDMOIBVector(m_pDictionary, l_ClassID, m_pMemberIter->m_PointerInfo.m_NumElements, l_Version);
                }
                else
                {   
					if(l_pBufferIter != NULL && LINE_CHECK(l_pBufferIter + sizeof(uint32), &l_ClassID, SIZEOF_CLASSID))
					{
						// Move ahead to skip the uint32 in front of the 
                        // subobject.
                        l_pBufferIter += sizeof(uint32);

						m_pMemberIter->m_PointerInfo.m_MemSize = m_pMemberIter->m_PointerInfo.m_NumElements = 1;                  
						m_pMemberIter->Ptr = new CDMOIBObject(m_pDictionary, l_pBufferIter);

						// Get the size of the object out of the buffer
                        l_pBufferIter += DMOIBSerialSize(l_pBufferIter);
					}
					else
					{
						m_pMemberIter->m_PointerInfo.m_MemSize = m_pMemberIter->m_PointerInfo.m_NumElements = 1;                  
						m_pMemberIter->Ptr = new CDMOIBObject(m_pDictionary, l_ClassID);
					}
					
					// If the UNS flag is set, then this is a baseclass object.
					if(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_UNS)
					{
					    SearchMode_t l_SearchMode = m_pDictionary->GetSearchMode();
					
					    m_pBaseClass = (CDMOIBObject*)m_pMemberIter->Ptr;
					    m_pDictionary->SetSearchMode(STATIC_DEFINITIONS_ONLY);
					    m_BaseClassMemberCount = m_pBaseClass->m_BaseClassMemberCount + DefinitionMemberCount(m_pDictionary->GetDefinition(l_ClassID));
					    m_pDictionary->SetSearchMode(l_SearchMode);
					}
                }

                if(m_pMemberDynStart == NULL)
                    m_pMemberDynStart = m_pMemberIter;                    
            }
            else if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR))
            {
                if(m_pMemberDynStart == NULL)
                    m_pMemberDynStart = m_pMemberIter;

				if(l_pBufferIter != NULL)
				{
					uint16 l_DynamicSize = 0;

					LINE_COPY(&l_DynamicSize , l_pBufferIter, sizeof(uint16));
					l_pBufferIter += sizeof(uint16);

					l_pBufferIter += l_DynamicSize * l_ByteWidth;
				}
            }
            else if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR))
            {
                uint16 l_StaticSize = 0;

                LINE_COPY(&l_StaticSize , l_pDefPtr, sizeof(uint16));
                l_pDefPtr += sizeof(uint16);

                m_pMemberIter->Ptr = new byte[l_StaticSize * l_ByteWidth];

                if(m_pMemberIter->Ptr != NULL)
                {
                    m_pMemberIter->m_PointerInfo.m_NumElements = l_StaticSize;
                    m_pMemberIter->m_PointerInfo.m_MemSize = l_StaticSize * l_ByteWidth;
                    m_StaticMemberSize = m_StaticMemberSize + m_pMemberIter->m_PointerInfo.m_MemSize;

                    memset(m_pMemberIter->Ptr, 0, m_pMemberIter->m_PointerInfo.m_MemSize);                    
                }

				if(l_pBufferIter != NULL)
				{
					l_pBufferIter += l_StaticSize * l_ByteWidth;
				}
            }
            else
            {
                m_StaticMemberSize = m_StaticMemberSize + l_ByteWidth;

				if(l_pBufferIter != NULL)
				{
					l_pBufferIter += l_ByteWidth;
				}
            }

            m_pMemberTranslation[l_Value] = l_MemberIndex++;
            m_pMemberIter++;
        }
        else
        {
            l_pDefEnd = l_pDefPtr;

            // If we are an object/array type, we need to skip extra 
            // bytes for the ClassID/Size
            if((*l_pDefEnd & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
                l_pDefPtr += sizeof(uint32);
            
            if(*l_pDefEnd & DMOIB_TYPE_FLAG_ARR)
                l_pDefPtr += sizeof(uint16);
            
            l_pDefPtr++;

            m_pMemberTranslation[l_Value] = m_MemberCount;
        }
    }

    // If we've gotten this far and m_pMemberDynStart is still NULL, it means
    // that there were no dynamic members. In this case, default it to 
    // MemberEnd so when we perform further accesses on this object, we will 
    // know there are no dynamic members.
    if(m_pMemberDynStart == NULL)
        m_pMemberDynStart = m_pMemberEnd;
    
}
/*
===============================================================================
    Method to return the object to an empty state and clean up memory 
    [PRIVATE]
===============================================================================
*/
void CDMOIBObject::Destruct()
{   
    if(m_pMemberTranslation != NULL && m_pMemberArray != NULL)
    {
        while(m_pMemberEnd-- != m_pMemberArray)
        {
            if(m_pMemberEnd->Ptr != NULL)
            {
                if((m_pMemberEnd->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
                {
                    if((m_pMemberEnd->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
                       (m_pMemberEnd->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR))
                    {
                        delete (CDMOIBVector*)m_pMemberEnd->Ptr;
                    }
                    else
                    {
                        delete (CDMOIBObject*)m_pMemberEnd->Ptr;
                    }
                }
                else if((m_pMemberEnd->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR) || 
                        (m_pMemberEnd->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR))
                {
                    delete[] (byte*)m_pMemberEnd->Ptr;
                }
            }
        }
        delete [] m_pMemberTranslation;
        delete [] m_pMemberArray;
    }

    m_pMemberArray       = NULL;
    m_pMemberIter        = NULL;
    m_pMemberEnd         = NULL;
    m_pMemberDynStart    = NULL;
    m_pMemberTranslation = NULL;
    m_MemberCount       = 0;
    m_StaticMemberSize  = 0;
    m_DataBitField      = 0;
    m_ClassID           = 0;
    m_Version           = 0;
}

/*
===============================================================================
    Method to copy all data from another CDMOIBObject to this one.
===============================================================================
*/
boolean CDMOIBObject::Copy(CDMOIBObject* ObjectCopy)
{
    boolean l_RetVal = FALSE;

    if(ObjectCopy != NULL && 
       ObjectCopy->m_ClassID == m_ClassID && 
       ObjectCopy->m_Version == m_Version)
    {
        uint16 l_Index = 0;

        for(m_pMemberIter  = m_pMemberArray, l_RetVal  = TRUE;
            m_pMemberIter != m_pMemberEnd && l_RetVal == TRUE && l_Index < ObjectCopy->m_MemberCount;
            m_pMemberIter++, l_Index++)
        {
            if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
            {
                if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
                   (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR))
                {
                    if(m_pMemberIter->Ptr != NULL)
                    {
                        if(ObjectCopy->m_pMemberArray[l_Index].Ptr != NULL)
                        {
                            l_RetVal = ((CDMOIBVector*)m_pMemberIter->Ptr)->Copy((CDMOIBVector*)ObjectCopy->m_pMemberArray[l_Index].Ptr);
                        }
                        else
                        {
                            ((CDMOIBVector*)m_pMemberIter->Ptr)->Clear();
                        }
                    }
                }
                else
                {
                    if(m_pMemberIter->Ptr != NULL)
                    {
                        if(ObjectCopy->m_pMemberArray[l_Index].Ptr != NULL)
                        {
                            l_RetVal = ((CDMOIBObject*)m_pMemberIter->Ptr)->Copy((CDMOIBObject*)ObjectCopy->m_pMemberArray[l_Index].Ptr);
                        }
                        else
                        {
                            ((CDMOIBObject*)m_pMemberIter->Ptr)->Reset();
                        }
                    }
                }
            }
            else if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
                    (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR))
            {
                l_RetVal = SetBuffer(ObjectCopy->m_pMemberArray[l_Index].Ptr, 
                             (uint8)(ObjectCopy->m_pMemberArray[l_Index].m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW)+1, 
                                    (ObjectCopy->m_pMemberArray[l_Index].m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_TXT) &&
                                    (ObjectCopy->m_pMemberArray[l_Index].m_PointerInfo.m_NumElements > 0) ? 
                                        ObjectCopy->m_pMemberArray[l_Index].m_PointerInfo.m_NumElements - 1 : 
                                        ObjectCopy->m_pMemberArray[l_Index].m_PointerInfo.m_NumElements,
                             (uint8)(ObjectCopy->m_pMemberArray[l_Index].m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP));
            }
            else
            {
                m_pMemberIter->m_MemberInfo = ObjectCopy->m_pMemberArray[l_Index].m_MemberInfo;
            }
        }
    }

    if(l_RetVal == FALSE)
        Reset();

    return l_RetVal;
}

/*
===============================================================================
    Methods for StringTable management
===============================================================================
*/
boolean CDMOIBObject::SetStringTable(CDMOIBStringTable* pStringTable)
{
    m_pStringTable = pStringTable;
    return TRUE;
}

CDMOIBStringTable* CDMOIBObject::GetStringTable()
{
    return m_pStringTable;
}

/*
===============================================================================
    Methods for BaseClass management
===============================================================================
*/
CDMOIBObject* CDMOIBObject::BaseClass()
{
    return m_pBaseClass;
}

CDMOIBObject* CDMOIBObject::Cast(uint32 ClassID)
{
    CDMOIBObject* l_pCompatibleClass = NULL;
        
    // If the ClassID matches our own, then we're compatible.
    if(m_ClassID == ClassID)
    {
        l_pCompatibleClass = this;
    }
    // Otherwise check the tree of baseclasses for a match.
    else if(m_pBaseClass != NULL)
    {
        l_pCompatibleClass = m_pBaseClass->Cast(ClassID);
    }
    
    return l_pCompatibleClass;
}

/*
===============================================================================
    Methods for ClassID information
===============================================================================
*/
uint32 CDMOIBObject::ClassID()
{
    return m_ClassID;
}
uint16 CDMOIBObject::Family()
{
    return GET_FAMILY(m_ClassID);
}
uint8 CDMOIBObject::Genus()
{
    return GET_GENUS(m_ClassID);
}
uint8 CDMOIBObject::Species()
{
    return GET_SPECIES(m_ClassID);
}

/*
===============================================================================
    Methods for Object information
===============================================================================
*/
uint16 CDMOIBObject::Version()
{
    return m_Version;
}
boolean CDMOIBObject::IsDirty()
{
    return (m_DataBitField & DIRTY_FLAG_MASK);
}
boolean CDMOIBObject::SetDirty(boolean Dirty)
{
    if(Dirty == TRUE)
        m_DataBitField |= DIRTY_FLAG_MASK;
    else
        m_DataBitField &= ~DIRTY_FLAG_MASK;

    return Dirty;
}
boolean CDMOIBObject::IsValid()
{
    return (m_DataBitField & VALID_FLAG_MASK);
}
boolean CDMOIBObject::SetValid(boolean Valid)
{
    if(Valid == TRUE)
        m_DataBitField |= VALID_FLAG_MASK;
    else
        m_DataBitField &= ~VALID_FLAG_MASK;

    return Valid;
}

/*
===============================================================================
    Methods for Object control
===============================================================================
*/
void CDMOIBObject::Reset(boolean ResetAll)
{
    m_DataBitField = 0;

    for(m_pMemberIter =  m_pMemberArray; 
        m_pMemberIter != m_pMemberEnd;
        m_pMemberIter++)
    {
        if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ &&
            m_pMemberIter->Ptr != NULL && ResetAll)
        {
            if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
               (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR))
            {
                ((CDMOIBVector*)m_pMemberIter->Ptr)->Clear();
            }
            else
            {
                ((CDMOIBObject*)m_pMemberIter->Ptr)->Reset(ResetAll);
            }
        }
        else if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR) || 
                (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) )
        {
            SetBuffer(NULL, 
                      (uint8)(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW)+1, 
                      0,
                      (uint8)(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP));
        }
        else
        {
            // Reset all data to 0 except the type info
            memset(m_pMemberIter->m_MemberInfo.m_Padding, 0, MAX_MEMBER_SIZE);
        }
    }
}

/*
===============================================================================
    Method to get the total member count
===============================================================================
*/
uint16 CDMOIBObject::MemberCount()
{
    if(m_pBaseClass != NULL)
    {
        // Our member count is the total member count of our baseclasses, -1
        // for the baseclass object itself.
        return m_MemberCount + m_pBaseClass->MemberCount() - 1;
    }
    else
    {
        return m_MemberCount;
    }
}
/*
===============================================================================
    Method to get the ClassName, if it's available
===============================================================================
*/
const ansichar* CDMOIBObject::GetClassText()
{
    return (m_pStringTable == NULL ? NULL : m_pStringTable->GetClassText(m_ClassID));
}
const ansichar* CDMOIBObject::GetClassText(uint8 LanguageID)
{
    return (m_pStringTable == NULL ? NULL : m_pStringTable->GetClassText(m_ClassID, LanguageID));
}
/*
===============================================================================
    Method to get the Definition for this object
===============================================================================
*/
void* CDMOIBObject::GetDefinition()
{
    return (void*)m_pDefinition;
}
/*
===============================================================================
    Method to determine if a the member specified by MemberID is available in
    this version of the object.
===============================================================================
*/
boolean CDMOIBObject::MemberAvailable(const uint16 MemberID)
{
    if(m_pBaseClass && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->MemberAvailable(MemberID);
    }
    else
    {
        FindMember(MemberID);
        return (m_pMemberIter != m_pMemberEnd);
    }
}

/*
===============================================================================
    Method to find a CDMOIBData_t member based on it's unique ID
    [PRIVATE]
===============================================================================
*/
void CDMOIBObject::FindMember(const uint16 MemberID)
{   
    uint16 l_MemberID = MemberID;
        
    if(m_pBaseClass != NULL && l_MemberID >= m_BaseClassMemberCount)
        l_MemberID -= m_BaseClassMemberCount;
            
    if(l_MemberID >= m_MemberCountTotal)
        m_pMemberIter = m_pMemberEnd;
    else
        m_pMemberIter = m_pMemberArray + m_pMemberTranslation[l_MemberID];
}
/*
===============================================================================
    Method to find a CDMOIBData_t member based on an index
    [PRIVATE]
===============================================================================
*/
void CDMOIBObject::FindMemberAt(uint16 Offset)
{  
    if(m_pBaseClass != NULL)
        // -1 for the BaseClass member in this object
        Offset -= (m_pBaseClass->MemberCount() - 1);
              
    if (Offset > m_MemberCount)
        Offset = m_MemberCount;

    m_pMemberIter = m_pMemberArray + Offset;
}
/*
===============================================================================
    Methods to access individual members, either by Offset or their unique ID
===============================================================================
*/
CDMOIBData_t& CDMOIBObject::GetMember(const uint16 MemberID, uint16* Size)
{
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->GetMember(MemberID, Size);
    }
    else
    {       
        if(Size != NULL)
            *Size = GetMemberNumElements(MemberID);
        else
            FindMember(MemberID);

        return *m_pMemberIter;
    }
}

CDMOIBData_t& CDMOIBObject::GetMemberAt(uint16 Offset, uint16* Size)
{
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->GetMemberAt(Offset, Size);
        }
    }

    if(Size != NULL)
        *Size = GetMemberAtNumElements(Offset);
    else
        FindMemberAt(Offset);

    return *m_pMemberIter;
}

/*
===============================================================================
    Methods to access individual members, either by Offset or their unique ID
===============================================================================
*/
CDMOIBData_t& CDMOIBObject::operator()(const uint16 MemberID)
{
    return GetMember(MemberID);
}
    
CDMOIBData_t& CDMOIBObject::operator[](uint16 Offset)
{
    return GetMemberAt(Offset);
}

CDMOIBData_t& CDMOIBObject::Member(const uint16 MemberID)
{
    return GetMember(MemberID);
}
    
CDMOIBData_t& CDMOIBObject::At(uint16 Offset)
{
    return GetMemberAt(Offset);
}

/*
===============================================================================
    Methods to determine the number of elements contained in a member.
===============================================================================
*/
uint16 CDMOIBObject::GetMemberNumElements(const uint16 MemberID)
{
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->GetMemberNumElements(MemberID);
    }
    else
    {
        FindMember(MemberID);

        if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
           (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR) )
            return m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_TXT && 
                   m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR &&
                   m_pMemberIter->m_PointerInfo.m_NumElements > 0 ? 
                   m_pMemberIter->m_PointerInfo.m_NumElements - 1 : 
                   m_pMemberIter->m_PointerInfo.m_NumElements;
        else
            return 1;
    }
}
uint16 CDMOIBObject::GetMemberAtNumElements(uint16 Offset)
{
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->GetMemberAtNumElements(Offset);
        }
    }
       
    FindMemberAt(Offset);

    if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
        (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR) )
        return m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_TXT && 
                m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR &&
                m_pMemberIter->m_PointerInfo.m_NumElements > 0 ? 
                m_pMemberIter->m_PointerInfo.m_NumElements - 1 : 
                m_pMemberIter->m_PointerInfo.m_NumElements;
    else
        return 1;
}
/*
===============================================================================
    Methods to get the type information for a member
===============================================================================
*/
uint8 CDMOIBObject::GetMemberType(const uint16 MemberID)
{
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->GetMemberType(MemberID);
    }
    else
    {
        FindMember(MemberID);
        return m_pMemberIter->m_MemberInfo.m_TypeInfo;
    }
}
uint8 CDMOIBObject::GetMemberAtType(uint16 Offset)
{
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->GetMemberAtType(Offset);
        }
    }

    FindMemberAt(Offset);
    return m_pMemberIter->m_MemberInfo.m_TypeInfo;
}

/*
===============================================================================
    Methods to get the byte size information for a member
===============================================================================
*/
uint16 CDMOIBObject::GetMemberSize(const uint16 MemberID)
{
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->GetMemberSize(MemberID);
    }
    else
    {
        uint16 l_MemberSize = 0;
        uint16 l_MemberID = MemberID;
    
        if(m_pBaseClass != NULL)
            l_MemberID -= m_BaseClassMemberCount;

        // Check to make sure MemberID is within the translation array bounds.
        if(l_MemberID < m_MemberCountTotal)
        {
            // Call GetMemberAtSize() with the extracted offset.
            l_MemberSize = GetMemberAtSize(m_pMemberTranslation[l_MemberID]);
        }
        
        return l_MemberSize;
    }
}
uint16 CDMOIBObject::GetMemberAtSize(uint16 Offset)
{
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->GetMemberAtSize(Offset);
        }
    }

    uint16 l_MemberSize = 0;

    FindMemberAt(Offset);

    if(m_pMemberIter != m_pMemberEnd)
    {
        l_MemberSize = (uint16)(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW);

        switch(l_MemberSize)
        {
            // Check for the 2 special cases of objects or TIME structs.
            case DMOIB_TYPE_FLAG_OBJ:
                l_MemberSize = sizeof(void*);
                break;
            case DMOIB_TYPE_FLAG_TIME:
                l_MemberSize = sizeof(MOIB_TIME);
                break;
            // If it's not a special case, then the byte-size is encoded, so decode
            // it by incrementing the masked value by 1.
            default:
                l_MemberSize++;
                break;
        }
    }

    return l_MemberSize;
}

/*
===============================================================================
    Methods to get the alias information for a member
===============================================================================
*/
uint32 CDMOIBObject::GetMemberAlias(const uint16 MemberID)
{
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->GetMemberAlias(MemberID);
    }
    else
    {
        uint16 l_MemberID       = MemberID;
        uint32 l_Alias          = OMITTED;
        uint32 l_DefSize        = DefinitionSize((void*)m_pDefinition);
        uint16 l_MemberCount    = DefinitionMemberCount((void*)m_pDefinition);
        
        if(m_pBaseClass != NULL)
            l_MemberID -= m_BaseClassMemberCount;
        
        // If the DefSize extends past the current m_pDefinitionRef, then there
        // is extra information available (Alias/Units/Scale).
        if((m_pDefinitionRef <= (m_pDefinition + l_DefSize - (l_MemberCount * SIZEOF_ALIAS))) && 
           (l_MemberID < l_MemberCount))
        {
            uint32      l_InfoBlockSize = (uint32)(((m_pDefinition + l_DefSize) - m_pDefinitionRef) / l_MemberCount);
            const byte* l_pAliasRef = m_pDefinitionRef + (l_MemberID * l_InfoBlockSize);

            LINE_COPY(&l_Alias, l_pAliasRef, SIZEOF_ALIAS);
        }

        return l_Alias;
    }
}
uint32 CDMOIBObject::GetMemberAtAlias(uint16 Offset)
{
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->GetMemberAtAlias(Offset);
        }
    }
     
    if(m_pBaseClass != NULL)
        // -1 for the BaseClass member in this object
        Offset -= (m_pBaseClass->MemberCount() - 1);
        
    uint16 l_MemberCount    = DefinitionMemberCount((void*)m_pDefinition);
    uint16 l_MemberID       = Offset;
        
    for(;(l_MemberID < l_MemberCount) && (m_pMemberTranslation[l_MemberID] != Offset); l_MemberID++);

    if(m_pBaseClass != NULL)
        l_MemberID += m_BaseClassMemberCount;
        
    return GetMemberAlias(l_MemberID);
}

/*
===============================================================================
    Methods to get the units information for a member
===============================================================================
*/
uint8 CDMOIBObject::GetMemberUnits(const uint16 MemberID)
{
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->GetMemberUnits(MemberID);
    }
    else
    {
        uint16 l_MemberID       = MemberID;
        uint8  l_Units          = (uint8)(OMITTED & 0x0FF);
        uint32 l_DefSize        = DefinitionSize((void*)m_pDefinition);
        uint16 l_MemberCount    = DefinitionMemberCount((void*)m_pDefinition);
        
        if(m_pBaseClass != NULL)
            l_MemberID -= m_BaseClassMemberCount;
        
        // If the DefSize extends past the current m_pDefinitionRef, then there
        // is extra information available (Alias/Units/Scale).
        if((m_pDefinitionRef <= (m_pDefinition + l_DefSize - (l_MemberCount * (SIZEOF_ALIAS + 
                                                                               SIZEOF_UNITS)))) && 
           (l_MemberID < l_MemberCount))
        {
            uint32      l_InfoBlockSize = (uint32)(((m_pDefinition + l_DefSize) - m_pDefinitionRef) / l_MemberCount);
            const byte* l_pUnitsRef = m_pDefinitionRef + (l_MemberID * l_InfoBlockSize) + SIZEOF_ALIAS;

            LINE_COPY(&l_Units, l_pUnitsRef, SIZEOF_UNITS);
        }

        return l_Units;
    }
}
uint8 CDMOIBObject::GetMemberAtUnits(uint16 Offset)
{
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->GetMemberAtUnits(Offset);
        }
    }

    if(m_pBaseClass != NULL)
        // -1 for the BaseClass member in this object
        Offset -= (m_pBaseClass->MemberCount() - 1);
        
    uint16 l_MemberCount    = DefinitionMemberCount((void*)m_pDefinition);
    uint16 l_MemberID       = Offset;
        
    for(;(l_MemberID < l_MemberCount) && (m_pMemberTranslation[l_MemberID] != Offset); l_MemberID++);

    if(m_pBaseClass != NULL)
        l_MemberID += m_BaseClassMemberCount;
        
    return GetMemberUnits(l_MemberID);
}

/*
===============================================================================
    Methods to get the scale coefficient information for a member
===============================================================================
*/
uint16 CDMOIBObject::GetMemberScaleCoefficient(const uint16 MemberID)
{
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->GetMemberScaleCoefficient(MemberID);
    }
    else
    {
        uint16 l_MemberID       = MemberID;
        uint16 l_ScaleC         = 1;
        uint32 l_DefSize        = DefinitionSize((void*)m_pDefinition);
        uint16 l_MemberCount    = DefinitionMemberCount((void*)m_pDefinition);
        
        if(m_pBaseClass != NULL)
            l_MemberID -= m_BaseClassMemberCount;
        
        // If the DefSize extends past the current m_pDefinitionRef, then there
        // is extra information available (Alias/Units/Scale).
        if((m_pDefinitionRef <= (m_pDefinition + l_DefSize - (l_MemberCount * (SIZEOF_ALIAS + 
                                                                               SIZEOF_UNITS + 
                                                                               SIZEOF_SCALE_COEFFICIENT)))) && 
           (l_MemberID < l_MemberCount))
        {
            uint32      l_InfoBlockSize = (uint32)(((m_pDefinition + l_DefSize) - m_pDefinitionRef) / l_MemberCount);
            const byte* l_pScaleCRef = m_pDefinitionRef + (l_MemberID * l_InfoBlockSize) + 
                                                          SIZEOF_ALIAS + 
                                                          SIZEOF_UNITS;

            LINE_COPY(&l_ScaleC, l_pScaleCRef, SIZEOF_SCALE_COEFFICIENT);
        }

        return l_ScaleC;
    }
}
uint16 CDMOIBObject::GetMemberAtScaleCoefficient(uint16 Offset)
{
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->GetMemberAtScaleCoefficient(Offset);
        }
    }

    if(m_pBaseClass != NULL)
        // -1 for the BaseClass member in this object
        Offset -= (m_pBaseClass->MemberCount() - 1);
        
    uint16 l_MemberCount    = DefinitionMemberCount((void*)m_pDefinition);
    uint16 l_MemberID       = Offset;
        
    for(;(l_MemberID < l_MemberCount) && (m_pMemberTranslation[l_MemberID] != Offset); l_MemberID++);

    if(m_pBaseClass != NULL)
        l_MemberID += m_BaseClassMemberCount;
        
    return GetMemberScaleCoefficient(l_MemberID);
}

/*
===============================================================================
    Methods to get the scale exponent information for a member
===============================================================================
*/
int8 CDMOIBObject::GetMemberScaleExponent(const uint16 MemberID)
{
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->GetMemberScaleExponent(MemberID);
    }
    else
    {
        uint16 l_MemberID       = MemberID;
        int8   l_ScaleE         = 0;
        uint32 l_DefSize        = DefinitionSize((void*)m_pDefinition);
        uint16 l_MemberCount    = DefinitionMemberCount((void*)m_pDefinition);
        
        if(m_pBaseClass != NULL)
            l_MemberID -= m_BaseClassMemberCount;
        
        // If the DefSize extends past the current m_pDefinitionRef, then there
        // is extra information available (Alias/Units/Scale).
        if((m_pDefinitionRef <= (m_pDefinition + l_DefSize - (l_MemberCount * (SIZEOF_ALIAS + 
                                                                               SIZEOF_UNITS + 
                                                                               SIZEOF_SCALE_COEFFICIENT + 
                                                                               SIZEOF_SCALE_EXPONENT)))) && 
           (l_MemberID < l_MemberCount))
        {
            uint32      l_InfoBlockSize = (uint32)(((m_pDefinition + l_DefSize) - m_pDefinitionRef) / l_MemberCount);
            const byte* l_pScaleERef = m_pDefinitionRef + (l_MemberID * l_InfoBlockSize) + 
                                                          SIZEOF_ALIAS + 
                                                          SIZEOF_UNITS + 
                                                          SIZEOF_SCALE_COEFFICIENT;

            LINE_COPY(&l_ScaleE, l_pScaleERef, SIZEOF_SCALE_EXPONENT);
        }

        return l_ScaleE;
    }
}
int8 CDMOIBObject::GetMemberAtScaleExponent(uint16 Offset)
{
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->GetMemberAtScaleExponent(Offset);
        }
    }

    if(m_pBaseClass != NULL)
        // -1 for the BaseClass member in this object
        Offset -= (m_pBaseClass->MemberCount() - 1);
        
    uint16 l_MemberCount    = DefinitionMemberCount((void*)m_pDefinition);
    uint16 l_MemberID       = Offset;
        
    for(;(l_MemberID < l_MemberCount) && (m_pMemberTranslation[l_MemberID] != Offset); l_MemberID++);

    if(m_pBaseClass != NULL)
        l_MemberID += m_BaseClassMemberCount;
        
    return GetMemberScaleExponent(l_MemberID);
}

/*
===============================================================================
    Methods to access descriptions for members, either by Offset or their ID
===============================================================================
*/
const ansichar* CDMOIBObject::GetMemberText(const uint16 MemberID, uint8 LanguageID)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->GetMemberText(MemberID, LanguageID);
    }
    else
    {        
        return (m_pStringTable == NULL ? NULL : m_pStringTable->GetMemberString(m_ClassID, MemberID, LanguageID));
    }
}
const ansichar* CDMOIBObject::GetMemberAtText(uint16 Offset, uint8 LanguageID)
{
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->GetMemberAtText(Offset);
        }
    }

    if(m_pBaseClass != NULL)
        // -1 for the BaseClass member in this object
        Offset -= (m_pBaseClass->MemberCount() - 1);
    
    if(m_pStringTable != NULL && Offset < m_MemberCount)
    {
        uint16 l_MemberID = Offset;
        
        while(m_pMemberTranslation[l_MemberID] != Offset) l_MemberID++;

        return m_pStringTable->GetMemberString(m_ClassID, l_MemberID, LanguageID);
    }
    else
    {
        return NULL;
    }
}

/*
===============================================================================
    Methods to set individual members by their unique ID
===============================================================================
    Signed integer values (All go through SetMember(int32) helper)
===============================================================================
*/
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const int8 Value)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, Value);
        }
    }
    
    FindMemberAt(Offset);
    return SetMember((int32)Value);
}
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const int16 Value)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, Value);
        }
    }

    FindMemberAt(Offset);
    return SetMember((int32)Value);
}
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const int32 Value)
{  
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, Value);
        }
    }

    FindMemberAt(Offset);
    return SetMember((int32)Value);
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const int8 Value)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, Value);
    }
    else
    {
        FindMember(MemberID);
        return SetMember((int32)Value);
    }
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const int16 Value)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, Value);
    }
    else
    {
        FindMember(MemberID);
        return SetMember((int32)Value);
    }
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const int32 Value)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, Value);
    }
    else
    {
        FindMember(MemberID);
        return SetMember((int32)Value);
    }
}
/*
===============================================================================
    Method to assign signed integer values to a member
    [PRIVATE]
===============================================================================
*/
boolean CDMOIBObject::SetMember(const int32 Value)
{   
    if(m_pMemberIter != m_pMemberEnd && IS_BASIC_TYPE(m_pMemberIter->m_MemberInfo.m_TypeInfo))
    {
        memset(m_pMemberIter->m_MemberInfo.m_Padding, 0, MAX_MEMBER_SIZE);

        if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP)) 
        { 
            if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) > 4) 
                m_pMemberIter->Float64 = (float64)Value; 
            else 
                m_pMemberIter->Float32 = (float32)Value; 
        } 
        else 
        { 
            // We just copy over the integer data for the size of the data 
            // member. We'll worry about sign extending it later in the Get()
            // method
            BYTE_COPY(&m_pMemberIter->Int32, (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW)+1, &Value, sizeof(Value)); 
        }

        return TRUE;
    }
    else
    {
        return FALSE;
    }
}
/*
===============================================================================
    Unsigned integer values (All go through SetMember(uint32) helper)
===============================================================================
*/
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const ansichar Value)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, Value);
        }
    }

    FindMemberAt(Offset);
    return SetMember((uint32)Value);
}
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const uint8 Value)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, Value);
        }
    }

    FindMemberAt(Offset);
    return SetMember((uint32)Value);
}
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const uint16 Value)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, Value);
        }
    }

    FindMemberAt(Offset);
    return SetMember((uint32)Value);
}
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const uint32 Value)
{  
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, Value);
        }
    }

    FindMemberAt(Offset);
    return SetMember((uint32)Value);
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const ansichar Value)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, Value);
    }
    else
    {
        FindMember(MemberID);
        return SetMember((uint32)Value);
    }
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const uint8 Value)
{
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, Value);
    }
    else
    {
        FindMember(MemberID);
        return SetMember((uint32)Value);
    }
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const uint16 Value)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, Value);
    }
    else
    {
        FindMember(MemberID);
        return SetMember((uint32)Value);
    }
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const uint32 Value)
{
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, Value);
    }
    else
    {
        FindMember(MemberID);
        return SetMember((uint32)Value);
    }
}
/*
===============================================================================
    Method to assign unsigned integer values to a member
    [PRIVATE]
===============================================================================
*/
boolean CDMOIBObject::SetMember(const uint32 Value)
{
    if(m_pMemberIter != m_pMemberEnd && IS_BASIC_TYPE(m_pMemberIter->m_MemberInfo.m_TypeInfo))
    {
        memset(m_pMemberIter->m_MemberInfo.m_Padding, 0, MAX_MEMBER_SIZE);

        if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP)) 
        { 
            if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) > 4) 
                m_pMemberIter->Float64 = (float64)Value; 
            else 
                m_pMemberIter->Float32 = (float32)Value; 
        } 
        else 
        { 
            // We just copy over the integer data for the size of the data 
            // member. We'll worry about sign extending it later in the Get()
            // method
            BYTE_COPY(&m_pMemberIter->Uint32, (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW)+1, &Value, sizeof(Value)); 
        }

        return TRUE;
    }
    else
    {
        return FALSE;
    }
}
/*
===============================================================================
    Floating point values (All go through SetMember(float64) helper)
===============================================================================
*/
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const float32 Value)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, Value);
        }
    }

    FindMemberAt(Offset);
    return SetMember((float64)Value);
}
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const float64 Value)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, Value);
        }
    }

    FindMemberAt(Offset);
    return SetMember((float64)Value);
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const float32 Value)
{ 
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, Value);
    }
    else
    {
        FindMember(MemberID);
        return SetMember((float64)Value);
    }
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const float64 Value)
{
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, Value);
    }
    else
    {
        FindMember(MemberID);
        return SetMember((float64)Value);
    }
}
/*
===============================================================================
    Method to assign floating point values to a member
    [PRIVATE]
===============================================================================
*/
boolean CDMOIBObject::SetMember(const float64 Value)
{
    if(m_pMemberIter != m_pMemberEnd && IS_BASIC_TYPE(m_pMemberIter->m_MemberInfo.m_TypeInfo))
    {
        memset(m_pMemberIter->m_MemberInfo.m_Padding, 0, MAX_MEMBER_SIZE);
        
        if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP)) 
        { 
            if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) > 4) 
                m_pMemberIter->Float64 = (float64)Value; 
            else 
                m_pMemberIter->Float32 = (float32)Value; 
        } 
        else if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_UNS)) 
        { 
            uint32 l_CastValue = (uint32)Value; 
            BYTE_COPY(m_pMemberIter, (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW)+1, &l_CastValue, sizeof(uint32)); 
        } 
        else 
        { 
            int32 l_CastValue = (int32)Value; 
            BYTE_COPY(m_pMemberIter, (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW)+1, &l_CastValue, sizeof(uint32)); 
        } 

        return TRUE;
    }
    else
    {
        return FALSE;
    }
}
/*
===============================================================================
    MOIB_TIME structure values (All go through SetMember(MOIB_TIME&) helper)
===============================================================================
*/
boolean CDMOIBObject::SetMember(const uint16 MemberID, const MOIB_TIME& Value)
{
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, Value);
    }
    else
    {
        FindMember(MemberID);
        return SetMember(Value);
    }
}

boolean CDMOIBObject::SetMemberAt(uint16 Offset, const MOIB_TIME& Value)
{
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, Value);
        }
    }

    FindMemberAt(Offset);
    return SetMember(Value);
}
/*
===============================================================================
    Method to assign MOIB_TIME structure values to a member
    [PRIVATE]
===============================================================================
*/
boolean CDMOIBObject::SetMember(const MOIB_TIME& Value)
{
    if((m_pMemberIter != m_pMemberEnd) && 
       (m_pMemberIter->m_MemberInfo.m_TypeInfo == DMOIB_TYPE_TIME))
    {     
        m_pMemberIter->Time = Value;
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

/*
===============================================================================
    All array types (All go through SetBuffer() helper)
===============================================================================
*/
boolean CDMOIBObject::SetMember(const uint16 MemberID, const int8* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, pValue, Size);
    }
    else
    {
        FindMember(MemberID);
        return SetBuffer((void*)pValue, sizeof(*pValue), Size);
    }
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const int16* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, pValue, Size);
    }
    else
    {
        FindMember(MemberID);
        return SetBuffer((void*)pValue, sizeof(*pValue), Size);
    }
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const int32* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, pValue, Size);
    }
    else
    {
        FindMember(MemberID);
        return SetBuffer((void*)pValue, sizeof(*pValue), Size);
    }
}

boolean CDMOIBObject::SetMember(const uint16 MemberID, const uint8* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, pValue, Size);
    }
    else
    {
        FindMember(MemberID);
        return SetBuffer((void*)pValue, sizeof(*pValue), Size);
    }
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const uint16* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, pValue, Size);
    }
    else
    {
        FindMember(MemberID);
        return SetBuffer((void*)pValue, sizeof(*pValue), Size);
    }
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const uint32* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, pValue, Size);
    }
    else
    {
        FindMember(MemberID);
        return SetBuffer((void*)pValue, sizeof(*pValue), Size);
    }
}

boolean CDMOIBObject::SetMember(const uint16 MemberID, const float32* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, pValue, Size);
    }
    else
    {
        FindMember(MemberID);
        return SetBuffer((void*)pValue, sizeof(*pValue), Size, DMOIB_TYPE_FLAG_FP);
    }
}
boolean CDMOIBObject::SetMember(const uint16 MemberID, const float64* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, pValue, Size);
    }
    else
    {
        FindMember(MemberID);
        return SetBuffer((void*)pValue, sizeof(*pValue), Size, DMOIB_TYPE_FLAG_FP);
    }
}

boolean CDMOIBObject::SetMember(const uint16 MemberID, const ansichar* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, pValue, Size);
    }
    else
    {
        FindMember(MemberID);
        return SetBuffer((void*)pValue, sizeof(*pValue), Size);
    }  
}

boolean CDMOIBObject::SetMember(uint16 MemberID, const MOIB_TIME* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL && MemberID < m_BaseClassMemberCount)
    {
        return m_pBaseClass->SetMember(MemberID, pValue, Size);
    }
    else
    {
        FindMember(MemberID);
        return SetBuffer((void*)pValue, DMOIB_TYPE_FLAG_TIME + 1, Size);    
    }
}
// ----- By Offset---------------------
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const int8* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, pValue, Size);
        }
    }

    FindMemberAt(Offset);
    return SetBuffer((void*)pValue, sizeof(*pValue), Size);
}
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const int16* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, pValue, Size);
        }
    }

    FindMemberAt(Offset);
    return SetBuffer((void*)pValue, sizeof(*pValue), Size);
}
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const int32* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, pValue, Size);
        }
    }

    FindMemberAt(Offset);
    return SetBuffer((void*)pValue, sizeof(*pValue), Size);
}

boolean CDMOIBObject::SetMemberAt(uint16 Offset, const uint8* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, pValue, Size);
        }
    }

    FindMemberAt(Offset);
    return SetBuffer((void*)pValue, sizeof(*pValue), Size);
}
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const uint16* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, pValue, Size);
        }
    }

    FindMemberAt(Offset);
    return SetBuffer((void*)pValue, sizeof(*pValue), Size);
}
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const uint32* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, pValue, Size);
        }
    }

    FindMemberAt(Offset);
    return SetBuffer((void*)pValue, sizeof(*pValue), Size);
}

boolean CDMOIBObject::SetMemberAt(uint16 Offset, const float32* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, pValue, Size);
        }
    }

    FindMemberAt(Offset);
    return SetBuffer((void*)pValue, sizeof(*pValue), Size, DMOIB_TYPE_FLAG_FP);
}
boolean CDMOIBObject::SetMemberAt(uint16 Offset, const float64* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, pValue, Size);
        }
    }

    FindMemberAt(Offset);
    return SetBuffer((void*)pValue, sizeof(*pValue), Size, DMOIB_TYPE_FLAG_FP);
}

boolean CDMOIBObject::SetMemberAt(uint16 Offset, const ansichar* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, pValue, Size);
        }
    }

    FindMemberAt(Offset);
    return SetBuffer((void*)pValue, sizeof(*pValue), Size);
}

boolean CDMOIBObject::SetMemberAt(uint16 Offset, const MOIB_TIME* pValue, uint16 Size)
{   
    if(m_pBaseClass != NULL)
    {
        if(Offset < m_pBaseClass->MemberCount())
        {
            return m_pBaseClass->SetMemberAt(Offset, pValue, Size);
        }
    }

    FindMemberAt(Offset);
    return SetBuffer((void*)pValue, DMOIB_TYPE_FLAG_TIME + 1, Size);
}

/*
===============================================================================
    Method to copy a generic buffer of data to the selected member
    [PRIVATE]

    Possible argument cases:
        ValueSize and IsFP are determined automatically, so pValue (pV) and
        ValueLength (VL) are the only variables from outside sources.

                Inputs                      Result
        -----------------------------------------------------------------------
        pV == NULL | VL == 0 | Reset buffer to NULL and 0 (static buffer reset)
        pV == NULL | VL  > 0 | Reset buffer to empty array of VL length ( '' )
        pV != NULL | VL == 0 | FAILURE
        pV != NULL | VL  > 0 | Copy pV to buffer by VL length (FAILURE if VL
                   |         | is larger than static buffer size)
===============================================================================
*/
boolean CDMOIBObject::SetBuffer(void* pValue, uint8 ValueSize, uint16 ValueLength, uint8 IsFP)
{
    boolean l_RetVal = FALSE;

    if(  m_pMemberIter != m_pMemberEnd && 
       ((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP) == IsFP) &&
       ((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW)+1) == ValueSize &&
       !(pValue != NULL && ValueLength == 0))
    { 
        // Check for special types
        switch(ValueSize)
        {
            case (DMOIB_TYPE_FLAG_TIME + 1):
                // This is a MOIB_TIME struct, so use a specific copy 
                // function and change the ByteWidth.
                ValueSize = sizeof(MOIB_TIME);
                break;
            default:
                break;
        }            

        // We are dealing with a buffer that has the same bytewidth as our
        // member, so we can copy the data properly.
                
        // ARRAY TYPE
        if( (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) && 
            m_pMemberIter->Ptr != NULL && 
            m_pMemberIter->m_PointerInfo.m_MemSize > 0)
        {
            // If we're dealing with text, then we have to leave the last
            // element of the array as 0 for the NULL terminator. So check if
            // it is text, and set our copy length to one less than the actual
            // length to account for the extra element.
            uint16 Length = ((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_TXT) ? ValueLength+1 : ValueLength);

            // The only failure condition here is if the copy length is too
            // large for the static buffer. Check to make sure it's within
            // limits.
            if(Length <= m_pMemberIter->m_PointerInfo.m_NumElements)
            {
                l_RetVal = TRUE;

                // We are an array type, so zero our buffer by default
                memset(m_pMemberIter->Ptr, 0, m_pMemberIter->m_PointerInfo.m_MemSize);
             
                // If we have enough space available to copy the entire buffer, 
                // then copy it. Otherwise it will remain zeroed out.
                if(pValue != NULL && ValueLength > 0)
                {
                    memcpy(m_pMemberIter->Ptr, pValue, (ValueLength * ValueSize));
                }
            }            
        }
        // POINTER TYPE
        else if(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR)
        {
            // If we are dealing with text, add an extra element for the NULL 
            // terminator.
            uint16 Length = ((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_TXT) ? ValueLength+1 : ValueLength);
            
#if (defined(DMOIB_FORCE_ARRAY_RESIZE) && (DMOIB_FORCE_ARRAY_RESIZE == 1))
            if(m_pMemberIter->Ptr != NULL)
#else
            if(m_pMemberIter->Ptr != NULL &&
              (m_pMemberIter->m_PointerInfo.m_MemSize < (Length * ValueSize) || 
               ValueLength == 0))               
#endif       
            {
                // If we have memory allocated already, but it isn't large 
                // enough to fit the pValue buffer, then delete it and set the
                // pointer to NULL.
                delete [] (byte*)m_pMemberIter->Ptr;
                m_pMemberIter->Ptr = NULL;
                m_pMemberIter->m_PointerInfo.m_MemSize = 0;
                m_pMemberIter->m_PointerInfo.m_NumElements = 0;
            }

            // In the case where the array is reset to empty (NULL), the proper
            // return value is TRUE. Check to see if the arguments specifed
            // are NULL and 0 and the internal array matches it as NULL and 0.
            if(m_pMemberIter->Ptr == NULL && pValue == NULL &&
               m_pMemberIter->m_PointerInfo.m_MemSize == 0 && ValueLength == 0)
            {
                l_RetVal = TRUE;
            }
            else if(ValueLength > 0)
            {
                if(m_pMemberIter->Ptr == NULL)
                {
                    // Either we haven't allocated any memory for this member 
                    // yet, or the memory was freed above. Either way, we 
                    // allocate new memory.
                    m_pMemberIter->m_PointerInfo.m_MemSize = (Length * ValueSize);
                    m_pMemberIter->Ptr = new byte[m_pMemberIter->m_PointerInfo.m_MemSize];
                }
                
                if(m_pMemberIter->Ptr != NULL)
                {
                    l_RetVal = TRUE;

                    // Either we are reusing the memory allocated earlier, or we have 
                    // new memory from the call above. Zero out the buffer and save
                    // the NumElements.                    
                    m_pMemberIter->m_PointerInfo.m_NumElements = Length;
                    memset(m_pMemberIter->Ptr, 0, m_pMemberIter->m_PointerInfo.m_MemSize);
                    
                    // If we were passed NULL but ValueLength > 0, then the 
                    // intention was to create a new, empty buffer, so don't
                    // do a memcpy.
                    if(pValue != NULL)
                    {
                        memcpy(m_pMemberIter->Ptr, pValue, (ValueLength * ValueSize));
                    }
                }
                else
                {
                    m_pMemberIter->m_PointerInfo.m_MemSize = 0;
                }
            }
        }
    }
    
    return l_RetVal;
}
/*
===============================================================================
    Method to get the size this object will take up in a serial bytestream
===============================================================================
*/
uint32 CDMOIBObject::SizeAll(uint16 Version)
{
    uint32 l_SerialSize = 
              sizeof(uint32)        /* ClassID */
              /* Size of the total serialized object */
            + DMOIBSerialSizeWidth(m_ClassID) 
            + sizeof(uint16)        /* Version */
            + sizeof(byte)          /* is dirty flag */
            + sizeof(uint16)        /* Size of the static data */
            /* Static data buffer space is calculated dynamically below */
            + sizeof(uint16);       /* CRC bytes */

    const byte* l_pDefPtr = SKIP_DEF_HEADER(m_pDefinition);
    uint16 l_MinVersion = 0;
    uint16 l_MaxVersion = 0;
    uint8  l_ByteWidth;        
    uint8  l_TypeInfo;

    boolean l_InVersion = FALSE;

    if(Version == HIGHEST_KNOWN_VERSION)
    {
        Version = m_Version;
    }

    for(uint16 l_MemberIter = 0;
        l_MemberIter < m_MemberCountTotal;
        l_pDefPtr++, l_MemberIter++)
    {
        LINE_COPY(&l_MinVersion, l_pDefPtr, sizeof(uint16));
        l_pDefPtr += sizeof(uint16);
        LINE_COPY(&l_MaxVersion, l_pDefPtr, sizeof(uint16));
        l_pDefPtr += sizeof(uint16); 
        
        // Boolean to tell us if the member is available
        l_InVersion = (l_MinVersion <= Version && Version <= l_MaxVersion);
        l_ByteWidth = (uint8)(*l_pDefPtr & DMOIB_TYPE_MASK_BW);
        l_TypeInfo = *l_pDefPtr;

        if(l_InVersion)
        {
            m_pMemberIter = m_pMemberArray + m_pMemberTranslation[l_MemberIter];
        
            if((l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
            {
                if(m_pMemberIter->Ptr == NULL)
                {
                    // If we have a sub-object or vector member that doesn't exist,
                    // then we can't serialize properly. So just set the size to 0,
                    // and break to return early.
                    l_SerialSize = 0;
                    break;
                }
                else
                {
                    if((l_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
                       (l_TypeInfo & DMOIB_TYPE_FLAG_PTR))
                    {
                        l_SerialSize += ((CDMOIBVector*)m_pMemberIter->Ptr)->SizeAll();
                    }
                    else
                    {
                        l_SerialSize += ((CDMOIBObject*)m_pMemberIter->Ptr)->SizeAll() + sizeof(uint32);
                    }
                }            
            }
            else if((l_TypeInfo & DMOIB_TYPE_FLAG_PTR))
            {
                // Check for special types
                switch(l_TypeInfo & DMOIB_TYPE_MASK_BW)
                {
                    case DMOIB_TYPE_FLAG_TIME:
                        l_SerialSize += (m_pMemberIter->m_PointerInfo.m_NumElements * MOIB_TIME_SIZE)
                                        + sizeof(uint16);  
                        break;
                    default:
                        l_SerialSize += (m_pMemberIter->m_PointerInfo.m_NumElements *  
                                        ((l_TypeInfo & DMOIB_TYPE_MASK_BW)+1))
                                        + sizeof(uint16);
                        break;
                }
            }
            else if((l_TypeInfo & DMOIB_TYPE_FLAG_ARR))
            {
                uint16 l_ArrayLength = 0;
                
                LINE_COPY(&l_ArrayLength, (l_pDefPtr+1), SIZEOF_ARRAY_SIZE);
                
                // Check for special types
                switch(l_TypeInfo & DMOIB_TYPE_MASK_BW)
                {
                    case DMOIB_TYPE_FLAG_TIME:
                        l_SerialSize += (l_ArrayLength * MOIB_TIME_SIZE);  
                        break;
                    default:
                        l_SerialSize += (l_ArrayLength * ((l_TypeInfo & DMOIB_TYPE_MASK_BW)+1));
                        break;
                }
            }
            else
            {
                // Check for special types
                switch(l_TypeInfo & DMOIB_TYPE_MASK_BW)
                {
                    case DMOIB_TYPE_FLAG_TIME:
                        l_SerialSize += MOIB_TIME_SIZE;  
                        break;
                    default:
                        l_SerialSize += ((l_TypeInfo & DMOIB_TYPE_MASK_BW)+1);
                        break;
                }
            }
        }
        
        if(l_TypeInfo & DMOIB_TYPE_FLAG_ARR)
            l_pDefPtr += sizeof(uint16);

        if((l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
            l_pDefPtr += sizeof(uint32);
    }

    return l_SerialSize;
}

/*
===============================================================================
    Method to put the object into a serial bytestream
===============================================================================
*/
uint32 CDMOIBObject::Serialize(uint8* pBuffer, uint16 Version)
{
    uint8* l_pBufferIter = pBuffer;
    uint8* l_pDynamicIter = pBuffer;
    uint32 l_SerialSize = 0;
    uint16 l_CRC = 0;
    uint8  l_ByteWidth;
    uint8  l_SizeWidth = DMOIBSerialSizeWidth(m_ClassID);
    uint16 l_StaticMemberSize = 0;
    
    const byte* l_pDefPtr = SKIP_DEF_HEADER(m_pDefinition);
    uint16 l_MinVersion = 0;
    uint16 l_MaxVersion = 0;    
    uint8  l_TypeInfo;
    uint16 l_MemberIter;   

    boolean l_InVersion = FALSE;
    
    if(pBuffer != NULL)
    {
        if(Version == HIGHEST_KNOWN_VERSION)
        {
            Version = m_Version;
        }
        
        /* Run through the definition once to precalculate the static member
           sizes */
        for(l_MemberIter = 0; //m_pMemberIter =  m_pMemberArray;
            l_MemberIter < m_MemberCountTotal; //m_pMemberIter != m_pMemberEnd;
            l_pDefPtr++, l_MemberIter++)
        {
            LINE_COPY(&l_MinVersion, l_pDefPtr, sizeof(uint16));
            l_pDefPtr += sizeof(uint16);
            LINE_COPY(&l_MaxVersion, l_pDefPtr, sizeof(uint16));
            l_pDefPtr += sizeof(uint16); 
            
            // Boolean to tell us if the member is available
            l_InVersion = (l_MinVersion <= Version && Version <= l_MaxVersion);
            l_TypeInfo = *l_pDefPtr;
            
            // Only calculate the size if it's not an object type, and not
            // a dynamic array.
            if((l_TypeInfo & DMOIB_TYPE_MASK_BW) != DMOIB_TYPE_FLAG_OBJ &&
              !(l_TypeInfo & DMOIB_TYPE_FLAG_PTR) && 
                l_InVersion)
            {
                if((l_TypeInfo & DMOIB_TYPE_FLAG_ARR))
                {
                    uint16 l_ArrayLength = 0;
                    
                    // Grab the static array length from the definition
                    LINE_COPY(&l_ArrayLength, (l_pDefPtr+1), SIZEOF_ARRAY_SIZE);
                    
                    // Check for special types
                    switch(l_TypeInfo & DMOIB_TYPE_MASK_BW)
                    {
                        case DMOIB_TYPE_FLAG_TIME:
                            l_StaticMemberSize += (l_ArrayLength * MOIB_TIME_SIZE);  
                            break;
                        default:
                            l_StaticMemberSize += (l_ArrayLength * ((l_TypeInfo & DMOIB_TYPE_MASK_BW)+1));
                            break;
                    }
                }
                else
                {
                    // Check for special types
                    switch(l_TypeInfo & DMOIB_TYPE_MASK_BW)
                    {
                        case DMOIB_TYPE_FLAG_TIME:
                            l_StaticMemberSize += MOIB_TIME_SIZE;  
                            break;
                        default:
                            l_StaticMemberSize += ((l_TypeInfo & DMOIB_TYPE_MASK_BW)+1);
                            break;
                    }
                }
            }
            
            if(l_TypeInfo & DMOIB_TYPE_FLAG_ARR)
                l_pDefPtr += sizeof(uint16);

            if((l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
                l_pDefPtr += sizeof(uint32);
        }
    
        LINE_COPY(l_pBufferIter, &m_ClassID, sizeof(uint32));
        l_pBufferIter += sizeof(uint32);
        // Just make the space for the SerialSize, but don't fill it in yet. We'll
        // fill it in at the end.    
        l_pBufferIter += l_SizeWidth;
        LINE_COPY(l_pBufferIter, &Version, sizeof(uint16));
        l_pBufferIter += sizeof(Version);
        LINE_COPY(l_pBufferIter, &m_DataBitField, sizeof(uint8));
        l_pBufferIter += sizeof(m_DataBitField);
        // Use the dynamically calculated static member size from above, based
        // on the version of the stream.
        LINE_COPY(l_pBufferIter, &l_StaticMemberSize, sizeof(uint16));
        l_pBufferIter += sizeof(l_StaticMemberSize);

        // Place the Dynamic member iterator at the end of the static data area.
        // This way, if we encounter a dynamic member out of order, before we have
        // finished the static members, we can enqueue it immediately.
        l_pDynamicIter = l_pBufferIter + l_StaticMemberSize;

        l_pDefPtr = SKIP_DEF_HEADER(m_pDefinition);
        
        for(l_MemberIter = 0; //m_pMemberIter =  m_pMemberArray;
            l_MemberIter < m_MemberCountTotal; //m_pMemberIter != m_pMemberEnd;
            l_pDefPtr++, l_MemberIter++)
        {
            LINE_COPY(&l_MinVersion, l_pDefPtr, sizeof(uint16));
            l_pDefPtr += sizeof(uint16);
            LINE_COPY(&l_MaxVersion, l_pDefPtr, sizeof(uint16));
            l_pDefPtr += sizeof(uint16); 
            
            // Boolean to tell us if the member is available
            l_InVersion = (l_MinVersion <= Version && Version <= l_MaxVersion);
            l_TypeInfo = *l_pDefPtr;

            if(l_InVersion)
            {
                m_pMemberIter = m_pMemberArray + m_pMemberTranslation[l_MemberIter];
                
                l_ByteWidth = (uint8)((l_TypeInfo & DMOIB_TYPE_MASK_BW)+1);

                // Check for sub objects, and handle them recursively.
                if((l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
                {
                    if(m_pMemberIter->Ptr == NULL)
                    {
                        // If we have a sub-object or vector member that doesn't 
                        // exist, then we can't serialize properly. So just 
                        // return 0 early.
                        return 0;
                    }
                    else
                    {
                        if( (l_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
                            (l_TypeInfo & DMOIB_TYPE_FLAG_PTR))
                        {
                            // Serialize the sub objects into the buffer and advance
                            l_pDynamicIter += ((CDMOIBVector*)m_pMemberIter->Ptr)->Serialize(l_pDynamicIter);
                        }
                        else
                        {
                            // Move forward 4 bytes to put us where we want to place the serialized object                
                            l_pDynamicIter += sizeof(uint32);

                            // Serialize the sub object into the buffer and save the size of it
                            l_SerialSize = ((CDMOIBObject*)m_pMemberIter->Ptr)->Serialize(l_pDynamicIter);

                            // Move back 4 bytes and put in the size that we just saved
                            LINE_COPY(l_pDynamicIter - sizeof(uint32), &l_SerialSize,  sizeof(uint32));

                            // Move forward by the size of the object to put us in position for the next member
                            l_pDynamicIter += l_SerialSize;
                        }
                    }                
                }
                // Otherwise we have normal data types or structures, so handle them
                // directly.
                else
                {   
                    void    (*l_pCopyFunc)(byte*, uint16, byte*, uint16);
                    uint8** l_pBufferPtr = &l_pBufferIter;
                    uint8   l_ElementWidth = l_ByteWidth;
                    byte*   l_pDataPtr = (byte*)&m_pMemberIter->m_MemberInfo;
                    byte*   l_pDataEnd;

                    // Check for special types
                    switch(l_TypeInfo & DMOIB_TYPE_MASK_BW)
                    {
                        case DMOIB_TYPE_FLAG_TIME:
                            // This is a MOIB_TIME struct, so use a specific copy 
                            // function and change the ByteWidth.
                            l_pCopyFunc = TimeToBuffer;
                            l_ElementWidth = sizeof(MOIB_TIME);
                            l_ByteWidth = MOIB_TIME_SIZE;
                            break;
                        default:
                            l_pCopyFunc = LINE_COPY_FP;
                            break;
                    }            

                    l_pDataEnd = l_pDataPtr + l_ElementWidth;
                    l_SerialSize = (uint16)(l_ByteWidth * m_pMemberIter->m_PointerInfo.m_NumElements);            

                    // Check for Pointer, Array, or single elements
                    switch(l_TypeInfo & (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_FLAG_ARR))
                    {
                        case DMOIB_TYPE_FLAG_PTR:
                            // Put the size of the ptr into the buffer
                            LINE_COPY(l_pDynamicIter, &l_SerialSize, sizeof(uint16));
                            l_pDynamicIter += sizeof(uint16);
                            l_pBufferPtr = &l_pDynamicIter;

                            // If we are dealing with an object that has a version < 200,
                            // it is a legacy object that has a bug with serialization.
                            // This bug does not convert the the data to the proper
                            // endianess form when serializing, it just does a bytecopy.
                            // To remain conpatible with legacy objects, we check the 
                            // version of our object here, and use a normal bytecopy for
                            // our serialization.
                            if(m_Version < 200)
                                l_pCopyFunc = BYTE_COPY_FP;

                        case DMOIB_TYPE_FLAG_ARR:
                            // Then put the ptr itself into the buffer, element-wise because
                            // of endianess
                            l_pDataPtr = (byte*)m_pMemberIter->Ptr;
                            l_pDataEnd = (byte*)m_pMemberIter->Ptr + l_SerialSize;

                        default:
                            for(;l_pDataPtr < l_pDataEnd; l_pDataPtr += l_ElementWidth)
                            {
                                l_pCopyFunc(*l_pBufferPtr, l_ByteWidth, l_pDataPtr, l_ElementWidth);
                                *l_pBufferPtr += l_ByteWidth;
                            }
                    }
                }
            }
            
            if(l_TypeInfo & DMOIB_TYPE_FLAG_ARR)
                l_pDefPtr += sizeof(uint16);

            if((l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
                l_pDefPtr += sizeof(uint32);
        }
    
        // DynamicIter points at the end of the buffer, so subtract the front of
        // the buffer from it to get our serialized size. Then enqueue that value
        // to the SerialSize position in the front
        l_SerialSize = (uint32)(l_pDynamicIter - pBuffer) // End - Front
                                + sizeof(uint16)        // + space for CRC
                                - sizeof(uint32)        // - space for ClassID
                                - l_SizeWidth;          // - space for Size itself

        LINE_COPY(pBuffer + sizeof(uint32), &l_SerialSize, l_SizeWidth);

        // Enqueue the CRC at the end of the Dynamic members.
        l_CRC = CalculateCRC(pBuffer, (uint32)(l_pDynamicIter - pBuffer));
        LINE_COPY(l_pDynamicIter, &l_CRC, sizeof(uint16));
        l_pDynamicIter += sizeof(uint16);
    }

    return (uint32)(l_pDynamicIter - pBuffer);
}
/*
===============================================================================
    Method to parse data from a serial bytestream into the object
===============================================================================
*/
uint32 CDMOIBObject::Deserialize(uint8* pBuffer)
{
    uint8* l_pBufferIter = pBuffer;
    uint8* l_pDynamicIter = pBuffer;    

    if(pBuffer != NULL && LINE_CHECK(l_pBufferIter, &m_ClassID, sizeof(uint32)) == TRUE)
    {
        const byte* l_pDefPtr;
        uint32 l_SerialSize = 0;
        uint16 l_CRC;
        uint16 l_Version = 0;
        uint16 l_MinVersion = 0;
        uint16 l_MaxVersion = 0;    
        uint16 l_StaticMemberSize = 0;
        uint8  l_ByteWidth;        
        uint8  l_TypeInfo;

        boolean l_InStream = FALSE;
        boolean l_InObject = FALSE;

        l_ByteWidth = DMOIBSerialSizeWidth(m_ClassID);
        l_SerialSize = DMOIBSerialSize(pBuffer);

        // Step past the ClassID we just checked and the size of the stream...
        l_pBufferIter += sizeof(uint32) + l_ByteWidth; 
        
        l_CRC = CalculateCRC(pBuffer, l_SerialSize - sizeof(uint16));

        if(LINE_CHECK(pBuffer + l_SerialSize - sizeof(uint16), &l_CRC, sizeof(uint16)) == FALSE)
        {
            Reset();
            return 0;
        }
        LINE_COPY(&l_Version, l_pBufferIter, sizeof(uint16));
        if(m_Version < l_Version)
        {
            Reset();
            return 0;
        }        

        // Step past the Version we just checked...
        l_pBufferIter += sizeof(m_Version); 
        LINE_COPY(&m_DataBitField, l_pBufferIter, sizeof(uint8));
        l_pBufferIter += sizeof(m_DataBitField);
        LINE_COPY(&l_StaticMemberSize, l_pBufferIter, sizeof(uint16));
        l_pBufferIter += sizeof(l_StaticMemberSize);

        l_pDynamicIter = l_pBufferIter + l_StaticMemberSize;

        l_pDefPtr = SKIP_DEF_HEADER(m_pDefinition);                    

        for(m_pMemberIter =  m_pMemberArray;
            m_pMemberIter != m_pMemberEnd;
            l_pDefPtr ++)
        {
            LINE_COPY(&l_MinVersion, l_pDefPtr, sizeof(uint16));
            l_pDefPtr += sizeof(uint16);
            LINE_COPY(&l_MaxVersion, l_pDefPtr, sizeof(uint16));
            l_pDefPtr += sizeof(uint16); 
            
            // Booleans to tell us if we're available in the Stream/Object
            l_InStream = (l_MinVersion <= l_Version && l_Version <= l_MaxVersion);
            l_InObject = (l_MinVersion <= m_Version && m_Version <= l_MaxVersion);
            l_ByteWidth = (uint8)(*l_pDefPtr & DMOIB_TYPE_MASK_BW);
            l_TypeInfo = *l_pDefPtr;

            if(l_InStream)
            {
                if(l_ByteWidth++ == DMOIB_TYPE_FLAG_OBJ)
                {
                    if((l_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
                       (l_TypeInfo & DMOIB_TYPE_FLAG_PTR))
                    {
                        if(l_InObject && (m_pMemberIter->Ptr != NULL))
                        {
                            // Get the objects
                            l_pDynamicIter += ((CDMOIBVector*)m_pMemberIter->Ptr)->Deserialize(l_pDynamicIter);
                        }
                        // If the vector isn't available in this version of the
                        // object, or it just wasn't created correctly, we 
                        // can't deserialize it, but we have to skip over it in
                        // the stream...
                        else
                        {
                            boolean l_bSkipCollectionSize;
                            uint32  l_ClassID;

                            // Grab the ClassID for this vector from the definition.
                            // This will eithwe be a specific ClassID, or 
                            // ANY_CLASS::CLASSID.
                            LINE_COPY(&l_ClassID, l_pDefPtr + 1, sizeof(uint32));

                            // Check to see if the ClassID is a specific one, and
                            // the vector size is dynamic. If both are true, then
                            // we're dealing with a legacy Collection type, and
                            // we have an extra uint16 value before each object
                            // element in the stream we need to handle.
                            l_bSkipCollectionSize = (l_ClassID != ANY_CLASS::CLASSID && 
                                                    (*l_pDefPtr & DMOIB_TYPE_FLAG_PTR));

                            l_SerialSize = 0;
                            // Grab the number of vector elements in the stream.
                            LINE_COPY(&l_SerialSize, l_pDynamicIter, sizeof(uint16));
                            l_pDynamicIter += sizeof(uint16);

                            // Iterate through the objects in the stream.
                            while(l_SerialSize-- > 0)
                            {
                                // If this boolean is TRUE, then we are dealing
                                // with a Collection-style vector, and we need
                                // to skip the uint16 value before each object.
                                // (This value is effectively the size of the
                                // following object, but we don't use/need it
                                // because there is a size ID in the object 
                                // itself.
                                if(l_bSkipCollectionSize == TRUE)
                                    l_pDynamicIter += sizeof(uint16);

                                // Extract the size from the object and move
                                // ahead that many bytes.
                                l_pDynamicIter += DMOIBSerialSize(l_pDynamicIter);
                            }
                        }

                        if(l_TypeInfo & DMOIB_TYPE_FLAG_ARR)
                            l_pDefPtr += sizeof(uint16);                        
                    }
                    else
                    {   
                        // Move ahead to skip the uint32 in front of the 
                        // subobject.
                        l_pDynamicIter += sizeof(uint32);
                        // Get the size of the object out of the buffer
                        l_SerialSize = DMOIBSerialSize(l_pDynamicIter);

                        if(l_InObject && (m_pMemberIter->Ptr != NULL))
                        {
                            // Then get the object itself
                            ((CDMOIBObject*)m_pMemberIter->Ptr)->Deserialize(l_pDynamicIter);
                        }

                        l_pDynamicIter += l_SerialSize;
                    }

                    l_pDefPtr += sizeof(uint32);
                }
                else
                {
                    byte*   l_pDataPtr = (byte*)&m_pMemberIter->m_MemberInfo;
                    byte*   l_pDataEnd = NULL;
                    uint8** l_pBufferPtr = &l_pBufferIter;
                    // Since the actual data member's width may not be the 
                    // same size as the serialized value, we need to store 
                    // it separately (By default it is the same, but may be 
                    // changed later).
                    uint8   l_ElementWidth = l_ByteWidth;
                    void    (*l_pCopyFunc)(byte*, uint16, byte*, uint16);
                    
                    l_SerialSize = 0;
                    
                    // Check for special types
                    switch(l_TypeInfo & DMOIB_TYPE_MASK_BW)
                    {
                        case DMOIB_TYPE_FLAG_TIME:
                            // This is a MOIB_TIME struct, so use a specific copy 
                            // function and change the ByteWidth.
                            l_pCopyFunc = BufferToTime;
                            l_ElementWidth = sizeof(MOIB_TIME);
                            l_ByteWidth = MOIB_TIME_SIZE;
                            break;
                        default:
                            // Standard data type, so use the standard copy
                            // function.
                            l_pCopyFunc = LINE_COPY_FP;
                            break;
                    }

                    switch(l_TypeInfo & (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_FLAG_ARR))
                    {
                        case DMOIB_TYPE_FLAG_PTR:
                            // Dynamic array, so we need to grab the size from
                            // the serialized stream.
                            l_pBufferPtr = &l_pDynamicIter;
                            LINE_COPY(&l_SerialSize, l_pDynamicIter, sizeof(uint16));
                            l_pDynamicIter += sizeof(uint16);
                            break;
                        case DMOIB_TYPE_FLAG_ARR:
                            // Static array, so we grab the size from the
                            // original definition, since it can NEVER change.
                            LINE_COPY(&l_SerialSize, l_pDefPtr + 1, sizeof(uint16));
                            l_SerialSize *= l_ByteWidth;
                            l_pDefPtr += sizeof(uint16);
                            break;
                        default:
                            // Single static element, so default the DataEnd
                            // pointer to one element past the start so we will
                            // only grab a single value from the stream later.
                            l_pDataEnd = l_pDataPtr + l_ElementWidth;
                            l_SerialSize = l_ByteWidth;                            
                            break;
                    }

                    if(l_InObject)
                    {
                        switch(l_TypeInfo & (DMOIB_TYPE_FLAG_PTR | DMOIB_TYPE_FLAG_ARR))
                        {
                            case DMOIB_TYPE_FLAG_PTR:
                                // Use the SetBuffer method with NULL as our 
                                // value array to create an empty array of the
                                // appropriate size.
                                SetBuffer(NULL, (uint8)(l_TypeInfo & DMOIB_TYPE_MASK_BW)+1, 
                                          (uint16)(l_SerialSize/l_ByteWidth) -
                                          ((l_TypeInfo & DMOIB_TYPE_FLAG_TXT) && 
                                           (l_SerialSize > 0) ? 1 : 0), 
                                          (uint8)(l_TypeInfo & DMOIB_TYPE_FLAG_FP));

                                // If we are dealing with an object that has a version < 200,
                                // it is a legacy object that has a bug with serialization.
                                // This bug does not convert the the data to the proper
                                // endianess form when serializing, it just does a bytecopy.
                                // To remain conpatible with legacy objects, we check the 
                                // version of our object here, and use a normal bytecopy for
                                // our serialization.
                                if(l_Version < 200)
                                    l_pCopyFunc = BYTE_COPY_FP;

                            case DMOIB_TYPE_FLAG_ARR:
                                // The data from pBuffer will always be the same length since it
                                // is a static array, so we can just copy the whole array, 
                                // element-by-element from pBuffer.
                                l_pDataPtr = (byte*)m_pMemberIter->Ptr;
                                l_pDataEnd = (byte*)m_pMemberIter->Ptr + l_SerialSize;

                            default:
                                // Once we get here, we will have either fallen through
                                // from the array types above, or straight to here as
                                // a single element type. Either way, all the variables
                                // we use for this loop should be set correctly, and
                                // will either copy a single element, or the number of
                                // elements specified for the array type.
                                if(l_pDataPtr != NULL)
                                {
                                    for(;l_pDataPtr < l_pDataEnd; l_pDataPtr += l_ElementWidth)
                                    {
                                        l_pCopyFunc(l_pDataPtr, l_ElementWidth, *l_pBufferPtr, l_ByteWidth);
                                        *l_pBufferPtr += l_ByteWidth;
                                    }
                                }
                        }
                    }
                    else
                    {
                        *l_pBufferPtr += l_SerialSize;
                    }
                }
            }
            else
            {
                if(l_TypeInfo & DMOIB_TYPE_FLAG_ARR)
                    l_pDefPtr += sizeof(uint16);

                if(l_ByteWidth == DMOIB_TYPE_FLAG_OBJ)
                    l_pDefPtr += sizeof(uint32);
            }

            if(l_InObject)
            {
                m_pMemberIter++;
            }            
        }

        // Move ahead 16 bits to account for the CRC value checked earlier.
        l_pDynamicIter += sizeof(uint16);
    }   

    return (uint32)(l_pDynamicIter - pBuffer);
}


/*
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    CDMOIBVector Class Methods

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
===============================================================================
    Constructor/Destructor
    NOTE: These are private, and can only be called by a CDMOIBObject instance.
    [PRIVATE]
===============================================================================
*/
CDMOIBVector::CDMOIBVector(CDMOIBDictionary* pDictionary, 
                           uint32 FixedClassID, 
                           uint16 FixedSize,
						   uint16 FixedVersion)
{
    Construct(pDictionary,
              FixedClassID,
              FixedSize,
			  FixedVersion);
}
CDMOIBVector::CDMOIBVector(CDMOIBVector* pVectorCopy)
{
    if(pVectorCopy != NULL)
    {
        Construct(pVectorCopy->m_pDictionary, 
                  pVectorCopy->m_FixedClassID, 
                  pVectorCopy->m_FixedSize,
				  pVectorCopy->m_FixedVersion);

        Copy(pVectorCopy);
    }
    else
    {
        // If we were given a bogus pointer to copy, just create a default
        // vector object that will accept anything.
        Construct(NULL, (uint32)ANY_CLASS::CLASSID, 0);
    }
}
CDMOIBVector::~CDMOIBVector()
{
    Destruct();
}

/*
===============================================================================
    Method to Initialize internal members and objects
    [PRIVATE]
===============================================================================
*/
void CDMOIBVector::Construct(CDMOIBDictionary* pDictionary, 
                             uint32 FixedClassID, 
                             uint16 FixedSize,
							 uint16 FixedVersion)
{
    m_pDictionary       = pDictionary;
    m_FixedClassID      = FixedClassID;
    m_FixedSize         = FixedSize;
	m_FixedVersion		= FixedVersion;
    m_NumElements       = 0;
    m_NumElementsMax    = m_FixedSize;
    m_ppObjectArray     = NULL; 

#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE) || (TARGET_XML_SUPPORT == XML_ALL))
    m_XMLUserData.m_pXMLParent          = NULL;
    m_XMLUserData.m_pXMLObject          = this;
    m_XMLUserData.m_XMLObjectType       = DMOIB_TYPE_VECTOR_PTR;
    m_XMLUserData.m_XMLCurrentIndex     = 0;
    m_XMLUserData.m_XMLContinueParsing  = TRUE;
#endif

    if(m_FixedSize != UNBOUNDED)
    {                
        m_ppObjectArray = new CDMOIBObject*[m_NumElementsMax];
        
        if(m_ppObjectArray == NULL)
        {
            m_NumElementsMax = 0;
            m_NumElements = 0;
        }
        else
        {
            if(m_FixedClassID != ANY_CLASS::CLASSID)
            {            
                while(m_NumElements < m_NumElementsMax)
                {
                    m_ppObjectArray[m_NumElements] = new CDMOIBObject(m_pDictionary, m_FixedClassID, m_FixedVersion);

                    // If we couldn't allocate a new object, then there's not
                    // much we can do for a fixed size/fixed ClassID vector.
                    // Break and we'll return early with a partially empty 
                    // vector and we'll have to check later in other methods.
                    if(m_ppObjectArray[m_NumElements] == NULL)
                        break;

                    m_NumElements++;
                }
            }
            else
            {
                memset(m_ppObjectArray, 0, m_FixedSize * sizeof(*m_ppObjectArray));
            }
        }        
    }
}
/*
===============================================================================
    Method to return the object to an empty state and clean up memory 
    [PRIVATE]
===============================================================================
*/
void CDMOIBVector::Destruct()
{
    Clear();

    if(m_ppObjectArray != NULL)
    {
        delete [] m_ppObjectArray;
    }   
}
/*
===============================================================================
    -- PUBLIC METHODS -- 
===============================================================================
    Method to copy all elements from another vector to this one.
===============================================================================
*/
boolean CDMOIBVector::Copy(CDMOIBVector* pVectorCopy)
{
    boolean l_RetVal = FALSE;

    if(pVectorCopy != NULL && 
       pVectorCopy->m_FixedSize == m_FixedSize && 
       pVectorCopy->m_FixedClassID == m_FixedClassID)
    {
        l_RetVal = TRUE;

        // Clear out the vector to get rid of any previous data
        Clear();

        if(m_FixedSize > 0 && m_FixedClassID != ANY_CLASS::CLASSID)
        {
            for(uint16 l_Index = 0; 
                l_Index < m_NumElements && l_RetVal == TRUE; 
                l_Index++)
            {
                l_RetVal = m_ppObjectArray[l_Index]->Copy(pVectorCopy->m_ppObjectArray[l_Index]);
            }
        }
        else
        {
            for(uint16 l_Index = 0; 
                l_Index < pVectorCopy->m_NumElements && l_RetVal == TRUE; 
                l_Index++)
            {
                l_RetVal = (Push(pVectorCopy->m_ppObjectArray[l_Index]) != NULL);
            }
        }
    }

    // If for some reason we couldn't complete the copy, clear out the Vector
    // again, so no pre-existing data gets carried over or 1/2 copied data.
    if(l_RetVal == FALSE)
        Clear();

    return l_RetVal;
}
/*
===============================================================================
    Method to get an DMOIBObject stored at Index. 
===============================================================================
*/
CDMOIBObject* CDMOIBVector::At(uint16 Index)
{
    if(Index < m_NumElements)
    {
        return m_ppObjectArray[Index];
    }
    else
    {
        return NULL;
    }
}
/*
===============================================================================
    Method to get the first object in the Vector. 
===============================================================================
*/
CDMOIBObject* CDMOIBVector::Front()
{
    if(m_NumElements > 0)
    {
        return m_ppObjectArray[0];
    }
    else
    {
        return NULL;
    }
}
/*
===============================================================================
    Method to get the last object in the Vector. 
===============================================================================
*/
CDMOIBObject* CDMOIBVector::Back()
{
    if(m_NumElements > 0)
    {
        return m_ppObjectArray[m_NumElements - 1];
    }
    else
    {
        return NULL;
    }
}

/*
===============================================================================
    Methods to append one object to the end of the Vector.
===============================================================================
*/
CDMOIBObject* CDMOIBVector::Push(CDMOIBObject* pDMOIBObject)
{
    return Insert(pDMOIBObject, m_NumElements);
}
CDMOIBObject* CDMOIBVector::Push(void* pDefinition)
{
    return Insert(pDefinition, m_NumElements);
}
CDMOIBObject* CDMOIBVector::Push(void* pDefinition, uint16 Version)
{
    return Insert(pDefinition, Version, m_NumElements);
}
CDMOIBObject* CDMOIBVector::Push(uint32 ClassID)
{
    return Insert(ClassID, m_NumElements);
}
CDMOIBObject* CDMOIBVector::Push(uint32 ClassID, uint16 Version)
{
    return Insert(ClassID, Version, m_NumElements);
}
/*
===============================================================================
    Method to remove the last object in the Vector.
===============================================================================
*/
void CDMOIBVector::Pop()
{
    if(m_NumElements > 0)
    {
        Erase(m_NumElements - 1);
    }
}

/*
===============================================================================
    Methods to insert a new object at Index. (All call private Insert() helper)
===============================================================================
*/
CDMOIBObject* CDMOIBVector::Insert(void* pDefinition, uint16 Index)
{
    return Insert(Index, NULL, pDefinition, (uint32)ANY_CLASS::CLASSID);
}
CDMOIBObject* CDMOIBVector::Insert(void* pDefinition, uint16 Version, uint16 Index)
{
    return Insert(Index, NULL, pDefinition, (uint32)ANY_CLASS::CLASSID, Version);
}
CDMOIBObject* CDMOIBVector::Insert(uint32 ClassID, uint16 Index)
{
    return Insert(Index, NULL, NULL, ClassID);
}
CDMOIBObject* CDMOIBVector::Insert(uint32 ClassID, uint16 Version, uint16 Index)
{
    return Insert(Index, NULL, NULL, ClassID, Version);
}
CDMOIBObject* CDMOIBVector::Insert(CDMOIBObject* pDMOIBObject, uint16 Index)
{
    return Insert(Index, pDMOIBObject, NULL, (uint32)ANY_CLASS::CLASSID);
}
/*
===============================================================================
    Methods to insert a new object at Index. 
    [PRIVATE]
===============================================================================
*/
CDMOIBObject* CDMOIBVector::Insert(uint16 Index, CDMOIBObject* pDMOIBObject, 
								   void* pDefinition, uint32 ClassID,
								   uint16 Version)
{
    if((pDMOIBObject != NULL || pDefinition != NULL || ClassID != ANY_CLASS::CLASSID) // Need one or the other
        && m_NumElements >= Index           // Make sure index is in bounds
        && m_NumElementsMax >= m_FixedSize) // Make sure inserts are allowed 
    {
        CDMOIBObject**  l_ppObjectArray = m_ppObjectArray;
        uint16          l_Iter;
        uint32          l_ClassID;

        if(pDMOIBObject != NULL)
            l_ClassID = pDMOIBObject->ClassID();
        else if(pDefinition != NULL)
            l_ClassID = DefinitionClassID(pDefinition);
        else
            l_ClassID = ClassID;

        if((m_NumElements < m_FixedSize || m_FixedSize == UNBOUNDED) && 
           (m_FixedClassID == ANY_CLASS::CLASSID || m_FixedClassID == l_ClassID))
        {
            if(m_FixedSize == UNBOUNDED && m_NumElementsMax == m_NumElements)
            {
                m_NumElementsMax++;
                l_ppObjectArray = new CDMOIBObject*[m_NumElementsMax];

                if(l_ppObjectArray == NULL)
                    // If we couldn't allocate a new object array, then we
                    // can't add any new elements, so just return NULL without
                    // doing anything more.
                    return NULL;

                // Since we are moving the array to a new space, copy all elements
                // over from the front of the array, maintaining their positions.
                for(l_Iter = 0; l_Iter < Index; l_Iter++)
                {
                    l_ppObjectArray[l_Iter] = m_ppObjectArray[l_Iter];
                }
            }   

            // Regardless of whether we have a new array or we are reusing the old
            // one, copy the elements from the back of the array in reverse to 
            // move them ahead one position.
            for(l_Iter = m_NumElements; l_Iter > Index; l_Iter--)
            {
                l_ppObjectArray[l_Iter] = m_ppObjectArray[l_Iter - 1];
            }

            // If the temp pointer doesn't match the persistent one, then we have
            // allocated a new array from before, so delete the old one and 
            // reassign the persistent one.
            if(l_ppObjectArray != m_ppObjectArray)
            {
                if(m_ppObjectArray != NULL)
                    delete [] m_ppObjectArray;

                m_ppObjectArray = l_ppObjectArray;
            }  

            if(pDMOIBObject != NULL)
                l_ppObjectArray[Index] = new CDMOIBObject(pDMOIBObject);
            else if(pDefinition != NULL)
                l_ppObjectArray[Index] = new CDMOIBObject(m_pDictionary, pDefinition, Version);
            else
                l_ppObjectArray[Index] = new CDMOIBObject(m_pDictionary, l_ClassID, Version);

            if(l_ppObjectArray[Index] != NULL)
                m_NumElements++;

            return l_ppObjectArray[Index];
        }
    }        

    return NULL;    
}
/*
===============================================================================
    Method to remove the number of objects specified by Count, starting at 
    Index.
===============================================================================
*/
void CDMOIBVector::Erase(uint16 Index, uint16 Count)
{
    if((m_FixedSize == UNBOUNDED || m_FixedClassID == ANY_CLASS::CLASSID) && 
       (Index + Count) <= m_NumElements)
    {
        uint16 l_IndexEnd = Index + Count;

        while(Index < l_IndexEnd)
        {
            delete m_ppObjectArray[Index++];
        }

        while(Index < m_NumElements)
        {
            m_ppObjectArray[Index - Count] = m_ppObjectArray[Index++];
        }

        m_NumElements = m_NumElements - Count;
    }
}
/*
===============================================================================
    Method to remove all objects from the Vector
===============================================================================
*/
void CDMOIBVector::Clear()
{
    if(m_FixedSize != UNBOUNDED && m_FixedClassID != ANY_CLASS::CLASSID)
    {   
        for(uint16 l_Index = 0; l_Index < m_NumElements; l_Index++)
        {
            m_ppObjectArray[l_Index]->Reset(TRUE);
        }
    }
    else
    {
        while(m_NumElements > 0)
        {
            delete m_ppObjectArray[--m_NumElements];
        }

        if(m_FixedSize != UNBOUNDED)
        {
            memset(m_ppObjectArray, 0, m_NumElementsMax * sizeof(*m_ppObjectArray));
        }
#if (defined(DMOIB_FORCE_ARRAY_RESIZE) && (DMOIB_FORCE_ARRAY_RESIZE == 1))
        else if(m_ppObjectArray != NULL)
        {
            delete [] m_ppObjectArray;
        }
#endif 
        
    }
}
/*
===============================================================================
    Method to report the number of elements in the Vector
===============================================================================
*/
uint16 CDMOIBVector::Size()
{
    return m_NumElements;
}
/*
===============================================================================
    Method to determine if the vector is empty (contains no elements)
===============================================================================
*/
boolean CDMOIBVector::Empty()
{
    return (m_NumElements == 0);
}
/*
===============================================================================
    Method to report the serial size of all objects in the Vector.
    [PRIVATE]
===============================================================================
*/
uint32 CDMOIBVector::SizeAll()
{    
    uint32 l_SerialSize = sizeof(uint16);
    
    if(m_FixedClassID != ANY_CLASS::CLASSID && m_FixedSize == UNBOUNDED)
        l_SerialSize += (sizeof(uint16) * m_NumElements);

    for(uint16 l_Index = 0; l_Index < m_NumElements; l_Index++)
    {
        l_SerialSize += m_ppObjectArray[l_Index]->SizeAll();
    }

    return l_SerialSize;
}
/*
===============================================================================
    Method to serialize the objects to a stream
    [PRIVATE]
===============================================================================
*/
uint32 CDMOIBVector::Serialize(uint8* pBuffer)
{    
    uint8*  l_pBuffer = pBuffer;
    uint32  l_SerialSize = 0;
    boolean l_AddCollectionSize = (m_FixedClassID != ANY_CLASS::CLASSID && 
                                   m_FixedSize == UNBOUNDED);

    if(l_pBuffer != NULL)
    {
        LINE_COPY(l_pBuffer, &m_NumElements, sizeof(uint16));
        l_pBuffer += sizeof(uint16);        
        
        for(uint16 l_Index = 0; l_Index < m_NumElements; l_Index++)
        {
            if(l_AddCollectionSize)
                l_pBuffer += sizeof(uint16);

            l_SerialSize = m_ppObjectArray[l_Index]->Serialize(l_pBuffer);

            if(l_AddCollectionSize)
            {
                if(l_SerialSize == 0)
                    l_pBuffer -= sizeof(uint16);
                else
                    LINE_COPY(l_pBuffer - sizeof(uint16), &l_SerialSize, sizeof(uint16));
            }

            l_pBuffer += l_SerialSize;
        }
    }
    
    return (uint32)(l_pBuffer - pBuffer);
}
/*
===============================================================================
    Method to deserialize a stream of objects
    [PRIVATE]
===============================================================================
*/
uint32 CDMOIBVector::Deserialize(uint8* Buffer)
{
    uint8*  l_pBuffer = Buffer;
    uint32  l_ClassID = 0;
    uint16  l_ObjectCount;   
    boolean l_bSkipSize = (m_FixedClassID != ANY_CLASS::CLASSID && 
                           m_FixedSize == UNBOUNDED);

    Clear();

    if(l_pBuffer != NULL)
    {
        LINE_COPY(&l_ObjectCount, l_pBuffer, sizeof(uint16));
        l_pBuffer += sizeof(uint16);

        if(m_FixedSize != UNBOUNDED && m_FixedClassID != ANY_CLASS::CLASSID)
        {
            if(m_NumElements == l_ObjectCount)
            {
                for(uint16 l_Index = 0; l_Index < l_ObjectCount; l_Index++)
                {
                    m_ppObjectArray[l_Index]->Deserialize(l_pBuffer);
                    l_pBuffer += DMOIBSerialSize(l_pBuffer);
                }
            }
        }
        else
        {
			uint16			l_Version;
            CDMOIBObject*	l_pObject;

            for(uint16 l_Index = 0; l_Index < l_ObjectCount; l_Index++)
            {
                if(l_bSkipSize)
                    l_pBuffer += sizeof(uint16);

                LINE_COPY(&l_ClassID, l_pBuffer, SIZEOF_CLASSID);

				// Grab the version of the object
				LINE_COPY(&l_Version, l_pBuffer + SIZEOF_CLASSID + DMOIBSerialSizeWidth(l_ClassID), SIZEOF_VERSION);

                l_pObject = Push(l_ClassID, l_Version);

                if(l_pObject != NULL)
                    l_pObject->Deserialize(l_pBuffer);

                l_pBuffer += DMOIBSerialSize(l_pBuffer);
            }
        }
    }    
    return (uint32)(l_pBuffer - Buffer);
}


/*
===============================================================================
===============================================================================
   All XML Support Functions/Data is described below
===============================================================================
===============================================================================
*/
#if((TARGET_XML_SUPPORT != XML_NONE))

typedef struct XMLTypeText_s
{
    uint8       Length;
    ansichar*   Text;

}XMLTypeText_t;

// %+.9E = 16 characters printed, plus the sign, ALWAYS
#define XML_FLOAT32_LENGTH  (16+1)
// %+.16E = 23 characters printed, plus the sign, ALWAYS
#define XML_FLOAT64_LENGTH  (23+1)
// (sizeof("YYYY/MM/DD HH:MM:SS") - sizeof('\0'))
//#define XML_TIME_LENGTH     (19) 

#define XML_TIME_LENGTH(Time)   (sizeof("// :: ") - sizeof('\0'))     \
                              + ((Time).year   > 9999 ? 5 : 4)      \
                              + ((Time).month  > 99 ? 3 : 2)        \
                              + ((Time).day    > 99 ? 3 : 2)        \
                              + ((Time).hour   > 99 ? 3 : 2)        \
                              + ((Time).minute > 99 ? 3 : 2)        \
                              + ((Time).second > 99 ? 3 : 2)        \
                              + ((Time).format > 99 ? 3 : 2)          

#define XML_INT_LENGTH(IntValue) XMLIntLength(IntValue)

/*
===============================================================================
    Macros for declaring XML Constants
    The XML macros are preprocessed in 2 phases, a PROTOTYPE phase and 
    IMPLEMENT phase. The PROTOTYPE is done first to declare any variables or
    constants needed buy the XML functions. IMPLEMENT is done next to actually
    create tables of strings used in XML output/parsing.
===============================================================================
*/
#define PROTOTYPE_END
#define PROTOTYPE(Macro)   PROTOTYPE##Macro

#define IMPLEMENT_END 
#define IMPLEMENT(Macro)   IMPLEMENT##Macro

#define XML_CONST(Name, Text) \
      (_XML_CONST)(Name, Text)

#define XML_TYPES(x, y) \
        const XMLTypeText_t XMLTypes[x][y] = \
        XML_TYPES_
#define XML_TYPES_(Internals) {Internals};

#define XML_CONSTANTS(Constants) \
        enum XMLConstants_e \
        { \
            PROTOTYPE Constants (_END) \
            XML_CONSTANTS_SIZE \
        }; \
        const ansichar* XMLConstants = \
        { \
            IMPLEMENT Constants (_END) \
        };

#define PROTOTYPE_XML_CONST(Name, Text) \
        Name, \
        Name##_SIZE = (sizeof(Text) - 1), \
        Name##_OUT  = Name + Name##_SIZE - 1, \
        PROTOTYPE

#define IMPLEMENT_XML_CONST(Name, Text) \
        Text \
        IMPLEMENT
/*
===============================================================================
*/
#define XML_WRITE(Buffer, Text) \
    memcpy(Buffer, Text, strlen(Text)); \
    Buffer += strlen(Text)

#define XML_FAST_WRITE(Dest, Src, Size) \
    { \
        register const ansichar* XMLConstBuffer = (const ansichar*)(Src); \
        switch((Size) >> 0x02) \
        { \
            case 5:  *((uint32*&)Dest)++ = *((uint32*&)XMLConstBuffer)++; \
            case 4:  *((uint32*&)Dest)++ = *((uint32*&)XMLConstBuffer)++; \
            case 3:  *((uint32*&)Dest)++ = *((uint32*&)XMLConstBuffer)++; \
            case 2:  *((uint32*&)Dest)++ = *((uint32*&)XMLConstBuffer)++; \
            case 1:  *((uint32*&)Dest)++ = *((uint32*&)XMLConstBuffer)++; \
        } \
        switch((Size) & 0x03) \
        { \
            case 3:  *((ansichar*&)Dest)++ = *XMLConstBuffer++; \
            case 2:  *((ansichar*&)Dest)++ = *XMLConstBuffer++; \
            case 1:  *((ansichar*&)Dest)++ = *XMLConstBuffer++; \
        } \
    }

#define XML_CONST_STR(Constant)     (XMLConstants + Constant)
#define XML_CONST_SIZE(Constant)    Constant##_SIZE
#define XML_CONST_WRITE(Buffer, Constant) \
        XML_FAST_WRITE(Buffer, XMLConstants + Constant, XML_CONST_SIZE(Constant))

#define XML_TYPE_GROUP(Group)       {Group},
#define XML_TYPE(Type)              {sizeof(#Type) - 1, #Type},
#define XML_TYPE_OMITTED            {0, NULL},
#define XML_TYPE_SIZE(TypeInfo)     (XML_TYPE_LOOKUP(TypeInfo).Length)
#define XML_TYPE_TEXT(TypeInfo)     (XML_TYPE_LOOKUP(TypeInfo).Text)
#define XML_TYPE_WRITE(Buffer, TypeInfo) \
        XML_FAST_WRITE(Buffer, XML_TYPE_TEXT(TypeInfo), XML_TYPE_SIZE(TypeInfo))

#define XML_TYPE_LOOKUP(TypeInfo) \
        (XMLTypes[ \
                   (((TypeInfo >> 0x05) & 0x01) * 0x03) + \
                    ((TypeInfo >> 0x03) & 0x03) \
                 ] \
                 [ \
                     (TypeInfo &  0x03) + \
                    ((TypeInfo >> 0x01) & 0x02) \
                 ])
/*
===============================================================================
*/
XML_TYPES(4,6)
(
    // Signed
    XML_TYPE_GROUP
    (                       // Bitmasks that resolve to these array offsets
        XML_TYPE(int8)      // 000 = offset 0
        XML_TYPE(int16)     // 001 = offset 1
        XML_TYPE(TIME)      // 010 = offset 2
        XML_TYPE(int32)     // 011 = offset 3
        XML_TYPE(OBJECT)    // 110 = offset 4
        XML_TYPE(int64)     // 111 = offset 5
    )
    // Text
    XML_TYPE_GROUP
    (
        XML_TYPE(ansichar)
        XML_TYPE(widechar)
        XML_TYPE_OMITTED
        XML_TYPE_OMITTED
        XML_TYPE_OMITTED
        XML_TYPE_OMITTED
    )
    // Unsigned
    XML_TYPE_GROUP
    (
        XML_TYPE(uint8)
        XML_TYPE(uint16)
        XML_TYPE_OMITTED
        XML_TYPE(uint32)
        XML_TYPE(OBJECT)    // Base class type
        XML_TYPE(uint64)
    )
    // Floating Point
    XML_TYPE_GROUP
    (
        XML_TYPE_OMITTED
        XML_TYPE_OMITTED
        XML_TYPE_OMITTED
        XML_TYPE(float32)
        XML_TYPE_OMITTED
        XML_TYPE(float64)
    )
)
/*
===============================================================================
*/

XML_CONSTANTS
(
    XML_CONST(TAG_DEFINITION        ,XML_TAG_DEFINITION        )
    XML_CONST(TAG_MEMBER            ,XML_TAG_MEMBER            )
    XML_CONST(TAG_MEMBERS           ,XML_TAG_MEMBERS           )
    XML_CONST(TAG_VALUE             ,XML_TAG_VALUE             )
    XML_CONST(TAG_ENUMERATION       ,XML_TAG_ENUMERATION       )
    XML_CONST(TAG_ENUM              ,XML_TAG_ENUM              )
    XML_CONST(TAG_CDATA_OPEN        ,XML_TAG_CDATA_OPEN        )
    XML_CONST(TAG_CDATA_CLOSE       ,XML_TAG_CDATA_CLOSE       )

    XML_CONST(ATT_FAMILY            ,XML_ATT_FAMILY            )
    XML_CONST(ATT_GENUS             ,XML_ATT_GENUS             )
    XML_CONST(ATT_SPECIES           ,XML_ATT_SPECIES           )
    XML_CONST(ATT_VERSION           ,XML_ATT_VERSION           )
    XML_CONST(ATT_TYPE              ,XML_ATT_TYPE              )
    XML_CONST(ATT_NAME              ,XML_ATT_NAME              )
    XML_CONST(ATT_MINVERSION        ,XML_ATT_MINVERSION        )
    XML_CONST(ATT_MAXVERSION        ,XML_ATT_MAXVERSION        )
    XML_CONST(ATT_SIZE              ,XML_ATT_SIZE              )
    XML_CONST(ATT_CLASS             ,XML_ATT_CLASS             )
    XML_CONST(ATT_DESCRIPTION       ,XML_ATT_DESCRIPTION       )
    XML_CONST(ATT_SELECTED          ,XML_ATT_SELECTED          )
    XML_CONST(ATT_DEFAULT_VALUE     ,XML_ATT_DEFAULT_VALUE     )
    XML_CONST(ATT_ALIAS             ,XML_ATT_ALIAS             )
    XML_CONST(ATT_UNITS             ,XML_ATT_UNITS             )
    XML_CONST(ATT_SCALE_COEFFICIENT ,XML_ATT_SCALE_COEFFICIENT )
    XML_CONST(ATT_SCALE_EXPONENT    ,XML_ATT_SCALE_EXPONENT    )
    XML_CONST(ATT_INHERITED         ,XML_ATT_INHERITED         )

    XML_CONST(CHAR_QUOTE            ,XML_CHAR_QUOTE            )
    XML_CONST(CHAR_OPENPAREN        ,XML_CHAR_OPENPAREN        )
    XML_CONST(CHAR_CLOSEPAREN       ,XML_CHAR_CLOSEPAREN       )
    XML_CONST(CHAR_LESSTHAN         ,XML_CHAR_LESSTHAN         )
    XML_CONST(CHAR_GREATERTHAN      ,XML_CHAR_GREATERTHAN      )
    XML_CONST(CHAR_SLASH            ,XML_CHAR_SLASH            )
    XML_CONST(CHAR_SPACE            ,XML_CHAR_SPACE            )
    XML_CONST(CHAR_RETURN           ,XML_CHAR_RETURN           )
    XML_CONST(CHAR_ARRAY_DELIM      ,XML_CHAR_ARRAY_DELIM      )
    XML_CONST(CHAR_TERMINATOR       ,XML_CHAR_TERMINATOR       )

    XML_CONST(VAL_UNBOUNDED         ,XML_VAL_UNBOUNDED         )
    XML_CONST(VAL_ANY_CLASS         ,XML_VAL_ANY_CLASS         )
    XML_CONST(VAL_CURRENT_VERSION   ,XML_VAL_CURRENT_VERSION   )
    XML_CONST(VAL_YES               ,XML_VAL_YES               )
)

/*
===============================================================================
*/


/*
===============================================================================
    Function to determine the textual length of an unsigned integer value
===============================================================================
*/
uint8 XMLIntLength(uint32 Value)
{
    uint8 l_RetVal = 0;
    do
    {
        Value = Value/10;
        l_RetVal++;
    }
    while(Value != 0);
    return l_RetVal;
}
/*
===============================================================================
    Function to determine the textual length of a signed integer value
===============================================================================
*/
uint8 XMLIntLength(int32 Value)
{
    uint8 l_RetVal = 0;

    if(Value < 0)
        l_RetVal++;
    do
    {
        Value = Value/10;
        l_RetVal++;
    }
    while(Value != 0);
    return l_RetVal;
}

/*
===============================================================================
    Functions to get the XML text from a Definition
===============================================================================
    Function to get XML text for Enumerations
===============================================================================
*/
const ansichar* XMLGetEnumerationString(const byte* pDefinition, uint16 Enumeration)
{
    if(pDefinition != NULL && DefinitionXMLSize((void*)pDefinition) > 0)
    {
        const byte* l_pTableHead = (pDefinition + DefinitionSize((void*)pDefinition));
        uint16*     l_pTableIter = (uint16*)l_pTableHead;
        uint16      l_Value;

        LINE_COPY(&l_Value, l_pTableIter, sizeof(uint16));
        l_pTableIter++;

        if(Enumeration > l_Value)
        {
            Enumeration = UINT16_MAX - Enumeration + l_Value;
            LINE_COPY(&l_Value, l_pTableIter, sizeof(uint16));
        }

        if(Enumeration < l_Value)
        {
            l_pTableIter += (Enumeration + 1) * 2;
            LINE_COPY(&l_Value, l_pTableIter, sizeof(uint16));
            return (const ansichar*)(l_pTableHead + l_Value);
        }
    }

    return NULL;
}
/*
===============================================================================
    Function to get XML text for Enumerators
===============================================================================
*/
const ansichar* XMLGetEnumeratorString(const byte* pDefinition, uint16 Enumeration, 
                                       uint16 Enumerator)
{
    if(pDefinition != NULL && DefinitionXMLSize((void*)pDefinition) > 0)
    {
        const byte* l_pTableHead = (pDefinition + DefinitionSize((void*)pDefinition));
        uint16*     l_pTableIter = (uint16*)l_pTableHead;
        uint16*     l_pTableEnd;
        uint16      l_Value;

        LINE_COPY(&l_Value, l_pTableIter, sizeof(uint16));
        l_pTableIter++;

        if(Enumeration > l_Value)
        {
            Enumeration = UINT16_MAX - Enumeration + l_Value;
            LINE_COPY(&l_Value, l_pTableIter, sizeof(uint16));
        }

        if(Enumeration < l_Value)
        {
            l_pTableIter += (Enumeration * 2) + 1;
            LINE_COPY(&l_Value, l_pTableIter, sizeof(uint16));

            if(l_Value == UINT16_MAX)
                return NULL;

            l_pTableIter = (uint16*)(l_pTableHead + l_Value);
            LINE_COPY(&l_Value, l_pTableIter, sizeof(uint16));

            l_pTableEnd = (uint16*)(l_pTableHead + l_Value);
            l_pTableIter++;

            while(l_pTableIter < l_pTableEnd)
            {
                if(LINE_CHECK(l_pTableIter++, &Enumerator, sizeof(uint16)))
                {
                    LINE_COPY(&l_Value, l_pTableIter, sizeof(uint16));
                    return (const ansichar*)(l_pTableHead + l_Value);
                }
                l_pTableIter++;
            }
        }
    }

    return NULL;
}
/*
===============================================================================
    Function to get XML text for the ClassName
===============================================================================
*/
const ansichar* XMLGetClassText(const byte* pDefinition)
{
    return XMLGetEnumerationString(pDefinition, SPECIES_ENUMERATION_ID);
}
/*
===============================================================================
    Function to get XML text for Species
===============================================================================
*/
const ansichar* XMLGetSpeciesString(const byte* pDefinition, uint8 Species)
{
    return XMLGetEnumeratorString(pDefinition, SPECIES_ENUMERATION_ID, Species);
}
/*
===============================================================================
    Function to get XML text for Members
===============================================================================
*/
const ansichar* XMLGetMemberString(const byte* pDefinition, uint16 MemberID)
{
    return XMLGetEnumerationString(pDefinition, MemberID);
}


#if ((TARGET_XML_SUPPORT == XML_SERIALIZE) || (TARGET_XML_SUPPORT == XML_ALL))
/*
===============================================================================
    Method to get the XML textual size of the object
===============================================================================
*/
uint32 CDMOIBObject::XMLSizeAll(uint16 Version)
{
    uint32 l_XMLSerialSize = DefinitionXMLSize((void*)m_pDefinition);    

    // Calculate extra XML size
    if(l_XMLSerialSize != 0)
    {           
        uint32 l_Value = 0;
        
        const byte* l_pDefPtr = SKIP_DEF_HEADER(m_pDefinition);
        uint16 l_MinVersion = 0;
        uint16 l_MaxVersion = 0;
        uint8  l_ByteWidth;        
        uint8  l_TypeInfo;

        boolean l_InVersion = FALSE;

        if(Version == HIGHEST_KNOWN_VERSION)
        {
            Version = m_Version;
        }

        for(uint16 l_MemberIter = 0; 
            l_MemberIter < m_MemberCountTotal; 
            l_pDefPtr++, l_MemberIter++)
        {
            LINE_COPY(&l_MinVersion, l_pDefPtr, sizeof(uint16));
            l_pDefPtr += sizeof(uint16);
            LINE_COPY(&l_MaxVersion, l_pDefPtr, sizeof(uint16));
            l_pDefPtr += sizeof(uint16); 
            
            // Boolean to tell us if the member is available
            l_InVersion = (l_MinVersion <= Version && Version <= l_MaxVersion);
            l_ByteWidth = (uint8)(*l_pDefPtr & DMOIB_TYPE_MASK_BW);
            l_TypeInfo = *l_pDefPtr;
            
            m_pMemberIter = m_pMemberArray + m_pMemberTranslation[l_MemberIter];

            if(l_InVersion && m_pMemberIter != m_pMemberEnd)
            {
                l_XMLSerialSize = l_XMLSerialSize 
                    + XML_CONST_SIZE(CHAR_LESSTHAN)
                    + XML_CONST_SIZE(TAG_MEMBER)
                    + XML_CONST_SIZE(CHAR_GREATERTHAN)
                    + XML_CONST_SIZE(CHAR_RETURN)

                    + XML_CONST_SIZE(CHAR_LESSTHAN)
                    + XML_CONST_SIZE(TAG_VALUE)
                    + XML_CONST_SIZE(CHAR_GREATERTHAN)

                    + XML_CONST_SIZE(CHAR_LESSTHAN)
                    + XML_CONST_SIZE(CHAR_SLASH)
                    + XML_CONST_SIZE(TAG_VALUE)
                    + XML_CONST_SIZE(CHAR_GREATERTHAN)
                    + XML_CONST_SIZE(CHAR_RETURN);

                if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
                {
                    if(m_pMemberIter->Ptr == NULL)
                    {
                        // If the pointer to the vector/object is NULL, then we 
                        // can't properly serialize this object, so return 0 to
                        // denote a failure.
                        return 0;
                    }
                    else
                    {
                        l_XMLSerialSize += XML_CONST_SIZE(CHAR_RETURN);

                        if( (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
                            (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR))
                        {
                            l_XMLSerialSize += ((CDMOIBVector*)m_pMemberIter->Ptr)->XMLSizeAll();
                        }
                        else
                        {
                            l_Value = ((CDMOIBObject*)m_pMemberIter->Ptr)->XMLSizeAll();
                            
                            if(l_Value == 0)
                            {
                                // If the subsequent SizeAll() failed, then we 
                                // can't properly serialize this object, so 
                                // return 0 to denote a failure.
                                return 0;
                            }
                            else
                            {
                                l_XMLSerialSize += l_Value - XML_CONST_SIZE(CHAR_TERMINATOR);
                            }
                        }
                    }                
                }
                else if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
                        (m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_PTR))
                {
                    l_XMLSerialSize += XML_CONST_SIZE(CHAR_LESSTHAN)
                                     + XML_CONST_SIZE(TAG_CDATA_OPEN)
                                     + XML_CONST_SIZE(TAG_CDATA_CLOSE)
                                     + XML_CONST_SIZE(CHAR_GREATERTHAN);

                    // MOIB_TIME
                    if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_TIME)
                    {
                        MOIB_TIME* l_pArrayIter = ((MOIB_TIME*)m_pMemberIter->Ptr);
                        MOIB_TIME* l_pArrayEnd  = ((MOIB_TIME*)m_pMemberIter->Ptr) + 
                                                   (m_pMemberIter->m_PointerInfo.m_NumElements);

                        while(l_pArrayIter < l_pArrayEnd)
                        {
                            l_XMLSerialSize += XML_TIME_LENGTH(*l_pArrayIter) + 
                                               XML_CONST_SIZE(CHAR_ARRAY_DELIM);
                            l_pArrayIter++;
                        }
                    }
                    // Text
                    else if(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_TXT)
                    {
                        if(m_pMemberIter->Ptr != NULL)
                        {
                            // Ansichar
                            if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) == 0)
                            {
                                l_XMLSerialSize += (uint32)strlen((ansichar*)m_pMemberIter->Ptr);
                            }
                            // Widechar
                            else
                            {
                                l_XMLSerialSize += (uint32)wcslen((widechar*)m_pMemberIter->Ptr);
                            }
                        }
                    }
                    // Floating Point
                    else if(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP)
                    {
                        // Float32
                        if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) == 3)
                        {
                            l_XMLSerialSize += m_pMemberIter->m_PointerInfo.m_NumElements * 
                                (XML_FLOAT32_LENGTH + XML_CONST_SIZE(CHAR_ARRAY_DELIM));
                        }
                        // Float64
                        else
                        {
                            l_XMLSerialSize += m_pMemberIter->m_PointerInfo.m_NumElements * 
                                (XML_FLOAT64_LENGTH + XML_CONST_SIZE(CHAR_ARRAY_DELIM));
                        }
                    }
                    // All other Integers
                    else
                    {
                        byte* l_pArrayIter = ((byte*)m_pMemberIter->Ptr);
                        byte  l_ByteWidth = (byte)((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) + 1);
                        byte* l_pArrayEnd  = ((byte*)m_pMemberIter->Ptr) + 
                                            (m_pMemberIter->m_PointerInfo.m_NumElements * l_ByteWidth);

                        // Unsigned
                        if(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_UNS)
                        {
                            while(l_pArrayIter < l_pArrayEnd)
                            {
                                l_Value = 0;
                                BYTE_COPY(&l_Value, sizeof(l_Value), l_pArrayIter, l_ByteWidth);
                                l_pArrayIter += l_ByteWidth;
                                l_XMLSerialSize += XML_INT_LENGTH(l_Value) + XML_CONST_SIZE(CHAR_ARRAY_DELIM);
                            }
                        }
                        // Signed
                        else
                        {
                            while(l_pArrayIter < l_pArrayEnd)
                            {   
                                l_Value = 0;
                                BYTE_COPY(&l_Value, sizeof(l_Value), l_pArrayIter, l_ByteWidth);

                                if( IS_NEGATIVE(l_Value, l_ByteWidth - 1) )
                                    l_Value = SIGN_EXTEND(l_Value, l_ByteWidth - 1);

                                l_pArrayIter += l_ByteWidth;
                                l_XMLSerialSize += XML_INT_LENGTH((int32)l_Value) 
                                                   + XML_CONST_SIZE(CHAR_ARRAY_DELIM);
                            }
                        }
                    }
                }
                else
                {
                    l_XMLSerialSize += XML_CONST_SIZE(CHAR_LESSTHAN)
                                     + XML_CONST_SIZE(TAG_CDATA_OPEN)
                                     + XML_CONST_SIZE(TAG_CDATA_CLOSE)
                                     + XML_CONST_SIZE(CHAR_GREATERTHAN);

                    // MOIB_TIME
                    if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_TIME)
                    {
                        l_XMLSerialSize += XML_TIME_LENGTH(m_pMemberIter->Time);
                    }
                    // Text
                    else if(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_TXT)
                    {
                        // Both ansichar and widechar are stored in uint32,
                        // so just use the value there.
                        if(WCTOB(m_pMemberIter->Uint32) != 0)
                            l_XMLSerialSize ++;
                    }
                    // Floating Point
                    else if(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_FP)
                    {
                        // Float32
                        if((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) == 3)
                        {
                            l_XMLSerialSize += XML_FLOAT32_LENGTH;
                        }
                        // Float64
                        else
                        {
                            l_XMLSerialSize += XML_FLOAT64_LENGTH;
                        }
                    }
                    // All other Integers
                    else
                    {
                        byte  l_ByteWidth = (byte)((m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_MASK_BW) + 1);

                        // Unsigned
                        if(m_pMemberIter->m_MemberInfo.m_TypeInfo & DMOIB_TYPE_FLAG_UNS)
                        {
                            l_XMLSerialSize += XML_INT_LENGTH(m_pMemberIter->Uint32);
                        }
                        // Signed
                        else
                        {
                            if(IS_NEGATIVE(m_pMemberIter->Int32, l_ByteWidth - 1))
                                l_XMLSerialSize += XML_INT_LENGTH((int32)SIGN_EXTEND(m_pMemberIter->Int32, l_ByteWidth - 1));
                            else
                                l_XMLSerialSize += XML_INT_LENGTH((int32)m_pMemberIter->Int32);
                        }
                    }                    
                }
            }
            
            if(l_TypeInfo & DMOIB_TYPE_FLAG_ARR)
                l_pDefPtr += sizeof(uint16);

            if((l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
                l_pDefPtr += sizeof(uint32);
        }

        // Add in any enumerations
        for(uint16* l_pIter = m_pMemberTranslation; l_pIter < (m_pMemberTranslation + m_MemberCountTotal); l_pIter++)
        {
            uint16          l_EnumeratorValue = GetMember((*l_pIter) + m_BaseClassMemberCount);
            const ansichar* l_pEnumerationString = XMLGetEnumerationString(m_pDefinition, 
                                                                           *l_pIter);
            const ansichar* l_pEnumeratorString = XMLGetEnumeratorString(m_pDefinition, 
                                                                         *l_pIter, 
                                                                         l_EnumeratorValue);

            if(*l_pIter < m_MemberCount && 
                l_pEnumerationString != NULL && 
                l_pEnumeratorString != NULL)
            {
                l_XMLSerialSize = l_XMLSerialSize 
                    + XML_CONST_SIZE(CHAR_LESSTHAN)
                    + XML_CONST_SIZE(TAG_ENUMERATION)

                    + XML_CONST_SIZE(CHAR_SPACE)
                    + XML_CONST_SIZE(ATT_NAME)
                    + XML_CONST_SIZE(CHAR_QUOTE)
                    + strlen(l_pEnumerationString)
                    + XML_CONST_SIZE(CHAR_QUOTE)

                    + XML_CONST_SIZE(CHAR_GREATERTHAN)
                    + XML_CONST_SIZE(CHAR_RETURN)
                    + XML_CONST_SIZE(CHAR_LESSTHAN)
                    + XML_CONST_SIZE(TAG_ENUM)

                    + XML_CONST_SIZE(CHAR_SPACE)
                    + XML_CONST_SIZE(ATT_NAME)
                    + XML_CONST_SIZE(CHAR_QUOTE)
                    + strlen(l_pEnumeratorString)
                    + XML_CONST_SIZE(CHAR_QUOTE)

                    + XML_CONST_SIZE(CHAR_SPACE)
                    + XML_CONST_SIZE(ATT_DEFAULT_VALUE)
                    + XML_CONST_SIZE(CHAR_QUOTE)
                    + XML_INT_LENGTH((uint32)l_EnumeratorValue)                    
                    + XML_CONST_SIZE(CHAR_QUOTE)

                    + XML_CONST_SIZE(CHAR_SPACE)
                    + XML_CONST_SIZE(ATT_SELECTED)
                    + XML_CONST_SIZE(CHAR_QUOTE)
                    + XML_CONST_SIZE(VAL_YES)
                    + XML_CONST_SIZE(CHAR_QUOTE)

                    + XML_CONST_SIZE(CHAR_SLASH)
                    + XML_CONST_SIZE(CHAR_GREATERTHAN)
                    + XML_CONST_SIZE(CHAR_RETURN)
                    + XML_CONST_SIZE(CHAR_LESSTHAN)
                    + XML_CONST_SIZE(CHAR_SLASH)
                    + XML_CONST_SIZE(TAG_ENUMERATION)
                    + XML_CONST_SIZE(CHAR_GREATERTHAN)
                    + XML_CONST_SIZE(CHAR_RETURN)
                    ;
            }
        }

        l_XMLSerialSize += XML_CONST_SIZE(CHAR_TERMINATOR);
    }

    return l_XMLSerialSize;
}
/*
===============================================================================
    Method to put the object into an XML text stream
===============================================================================
*/
uint32 CDMOIBObject::XMLSerialize(ansichar* pXMLBuffer, uint16 Version)
{
    ansichar*   l_pBuffer = pXMLBuffer;    
    
    // Check to see if we have strings available in our definition. If we 
    // don't, then we can't serialize to XML.
    if(pXMLBuffer != NULL && DefinitionXMLSize((void*)m_pDefinition) != 0)
    {    
        uint32          l_Value = 0;
        uint16          l_MemberOffset = 0;
        uint16          l_MemberCount = DefinitionMemberCount((void*)m_pDefinition);
        uint8           l_TypeInfo = 0;
        const ansichar* l_pDefinitionString = NULL;
        
        if(Version == HIGHEST_KNOWN_VERSION)
        {
            Version = m_Version;
        }

        XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
        XML_CONST_WRITE(l_pBuffer, TAG_DEFINITION);
        //==============================
        XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
        XML_CONST_WRITE(l_pBuffer, ATT_CLASS);
        XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
        // ClassName
        //==============================
        // Check to see if we have a ClassName string available. If not, exit
        // immediately with a value of 0 to signify a failure.
        if((l_pDefinitionString = XMLGetClassText(m_pDefinition)) == NULL)
        {
            return 0;
        }

        XML_WRITE(l_pBuffer, l_pDefinitionString);
        //==============================
        XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
        XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
        XML_CONST_WRITE(l_pBuffer, ATT_VERSION);
        XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
        // Version
        //==============================
        l_Value = sprintf((ansichar*)l_pBuffer, "%u", Version);
        l_pBuffer += l_Value;
        //==============================
        XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
        XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
        XML_CONST_WRITE(l_pBuffer, ATT_FAMILY);
        XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
        // Family
        //==============================
        l_Value = sprintf((ansichar*)l_pBuffer, "%u", GET_FAMILY(m_ClassID));
        l_pBuffer += l_Value;
        //==============================
        XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
        XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
        XML_CONST_WRITE(l_pBuffer, ATT_GENUS);
        XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
        // Genus
        //==============================
        l_Value = sprintf((ansichar*)l_pBuffer, "%u", GET_GENUS(m_ClassID));
        l_pBuffer += l_Value;
        //==============================
        XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
        XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
        XML_CONST_WRITE(l_pBuffer, ATT_SPECIES);
        XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
        // Species
        //==============================
        l_Value = sprintf((ansichar*)l_pBuffer, "%u", GET_SPECIES(m_ClassID));
        l_pBuffer += l_Value;
        //==============================
        XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);    
        XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
        XML_CONST_WRITE(l_pBuffer, CHAR_RETURN);
        XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
        XML_CONST_WRITE(l_pBuffer, TAG_MEMBERS);
        XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
        XML_CONST_WRITE(l_pBuffer, CHAR_RETURN);

        for(const byte *l_pDefPtr = SKIP_DEF_HEADER(m_pDefinition);
            l_MemberOffset < l_MemberCount;
            l_MemberOffset ++)
        {
            uint16 l_MinVersion;
            uint16 l_MaxVersion;
            boolean l_InVersion = FALSE;

            // Check to see if we are greater than the Min Version
            LINE_COPY(&l_MinVersion, l_pDefPtr, sizeof(uint16));
            l_pDefPtr += sizeof(uint16);

            // Check to see if we are less than the Max Version
            LINE_COPY(&l_MaxVersion, l_pDefPtr, sizeof(uint16));
            l_pDefPtr += sizeof(uint16);

            l_InVersion = (l_MinVersion <= Version && Version <= l_MaxVersion);
            l_TypeInfo = *l_pDefPtr++;

            XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
            XML_CONST_WRITE(l_pBuffer, TAG_MEMBER);
            XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
            XML_CONST_WRITE(l_pBuffer, ATT_TYPE);
            XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
            XML_TYPE_WRITE(l_pBuffer, l_TypeInfo);
            XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);        

            if((l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
            {
                // get the ClassID
                LINE_COPY(&l_Value, l_pDefPtr, sizeof(uint32));
                l_pDefPtr += sizeof(uint32);

                XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
                XML_CONST_WRITE(l_pBuffer, ATT_CLASS);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);

                if(l_Value == ANY_CLASS::CLASSID)
                {
                    XML_CONST_WRITE(l_pBuffer, VAL_ANY_CLASS);
                }
                else
                {
                    // Check to see if we have a ClassName string available. If not, exit
                    // immediately with a value of 0 to signify a failure.
                    if((l_pDefinitionString = XMLGetClassText((const byte*)(m_pDictionary == NULL ? NULL : m_pDictionary->GetDefinition(l_Value)))) == NULL)
                    {
                        return 0;
                    }
                    XML_WRITE(l_pBuffer, l_pDefinitionString);
                }

                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);                        
            }

            if(l_TypeInfo >= DMOIB_TYPE_FLAG_PTR)
            {
                XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
                XML_CONST_WRITE(l_pBuffer, ATT_SIZE);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);            

                if(l_TypeInfo & DMOIB_TYPE_FLAG_PTR)
                {                
                    XML_CONST_WRITE(l_pBuffer, VAL_UNBOUNDED);
                }
                else if(l_TypeInfo & DMOIB_TYPE_FLAG_ARR)
                {
                    l_Value = 0;
                    LINE_COPY(&l_Value, l_pDefPtr, sizeof(uint16));
                    l_pDefPtr += sizeof(uint16);                        

                    l_Value = sprintf((ansichar*)l_pBuffer, "%d", l_Value);
                    l_pBuffer += l_Value;
                }                    

                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE); 
            }

            if(l_TypeInfo != DMOIB_TYPE_BASE_CLASS)
            {
                XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
                XML_CONST_WRITE(l_pBuffer, ATT_NAME);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);

                // Check to see if we have a MemberName string available. If not, exit
                // immediately with a value of 0 to signify a failure.
                if((l_pDefinitionString = XMLGetMemberString(m_pDefinition, l_MemberOffset)) == NULL)
                {
                    return 0;
                }
                XML_WRITE(l_pBuffer, l_pDefinitionString);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
            }
            else
            {
                XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
                XML_CONST_WRITE(l_pBuffer, ATT_INHERITED);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
                XML_CONST_WRITE(l_pBuffer, VAL_YES);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
            }

            XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
            XML_CONST_WRITE(l_pBuffer, ATT_MINVERSION);
            XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
            // MinVersion
            //==============================
            l_Value = sprintf((ansichar*)l_pBuffer, "%d", l_MinVersion);
            l_pBuffer += l_Value;
            //==============================
            XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
            XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
            XML_CONST_WRITE(l_pBuffer, ATT_MAXVERSION);
            XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
            // Version
            //==============================
            if(l_MaxVersion == HIGHEST_KNOWN_VERSION)
            {
                XML_CONST_WRITE(l_pBuffer, VAL_CURRENT_VERSION);
            }
            else
            {
                l_Value = sprintf((ansichar*)l_pBuffer, "%d", l_MaxVersion);
                l_pBuffer += l_Value;
            }
            //==============================
            XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);


            // Write the Alias value, if available
            if((l_Value = GetMemberAlias(l_MemberOffset + m_BaseClassMemberCount)) != OMITTED)
            {
                XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
                XML_CONST_WRITE(l_pBuffer, ATT_ALIAS);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);            

                l_Value = sprintf((ansichar*)l_pBuffer, "%d", l_Value);
                l_pBuffer += l_Value;                

                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE); 
            }

            // Write the Units value, if available
            if((l_Value = GetMemberUnits(l_MemberOffset + m_BaseClassMemberCount)) != (uint8)(OMITTED & 0x0FF))
            {
                XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
                XML_CONST_WRITE(l_pBuffer, ATT_UNITS);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);            

                l_Value = sprintf((ansichar*)l_pBuffer, "%d", l_Value);
                l_pBuffer += l_Value;                

                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE); 
            }
            
            uint16  l_ScaleC = GetMemberScaleCoefficient(l_MemberOffset + m_BaseClassMemberCount);
            int8    l_ScaleE = GetMemberScaleExponent(l_MemberOffset + m_BaseClassMemberCount);

            // Write the Scale coefficient and exponent values only if both 
            // are available.
            if(l_ScaleC != 1 || l_ScaleE != 0)
            {
                XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
                XML_CONST_WRITE(l_pBuffer, ATT_SCALE_COEFFICIENT);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);            

                l_Value = sprintf((ansichar*)l_pBuffer, "%u", l_ScaleC);
                l_pBuffer += l_Value;                

                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE); 
            
                XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
                XML_CONST_WRITE(l_pBuffer, ATT_SCALE_EXPONENT);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);            

                l_Value = sprintf((ansichar*)l_pBuffer, "%d", l_ScaleE);
                l_pBuffer += l_Value;

                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
            }

            m_pMemberIter = m_pMemberArray + m_pMemberTranslation[l_MemberOffset];

            if(l_InVersion && m_pMemberIter != m_pMemberEnd)
            {
                byte  l_ByteWidth = (byte)(l_TypeInfo & DMOIB_TYPE_MASK_BW);

                XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
                XML_CONST_WRITE(l_pBuffer, CHAR_RETURN);
                XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
                XML_CONST_WRITE(l_pBuffer, TAG_VALUE);
                XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
               

                // Check for special types
                switch(l_ByteWidth)
                {
                    case DMOIB_TYPE_FLAG_TIME:
                        l_ByteWidth = sizeof(MOIB_TIME);
                        break;
                    default:
                        l_ByteWidth++;
                        break;
                } 

                if((l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
                {
                    if(m_pMemberIter->Ptr == NULL)
                    {
                        return 0;
                    }
                    else
                    {
                        XML_CONST_WRITE(l_pBuffer, CHAR_RETURN);

                        if((l_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
                           (l_TypeInfo & DMOIB_TYPE_FLAG_PTR))
                        {          
                            l_pBuffer += ((CDMOIBVector*)m_pMemberIter->Ptr)->XMLSerialize(l_pBuffer);
                        }
                        else
                        {                    
                            l_Value = ((CDMOIBObject*)m_pMemberIter->Ptr)->XMLSerialize(l_pBuffer);

                            if(l_Value == 0)
                            {
                                return 0;
                            }

                            l_pBuffer += l_Value - XML_CONST_SIZE(CHAR_TERMINATOR);
                        }
                    }
                    
                }
                else if((l_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
                        (l_TypeInfo & DMOIB_TYPE_FLAG_PTR))
                {
                    byte* l_pArrayIter = ((byte*)m_pMemberIter->Ptr);
                    byte* l_pArrayEnd  = ((byte*)m_pMemberIter->Ptr) + (m_pMemberIter->m_PointerInfo.m_NumElements * l_ByteWidth);

                    XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
                    XML_CONST_WRITE(l_pBuffer, TAG_CDATA_OPEN);

                    // MOIB_TIME
                    if((l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_TIME)
                    {
                        while(l_pArrayIter < l_pArrayEnd)
                        {
                            timetostr(*((MOIB_TIME*)l_pArrayIter), l_pBuffer);
                            l_pBuffer += XML_TIME_LENGTH(*(MOIB_TIME*)l_pArrayIter);
                            XML_CONST_WRITE(l_pBuffer, CHAR_ARRAY_DELIM);
                            l_pArrayIter += l_ByteWidth;
                        }  
                    }
                    // Text
                    else if(l_TypeInfo & DMOIB_TYPE_FLAG_TXT)
                    {
                        // Ansichar
                        if(l_ByteWidth == 1)
                        {
                            ansichar* l_pStringIter = (ansichar*)m_pMemberIter->Ptr;

                            while(l_pStringIter != NULL && *l_pStringIter != 0)
                            {
                                *l_pBuffer++ = *l_pStringIter++;
                            }
                        }
                        // Widechar
                        else
                        {
                            widechar* l_pStringIter = (widechar*)m_pMemberIter->Ptr;

                            while(l_pStringIter != NULL && WCTOB(*l_pStringIter) != 0)
                            {
                                *l_pBuffer++ = WCTOB(*l_pStringIter);
                                *l_pStringIter++;
                            }
                        }
                    }
                    // Floating Point
                    else if(l_TypeInfo & DMOIB_TYPE_FLAG_FP)
                    {
                        // Float32
                        if(l_ByteWidth == 4)
                        {
                            while(l_pArrayIter < l_pArrayEnd)
                            {
                                l_Value = sprintf((ansichar*)l_pBuffer, "%+.9E", *(float32*)l_pArrayIter);
                                l_pBuffer += l_Value;
                                XML_CONST_WRITE(l_pBuffer, CHAR_ARRAY_DELIM);
                                l_pArrayIter += l_ByteWidth;
                            }                        
                        }
                        // Float64
                        else
                        {
                            while(l_pArrayIter < l_pArrayEnd)
                            {
                                l_Value = sprintf((ansichar*)l_pBuffer, "%+.16E", *(float64*)l_pArrayIter);
                                l_pBuffer += l_Value;
                                XML_CONST_WRITE(l_pBuffer, CHAR_ARRAY_DELIM);
                                l_pArrayIter += l_ByteWidth;
                            }
                        }
                    }
                    // All other Integers
                    else
                    {
                        // Unsigned
                        if(l_TypeInfo & DMOIB_TYPE_FLAG_UNS)
                        {
                            while(l_pArrayIter < l_pArrayEnd)
                            {
                                l_Value = 0;
                                BYTE_COPY(&l_Value, sizeof(l_Value), l_pArrayIter, l_ByteWidth);
                                l_Value = sprintf((ansichar*)l_pBuffer, "%lu", l_Value);
                                l_pBuffer += l_Value;
                                XML_CONST_WRITE(l_pBuffer, CHAR_ARRAY_DELIM);
                                l_pArrayIter += l_ByteWidth;
                            }
                        }
                        // Signed
                        else
                        {
                            while(l_pArrayIter < l_pArrayEnd)
                            {   
                                l_Value = 0;
                                BYTE_COPY(&l_Value, sizeof(l_Value), l_pArrayIter, l_ByteWidth);

                                if( IS_NEGATIVE(l_Value, l_ByteWidth - 1) )
                                    l_Value = SIGN_EXTEND(l_Value, l_ByteWidth - 1);

                                l_Value = sprintf((ansichar*)l_pBuffer, "%d", l_Value);
                                l_pBuffer += l_Value;
                                XML_CONST_WRITE(l_pBuffer, CHAR_ARRAY_DELIM);
                                l_pArrayIter += l_ByteWidth;
                            }
                        }
                    }

                    XML_CONST_WRITE(l_pBuffer, TAG_CDATA_CLOSE);
                    XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
                }
                else
                {

                    XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
                    XML_CONST_WRITE(l_pBuffer, TAG_CDATA_OPEN);

                    // MOIB_TIME
                    if((l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_TIME)
                    {
                        timetostr(m_pMemberIter->Time, l_pBuffer);
                        l_pBuffer += XML_TIME_LENGTH(m_pMemberIter->Time);
                    }
                    // Text
                    else if(l_TypeInfo & DMOIB_TYPE_FLAG_TXT)
                    {
                        // Both ansichar and widechar are stored in uint32,
                        // so just use the value there.
                        *l_pBuffer = WCTOB(m_pMemberIter->Uint32);

                        if(*l_pBuffer != 0)
                            l_pBuffer++;
                    }
                    // Floating Point
                    else if(l_TypeInfo & DMOIB_TYPE_FLAG_FP)
                    {
                        // Float32
                        if(l_ByteWidth == 4)
                        {
                            l_Value = sprintf((ansichar*)l_pBuffer, "%+.9E", m_pMemberIter->Float32);
                            l_pBuffer += l_Value;
                        }
                        // Float64
                        else
                        {
                            l_Value = sprintf((ansichar*)l_pBuffer, "%+.16E", m_pMemberIter->Float64);
                            l_pBuffer += l_Value;
                        }
                    }
                    // All other Integers
                    else
                    {
                        // Unsigned
                        if(l_TypeInfo & DMOIB_TYPE_FLAG_UNS)
                        {
                            l_Value = sprintf((ansichar*)l_pBuffer, "%lu", m_pMemberIter->Uint32);
                            l_pBuffer += l_Value;
                        }
                        // Signed
                        else
                        {
                            l_Value = 0;
                            BYTE_COPY(&l_Value, sizeof(l_Value), &m_pMemberIter->Int32, l_ByteWidth);

                            if( IS_NEGATIVE(l_Value, l_ByteWidth - 1) )
                                l_Value = SIGN_EXTEND(l_Value, l_ByteWidth - 1);

                            l_Value = sprintf((ansichar*)l_pBuffer, "%d", l_Value);
                            l_pBuffer += l_Value;
                        }
                    }           

                    XML_CONST_WRITE(l_pBuffer, TAG_CDATA_CLOSE);
                    XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
                }                   

                XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
                XML_CONST_WRITE(l_pBuffer, CHAR_SLASH);
                XML_CONST_WRITE(l_pBuffer, TAG_VALUE);
                XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
                XML_CONST_WRITE(l_pBuffer, CHAR_RETURN);

                XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
                XML_CONST_WRITE(l_pBuffer, CHAR_SLASH);
                XML_CONST_WRITE(l_pBuffer, TAG_MEMBER);
                XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
                XML_CONST_WRITE(l_pBuffer, CHAR_RETURN);
            }
            else
            {
                XML_CONST_WRITE(l_pBuffer, CHAR_SLASH);
                XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
                XML_CONST_WRITE(l_pBuffer, CHAR_RETURN);
            }
        }

        XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
        XML_CONST_WRITE(l_pBuffer, CHAR_SLASH);
        XML_CONST_WRITE(l_pBuffer, TAG_MEMBERS);
        XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
        XML_CONST_WRITE(l_pBuffer, CHAR_RETURN);

        // Add in any enumerations
        for(uint16* l_pIter = m_pMemberTranslation; l_pIter < (m_pMemberTranslation + m_MemberCountTotal); l_pIter++)
        {
            uint16          l_EnumeratorValue = GetMember((*l_pIter) + m_BaseClassMemberCount);
            const ansichar* l_pEnumerationString = XMLGetEnumerationString(m_pDefinition, 
                                                                           *l_pIter);
            const ansichar* l_pEnumeratorString = XMLGetEnumeratorString(m_pDefinition, 
                                                                         *l_pIter, 
                                                                         l_EnumeratorValue);

            if(*l_pIter < m_MemberCount && 
                l_pEnumerationString != NULL && 
                l_pEnumeratorString != NULL)
            {
                XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
                XML_CONST_WRITE(l_pBuffer, TAG_ENUMERATION);

                XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
                XML_CONST_WRITE(l_pBuffer, ATT_NAME);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
                XML_WRITE(l_pBuffer, l_pEnumerationString);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);

                XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
                XML_CONST_WRITE(l_pBuffer, CHAR_RETURN);
                XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
                XML_CONST_WRITE(l_pBuffer, TAG_ENUM);

                XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
                XML_CONST_WRITE(l_pBuffer, ATT_NAME);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
                XML_WRITE(l_pBuffer, l_pEnumeratorString);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);

                XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
                XML_CONST_WRITE(l_pBuffer, ATT_DEFAULT_VALUE);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
                l_pBuffer += sprintf((ansichar*)l_pBuffer, "%u", l_EnumeratorValue);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);

                XML_CONST_WRITE(l_pBuffer, CHAR_SPACE);
                XML_CONST_WRITE(l_pBuffer, ATT_SELECTED);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);
                XML_CONST_WRITE(l_pBuffer, VAL_YES);
                XML_CONST_WRITE(l_pBuffer, CHAR_QUOTE);

                XML_CONST_WRITE(l_pBuffer, CHAR_SLASH);
                XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
                XML_CONST_WRITE(l_pBuffer, CHAR_RETURN);
                XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
                XML_CONST_WRITE(l_pBuffer, CHAR_SLASH);
                XML_CONST_WRITE(l_pBuffer, TAG_ENUMERATION);
                XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
                XML_CONST_WRITE(l_pBuffer, CHAR_RETURN);
            }
        }

        XML_CONST_WRITE(l_pBuffer, CHAR_LESSTHAN);
        XML_CONST_WRITE(l_pBuffer, CHAR_SLASH);
        XML_CONST_WRITE(l_pBuffer, TAG_DEFINITION);
        XML_CONST_WRITE(l_pBuffer, CHAR_GREATERTHAN);
        XML_CONST_WRITE(l_pBuffer, CHAR_RETURN);

        XML_CONST_WRITE(l_pBuffer, CHAR_TERMINATOR);
    }

    return (uint32)(l_pBuffer - pXMLBuffer);
}
/*
===============================================================================
    Method to get the XML textual size of the vector
===============================================================================
*/
uint32 CDMOIBVector::XMLSizeAll()
{
    uint32 l_XMLSerialSize = 0;
    uint32 l_XMLObjectSize = 0;
    
    for(uint16 l_Index = 0; l_Index < m_NumElements; l_Index++)
    {
        l_XMLObjectSize = m_ppObjectArray[l_Index]->XMLSizeAll();

        if(l_XMLObjectSize > 0)
            l_XMLObjectSize -= XML_CONST_SIZE(CHAR_TERMINATOR);

        l_XMLSerialSize += l_XMLObjectSize;
    }

    return l_XMLSerialSize;
}
/*
===============================================================================
    Method to put the vector into an XML text stream
===============================================================================
*/
uint32 CDMOIBVector::XMLSerialize(ansichar* pXMLBuffer)
{
    ansichar* l_pBuffer = pXMLBuffer;
    uint32 l_XMLObjectSize = 0;
    
    if(pXMLBuffer != NULL)
    {
        for(uint16 l_Index = 0; l_Index < m_NumElements; l_Index++)
        {
            l_XMLObjectSize = m_ppObjectArray[l_Index]->XMLSerialize(l_pBuffer);

            if(l_XMLObjectSize > 0)
                l_XMLObjectSize -= XML_CONST_SIZE(CHAR_TERMINATOR);

            l_pBuffer += l_XMLObjectSize;
        }
    }

    return (uint32)(l_pBuffer - pXMLBuffer);
}
#endif

#if ((TARGET_XML_SUPPORT == XML_DESERIALIZE) || (TARGET_XML_SUPPORT == XML_ALL))

/*
===============================================================================
    Method to get the object from an XML text stream
===============================================================================
*/
uint32 CDMOIBObject::XMLDeserialize(ansichar* pXMLBuffer)
{  
    uint32          l_RetVal = 0;

    if(pXMLBuffer != NULL && DefinitionXMLSize((void*)m_pDefinition) > 0)
    {
        XML_Parser      l_XMLParser = XML_ParserCreate(NULL);    
        XMLUserData_t*  l_pXMLUserData = &m_XMLUserData;

        m_XMLUserData.m_pXMLParent          = NULL;
        m_XMLUserData.m_XMLCurrentIndex     = 0;
        m_XMLUserData.m_XMLContinueParsing  = TRUE;

        XML_SetUserData(l_XMLParser, &l_pXMLUserData);
        XML_SetElementHandler(l_XMLParser, 
                              (XML_StartElementHandler)XMLStartElement, 
                              (XML_EndElementHandler)XMLEndElement);
        XML_SetCharacterDataHandler(l_XMLParser, 
                                    (XML_CharacterDataHandler)XMLCharacterData);

        l_RetVal = XML_Parse(l_XMLParser, pXMLBuffer, (uint32)strlen(pXMLBuffer), TRUE);

        XML_ParserFree(l_XMLParser);

        // Check to see if we had to stop parsing prematurely.
        if(l_pXMLUserData != NULL && l_pXMLUserData->m_XMLContinueParsing == FALSE)
            // If we did, then force the return value to 0 to denote failure.
            l_RetVal = 0;

        if(l_RetVal == 0)
            // If the Parsing failed in any way, reset the entire object so
            // the object doesn't have corrupt data.
            Reset();
    }

    return l_RetVal;
}

/*
===============================================================================
    Callback function to parse the starting tag name of an XML element
===============================================================================
*/
void XMLStartElement(void* pUserData, void* pTagName, void *pAttributes)
{    
    XMLUserData_t**     l_ppXMLUserData = (XMLUserData_t**)pUserData;
    const XML_Char*     l_pTagName      = (const XML_Char*)pTagName;
    const XML_Char**    l_pAttributes   = (const XML_Char**)pAttributes;

    if(l_ppXMLUserData != NULL && (*l_ppXMLUserData) != NULL && 
       (*l_ppXMLUserData)->m_XMLContinueParsing == TRUE && 
       (*l_ppXMLUserData)->m_pXMLObject != NULL)
    {
        switch(l_pTagName[0])
        {
            case 'D': // DEFINITION tag
            case 'd': // definition tag
                if(strncmp(l_pTagName, XML_CONST_STR(TAG_DEFINITION), XML_CONST_SIZE(TAG_DEFINITION)) == 0)
                {
                    const ansichar* l_pDefinitionString = NULL;
                    const XML_Char* l_pClassName        = NULL;                    
                    uint16          l_Version           = 0;
                    uint16          l_Family            = FmMAX;
                    uint8           l_Genus             = GnMAX;
                    uint8           l_Species           = 0xFF;
                    CDMOIBObject*   l_pObject           = (CDMOIBObject*)((*l_ppXMLUserData)->m_pXMLObject);

                    // In case we hit a DEFINITION tag inside a VALUE tag, reset
                    // the VALUE flag.
                    (*l_ppXMLUserData)->m_XMLIsValue = FALSE;
                    
                    while(l_pAttributes[0] != NULL)
                    {
                        // Do a quick check on the first character of the Attribute 
                        // string, so we don't have to check the full strings if we
                        // don't need to.
                        switch(*l_pAttributes[0])
                        {
                            case 'c': // class
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_CLASS), XML_CONST_SIZE(ATT_CLASS)-1) != 0)
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
                                l_pClassName = l_pAttributes[1];
                                break;                         
                            case 'v': // version
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_VERSION), XML_CONST_SIZE(ATT_VERSION)-1) != 0)
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
                                l_Version = (uint16)strtoul(l_pAttributes[1], NULL, 10);
                                break;
                            case 'f': // family
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_FAMILY), XML_CONST_SIZE(ATT_FAMILY)-1) != 0)
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
                                l_Family = (uint16)strtoul(l_pAttributes[1], NULL, 10);
                                break;
                            case 'g': // genus
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_GENUS), XML_CONST_SIZE(ATT_GENUS)-1) != 0)
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
                                l_Genus = (uint8)strtoul(l_pAttributes[1], NULL, 10);
                                break;
                            case 's': // species
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_SPECIES), XML_CONST_SIZE(ATT_SPECIES)-1) != 0)
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
                                l_Species = (uint8)strtoul(l_pAttributes[1], NULL, 10);
                                break;
                            default:
                            {
                                (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                return;
                            }
                        }
                        l_pAttributes += 2;
                    }

                    // We're done with the DEFINITION tag, so check if we are 
                    // currently a Vector, or a regular object
                    if((*l_ppXMLUserData)->m_XMLObjectType == DMOIB_TYPE_VECTOR_PTR)
                    {
                        l_pObject = ((CDMOIBVector*)((*l_ppXMLUserData)->m_pXMLObject))->At((*l_ppXMLUserData)->m_XMLCurrentIndex);

                        // If there was no object at the index, then try to 
                        // Insert one.
                        if(l_pObject == NULL)
                            l_pObject = ((CDMOIBVector*)((*l_ppXMLUserData)->m_pXMLObject))->
                                        Insert(MAKE_CLASSID(l_Family, l_Genus, l_Species), l_Version, (*l_ppXMLUserData)->m_XMLCurrentIndex);

                        // Either we had an object at the index, or pushed
                        // one in, so mark up our pUserData to reflect it.
                        if(l_pObject != NULL)
                        {
                            (*l_ppXMLUserData)->m_XMLCurrentIndex++;

                            l_pObject->m_XMLUserData.m_pXMLParent = (*l_ppXMLUserData);
                            l_pObject->m_XMLUserData.m_XMLCurrentIndex = 0;
                            l_pObject->m_XMLUserData.m_XMLContinueParsing = TRUE;
                            (*l_ppXMLUserData) = &l_pObject->m_XMLUserData;
                        }
                    }
                    
                    if(   l_pObject == NULL
                       || l_pObject->Version() <  l_Version 
                       || l_pObject->Family()  != l_Family
                       || l_pObject->Genus()   != l_Genus
                       || l_pObject->Species() != l_Species
                       || (l_pDefinitionString = XMLGetClassText(l_pObject->m_pDefinition)) == NULL
                       || (strcmp(l_pClassName, l_pDefinitionString) != 0))
                    {                        
                        (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                        return;
                    }                    
                }
                else
                {
                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                    return;
                }
                break;
            case 'M': // MEMBER(S) tag                
            case 'm': // member(s) tag 
                if((*l_ppXMLUserData)->m_XMLObjectType == DMOIB_TYPE_VECTOR_PTR)
                {
                    // We shouldn't run into a MEMBER(S) tag if we're processing a
                    // Vector.
                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                    return;
                }
                else if(strncmp(l_pTagName, XML_CONST_STR(TAG_MEMBERS), XML_CONST_SIZE(TAG_MEMBERS)) == 0)
                {
                    // Don't need to do anything for MEMBERS tag, just make sure it's right.
                    break;
                }
                else if(strncmp(l_pTagName, XML_CONST_STR(TAG_MEMBER), XML_CONST_SIZE(TAG_MEMBER)) == 0)
                {
                    while(l_pAttributes[0] != NULL)
                    {
                        // Do a quick check on the first character of the Attribute 
                        // string, so we don't have to check the full strings if we
                        // don't need to.

                        // NOTE: For now, all we will check is to make sure that the
                        // member name matches the dictionary's entry. We could 
                        // check the type, size, class, and versions in the future.
                        // But for now we will rely on the dictionary as the 
                        // "official" source of member information, and make all
                        // judgements based on it.
                        switch(*l_pAttributes[0])
                        {
                            case 't': // type
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_TYPE), XML_CONST_SIZE(ATT_TYPE)-1) != 0)
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
                                break;                         
                            case 'n': // name
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_NAME), XML_CONST_SIZE(ATT_NAME)-1) != 0)
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
								else
								{
                                    // Grab a reference to the definition with 
                                    // the XML names in it.
									const byte*     l_pDefintion = ((CDMOIBObject*)((*l_ppXMLUserData)->m_pXMLObject))->m_pDefinition;
									const ansichar* l_pMemberName = NULL;

                                    // Reset the current index iterator to the 
                                    // first member in the object.
                                    (*l_ppXMLUserData)->m_XMLCurrentIndex = 0;

                                    // Iterate through all member names with a
                                    // linear search until a match is found
                                    // for the current XML name.
									while((l_pMemberName = XMLGetMemberString(l_pDefintion, (*l_ppXMLUserData)->m_XMLCurrentIndex))!= NULL && 
                                          (strcmp(l_pAttributes[1], l_pMemberName) != 0))
                                    {
                                        (*l_ppXMLUserData)->m_XMLCurrentIndex++;
                                    }
								}
                                break;
                            case 'c': // class
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_CLASS), XML_CONST_SIZE(ATT_CLASS)-1) != 0)
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
                                break;
                            case 's': // size/scales
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_SIZE), XML_CONST_SIZE(ATT_SIZE)-1) != 0
                                && strncmp(l_pAttributes[0], XML_CONST_STR(ATT_SCALE_COEFFICIENT), XML_CONST_SIZE(ATT_SCALE_COEFFICIENT)-1) != 0
                                && strncmp(l_pAttributes[0], XML_CONST_STR(ATT_SCALE_EXPONENT), XML_CONST_SIZE(ATT_SCALE_EXPONENT)-1) != 0
                                   )
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
                                break;
                            case 'm': // versions
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_MINVERSION), XML_CONST_SIZE(ATT_MINVERSION)-1) != 0 &&
                                   strncmp(l_pAttributes[0], XML_CONST_STR(ATT_MAXVERSION), XML_CONST_SIZE(ATT_MAXVERSION)-1) != 0)
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
                                break;
                            case 'a': // alias
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_ALIAS), XML_CONST_SIZE(ATT_ALIAS)-1) != 0)
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
                                break;
                            case 'u': // units
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_UNITS), XML_CONST_SIZE(ATT_UNITS)-1) != 0)
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
                                break;
                            case 'i': // inherited
                                if(strncmp(l_pAttributes[0], XML_CONST_STR(ATT_INHERITED), XML_CONST_SIZE(ATT_INHERITED)-1) != 0)
                                {
                                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                    return;
                                }
                                // This member says it's inherited, so check to
                                // see if we have a valid BaseClass. If we do,
                                // then that base class is at member index 0,
                                // so reset the iterator.
                                else if(((CDMOIBObject*)((*l_ppXMLUserData)->m_pXMLObject))->BaseClass() != NULL)
                                {
                                    // Reset the current index iterator to the 
                                    // first member in the object.
                                    (*l_ppXMLUserData)->m_XMLCurrentIndex = 0;
                                }
                                break;
                            default:
                            {
                                (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                return;
                            }
                        }
                        l_pAttributes += 2;
                    }    
                    
                }
                else
                {
                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                    return;
                }
                break;
            case 'V': // VALUE
            case 'v': // value
                if(strncmp(l_pTagName, XML_CONST_STR(TAG_VALUE), XML_CONST_SIZE(TAG_VALUE)) == 0)
                {
                    // Flag that we are inside the VALUE tag, so we know now we
                    // can't skip anymore whitespace
                    (*l_ppXMLUserData)->m_XMLIsValue = TRUE;
                    
                    // Check the type of the current member. If it is unavailable,
                    // this will fall through. Otherwise handle the special cases
                    // (sub objects and vectors).
                    switch(((CDMOIBObject*)((*l_ppXMLUserData)->m_pXMLObject))->GetMemberType((*l_ppXMLUserData)->m_XMLCurrentIndex + 
                                                                                              ((CDMOIBObject*)((*l_ppXMLUserData)->m_pXMLObject))->m_BaseClassMemberCount))
                    {
                        case DMOIB_TYPE_VECTOR_PTR:
                        case DMOIB_TYPE_VECTOR_ARR:
                        {
                            CDMOIBVector* l_pObject = ((CDMOIBObject*)((*l_ppXMLUserData)->m_pXMLObject))->
                                                    GetMember((*l_ppXMLUserData)->m_XMLCurrentIndex + 
                                                              ((CDMOIBObject*)((*l_ppXMLUserData)->m_pXMLObject))->m_BaseClassMemberCount);

                            if(l_pObject == NULL)
                            {
                                (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                return;
                            }
                            else
                            {
                                l_pObject->Clear();
                                l_pObject->m_XMLUserData.m_pXMLParent = (*l_ppXMLUserData);
                                l_pObject->m_XMLUserData.m_XMLCurrentIndex = 0;
                                l_pObject->m_XMLUserData.m_XMLContinueParsing = TRUE;
                                (*l_ppXMLUserData) = &l_pObject->m_XMLUserData;
                            }

                            break;
                        }
                        case DMOIB_TYPE_OBJECT:
                        case DMOIB_TYPE_BASE_CLASS:
                        {
                            CDMOIBObject* l_pObject = ((CDMOIBObject*)((*l_ppXMLUserData)->m_pXMLObject))->
                                                    GetMember((*l_ppXMLUserData)->m_XMLCurrentIndex + 
                                                              ((CDMOIBObject*)((*l_ppXMLUserData)->m_pXMLObject))->m_BaseClassMemberCount);

                            if(l_pObject == NULL)
                            {
                                (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                                return;
                            }
                            else
                            {
                                l_pObject->m_XMLUserData.m_pXMLParent = (*l_ppXMLUserData);
                                l_pObject->m_XMLUserData.m_XMLCurrentIndex = 0;
                                l_pObject->m_XMLUserData.m_XMLContinueParsing = TRUE;
                                (*l_ppXMLUserData) = &l_pObject->m_XMLUserData;
                            }
                            
                            break;
                        }
                        default:
                            break;
                    }
                }
                else
                {
                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                    return;
                }
                break;
            case 'E': // ENUMERATION/ENUM tags
            case 'e': // enumeration/enum tags
                if(strncmp(l_pTagName, XML_CONST_STR(TAG_ENUMERATION), XML_CONST_SIZE(TAG_ENUMERATION)) == 0)
                {
                    ;//
                }
                else if(strncmp(l_pTagName, XML_CONST_STR(TAG_ENUM), XML_CONST_SIZE(TAG_ENUM)) == 0)
                {
                    ;//
                }
                else
                {
                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                    return;
                }
                break;
            default:
            {
                (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                return;
            }
        }
    }
}
/*
===============================================================================
    Callback function to parse the ending tag name of an XML element
===============================================================================
*/
void XMLEndElement(void* pUserData, void* pTagName)
{
    XMLUserData_t** l_ppXMLUserData = (XMLUserData_t**)pUserData;
    const XML_Char* l_pTagName = (const XML_Char*)pTagName;

    if(l_ppXMLUserData != NULL && (*l_ppXMLUserData) != NULL && 
       (*l_ppXMLUserData)->m_XMLContinueParsing == TRUE && 
       (*l_ppXMLUserData)->m_pXMLObject != NULL)
    {
        switch(l_pTagName[0])
        {
            case 'D': // DEFINITION tag
            case 'd': // definition tag
                if(strncmp(l_pTagName, XML_CONST_STR(TAG_DEFINITION), XML_CONST_SIZE(TAG_DEFINITION)) != 0)
                {
                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                    return;
                }
                else
                {
                    (*l_ppXMLUserData) = (*l_ppXMLUserData)->m_pXMLParent;
                }
                break;
            case 'M': // MEMBER(S) tag
            case 'm': // member(s) tag 
                if(strncmp(l_pTagName, XML_CONST_STR(TAG_MEMBERS), XML_CONST_SIZE(TAG_MEMBERS)) == 0)
                {
                    ;//
                }
                else if(strncmp(l_pTagName, XML_CONST_STR(TAG_MEMBER), XML_CONST_SIZE(TAG_MEMBER)) == 0)
                {
                    // If we are a Vector, we need to step up one more level
                    // when the VAR tag closes.
                    if((*l_ppXMLUserData)->m_XMLObjectType == DMOIB_TYPE_VECTOR_PTR)
                    {
                        (*l_ppXMLUserData) = (*l_ppXMLUserData)->m_pXMLParent;
                    }

                    (*l_ppXMLUserData)->m_XMLCurrentIndex++;
                }
                else
                {
                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                    return;
                }
                break;
            case 'V': // VALUE tag
            case 'v': // value tag
                if(strncmp(l_pTagName, XML_CONST_STR(TAG_VALUE), XML_CONST_SIZE(TAG_VALUE)) == 0)
                {
                    (*l_ppXMLUserData)->m_XMLIsValue = FALSE;
                }
                else
                {
                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                    return;
                }
                break;
            case 'E': // ENUMERATION/ENUM tags
            case 'e': // enumeration/enum tags
                if(strncmp(l_pTagName, XML_CONST_STR(TAG_ENUMERATION), XML_CONST_SIZE(TAG_ENUMERATION)) == 0)
                {
                    ;//
                }
                else if(strncmp(l_pTagName, XML_CONST_STR(TAG_ENUM), XML_CONST_SIZE(TAG_ENUM)) == 0)
                {
                    ;//
                }
                else
                {
                    (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                    return;
                }
                break;
            default:
            {
                (*l_ppXMLUserData)->m_XMLContinueParsing = FALSE;
                return;
            }
        }
    }
}
/*
===============================================================================
    Callback function to parse the character data (CDATA) of an XML element
===============================================================================
*/
void XMLCharacterData(void* pUserData, void* pCDataString, int CDataLength)
{
    XMLUserData_t**     l_ppXMLUserData = ((XMLUserData_t**)pUserData);
    const XML_Char*     l_pCDataString  = ((const XML_Char*)pCDataString);
    const XML_Char*     l_pCDataEnd     = ((const XML_Char*)pCDataString) + CDataLength;

    if(l_ppXMLUserData != NULL && (*l_ppXMLUserData) != NULL && 
       (*l_ppXMLUserData)->m_XMLContinueParsing == TRUE && 
       (*l_ppXMLUserData)->m_pXMLObject != NULL && 
       ((*l_ppXMLUserData)->m_XMLObjectType == DMOIB_TYPE_OBJECT || (*l_ppXMLUserData)->m_XMLObjectType == DMOIB_TYPE_BASE_CLASS))
    {
        if((*l_ppXMLUserData)->m_XMLIsValue == TRUE)
        {
            CDMOIBObject*   l_pObject = ((CDMOIBObject*)((*l_ppXMLUserData)->m_pXMLObject));
            uint8           l_TypeInfo;
            uint8           l_ByteWidth;

            l_TypeInfo = l_pObject->GetMemberType((*l_ppXMLUserData)->m_XMLCurrentIndex + l_pObject->m_BaseClassMemberCount);
            l_ByteWidth = (uint8)(l_TypeInfo & DMOIB_TYPE_MASK_BW) + 1;
            
            if(l_TypeInfo == DMOIB_TYPE_UNKNOWN || (l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_OBJ)
            {
                // Do nothing for object members. They will be processed in
                // order with the next step.
            }            
            else if((l_TypeInfo & DMOIB_TYPE_FLAG_ARR) || 
                    (l_TypeInfo & DMOIB_TYPE_FLAG_PTR))
            {   
                uint16  l_ArraySize = 0;
                byte*   l_pDataPtr;

                // If we are dealing with Text, then we don't have any delimiters
                // in the CData, so just grab the size straight-up.
                if(l_TypeInfo & DMOIB_TYPE_FLAG_TXT)
                {
                    l_ArraySize = (uint16)CDataLength;
                }
                // Otherwise we need to calculate the length by counting 
                // delimiters
                else
                {
                    while(l_pCDataString < l_pCDataEnd)
                    {
                        if(*l_pCDataString == *XML_CONST_STR(CHAR_ARRAY_DELIM))
                            l_ArraySize++;

                        l_pCDataString++;
                    }
                    l_pCDataString = ((const XML_Char*)pCDataString);
                }

                // Create a zeroed-out array of the specified size that we will
                // fill in directly ourselves.
                l_pObject->SetBuffer(NULL, l_ByteWidth, l_ArraySize, (uint8)(l_TypeInfo & DMOIB_TYPE_FLAG_FP));

                if((l_pDataPtr = (byte*)l_pObject->m_pMemberIter->Ptr) != NULL)
                {
                    // MOIB_TIME
                    if((l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_TIME)
                    {
                        while(l_pCDataString < l_pCDataEnd)
                        {
                            xmltotime((ansichar*)l_pCDataString, *((MOIB_TIME*)l_pDataPtr));
                            l_pCDataString += XML_TIME_LENGTH(*((MOIB_TIME*)l_pDataPtr)) + XML_CONST_SIZE(CHAR_ARRAY_DELIM);
                            l_pDataPtr += sizeof(MOIB_TIME);
                        }
                    }
                    // Text
                    else if(l_TypeInfo & DMOIB_TYPE_FLAG_TXT)
                    {
                        // Ansichar
                        if(l_ByteWidth == 1)
                        {
                            strncpy((ansichar*)l_pDataPtr, l_pCDataString, CDataLength);
                        }
                        // Widechar
                        else
                        {
                            strtowcs((widechar*)l_pDataPtr, l_pCDataString, CDataLength);
                        }                        
                    }
                    // Floating Point
                    else if(l_TypeInfo & DMOIB_TYPE_FLAG_FP)
                    {
                        // Float32
                        if(l_ByteWidth == 4)
                        {
                            while(l_pCDataString < l_pCDataEnd)
                            {
                                *((float32*)l_pDataPtr) = (float32)atof(l_pCDataString);
                                l_pCDataString += XML_FLOAT32_LENGTH + XML_CONST_SIZE(CHAR_ARRAY_DELIM);
                                l_pDataPtr += l_ByteWidth;
                            }
                        }
                        // Float64
                        else
                        {
                            while(l_pCDataString < l_pCDataEnd)
                            {
                                *((float64*)l_pDataPtr) = (float64)atof(l_pCDataString);
                                l_pCDataString += XML_FLOAT64_LENGTH + XML_CONST_SIZE(CHAR_ARRAY_DELIM);
                                l_pDataPtr += l_ByteWidth;
                            }
                        }
                    }
                    // All other Integers
                    else
                    {
                        // Unsigned
                        if(l_TypeInfo & DMOIB_TYPE_FLAG_UNS)
                        {
                            uint32 l_UintData;

                            while(l_pCDataString < l_pCDataEnd)
                            {
                                l_UintData = strtoul(l_pCDataString, NULL, 10);
                                BYTE_COPY(l_pDataPtr, l_ByteWidth, &l_UintData, sizeof(uint32));
                                l_pCDataString += XML_INT_LENGTH(l_UintData) + XML_CONST_SIZE(CHAR_ARRAY_DELIM);
                                l_pDataPtr += l_ByteWidth;
                            }
                        }
                        // Signed
                        else
                        {
                            int32 l_IntData;

                            while(l_pCDataString < l_pCDataEnd)
                            {
                                l_IntData = strtol(l_pCDataString, NULL, 10);
                                BYTE_COPY(l_pDataPtr, l_ByteWidth, &l_IntData, sizeof(int32));
                                l_pCDataString += XML_INT_LENGTH(l_IntData) + XML_CONST_SIZE(CHAR_ARRAY_DELIM);
                                l_pDataPtr += l_ByteWidth;
                            }
                        }
                    } 
                }
            }
            else
            {
                // MOIB_TIME
                if((l_TypeInfo & DMOIB_TYPE_MASK_BW) == DMOIB_TYPE_FLAG_TIME)
                {
                    xmltotime((ansichar*)l_pCDataString, l_pObject->m_pMemberIter->Time);
                }
                // Text
                else if(l_TypeInfo & DMOIB_TYPE_FLAG_TXT)
                {   
                    // Regardless of whether this is an ansichar or widechar
                    // member, the XML will be an ansichar. So just copy the
                    // ansichar from the string at this point into our generic
                    // integer space.
                    l_pObject->m_pMemberIter->Uint32 = *l_pCDataString;
                }
                // Floating Point
                else if(l_TypeInfo & DMOIB_TYPE_FLAG_FP)
                {
                    // Float32
                    if(l_ByteWidth == 4)
                    {
                        l_pObject->m_pMemberIter->Float32 = (float32)atof(l_pCDataString);
                    }
                    // Float64
                    else
                    {
                        l_pObject->m_pMemberIter->Float64 = (float64)atof(l_pCDataString);
                    }
                }
                // All other Integers
                else
                {
                    // Unsigned
                    if(l_TypeInfo & DMOIB_TYPE_FLAG_UNS)
                    {
                        uint32 l_UintData = strtoul(l_pCDataString, NULL, 10);

                        BYTE_COPY(&l_pObject->m_pMemberIter->Uint32, l_ByteWidth, &l_UintData, sizeof(uint32));
                    }
                    // Signed
                    else
                    {
                        int32 l_IntData = strtol(l_pCDataString, NULL, 10);

                        BYTE_COPY(&l_pObject->m_pMemberIter->Int32, l_ByteWidth, &l_IntData, sizeof(int32));
                    }
                } 
            }      
        }
    }
}
#endif


#endif


